namespace CRM_Perf_BenchMark
{
    using System;
    using System.Collections.Generic;
    using Microsoft.VisualStudio.TestTools.WebTesting;
    using System.Net;
    using System.Text;
    using System.Xml;
    using System.Web;
    
    /// <summary>
    /// 1.	Start at Workplace homepage
    /// 2.	Select Reports from Left Pane
    /// 3.	Select Sales History. (Double Click)
    /// 4.	Click Run Report
    /// </summary>

    public class AccountSummaryReport : CrmWebTest
    {
        public void AccountSummaryReport_PreWebTest(object sender, PreWebTestEventArgs e)
        {
            EntityRequest er = new EntityRequest();
            er.Type = EntityNames.Users;
            er.ReturnAs = EntityNames.Users;
            er.ParentID = EntityManager.Instance.OrgId;

            System.Collections.Hashtable userProps = new System.Collections.Hashtable();
            if (e.WebTest.Context.ContainsKey("UserRole"))
            {
                userProps.Add("role", e.WebTest.Context["UserRole"]);
                er.Props = userProps;
            }

            System.Collections.Hashtable Entities = EntityManager.Instance.GetEntities(er);

            user = (CRMEntity)Entities[EntityNames.Users];

            crmServer = user["CrmServer"];
            tokenKey = EntityManager.Instance.ReadTokenKey(user["organizationid"]);

            e.WebTest.UserName = user["domainName"];
            e.WebTest.Password = user["userpassword"];
            
            er = new EntityRequest();
            er.Type = EntityNames.Reports;
            er.ParentID = new Guid(user["organizationid"]);
            er.ReturnAs = EntityNames.Reports;

            System.Collections.Hashtable reportProps = new System.Collections.Hashtable();
            reportProps.Add("filename", "Account Summary.rdl");
            er.Props = reportProps;

            report = (CRMEntity)(EntityManager.Instance.GetEntities(er)[EntityNames.Reports]);
            EntityManager.Instance.FreeEntity(report);            
        }

        public void AccountSummaryReport_PostWebTest(object sender, PostWebTestEventArgs e)
        {
            EntityManager.Instance.FreeEntity(user);
        }

        public AccountSummaryReport()
        {
            WebRequest.RegisterPrefix("crm", new crmRequestFactory());

            this.Proxy = null;
            PreWebTest += new EventHandler<PreWebTestEventArgs>(AccountSummaryReport_PreWebTest);
            PostWebTest += new EventHandler<PostWebTestEventArgs>(AccountSummaryReport_PostWebTest);
        }

        public override IEnumerator<WebTestRequest> GetRequestEnumerator()
        {
            this.BeginTransaction("NavReportHomepage");

            CRMReports_home_reports_aspx reports = new CRMReports_home_reports_aspx(user);
            yield return (reports);

            this.EndTransaction("NavReportHomepage");

            this.BeginTransaction("OpenAccountSummaryReport");

            crmreports_viewer_viewer_aspx viewer = new crmreports_viewer_viewer_aspx(user);
            viewer.id = new Guid(report["reportid"]);
            viewer.helpId = report["filename"];
            viewer.action = "filter";
            yield return (viewer);

            this.EndTransaction("OpenAccountSummaryReport");

            this.BeginTransaction("AccountSummaryReport");

            AdvancedFind.AdvancedFind af = new AdvancedFind.AdvancedFind();
            WebTestRequest wtr = null;

            try { af.GetDefaultAdvancedFindView("account", false); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);
            }
            yield return (wtr);

            wtr = null;
            try { af.GetEntityFieldList("account"); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
            }
            yield return (wtr);

            try { af.GetLinkedEntityList("account"); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);
                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
            }
            yield return (wtr);

            CRMReports_rsviewer_reportviewer_aspx rsviewer = new CRMReports_rsviewer_reportviewer_aspx(crmServer);
            rsviewer.id = new Guid(report["reportid"]);
            rsviewer.uniquename = user["organizationName"];
            rsviewer.isScheduledReport = false;
            string temp = "%3CReportFilter%3E%3CReportEntity+paramname%3D%22CRM_FilteredAccount%22+displayname%3D%22Accounts%22%3E%3Cfetch+version%3D%221.0%22+output-format%3D%22xml-platform%22+mapping%3D%22logical%22+distinct%3D%22false%22%3E%3Centity+name%3D%22account%22%3E%3Call-attributes%2F%3E%3C%2Fentity%3E%3C%2Ffetch%3E%0D%0A%3C%2FReportEntity%3E%3C%2FReportFilter%3E";
            temp = HttpUtility.UrlDecode(temp);
            rsviewer.CRM_filter = temp;
            yield return (rsviewer);

            int Idx = rsviewer.lastResponse.BodyString.IndexOf(idxMarker);
            string match = rsviewer.lastResponse.BodyString.Substring(Idx + 13);
            string redirect = match.Substring(0, match.IndexOf('"'));
            redirect = redirect.Replace("&amp;", "&");

            Reserved_ReportViewerWebControl_axd reportViewer = new Reserved_ReportViewerWebControl_axd(crmServer);
            reportViewer.Url = reportViewer.Url + redirect;
            yield return (reportViewer);

            Idx = reportViewer.lastResponse.BodyString.IndexOf(idxMarker2);
            string match2 = reportViewer.lastResponse.BodyString.Substring(Idx + 12);
            string redirect2 = match2.Substring(0, match2.IndexOf('"'));
            redirect2 = redirect2.Replace("&amp;", "&");

            reportViewer = new Reserved_ReportViewerWebControl_axd(crmServer);
            reportViewer.Url = reportViewer.Url + redirect2;
            yield return (reportViewer);

            string partTwo = reportViewer.lastResponse.BodyString;
            Idx = partTwo.IndexOf(idxMarker3);
            while (Idx > 0)
            {
                string match3 = partTwo.Substring(Idx + 5);
                string redirect3 = match3.Substring(0, match3.IndexOf('"'));
                partTwo = match3.Substring(match3.IndexOf('"') + 1);
                redirect3 = redirect3.Replace("&amp;", "&");

                reportViewer = new Reserved_ReportViewerWebControl_axd(crmServer);
                reportViewer.Url = reportViewer.Url + redirect3;
                yield return (reportViewer);

                Idx = partTwo.IndexOf(idxMarker3);
            }

            this.EndTransaction("AccountSummaryReport");
        }

        private string idxMarker = "<iframe src=\"/Reserved.ReportViewerWebControl.axd?";
        private string idxMarker2 = "<frame src=\"/Reserved.ReportViewerWebControl.axd?";
        private string idxMarker3 = "SRC=\"/Reserved.ReportViewerWebControl.axd?";
        private CRMEntity user;
        private CRMEntity report;
        private string crmServer;

        WRPCInfo rpcInfo;
        WebTestRequestHeader headerItem;
        private string tokenKey;
    }
}



