﻿namespace CRM_Perf_BenchMark
{
	using System;
	using System.Collections.Generic;
	using Microsoft.VisualStudio.TestTools.WebTesting;
	using System.Net;

    /// <summary>
    ///1. Start point Homepage
    ///2. Select Tools, Advanced Find
    ///3. Select Accounts
    ///4. Fields – Account Name
    ///5. Account Name Contains "Random string"
    ///6. Select Find Search
    /// </summary>

	public class AdvancedFindAccount : CrmWebTest
	{
		public void AdvancedFindAccount_PreWebTest(object sender, PreWebTestEventArgs e)
		{
            EntityRequest er = new EntityRequest();
            er.Type = EntityNames.Users;
            er.ReturnAs = EntityNames.Users;
            er.ParentID = EntityManager.Instance.OrgId;

            System.Collections.Hashtable userProps = new System.Collections.Hashtable();
            if (e.WebTest.Context.ContainsKey("UserRole"))
            {
                userProps.Add("role", e.WebTest.Context["UserRole"]);
                er.Props = userProps;
            }

            System.Collections.Hashtable Entities = EntityManager.Instance.GetEntities(er);

            user = (CRMEntity)Entities[EntityNames.Users];

            e.WebTest.UserName = user["domainName"];
            e.WebTest.Password = user["userpassword"];
            
            crmServer = user["CrmServer"];
            tokenKey = EntityManager.Instance.ReadTokenKey(user["organizationid"]);
		}

		public void AdvancedFindAccount_PostWebTest(object sender, PostWebTestEventArgs e)
		{
			EntityManager.Instance.FreeEntity(user);
		}

		public AdvancedFindAccount()
		{
            WebRequest.RegisterPrefix("crm", new crmRequestFactory());
			this.Proxy = null;            
			PreWebTest += new EventHandler<PreWebTestEventArgs>(AdvancedFindAccount_PreWebTest);
			PostWebTest += new EventHandler<PostWebTestEventArgs>(AdvancedFindAccount_PostWebTest);
		}

		public override IEnumerator<WebTestRequest> GetRequestEnumerator()
		{
            this.BeginTransaction("AdvancedFindAccount");

            AdvancedFind_AdvFind_aspx AdvFind_aspx = new AdvancedFind_AdvFind_aspx(user);
			AdvFind_aspx.EntityCode = 9100;
            AdvFind_aspx.QueryId = new Guid("{687635FF-5117-40C8-9A04-3467307134E1}");
			AdvFind_aspx.ViewType = 1039;
			yield return (AdvFind_aspx);

			AdvancedFind.AdvancedFind af = new AdvancedFind.AdvancedFind();
			WebTestRequest wtr = null;

			try { af.GetQueryList("report", 0, true, true, false); }
			catch (crmException x)
			{
				wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);
			}
			yield return (wtr);

            try { af.GetSystemQuery("{687635FF-5117-40C8-9A04-3467307134E1}"); }
			catch (crmException x)
			{
				wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);
			}
			yield return (wtr);

			try { af.GetDefaultAdvancedFindView("report", false); }
			catch (crmException x)
			{
				wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);
			}
			yield return (wtr);

			try { af.GetLinkedEntityList("report"); }
			catch (crmException x)
			{
				wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
			}
			yield return (wtr);

			try { af.GetEntityFieldList("report"); }
			catch (crmException x)
			{
				wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
			}
			yield return (wtr);

			// Redo the 5 calls to advancedFind webservice for activityparty
			af = new AdvancedFind.AdvancedFind();
			wtr = null;

			try { af.GetEntityFieldList("reportvisibility"); }
			catch (crmException x)
			{
				wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
			}
			yield return (wtr);

			try { af.GetLinkedEntityList("reportvisibility"); }
			catch (crmException x)
			{
				wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
			}
			yield return (wtr);

			try { af.GetQueryList("account", 0, true, true, false); }
			catch (crmException x)
			{
				wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
			}
			yield return (wtr);

			try { af.GetSystemQuery("{00000000-0000-0000-00AA-000000666000}"); }
			catch (crmException x)
			{
				wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
			}
			yield return (wtr);

			try { af.GetDefaultAdvancedFindView("account", false); }
			catch (crmException x)
			{
				wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
			}
			yield return (wtr);

			af = new AdvancedFind.AdvancedFind();
			wtr = null;

			try { af.GetEntityFieldList("account"); }
			catch (crmException x)
			{
				wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
			}
			yield return (wtr);

			try { af.GetLinkedEntityList("account"); }
			catch (crmException x)
			{
				wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
			}
			yield return (wtr);

            AdvancedFind_fetchData_aspx fetchData_aspx = new AdvancedFind_fetchData_aspx(user);

			fetchData_aspx.FetchXml = fetchXml;
			fetchData_aspx.LayoutXml = layoutXml;
			fetchData_aspx.EntityName = entityName;
			fetchData_aspx.ViewId = viewId;
            fetchData_aspx.DefaultAdvFindViewId = viewId;
			fetchData_aspx.ViewType = viewType;
			fetchData_aspx.SortCol = sortCol;
			fetchData_aspx.SortDescend = sortDescend;

			yield return (fetchData_aspx);

			this.EndTransaction("AdvancedFindAccount");
		}

        public string fetchXml = @"<fetch version=""1.0"" output-format=""xml-platform"" mapping=""logical"" distinct=""false""><entity name=""account""><attribute name=""name""/><attribute name=""primarycontactid""/><attribute name=""telephone1""/><attribute name=""accountid""/><order attribute=""name"" descending=""false""/><filter type=""and""><condition attribute=""name"" operator=""like"" value=""&#x25;" + Utils.GetRandomString(4,4) + @"a&#x25;""/></filter></entity></fetch>";
		public string layoutXml = @"<grid name=""resultset"" object=""1"" jump=""name"" select=""1"" icon=""1"" preview=""1""><row name=""result"" id=""accountid""><cell name=""name"" width=""300"" /><cell name=""primarycontactid"" width=""150"" /><cell name=""telephone1"" width=""100"" /></row></grid>";
		public string entityName = "account";
		public Guid viewId = new Guid("{00000000-0000-0000-00AA-000000666000}");
		public int viewType = 1039;
		public string sortCol = "name";
		public bool sortDescend = false;

		private CRMEntity user;
        private string crmServer;
        private string tokenKey;
        private WRPCInfo rpcInfo;
        private WebTestRequestHeader headerItem;
	}
}
