namespace CRM_Perf_BenchMark
{
    using System;
    using System.Collections.Generic;
    using Microsoft.VisualStudio.TestTools.WebTesting;
    using System.Net;
    using System.Text;
    using System.Xml;
    
    /// <summary>
    /// 1. Start point Sales Tab
    /// 2. Click on Accounts
    /// 3. Insert name criteria in Find box and Select Find button
    /// 4. Open an Account
    /// 5. Select Actions > Assign from menu
    /// 6. Click Assign to another user
    /// 7. Select a User
    /// 8. Click Ok
    /// 9. Save and Close
    /// </summary>

    public class AssignAccount : CrmWebTest
    {
        public void AssignAccount_PreWebTest(object sender, PreWebTestEventArgs e)
        {
            EntityRequest er = new EntityRequest();
            er.Type = EntityNames.Users;
            er.ReturnAs = EntityNames.Users;
            er.ParentID = EntityManager.Instance.OrgId;

            System.Collections.Hashtable Props = new System.Collections.Hashtable();

            if (e.WebTest.Context.ContainsKey("UserRole"))
            {
                Props.Add("role", e.WebTest.Context["UserRole"]);
            }
            
            EntityRequest AccountEr = new EntityRequest();
            AccountEr.Type = EntityNames.Accounts;
            AccountEr.ParentID = Guid.NewGuid();
            AccountEr.ReturnAs = EntityNames.Accounts;

            Props.Add(EntityNames.Accounts, new EntityRequest[] { AccountEr });

            er.Props = Props;

            System.Collections.Hashtable Entities = EntityManager.Instance.GetEntities(er);

            user = (CRMEntity)Entities[EntityNames.Users];
            account = (CRMEntity)Entities[EntityNames.Accounts];
            crmServer = user["CrmServer"];
            tokenKey = EntityManager.Instance.ReadTokenKey(user["organizationid"]);

            e.WebTest.UserName = user["domainName"];
            e.WebTest.Password = user["userpassword"];
            
            user2 = EntityManager.Instance.GetNextUser(new Guid(user["organizationid"]));            
        }
        
        public void AssignAccount_PostWebTest(object sender, PostWebTestEventArgs e)
        {
            try
            {
                if (success)
                {
                    string newAcctId = account[EntityIDNames.Account];
                    if (newAcctId != null)
                    {
                        Guid g = EntityManager.Instance.GetOwnerGuid(new Guid(user2[EntityIDNames.User]), new Guid(user2["OrganizationId"]));
                        EntityManager.Instance.AddEntity(EntityNames.Accounts, g, new string[] { "OwningUser", "AccountID", "name", "EntityManagerOwningUser" }, new string[] { user2["systemuserid"], account[EntityIDNames.Account], account["name"], g.ToString() });
                    }
                }
            }
            finally
            {
                EntityManager.Instance.FreeEntity(user);
                EntityManager.Instance.FreeEntity(user2);
                EntityManager.Instance.DeleteEntity(account);                
            }
        }

        private void AssignAccount_ValidateResponse(object sender, ValidationEventArgs e)
        {
            if (e.Response.StatusCode == HttpStatusCode.OK)
            {
                e.IsValid = true;
                success = true;
            }
            else
            {
                e.IsValid = false;
                success = false;
            }
        }

        public AssignAccount()
        {
            WebRequest.RegisterPrefix("crm", new crmRequestFactory());
            string random = Utils.GetRandomString(2, 2);
            gridXml = GridXmlHelper.AccountSearchView(-1, searchAccountView, random, null);
            refreshGridXml = GridXmlHelper.AccountPageSearchViewRefresh(-1, searchAccountView, random, null);

            this.Proxy = null;            
            PreWebTest += new EventHandler<PreWebTestEventArgs>(AssignAccount_PreWebTest);
            PostWebTest += new EventHandler<PostWebTestEventArgs>(AssignAccount_PostWebTest);
        }

        public override IEnumerator<WebTestRequest> GetRequestEnumerator()
        {
            this.BeginTransaction("NavAccountHomepage");

            // Get the home page
            _root_homepage_aspx hp = new _root_homepage_aspx(user);
            hp.etc = 1;
            yield return (hp);

            this.EndTransaction("NavAccountHomepage");

            this.BeginTransaction("SearchAccounts");

            // Search for the Account name
            AppGridWebService.AppGridWebService ag = new AppGridWebService.AppGridWebService();
            WebTestRequest wtr = null;

            try { ag.Reset(gridXml, gridId); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
            }
            yield return (wtr);

            this.EndTransaction("SearchAccounts");

            this.BeginTransaction("OpenAccount");

            // simulate double clicking of account
            sfa_accts_edit_aspx dp = new sfa_accts_edit_aspx(user);
            dp.id = account[EntityIDNames.Account];
            yield return (dp);

            referer = dp.lastResponse.ResponseUri.AbsoluteUri;

            _forms_FormScript_js_aspx fs = new _forms_FormScript_js_aspx(user);
            fs.etc = 1;
            fs.FormType = "main";
            fs.FormName = "crmForm";
            headerItem = new WebTestRequestHeader();
            headerItem.Name = "referer";
            headerItem.Value = referer;
            fs.Headers.Add(headerItem);
            yield return (fs);                        

            this.EndTransaction("OpenAccount");
           
            this.BeginTransaction("AssignAccount");

            _grid_cmds_dlg_frmassign_aspx af = new _grid_cmds_dlg_frmassign_aspx(user);
            af.pId = new Guid(account[EntityIDNames.Account]);
            af.pType = 1;
            af.iObjtype = 1;
            af.iTotal = 1;
            yield return (af);

            // Lookup for a user when assigning to other user
            _controls_lookup_lookupsingle_aspx lookup = new _controls_lookup_lookupsingle_aspx(user);
            lookup.Class = "BasicOwner";
            lookup.objecttypes = "8";
            lookup.ShowNewButton = 1;
            lookup.ShowPropButton = 1;
            lookup.browse = 0;
            lookup.DefaultType = 0;
            yield return (lookup);

            // account update form
            dp = new sfa_accts_edit_aspx(user);
            dp.id = new Guid(account[EntityIDNames.Account]).ToString("B");
            dp.crmFormSubmitXml = "<account></account>";
            dp.CrmFormUserModified = false;
            dp.crmFormSubmitMode = 47;
            dp.crmFormSubmitId = new Guid(account[EntityIDNames.Account]).ToString("B");
            dp.crmFormSubmitObjectType = 1;
            dp.crmFormSubmitSecurity = 852023;
            dp.crmFormSubmitOnline = 1;            
            dp.assignOwnerId = new Guid(user2["systemuserid"]);
            dp.assignOwnerType = 8;

            rpcInfo = Utils.GenerateWRPCToken(sfa_accts_edit_aspx.url, user, tokenKey);
            dp.CRMWRPCToken = rpcInfo.token;
            dp.CRMWRPCTokenTimeStamp = rpcInfo.timeStamp;
            dp.ValidateResponse += this.AssignAccount_ValidateResponse;
            yield return (dp);

            referer = dp.lastResponse.ResponseUri.AbsoluteUri;

            fs = new _forms_FormScript_js_aspx(user);
            fs.etc = 1;
            fs.FormType = "main";
            fs.FormName = "crmForm";
            headerItem = new WebTestRequestHeader();
            headerItem.Name = "referer";
            headerItem.Value = referer;
            fs.Headers.Add(headerItem);
            yield return (fs);

            ag = new AppGridWebService.AppGridWebService();

            try { ag.Refresh(refreshGridXml); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
            }
            yield return (wtr);

            this.EndTransaction("AssignAccount");            
        }

        private string gridXml;
        private string refreshGridXml;
        private static string gridId = "crmGrid";
        private static string searchAccountView = "2D1187C4-23FE-4BB5-9647-43BB1C6DDBD1";
        private bool success;

        private WebTestRequestHeader headerItem;
        private WRPCInfo rpcInfo;
        private string referer;

        private CRMEntity user;
        private CRMEntity user2;
        private CRMEntity account;
        private string crmServer;
        private string tokenKey;
    }
}