namespace CRM_Perf_BenchMark
{
    using System;
    using System.Collections.Generic;
    using Microsoft.VisualStudio.TestTools.WebTesting;
    using System.Net;
    using System.Text;
    using System.Xml;
    
    /// <summary>
    /// 1. Start point Sales Tab
    /// 2. Click on Contacts
    /// 3. Insert name criteria in Find box and Select Find button
    /// 4. Open a Contact
    /// 5. Select Actions > Assign from menu
    /// 6. Click Assign to another user
    /// 7. Select a User
    /// 8. Click Ok
    /// 9. Save and Close
    /// </summary>

    public class AssignContact : CrmWebTest
    {
        public void AssignContact_PreWebTest(object sender, PreWebTestEventArgs e)
        {
            EntityRequest er = new EntityRequest();
            er.Type = EntityNames.Users;
            er.ReturnAs = EntityNames.Users;
            er.ParentID = EntityManager.Instance.OrgId;

            System.Collections.Hashtable Props = new System.Collections.Hashtable();

            if (e.WebTest.Context.ContainsKey("UserRole"))
            {
                Props.Add("role", e.WebTest.Context["UserRole"]);
            }
            
            EntityRequest ContactEr = new EntityRequest();
            ContactEr.Type = EntityNames.Contacts;
            ContactEr.ParentID = Guid.NewGuid();
            ContactEr.ReturnAs = EntityNames.Contacts;

            Props.Add(EntityNames.Contacts, new EntityRequest[] { ContactEr });

            er.Props = Props;

            System.Collections.Hashtable Entities = EntityManager.Instance.GetEntities(er);

            user = (CRMEntity)Entities[EntityNames.Users];
            contact = (CRMEntity)Entities[EntityNames.Contacts];
            crmServer = user["CrmServer"];
            tokenKey = EntityManager.Instance.ReadTokenKey(user["organizationid"]);

            e.WebTest.UserName = user["domainName"];
            e.WebTest.Password = user["userpassword"];

            user2 = EntityManager.Instance.GetNextUser(new Guid(user["organizationid"]));            
        }
                
        public void AssignContact_PostWebTest(object sender, PostWebTestEventArgs e)
        {
            try
            {
                if (success)
                {
                    string newContactId = contact[EntityIDNames.Contact];
                    if (newContactId != null)
                    {
                        Guid g = EntityManager.Instance.GetOwnerGuid(new Guid(user2[EntityIDNames.User]), new Guid(user2["OrganizationId"]));
                        EntityManager.Instance.AddEntity(EntityNames.Contacts, g, new string[] { "OwningUser", "ContactID", "EntityManagerOwningUser" }, new string[] { user2["systemuserid"], contact[EntityIDNames.Contact], g.ToString() });
                    }
                }
            }
            finally
            {
                EntityManager.Instance.FreeEntity(user);
                EntityManager.Instance.FreeEntity(user2);
                EntityManager.Instance.DeleteEntity(contact);
            }
        }

        private void AssignContact_ValidateResponse(object sender, ValidationEventArgs e)
        {
            if (e.Response.StatusCode == HttpStatusCode.OK)
            {
                e.IsValid = true;
                success = true;
            }
            else
            {
                e.IsValid = false;
                success = false;
            }
        }

        public AssignContact()
        {
            WebRequest.RegisterPrefix("crm", new crmRequestFactory());
            string random = Utils.GetRandomString(2, 2);
            gridXml = GridXmlHelper.ContactSearchView(-1, searchContactsView, random, null);
            gridRefreshXml = GridXmlHelper.ContactPageSearchViewRefresh(-1, searchContactsView, random, null);

            this.Proxy = null;            
            PreWebTest += new EventHandler<PreWebTestEventArgs>(AssignContact_PreWebTest);
            PostWebTest += new EventHandler<PostWebTestEventArgs>(AssignContact_PostWebTest);
        }

        public override IEnumerator<WebTestRequest> GetRequestEnumerator()
        {
            this.BeginTransaction("NavContactHomepage");

            // Get the home page
            _root_homepage_aspx hp = new _root_homepage_aspx(user);
            hp.etc = 2;
            yield return (hp);

            this.EndTransaction("NavContactHomepage");

            this.BeginTransaction("SearchContacts");

            // Search for the Contact name
            AppGridWebService.AppGridWebService ag = new AppGridWebService.AppGridWebService();
            WebTestRequest wtr = null;

            try { ag.Reset(gridXml, gridId); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
            }
            yield return (wtr);

            this.EndTransaction("SearchContacts");

            this.BeginTransaction("OpenContact");

            // simulate double clicking of contact
            sfa_conts_edit_aspx dp = new sfa_conts_edit_aspx(user);
            dp.id = contact[EntityIDNames.Contact];
            yield return (dp);

            referer = dp.lastResponse.ResponseUri.ToString();

            _forms_FormScript_js_aspx fs = new _forms_FormScript_js_aspx(user);
            fs.etc = 1;
            fs.FormType = "main";
            fs.FormName = "crmForm";
            headerItem = new WebTestRequestHeader();
            headerItem.Name = "referer";
            headerItem.Value = referer;
            fs.Headers.Add(headerItem);
            yield return (fs);

            this.EndTransaction("OpenContact");
            this.BeginTransaction("AssignContact");

            _grid_cmds_dlg_frmassign_aspx af = new _grid_cmds_dlg_frmassign_aspx(user);
            af.pId = new Guid(contact[EntityIDNames.Contact]);
            af.pType = 2;
            af.iObjtype = 2;
            af.iTotal = 1;
            yield return (af);

            // Lookup for a user when assigning to other user
            _controls_lookup_lookupsingle_aspx lookup = new _controls_lookup_lookupsingle_aspx(user);
            lookup.Class = "BasicOwner";
            lookup.objecttypes = "8";
            lookup.browse = 0;
            lookup.ShowNewButton = 1;
            lookup.ShowPropButton = 1;
            lookup.DefaultType = 0;
            yield return (lookup);

            // contact update form
            dp = new sfa_conts_edit_aspx(user);
            dp.id = new Guid(contact[EntityIDNames.Contact]).ToString("B");
            dp.crmFormSubmitXml = "<contact></contact>"; 
            dp.crmFormSubmitMode = 47;
            dp.crmFormSubmitId = new Guid(contact[EntityIDNames.Contact]).ToString("B");
            dp.crmFormUserModified = true;
            dp.crmFormSubmitObjectType = 2;
            dp.crmFormSubmitSecurity = 852023;
            dp.crmFormSubmitOnline = 1;            
            dp.assignOwnerId = new Guid(user2["systemuserid"]);
            dp.assignOwnerType = 8;

            rpcInfo = Utils.GenerateWRPCToken(sfa_conts_edit_aspx.url, user, tokenKey);
            dp.CRMWRPCToken = rpcInfo.token;
            dp.CRMWRPCTokenTimeStamp = rpcInfo.timeStamp;
            dp.ValidateResponse += this.AssignContact_ValidateResponse;
            yield return (dp);

            referer = dp.lastResponse.ResponseUri.ToString();

            fs = new _forms_FormScript_js_aspx(user);
            fs.etc = 1;
            fs.FormType = "main";
            fs.FormName = "crmForm";
            headerItem = new WebTestRequestHeader();
            headerItem.Name = "referer";
            headerItem.Value = referer;
            fs.Headers.Add(headerItem);
            yield return (fs);

            ag = new AppGridWebService.AppGridWebService();
            wtr = null;

            try { ag.Refresh(gridRefreshXml); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
            }
            yield return (wtr);
                        

            this.EndTransaction("AssignContact");            
        }

        private string gridXml;
        private string gridRefreshXml;
        private static string gridId = "crmGrid";
        private static string searchContactsView = "8DF19B44-A073-40C3-9D6D-EE1355D8C4BA";
        private bool success;

        private CRMEntity user;
        private CRMEntity user2;
        private CRMEntity contact;
        private string crmServer;
        private string tokenKey;
        private string referer;

        private WRPCInfo rpcInfo;
        private WebTestRequestHeader headerItem;
    }
}