namespace CRM_Perf_BenchMark
{
    using System;
    using System.Collections.Generic;
    using Microsoft.VisualStudio.TestTools.WebTesting;
    using System.Net;
    using System.Text;
    using System.Xml;
    
	/// <summary>
	/// 1. Starts at Sales homepage
	/// 2. Click on Opportunities
	/// 3. Select Open Opportunities view
	/// 4. Add random string of two characters in look for textbox and click find
	/// 5. Double click an opportunity
	/// 6. Select Assign from Actions menu
	/// 7. Select Assign to another user radio button
	/// 8. Select a user from the search menu
	/// 9. Click OK
	/// 10. Click Save & Close
	/// </summary>
    
    public class AssignOpportunity : CrmWebTest
    {
        public void AssignOpportunity_PreWebTest(object sender, PreWebTestEventArgs e)
        {
			EntityRequest er = new EntityRequest();
			er.Type = EntityNames.Users;
			er.ReturnAs = EntityNames.Users;
			er.ParentID = EntityManager.Instance.OrgId;

			System.Collections.Hashtable Props = new System.Collections.Hashtable();

            if (e.WebTest.Context.ContainsKey("UserRole"))
            {
                Props.Add("role", e.WebTest.Context["UserRole"]);
            }
            
			EntityRequest OpportunityEr = new EntityRequest();
			OpportunityEr.Type = EntityNames.Opportunities;
			OpportunityEr.ParentID = Guid.NewGuid();
			OpportunityEr.ReturnAs = EntityNames.Opportunities;

			Props.Add(EntityNames.Opportunities, new EntityRequest[] { OpportunityEr });

			er.Props = Props;

			System.Collections.Hashtable Entities = EntityManager.Instance.GetEntities(er);

			user = (CRMEntity)Entities[EntityNames.Users];
			opportunity = (CRMEntity)Entities[EntityNames.Opportunities];
            crmServer = user["CrmServer"];
            tokenKey = EntityManager.Instance.ReadTokenKey(user["organizationid"]);
            
            user2 = EntityManager.Instance.GetNextUser(new Guid(user["organizationid"]));

            e.WebTest.UserName = user["domainName"];
            e.WebTest.Password = user["userpassword"];            
        }

        public void AssignOpportunity_PostWebTest(object sender, PostWebTestEventArgs e)
        {
            try
            {
                if (success)
                {
                    string newOppId = opportunity[EntityIDNames.Contact];
                    if (newOppId != null)
                    {
                        Guid g = EntityManager.Instance.GetOwnerGuid(new Guid(user2[EntityIDNames.User]), new Guid(user2["OrganizationId"]));
                        EntityManager.Instance.AddEntity(EntityNames.Opportunities, g, new string[] { "OwningUser", "OpportunityID", "EntityManagerOwningUser" }, new string[] { user2["systemuserid"], opportunity[EntityIDNames.Opportunities], g.ToString() });
                    }
                }
            }
            finally
            {
                EntityManager.Instance.FreeEntity(user);
                EntityManager.Instance.FreeEntity(user2);
                EntityManager.Instance.DeleteEntity(opportunity);
            }
        }

        private void AssignOpportunity_ValidateResponse(object sender, ValidationEventArgs e)
        {
            if (e.Response.StatusCode == HttpStatusCode.OK)
            {
                e.IsValid = true;
                success = true;
            }
            else
            {
                e.IsValid = false;
                success = false;
            }
        }

        public AssignOpportunity()
        {
            WebRequest.RegisterPrefix("crm", new crmRequestFactory());
            string random = Utils.GetRandomString(2, 2);
            gridXml = GridXmlHelper.OpportunityPageView(-1, openOpportunitiesView, null);
            gridXml2 = GridXmlHelper.OpportunitySearchView(-1, searchOppView, random, null);
            gridRefreshXml = GridXmlHelper.OpportunityPageSearchViewRefresh(-1, searchOppView, random, null);

            this.Proxy = null;            
            PreWebTest += new EventHandler<PreWebTestEventArgs>(AssignOpportunity_PreWebTest);
            PostWebTest += new EventHandler<PostWebTestEventArgs>(AssignOpportunity_PostWebTest);
        }

        public override IEnumerator<WebTestRequest> GetRequestEnumerator()
        {
            AppGridWebService.AppGridWebService ag;
            WebTestRequest wtr;

            this.BeginTransaction("NavOpportunityHomepage");

            // Get the home page
            _root_homepage_aspx hp = new _root_homepage_aspx(user);
            hp.etc = 3;
            yield return (hp);

            this.EndTransaction("NavOpportunityHomepage");

            if (ConfigSettings.Default.alltasks == true)
            {
                this.BeginTransaction("NavOpenOpportunities");

                // Open Opportunities view.   
                ag = new AppGridWebService.AppGridWebService();
                wtr = null;

                try { ag.Reset(gridXml, gridId); }
                catch (crmException x)
                {
                    wtr = (CrmRequest)x.wtr;
                    wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                    string urlPath = wtr.Url.Remove(0, crmServer.Length);
                    urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                    rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                    headerItem = new WebTestRequestHeader();
                    headerItem.Name = "crmwrpctokentimestamp";
                    headerItem.Value = rpcInfo.timeStamp.ToString();
                    wtr.Headers.Add(headerItem);
                    headerItem = new WebTestRequestHeader();
                    headerItem.Name = "crmwrpctoken";
                    headerItem.Value = rpcInfo.token;
                    wtr.Headers.Add(headerItem);
                    wtr.Method = "Post";
                }
                yield return (wtr);

                this.EndTransaction("NavOpenOpportunities");
            }

			this.BeginTransaction("SearchOpportunities");

            //Search for opportunities with a random string name.
			ag = new AppGridWebService.AppGridWebService();
			wtr = null;

			try { ag.Reset(gridXml2, gridId); }
			catch (crmException x)
			{
				wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
			}
			yield return (wtr);

            this.EndTransaction("SearchOpportunities");

            this.BeginTransaction("OpenOpportunity");

            // simulate double clicking of opportunity
            sfa_opps_edit_aspx dp = new sfa_opps_edit_aspx(user);
            dp.id = opportunity["opportunityid"];
            yield return (dp);

            referer = dp.lastResponse.ResponseUri.ToString();

            _forms_FormScript_js_aspx fs = new _forms_FormScript_js_aspx(user);
            fs.etc = 1;
            fs.FormType = "main";
            fs.FormName = "crmForm";
            headerItem = new WebTestRequestHeader();
            headerItem.Name = "referer";
            headerItem.Value = referer;
            fs.Headers.Add(headerItem);
            yield return (fs);

            this.EndTransaction("OpenOpportunity");
            
			this.BeginTransaction("AssignOpportunity");

            _grid_cmds_dlg_frmassign_aspx af = new _grid_cmds_dlg_frmassign_aspx(user);
            af.pId = new Guid(opportunity["opportunityid"]);
            af.pType = 3;
            af.iObjtype = 3;
            af.iTotal = 1;
            yield return (af);

            _controls_lookup_lookupsingle_aspx ls = new _controls_lookup_lookupsingle_aspx(user);
			ls.Class = "BasicOwner";
			ls.objecttypes = "8";
			ls.browse = 0;
            ls.ShowNewButton = 1;
            ls.ShowPropButton = 1;
			ls.DefaultType = 0;
			yield return (ls);

            // opportunity update form
            dp = new sfa_opps_edit_aspx(user);
            dp.crmFormSubmitXml = "<opportunity></opportunity>";
                        
            dp.crmFormSubmitMode = 47;
            dp.crmFormSubmitId = new Guid(opportunity["opportunityid"]);
            dp.crmFormUserModified = true;
            dp.crmFormSubmitObjectType = 3;
            dp.crmFormSubmitSecurity = 852023;
            dp.crmFormSubmitOnline = 1;
            
            dp.id = opportunity["opportunityid"];
            dp.assignOwnerId = new Guid(user2["systemuserid"]);
            dp.assignOwnerType = 8;
            rpcInfo = Utils.GenerateWRPCToken(sfa_opps_edit_aspx.url, user, tokenKey);
            dp.CRMWRPCToken = rpcInfo.token;
            dp.CRMWRPCTokenTimeStamp = rpcInfo.timeStamp;
            dp.ValidateResponse += this.AssignOpportunity_ValidateResponse;
            yield return (dp);

            referer = dp.lastResponse.ResponseUri.ToString();

            fs = new _forms_FormScript_js_aspx(user);
            fs.etc = 1;
            fs.FormType = "main";
            fs.FormName = "crmForm";
            headerItem = new WebTestRequestHeader();
            headerItem.Name = "referer";
            headerItem.Value = referer;
            fs.Headers.Add(headerItem);
            yield return (fs);

            ag = new AppGridWebService.AppGridWebService();
            wtr = null;

            try { ag.Refresh(gridRefreshXml); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
            }
            yield return (wtr);

            this.EndTransaction("AssignOpportunity");
        }

        private string gridXml;
        private string gridXml2;
        private string gridRefreshXml;
        private static string gridId = "crmGrid";
        private string referer;

        private CRMEntity user;
        private CRMEntity user2;
        private CRMEntity opportunity;
        private string crmServer;
        private string tokenKey;
        private bool success;
		private static string searchOppView = "3F6D889E-6549-4025-9CE0-4C4103EFB6A3";
		private static string openOpportunitiesView = "00000000-0000-0000-00AA-000010003001";

        private WRPCInfo rpcInfo;
        private WebTestRequestHeader headerItem;
    }
}