namespace CRM_Perf_BenchMark
{
    using System;
    using System.Collections.Generic;
    using Microsoft.VisualStudio.TestTools.WebTesting;
    using System.Net;
    using System.Text;
    using System.Xml;
    
    /// <summary>
    /// Start point Settings
    /// 1.	Select Users
    /// 2.	Double click single user to open
    /// 3.	Select roles 
    /// 4.	Click Manage Roles
    /// 5.	Add Role by selecting check box
    /// 6.	OK
    /// 7.	Click Save & Close
    /// </summary>
   
    public class ChangeRoleForUser : CrmWebTest
    {
        public void ChangeRoleForUser_PreWebTest(object sender, PreWebTestEventArgs e)
        {
            EntityRequest er = new EntityRequest();
            er.Type = EntityNames.Users;
            er.ReturnAs = EntityNames.Users;
            er.ParentID = EntityManager.Instance.OrgId;

            System.Collections.Hashtable userProps = new System.Collections.Hashtable();
            if (e.WebTest.Context.ContainsKey("UserRole"))
            {
                userProps.Add("role", e.WebTest.Context["UserRole"]);
                er.Props = userProps;
            }
            
            System.Collections.Hashtable Entities = EntityManager.Instance.GetEntities(er);

            user = (CRMEntity)Entities[EntityNames.Users];

            e.WebTest.UserName = user["domainName"];
            e.WebTest.Password = user["userpassword"];
            
            role = EntityManager.Instance.GetNextORGEntity(EntityNames.Roles, new Guid(user["organizationid"]));
            businessunit = EntityManager.Instance.GetNextORGEntity(EntityNames.BusinessUnits, new Guid(user["organizationid"]));
            crmServer = user["CrmServer"];
            tokenKey = EntityManager.Instance.ReadTokenKey(user["organizationid"]);
        }
       
        public void ChangeRoleForUser_PostWebTest(object sender, PostWebTestEventArgs e)
        {
            EntityManager.Instance.FreeEntity(user);
            EntityManager.Instance.FreeEntity(role);
            EntityManager.Instance.FreeEntity(businessunit);
        }

        public ChangeRoleForUser()
        {
            WebRequest.RegisterPrefix("crm", new crmRequestFactory());
            this.Proxy = null;            
            PreWebTest += new EventHandler<PreWebTestEventArgs>(ChangeRoleForUser_PreWebTest);
            PostWebTest += new EventHandler<PostWebTestEventArgs>(ChangeRoleForUser_PostWebTest);
        }

        public override IEnumerator<WebTestRequest> GetRequestEnumerator()
        {
            ChangeRoleForUser.gridXml = GridXmlHelper.UserRolePageRefresh(user[EntityIDNames.User]);

            this.BeginTransaction("ChangeUserRole");

            tools_Admin_admin_aspx adminpage = new tools_Admin_admin_aspx(user);
            adminpage.pid = 07;
            yield return (adminpage);

            tools_business_home_user_aspx hu = new tools_business_home_user_aspx(user);
            yield return (hu);

            biz_users_edit_aspx bu = new biz_users_edit_aspx(user);
            bu.id = new Guid(user[EntityIDNames.User]);
            yield return(bu);

            referer = bu.lastResponse.ResponseUri.AbsoluteUri;

            _forms_FormScript_js_aspx fs = new _forms_FormScript_js_aspx(user);
            fs.etc = 1;
            fs.FormType = "main";
            fs.FormName = "crmForm";
            headerItem = new WebTestRequestHeader();
            headerItem.Name = "referer";
            headerItem.Value = referer;
            fs.Headers.Add(headerItem);
            yield return (fs);

            biz_users_areas_aspx ba = new biz_users_areas_aspx(user);
            ba.oId = new Guid(user[EntityIDNames.User]);
            ba.oType = 8;
            ba.security = 65591;
            ba.tabSet = "areaRoles";
            yield return (ba);

            _grid_cmds_dlg_role_aspx dr = new _grid_cmds_dlg_role_aspx(user);
            dr.iObjType = 8;
            dr.iTotal = 1;
            yield return (dr);

            Guid userGuid = new Guid(user[EntityIDNames.User]);
            Guid[] userGuids = new Guid[] { userGuid };
            UserManager.UserManager umws = new UserManager.UserManager();
            CrmRequest getRoles = null;
            try { umws.GetUserRoles(userGuids); }
            catch (crmException x)
            {
                getRoles = (CrmRequest)x.wtr;
                getRoles.Url = Utils.ReplaceServerName(getRoles.Url, user);
            }
            yield return (getRoles);
                        
            dr = new _grid_cmds_dlg_role_aspx(user);
            dr.iObjType = 8;
            dr.iTotal = 1;
            dr.iIndex = 0;
            dr.iId = new Guid(user[EntityIDNames.User]).ToString("B");
            
            ChangeRoleForUser.textXml = ChangeRoleForUser.textXml.Replace("ROLE_GUID", role[EntityIDNames.Role]);
            ChangeRoleForUser.textXml = ChangeRoleForUser.textXml.Replace("BUSINESS_GUID", businessunit[EntityIDNames.BusinessUnit]);
            
            StringHttpBody httpBody = new StringHttpBody();
            httpBody.BodyString = ChangeRoleForUser.textXml;
            httpBody.ContentType = "text/xml";
            dr.Body = httpBody;

            rpcInfo = Utils.GenerateWRPCToken(_grid_cmds_dlg_role_aspx.url, user, tokenKey);

            headerItem = new WebTestRequestHeader();
            headerItem.Name = "CRMWRPCTokenTimeStamp";
            headerItem.Value = rpcInfo.timeStamp.ToString();
            dr.Headers.Add(headerItem);
            headerItem = new WebTestRequestHeader();
            headerItem.Name = "CRMWRPCToken";
            headerItem.Value = rpcInfo.token;
            dr.Headers.Add(headerItem);
            dr.Method = "Post";
            yield return (dr);

            AppGridWebService.AppGridWebService agws = new AppGridWebService.AppGridWebService();
            WebTestRequest wtrRefresh = null;
            try { agws.Refresh(gridXml); }
            catch (crmException x)
            {
                wtrRefresh = x.wtr;
                wtrRefresh.Url = Utils.ReplaceServerName(wtrRefresh.Url, user);

                string urlPath = wtrRefresh.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtrRefresh.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtrRefresh.Headers.Add(headerItem);
                wtrRefresh.Method = "Post";
            }
            yield return (wtrRefresh);

            this.EndTransaction("ChangeUserRole");
        }

        private static string textXml = @"<roles><add><roleid bid=""{BUSINESS_GUID}"">{ROLE_GUID}</roleid></add></roles>";

        private const string newIdMarker = @"<roleid>";
        private static string gridXml;
        private CRMEntity user;
        private CRMEntity role;
        private CRMEntity businessunit;
        private string crmServer;
        private string tokenKey;
        private string referer;

        private WRPCInfo rpcInfo;
        private WebTestRequestHeader headerItem;
    }
}