namespace CRM_Perf_BenchMark
{
    using System;
    using System.Collections.Generic;
    using Microsoft.VisualStudio.TestTools.WebTesting;
    using System.Net;
    using System.Text;
    using System.Xml;
    
    /// <summary>
    /// 1. Starts at Sales homepage
    /// 2. Click on Leads
    /// 3. Select Open Leads view
    /// 4. Add random string of two characters in look for textbox and click find
    /// 5. Double click a lead
    /// 6. Press Convert Lead button
    /// 7. Select Qualify & Convert into the following record(s)
    /// 8. Check boxes opportunity and account
    /// 9. Select Ok
    /// 10. Click Close
    /// </summary>

    public class ConvertleadToOppWithAccount : CrmWebTest
    {
        public void ConvertleadToOppWithAccount_PreWebTest(object sender, PreWebTestEventArgs e)
        {
			EntityRequest er = new EntityRequest();
			er.Type = EntityNames.Users;
			er.ReturnAs = EntityNames.Users;
			er.ParentID = EntityManager.Instance.OrgId;

			System.Collections.Hashtable Props = new System.Collections.Hashtable();

            if (e.WebTest.Context.ContainsKey("UserRole"))
            {
                Props.Add("role", e.WebTest.Context["UserRole"]);
            }           

			EntityRequest LeadEr = new EntityRequest();
			LeadEr.Type = EntityNames.Leads;
			LeadEr.ParentID = Guid.NewGuid();
			LeadEr.ReturnAs = EntityNames.Leads;

			Props.Add(EntityNames.Accounts, new EntityRequest[] { LeadEr });

			er.Props = Props;

			System.Collections.Hashtable Entities = EntityManager.Instance.GetEntities(er);

			user = (CRMEntity)Entities[EntityNames.Users];
			lead = (CRMEntity)Entities[EntityNames.Leads];

            crmServer = user["CrmServer"];
            tokenKey = EntityManager.Instance.ReadTokenKey(user["organizationid"]);

            er = new EntityRequest();
            er.Type = EntityNames.TransactionCurrency;
            er.ReturnAs = EntityNames.TransactionCurrency;
            er.ParentID = new Guid(user["organizationid"]);
            System.Collections.Hashtable currencies = EntityManager.Instance.GetEntities(er);
            currency = (CRMEntity)currencies[EntityNames.TransactionCurrency];

            e.WebTest.UserName = user["domainName"];
            e.WebTest.Password = user["userpassword"];            
        }
        
        public void ConvertleadToOppWithAccount_PostWebTest(object sender, PostWebTestEventArgs e)
        {
            EntityManager.Instance.FreeEntity(user);
            EntityManager.Instance.FreeEntity(lead);
            EntityManager.Instance.FreeEntity(currency);
        }

        public ConvertleadToOppWithAccount()
        {
            WebRequest.RegisterPrefix("crm", new crmRequestFactory());
            string random = Utils.GetRandomString(2, 2);
            gridXml = GridXmlHelper.LeadPageView(-1, openLeadsView, null);
            gridXml2 = GridXmlHelper.LeadSearchView(-1, searchLeadsView, random, null);
            gridXml3 = GridXmlHelper.LeadPageSearchViewRefresh(-1, searchLeadsView, random, null);

            this.Proxy = null;            
            PreWebTest += new EventHandler<PreWebTestEventArgs>(ConvertleadToOppWithAccount_PreWebTest);
            PostWebTest += new EventHandler<PostWebTestEventArgs>(ConvertleadToOppWithAccount_PostWebTest);
        }

        public override IEnumerator<WebTestRequest> GetRequestEnumerator()
        {
            AppGridWebService.AppGridWebService ag;
            WebTestRequest wtr;

			this.BeginTransaction("NavLeadHomepage");

            // Get the home page
            _root_homepage_aspx hp = new _root_homepage_aspx(user);
            hp.etc = 4;
            yield return (hp);

			this.EndTransaction("NavLeadHomepage");

            if (ConfigSettings.Default.alltasks == true)
            {
                this.BeginTransaction("NavOpenLeads");

                
                ag = new AppGridWebService.AppGridWebService();
                wtr = null;

                try { ag.Reset(gridXml, gridId); }
                catch (crmException x)
                {
                    wtr = (CrmRequest)x.wtr;
                    wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                    string urlPath = wtr.Url.Remove(0, crmServer.Length);
                    urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                    rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                    headerItem = new WebTestRequestHeader();
                    headerItem.Name = "crmwrpctokentimestamp";
                    headerItem.Value = rpcInfo.timeStamp.ToString();
                    wtr.Headers.Add(headerItem);
                    headerItem = new WebTestRequestHeader();
                    headerItem.Name = "crmwrpctoken";
                    headerItem.Value = rpcInfo.token;
                    wtr.Headers.Add(headerItem);
                    wtr.Method = "Post";
                }
                yield return (wtr);

                this.EndTransaction("NavOpenLeads");
            }

            this.BeginTransaction("SearchLeads");

            // Search for the lead name
            ag = new AppGridWebService.AppGridWebService();
            wtr = null;
            try { ag.Reset(gridXml2, gridId); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
            }
            yield return (wtr);

            this.EndTransaction("SearchLeads");
            
            this.BeginTransaction("OpenLead");

            // simulate double clicking of lead
            sfa_leads_edit_aspx dp = new sfa_leads_edit_aspx(user);
            dp.id = lead[EntityIDNames.Lead];
            yield return (dp);

            referer = dp.lastResponse.ResponseUri.AbsoluteUri;

            _forms_FormScript_js_aspx formScript = new _forms_FormScript_js_aspx(user);
            formScript.etc = 4;
            formScript.FormType = "main";
            formScript.FormName = "crmForm";
            headerItem = new WebTestRequestHeader();
            headerItem.Name = "referer";
            headerItem.Value = referer;
            formScript.Headers.Add(headerItem);
            yield return (formScript);

            this.EndTransaction("OpenLead");
            
            this.BeginTransaction("ConvertLead");

            sfa_leads_dialogs_conv_lead_aspx conv = new sfa_leads_dialogs_conv_lead_aspx(user);
            yield return (conv);
            
            // lead conversion form
            dp = new sfa_leads_edit_aspx(user);
            
            dp.crmFormSubmitXml = "<lead></lead>";
            dp.crmFormSubmitMode = 16;
            dp.crmFormSubmitObjectType = 4;
            dp.crmFormSubmitId = lead["leadid"];
            dp.crmFormUserModified = false;
            dp.crmFormSubmitSecurity = 852023;
            dp.crmFormSubmitOnline = 1;            
            dp.id = lead["leadid"];
            dp.qlShowNew = false;
            dp.qlCreateAccount = true;
            dp.qlCreateContact = false;
            dp.qlCreateOpportunity = true;
            dp.qlOppCurrencyId = new Guid(currency[EntityIDNames.TransactionCurrency]);
            rpcInfo = Utils.GenerateWRPCToken(sfa_leads_edit_aspx.url, user, tokenKey);
            dp.CRMWRPCToken = rpcInfo.token;
            dp.CRMWRPCTokenTimeStamp = rpcInfo.timeStamp;
            yield return (dp);

            referer = dp.lastResponse.ResponseUri.AbsoluteUri;

            formScript = new _forms_FormScript_js_aspx(user);
            formScript.etc = 4;
            formScript.FormType = "main";
            formScript.FormName = "crmForm";
            headerItem = new WebTestRequestHeader();
            headerItem.Name = "referer";
            headerItem.Value = referer;
            formScript.Headers.Add(headerItem);
            yield return (formScript);

            // refresh of lead list page
            AppGridWebService.AppGridWebService agws = new AppGridWebService.AppGridWebService();
            WebTestRequest wtrRefresh = null;
            try { agws.Refresh(gridXml3); }
            catch (crmException x)
            {
                wtrRefresh = x.wtr;
                wtrRefresh.Url = Utils.ReplaceServerName(wtrRefresh.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
            }
            yield return (wtrRefresh);

            this.EndTransaction("ConvertLead");
        }

        private string gridXml;
        private string gridXml2;
        private string gridXml3;
        private static string gridId = "crmGrid";
        private static string openLeadsView = "00000000-0000-0000-00AA-000010001006";
        private static string searchLeadsView = "F1A7C4D9-F439-46DA-8B69-ECC133214695";

        private CRMEntity user;
        private CRMEntity lead;
        private CRMEntity currency;
        private string crmServer;
        private string tokenKey;
        private string referer;

        private WRPCInfo rpcInfo;
        private WebTestRequestHeader headerItem;
    }
}