namespace CRM_Perf_BenchMark
{
    using System;
    using System.Collections.Generic;
    using Microsoft.VisualStudio.TestTools.WebTesting;
    using System.Net;
    using System.Text;
    using System.Xml;
    
    /// <summary>
    /// Start Point Settings
    /// 1.	Select Security Roles
    /// 2.	Highlight Role
    /// 3.	Click on More Actions and select Copy Role
    /// 4.	Name the new role and click OK
    /// </summary>
    
    public class CopyRole : CrmWebTest
    {
        public void CopyRole_PreWebTest(object sender, PreWebTestEventArgs e)
        {
            EntityRequest er = new EntityRequest();
            er.Type = EntityNames.Users;
            er.ReturnAs = EntityNames.Users;
            er.ParentID = EntityManager.Instance.OrgId;

            System.Collections.Hashtable userProps = new System.Collections.Hashtable();
            if (e.WebTest.Context.ContainsKey("UserRole"))
            {
                userProps.Add("role", e.WebTest.Context["UserRole"]);
                er.Props = userProps;
            }
            
            System.Collections.Hashtable Entities = EntityManager.Instance.GetEntities(er);

            user = (CRMEntity)Entities[EntityNames.Users];

            e.WebTest.UserName = user["domainName"];
            e.WebTest.Password = user["userpassword"];

			er = new EntityRequest();
			er.Type = EntityNames.Roles;
			er.ParentID = new Guid(user["organizationid"]);
			er.ReturnAs = EntityNames.Roles;

			role = (CRMEntity)(EntityManager.Instance.GetEntities(er)[EntityNames.Roles]);

            crmServer = user["CrmServer"];
            tokenKey = EntityManager.Instance.ReadTokenKey(user["organizationid"]);
        }
        
        public void CopyRole_PostWebTest(object sender, PostWebTestEventArgs e)
        {
            EntityManager.Instance.FreeEntity(user);
            EntityManager.Instance.FreeEntity(role);
        }

        public CopyRole()
        {
            WebRequest.RegisterPrefix("crm", new crmRequestFactory());
            gridXml = GridXmlHelper.RolePageViewRefresh(-1, rollRefreshView, null);

            this.Proxy = null;            
            PreWebTest += new EventHandler<PreWebTestEventArgs>(CopyRole_PreWebTest);
            PostWebTest += new EventHandler<PostWebTestEventArgs>(CopyRole_PostWebTest);
        }

        public override IEnumerator<WebTestRequest> GetRequestEnumerator()
        {
            this.BeginTransaction("CopyRole");

            tools_Admin_admin_aspx admin = new tools_Admin_admin_aspx(user);
            admin.pid = 04;
            yield return (admin);
                        
            tools_business_home_role_aspx dp = new tools_business_home_role_aspx(user);
            yield return (dp);

            _grid_cmds_dlg_clonerole_aspx clone = new _grid_cmds_dlg_clonerole_aspx(user);
            clone.roleId = new Guid(role[EntityIDNames.Role]);
            yield return (clone);

            string referer = clone.lastResponse.ResponseUri.AbsoluteUri;

            clone = new _grid_cmds_dlg_clonerole_aspx(user);
            clone.iObjType = 1036;
            clone.iTotal = 1;
            clone.iIndex = 0;
            clone.iId = "undefined";

            string roleXml = "<role><name>"+Utils.GetRandomString(5,7)+ "</name><roletoclone>ROLE_GUID</roletoclone></role>";
            roleXml = roleXml.Replace("ROLE_GUID", "{" + role[EntityIDNames.Role].ToUpper() + "}");

            StringHttpBody stringHttpBody = new StringHttpBody();
            stringHttpBody.BodyString = roleXml;
            stringHttpBody.ContentType = "text/xml";
            clone.Body = stringHttpBody;

            rpcInfo = Utils.GenerateWRPCToken(_grid_cmds_dlg_clonerole_aspx.url, user, tokenKey);

            headerItem = new WebTestRequestHeader();
            headerItem.Name = "CRMWRPCTokenTimeStamp";
            headerItem.Value = rpcInfo.timeStamp.ToString();
            clone.Headers.Add(headerItem);
            headerItem = new WebTestRequestHeader();
            headerItem.Name = "CRMWRPCToken";
            headerItem.Value = rpcInfo.token;
            clone.Headers.Add(headerItem);
            headerItem = new WebTestRequestHeader();
            headerItem.Name = "referer";
            headerItem.Value = referer;
            clone.Headers.Add(headerItem);
            clone.Method = "Post";
            yield return (clone);

            //get the newly created id out of the post reply
            int idIdx = clone.lastResponse.BodyString.IndexOf(newIdMarker);
            string tempNewRoleId = clone.lastResponse.BodyString.Substring(idIdx + (CopyRole.newIdMarker.Length), 36);
            newRoleId = new Guid(tempNewRoleId).ToString();

            AppGridWebService.AppGridWebService agws = new AppGridWebService.AppGridWebService();
            WebTestRequest wtrRefresh = null;
            try { agws.Refresh(gridXml); }
            catch (crmException x)
            {
                wtrRefresh = x.wtr;
                wtrRefresh.Url = Utils.ReplaceServerName(wtrRefresh.Url, user);

                string urlPath = wtrRefresh.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtrRefresh.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtrRefresh.Headers.Add(headerItem);
                wtrRefresh.Method = "Post";
            }
            yield return (wtrRefresh);

            this.EndTransaction("CopyRole");
        }

        private const string newIdMarker = @"<roleid>&#123;";
        private string gridXml;
        private CRMEntity user;
        private CRMEntity role;
        private string newRoleId;
        private string crmServer;
        private string tokenKey;
        private static string rollRefreshView = "00000000-0000-0000-00AA-000010001214";

        private WRPCInfo rpcInfo;
        private WebTestRequestHeader headerItem;
    }
}