namespace CRM_Perf_BenchMark
{
    using System;
    using System.Collections.Generic;
    using Microsoft.VisualStudio.TestTools.WebTesting;
    using System.Net;
    using System.Text;
    using System.Xml;

    /// <summary>
    /// 1.	Start at Marketing homepage
    /// 2.  Open Campaign homepage
    /// 3.	Click on "New"
    /// 4.	Fill data and click "Save"
    /// 5.	Click on "Campaign Activities" on left pane
    /// 6.	Click on "New"
    /// 7.	Fill Channel, subject and description and click "Save and Close"
    /// </summary>
    
    public class CreateCampaignActivity : CrmWebTest
    {
        public void CreateCampaignActivity_PreWebTest(object sender, PreWebTestEventArgs e)
        {
            EntityRequest er = new EntityRequest();
            er.Type = EntityNames.Users;
            er.ReturnAs = EntityNames.Users;
            er.ParentID = EntityManager.Instance.OrgId;

            System.Collections.Hashtable Props = new System.Collections.Hashtable();

            if (e.WebTest.Context.ContainsKey("UserRole"))
            {
                Props.Add("role", e.WebTest.Context["UserRole"]);
            }
            
            System.Collections.Hashtable Entities = EntityManager.Instance.GetEntities(er);

            user = (CRMEntity)Entities[EntityNames.Users];
            crmServer = user["CrmServer"];
            tokenKey = EntityManager.Instance.ReadTokenKey(user["organizationid"]);

            orgId = new Guid(user["organizationid"]);

            e.WebTest.UserName = user["domainName"];
            e.WebTest.Password = user["userpassword"];            
        }

        public void CreateCampaignActivity_PostWebTest(object sender, PostWebTestEventArgs e)
        {
            try
            {
                if (newCampaignId != null)
                {
                    Guid g = EntityManager.Instance.GetOwnerGuid(new Guid(user[EntityIDNames.User]), new Guid(user["OrganizationId"]));
                    EntityManager.Instance.AddEntity(EntityNames.Campaigns, g, new string[] { "OwningUser", "CampaignID", "EntityManagerOwningUser" }, new string[] { user["systemuserid"], newCampaignId.ToString() , g.ToString()});
                }
            }
            catch (System.FormatException fe)
            {
                System.Diagnostics.Trace.WriteLine("Failed to convert bogus data into Guid. " + fe.ToString());
            }
            finally
            {
                EntityManager.Instance.FreeEntity(user);
            }
        }

        private void CreateCampaignActivity_ValidateResponse(object sender, ValidationEventArgs e)
        {
            string ID = null;

            int idIdx = e.Response.BodyString.IndexOf(newIdMarker);
            ID = e.Response.BodyString.Substring(idIdx + (CreateCampaignActivity.newIdMarker.Length), 36);
            try
            {
                ID = new Guid(ID).ToString();
            }
            catch (System.FormatException fe)
            {
                System.Diagnostics.Trace.WriteLine("Format exception: " + fe.ToString());
                e.IsValid = false;
                return;
            }
            e.IsValid = true;
        }

        public CreateCampaignActivity()
        {
            WebRequest.RegisterPrefix("crm", new crmRequestFactory());
            refreshGridCampaign = GridXmlHelper.CampaignPageViewRefresh(-1, myCampaignViewId, null);

            this.Proxy = null;
            PreWebTest += new EventHandler<PreWebTestEventArgs>(CreateCampaignActivity_PreWebTest);
            PostWebTest += new EventHandler<PostWebTestEventArgs>(CreateCampaignActivity_PostWebTest);
        }

        public override IEnumerator<WebTestRequest> GetRequestEnumerator()
        {
            this.BeginTransaction("NavCampaignHomepage");

            MA_home_camps_aspx hp = new MA_home_camps_aspx(user);            
            yield return (hp);

            this.EndTransaction("NavCampaignHomepage");
                       
            this.BeginTransaction("CreateCampaign");

            ma_camps_edit_aspx campPage = new ma_camps_edit_aspx(user);
            yield return (campPage);

            referer = campPage.lastResponse.ResponseUri.AbsoluteUri;

            _forms_FormScript_js_aspx formScript = new _forms_FormScript_js_aspx(user);
            formScript.etc = 4400;
            formScript.FormType = "main";
            formScript.FormName = "crmForm";
            headerItem = new WebTestRequestHeader("referer", referer);
            formScript.Headers.Add(headerItem);
            yield return (formScript);

            campPage = new ma_camps_edit_aspx(user);            
            campPage.crmFormSubmitXml = EntityXmlHelper.GetCreateCampaignXml(new Guid(user[EntityIDNames.User]), 8);
            campPage.crmFormSubmitMode = 1;
            campPage.crmFormUserModified = true;
            campPage.crmFormSubmitObjectType = 4400;
            campPage.crmFormSubmitSecurity = 852023;
            campPage.crmFormSubmitOnline = 1;

            rpcInfo = Utils.GenerateWRPCToken(ma_camps_edit_aspx.url, user, tokenKey);
            campPage.CRMWRPCToken = rpcInfo.token;
            campPage.CRMWRPCTokenTimeStamp = rpcInfo.timeStamp;
            campPage.ValidateResponse += this.CreateCampaignActivity_ValidateResponse;
            yield return (campPage);

            //get the newly created campaign id out of the post reply
            int idIdx = campPage.lastResponse.BodyString.IndexOf(newIdMarker);
            string tempNewCampaignId = campPage.lastResponse.BodyString.Substring(idIdx + (CreateCampaignActivity.newIdMarker.Length), 36);
            newCampaignId = new Guid(tempNewCampaignId);
            
            AppGridWebService.AppGridWebService agws = new AppGridWebService.AppGridWebService();
            CrmRequest wtrRefresh = null;
            try { agws.Refresh(refreshGridCampaign); }
            catch (crmException x)
            {
                wtrRefresh = (CrmRequest)x.wtr;
                wtrRefresh.Url = Utils.ReplaceServerName(wtrRefresh.Url, user);

                string urlPath = wtrRefresh.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtrRefresh.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtrRefresh.Headers.Add(headerItem);
                wtrRefresh.Method = "Post";
            }
            yield return (wtrRefresh);

            this.EndTransaction("CreateCampaign");

            this.BeginTransaction("OpenCampaign");

            ma_camps_edit_aspx openCamp = new ma_camps_edit_aspx(user);
            openCamp.id = newCampaignId;
            yield return (openCamp);

            referer = openCamp.lastResponse.ResponseUri.AbsoluteUri;

            formScript = new _forms_FormScript_js_aspx(user);
            formScript.etc = 4400;
            formScript.FormType = "main";
            formScript.FormName = "crmForm";
            headerItem = new WebTestRequestHeader("referer", referer);
            formScript.Headers.Add(headerItem);
            yield return (formScript);

            DateTimeService.DateTimeWebService dtws = new DateTimeService.DateTimeWebService();
            CrmRequest wtr = null;
            string today = DateTime.Today.ToString();
            try { dtws.GetFormattedDate(today); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);
            }
            yield return (wtr);

            yield return (wtr);

            this.EndTransaction("OpenCampaign");
            
            this.BeginTransaction("CreateCampaignActivity");

            ma_camps_areas_aspx createCampaignAct = new ma_camps_areas_aspx(user);
            createCampaignAct.oId = newCampaignId;
            createCampaignAct.oType = 4400;
            createCampaignAct.security = 852023;
            createCampaignAct.tabSet = "areaCampaignActivities";
            yield return (createCampaignAct);

            ma_campaignactivity_edit_aspx campActEdit = new ma_campaignactivity_edit_aspx(user);
            campActEdit._CreateFromType = 4400;
            campActEdit._CreateFromId = newCampaignId;
            yield return (campActEdit);

            referer = campActEdit.lastResponse.ResponseUri.AbsoluteUri;

            formScript = new _forms_FormScript_js_aspx(user);
            formScript.etc = 4402;
            formScript.FormType = "main";
            formScript.FormName = "crmForm";
            headerItem = new WebTestRequestHeader("referer", referer);
            formScript.Headers.Add(headerItem);
            yield return (formScript);

            campActEdit = new ma_campaignactivity_edit_aspx(user);
            campActEdit._CreateFromType = 4400;
            campActEdit._CreateFromId = newCampaignId;
            campActEdit.crmFormSubmitXml = EntityXmlHelper.GetCreateCampaignActivityXml(new Guid(user[EntityIDNames.User]), 8, newCampaignId, 4400, 5); //ca.Serialize();
            campActEdit.crmFormSubmitMode = 1;
            campActEdit.crmFormSubmitId = null;
            campActEdit.crmFormUserModified = true;
            campActEdit.crmFormSubmitObjectType = 4402;
            campActEdit.crmFormSubmitSecurity = 852023;
            campActEdit.crmFormSubmitOnline = 1;

            rpcInfo = Utils.GenerateWRPCToken(ma_campaignactivity_edit_aspx.url, user, tokenKey);
            campActEdit.CRMWRPCToken = rpcInfo.token;
            campActEdit.CRMWRPCTokenTimeStamp = rpcInfo.timeStamp;
            campActEdit.ValidateResponse += this.CreateCampaignActivity_ValidateResponse;
            yield return (campActEdit);

            //get the newly created list id out of the post reply
            idIdx = campActEdit.lastResponse.BodyString.IndexOf(newIdMarker);
            string tempNewCampaignActivityId = campActEdit.lastResponse.BodyString.Substring(idIdx + (CreateCampaignActivity.newIdMarker.Length), 36);
            newCampaignActivityId = new Guid(tempNewCampaignActivityId);
                        
            // Make the CampaignActivity Grid for Campaign
            tempNewCampaignId = newCampaignId.ToString();
            listGrid = GridXmlHelper.CampaignActivityGridForCampaignViewRefresh(-1, CampActInCampaignViewId, null, tempNewCampaignId, "4400");
            agws = new AppGridWebService.AppGridWebService();
            wtrRefresh = null;
            try { agws.Refresh(listGrid); }
            catch (crmException x)
            {
                wtrRefresh = (CrmRequest)x.wtr;
                wtrRefresh.Url = Utils.ReplaceServerName(wtrRefresh.Url, user);

                string urlPath = wtrRefresh.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtrRefresh.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtrRefresh.Headers.Add(headerItem);
                wtrRefresh.Method = "Post";
            }
            yield return (wtrRefresh);

            this.EndTransaction("CreateCampaignActivity");
        }

        private const string newIdMarker = @"separator + ""id=";
        private const string miniCampIdMarker = @"<CreateMiniCampaignResult>";
        private CRMEntity user;
        private Guid newCampaignId;
        private Guid newCampaignActivityId;
        private string refreshGridCampaign;
        private string listGrid;
        private string crmServer;
        private string tokenKey;
        private static string myCampaignViewId = "7F68A34B-26F1-430F-B3EF-7079F2925C69";
        private static string CampActInCampaignViewId = "89D107AB-42F5-4164-B7D9-AA83CE170673";

        private Guid orgId;
        private string referer;
        private WRPCInfo rpcInfo;
        private WebTestRequestHeader headerItem;
    }
}
