namespace CRM_Perf_BenchMark
{
    using System;
    using System.Collections.Generic;
    using Microsoft.VisualStudio.TestTools.WebTesting;
    using System.Net;
    using System.Text;
    using System.Xml;
    
    /// <summary>
    /// 1. Start at Workplace homepage
    /// 2. Select Activities
    /// 3. Select New
    /// 4. Highlight Email and select OK
    /// 5. Click the To lookup button
    /// 6. Select User from Lookfor dropdown
    /// 7. Click Find button
    /// 8. Select User from Available records and click OK
    /// 9. Enter a subject (datetime) and a description (randomstring)
    /// 10.	Click Regarding lookup
    /// 11.	Select an Account from the list and click OK
    /// 12.	Save&Close
    /// </summary>
    
    public class CreateEmail : CrmWebTest
    {
        public void CreateEmail_PreWebTest(object sender, PreWebTestEventArgs e)
        {
			EntityRequest er = new EntityRequest();
			er.Type = EntityNames.Users;
			er.ReturnAs = EntityNames.Users;
			er.ParentID = EntityManager.Instance.OrgId;

			System.Collections.Hashtable Props = new System.Collections.Hashtable();

            if (e.WebTest.Context.ContainsKey("UserRole"))
            {
                Props.Add("role", e.WebTest.Context["UserRole"]);
            }
            
			EntityRequest AccountEr = new EntityRequest();
			AccountEr.Type = EntityNames.Accounts;
			AccountEr.ParentID = Guid.NewGuid();
			AccountEr.ReturnAs = EntityNames.Accounts;

			Props.Add(EntityNames.Accounts, new EntityRequest[] { AccountEr });

			er.Props = Props;

			System.Collections.Hashtable Entities = EntityManager.Instance.GetEntities(er);

			user = (CRMEntity)Entities[EntityNames.Users];
			account = (CRMEntity)Entities[EntityNames.Accounts];
            crmServer = user["CrmServer"];
            tokenKey = EntityManager.Instance.ReadTokenKey(user["organizationid"]);
            orgId = new Guid(user["organizationid"]);

            e.WebTest.UserName = user["domainName"];
            e.WebTest.Password = user["userpassword"];
            
            user2 = EntityManager.Instance.GetNextUser(new Guid(user["organizationid"]));            
        }
        
        public void CreateEmail_PostWebTest(object sender, PostWebTestEventArgs e)
        {
            EntityManager.Instance.FreeEntity(user);
            EntityManager.Instance.FreeEntity(user2);
            EntityManager.Instance.FreeEntity(account);
        }

        public CreateEmail()
        {
            WebRequest.RegisterPrefix("crm", new crmRequestFactory());
            gridXml = GridXmlHelper.ActivitiesPageViewRefresh(-1, activitiesRefreshView, null);

            this.Proxy = null;            
            PreWebTest += new EventHandler<PreWebTestEventArgs>(CreateEmail_PreWebTest);
            PostWebTest += new EventHandler<PostWebTestEventArgs>(CreateEmail_PostWebTest);
        }
        
        public override IEnumerator<WebTestRequest> GetRequestEnumerator()
        {
			this.BeginTransaction("NavActivityHomepage");

            // Get the home page
            workplace_home_activities_aspx hp = new workplace_home_activities_aspx(user);
            yield return (hp);

			this.EndTransaction("NavActivityHomepage");

			this.BeginTransaction("CreateEmail");

            activities_dlg_create_aspx createPage = new activities_dlg_create_aspx(user);
            yield return (createPage);

            activities_email_edit_aspx editPage = new activities_email_edit_aspx(user);
            yield return (editPage);

            referer = editPage.lastResponse.ResponseUri.AbsoluteUri;

            _forms_FormScript_js_aspx formScript = new _forms_FormScript_js_aspx(user);
            formScript.etc = 4202;
            formScript.FormType = "main";
            formScript.FormName = "crmForm";
            headerItem = new WebTestRequestHeader("referer", referer);
            formScript.Headers.Add(headerItem);
            yield return (formScript);

            _controls_emailbody_msgbody_aspx msgBody = new _controls_emailbody_msgbody_aspx(user);
            msgBody.entityType = "email";
            yield return (msgBody);

            _controls_lookup_lookupmulti_aspx lookup = new _controls_lookup_lookupmulti_aspx(user);
            lookup.Class = "ActivityRecipient";
            lookup.objecttypes = "1,2,4,8,2020";
            lookup.browse = 0;
            lookup.ShowNewButton = 1;
            lookup.ShowPropButton = 1;
            lookup.DefaultType = 0;
            yield return (lookup);

            _controls_lookup_lookupdata_aspx lookupdata = new _controls_lookup_lookupdata_aspx(user);            
            lookupdata.ObjectType = 1;
            lookupdata.Class = "ActivityRecipient";
            lookupdata.objecttypes = "1,2,4,8,2020";
            lookupdata.browse = 0;
            lookupdata.ShowNewButton = 1;
            lookupdata.ShowPropButton = 1;
            lookupdata.DefaultType = 0;
            lookupdata.Mode = 4;
            lookupdata.ColumnsOnly = 1;
            yield return (lookupdata);

            lookupdata = new _controls_lookup_lookupdata_aspx(user);
            lookupdata.Class = "ActivityRecipient";
            lookupdata.objecttypes = "1,2,4,8,2020";
            lookupdata.browse = 0;
            lookupdata.ShowNewButton = 1;
            lookupdata.ShowPropButton = 1;
            lookupdata.DefaultType = 0;
            lookupdata.Mode = 4;
            lookupdata.ObjectType = 8;
            lookupdata.searchvalue = null;
            yield return (lookupdata);

            _controls_lookup_lookupsingle_aspx lookupsingle = new _controls_lookup_lookupsingle_aspx(user);
            lookupsingle.Class = "ActivityRegarding";
            lookupsingle.objecttypes = "1,2,3,4,112,1010,1084,1088,1090,4402,4406,4700";
            lookupsingle.browse = 0;
            lookupsingle.ShowNewButton = 1;
            lookupsingle.ShowPropButton = 1;
            lookupsingle.DefaultType = 0;
            yield return (lookupsingle);

            editPage = new activities_email_edit_aspx(user);            
            editPage.crmFormSubmitXml = EntityXmlHelper.GetCreateEmailXml(new Guid(user["systemuserid"]), 8, new Guid(user2["systemuserid"]), 8, new Guid(account["accountid"]), 1, new Guid(user["systemuserid"]), 8);
            editPage.crmFormSubmitMode = 2;
            editPage.crmFormUserModified = true;
            editPage.crmFormSubmitObjectType = 4202;
            editPage.crmFormSubmitSecurity = 852023;
            editPage.crmFormSubmitOnline = 1;   
            yield return (editPage);

            AppGridWebService.AppGridWebService ag = new AppGridWebService.AppGridWebService();
            CrmRequest wtr = null;

            try { ag.Refresh(gridXml); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
            }
            yield return (wtr);

            this.EndTransaction("CreateEmail");
        }

		private const string newIdMarker = @"SetQueryString(sLocation,""id"",""";
        private string gridXml;
        private CRMEntity user;
        private CRMEntity user2;
        private CRMEntity account;
        private string crmServer;
        private string tokenKey;
        private string activitiesRefreshView = "00000000-0000-0000-00AA-000010001899";
        private Guid orgId;

        private string referer;
        private WRPCInfo rpcInfo;
        private WebTestRequestHeader headerItem;
    }
}