namespace CRM_Perf_BenchMark
{
    using System;
    using System.Collections.Generic;
    using Microsoft.VisualStudio.TestTools.WebTesting;
    using System.Net;
    using System.Text;
    using System.Xml;    

    /// <summary>
    /// 1. Start at Marketing Tab
    /// 2. Select Marketing Lists 
    /// 3. Select new button
    /// 4. Enter data in form (name, member type account)
    /// 5. Save & close
    /// </summary>

    public class CreateNewList : CrmWebTest
    {
        public void CreateNewList_PreWebTest(object sender, PreWebTestEventArgs e)
        {
            EntityRequest er = new EntityRequest();
            er.Type = EntityNames.Users;
            er.ReturnAs = EntityNames.Users;
            er.ParentID = EntityManager.Instance.OrgId;

            System.Collections.Hashtable userProps = new System.Collections.Hashtable();
            if (e.WebTest.Context.ContainsKey("UserRole"))
            {
                userProps.Add("role", e.WebTest.Context["UserRole"]);
                er.Props = userProps;
            }
                       

            System.Collections.Hashtable Entities = EntityManager.Instance.GetEntities(er);

            user = (CRMEntity)Entities[EntityNames.Users];

            crmServer = user["CrmServer"];
            tokenKey = EntityManager.Instance.ReadTokenKey(user["organizationid"]);

            orgId = new Guid(user["organizationid"]);
            ownerId = new Guid(user["systemuserid"]);

            e.WebTest.UserName = user["domainName"];
            e.WebTest.Password = user["userpassword"];            
        }
       
        public void CreateNewList_PostWebTest(object sender, PostWebTestEventArgs e)
        {
			try
			{
                if (newListId != null)
                {
                    newListId = new Guid(newListId).ToString();
                    Guid g = EntityManager.Instance.GetOwnerGuid(new Guid(user[EntityIDNames.User]), new Guid(user["OrganizationId"]));
                    EntityManager.Instance.AddEntity(EntityNames.Lists, g, new string[] { "OwningUser", "ListID", "EntityManagerOwningUser" }, new string[] { user["systemuserid"], newListId, g.ToString() });
                }
			}
            catch (System.FormatException fe)
            {
                System.Diagnostics.Trace.WriteLine("Failed to convert bogus data into Guid. " + fe.ToString());
            }
			finally
			{
				EntityManager.Instance.FreeEntity(user);
			}
        }

		private void NewList_ValidateResponse(object sender, ValidationEventArgs e)
		{
			string ID = null;

			int idIdx = e.Response.BodyString.IndexOf(newIdMarker);
			ID = e.Response.BodyString.Substring(idIdx + (CreateNewList.newIdMarker.Length), 36);
			try
			{
				ID = new Guid(ID).ToString();
			}
			catch (System.FormatException fe)
			{
				System.Diagnostics.Trace.WriteLine("Format exception: " + fe.ToString());
				e.IsValid = false;
				return;
			}
			e.IsValid = true;
		}

        public CreateNewList()
        {
            WebRequest.RegisterPrefix("crm", new crmRequestFactory());
            gridXml = GridXmlHelper.ListPageViewRefresh(-1, myListViewId, null);

            this.Proxy = null;            
            PreWebTest += new EventHandler<PreWebTestEventArgs>(CreateNewList_PreWebTest);
            PostWebTest += new EventHandler<PostWebTestEventArgs>(CreateNewList_PostWebTest);
        }

        public override IEnumerator<WebTestRequest> GetRequestEnumerator()
        {
			this.BeginTransaction("NavListHomepage");

            _root_homepage_aspx hp = new _root_homepage_aspx(user);
            hp.etc = 4300;
            yield return (hp);

			this.EndTransaction("NavListHomepage");

			this.BeginTransaction("CreateList");

            ma_lists_edit_aspx ma = new ma_lists_edit_aspx(user);
            yield return (ma);

            referer = ma.lastResponse.ResponseUri.AbsoluteUri;

            _forms_FormScript_js_aspx formScript = new _forms_FormScript_js_aspx(user);
            formScript.etc = 4300;
            formScript.FormType = "main";
            formScript.FormName = "crmForm";
            headerItem = new WebTestRequestHeader("referer", referer);
            formScript.Headers.Add(headerItem);
            yield return (formScript);

            ma = new ma_lists_edit_aspx(user);
            ma.crmFormSubmitXml = EntityXmlHelper.GetCreateListXml(ownerId, 8);

            // Mode should be 2 but if it is 2 we can't capture the Id of the created list. Hence setting it to 1
            ma.crmFormSubmitMode = 1;
            ma.crmFormUserModified = true;
            ma.crmFormSubmitId = null;
            ma.crmFormSubmitObjectType = 4300;
            ma.crmFormSubmitSecurity = 852023;
            ma.crmFormSubmitOnline = 1;

            rpcInfo = Utils.GenerateWRPCToken(ma_lists_edit_aspx.url, user, tokenKey);
            ma.CRMWRPCToken = rpcInfo.token;
            ma.CRMWRPCTokenTimeStamp = rpcInfo.timeStamp;
			ma.ValidateResponse += this.NewList_ValidateResponse;
            yield return (ma);

            //get the newly created list id out of the post reply
            int idIdx = ma.lastResponse.BodyString.IndexOf(newIdMarker);
            string tempNewListId = ma.lastResponse.BodyString.Substring(idIdx + (CreateNewList.newIdMarker.Length), 36);
			newListId = new Guid(tempNewListId).ToString();

            AppGridWebService.AppGridWebService agws = new AppGridWebService.AppGridWebService();
            WebTestRequest wtrRefresh = null;
            try { agws.Refresh(gridXml); }
            catch (crmException x)
            {
                wtrRefresh = x.wtr;
                wtrRefresh.Url = Utils.ReplaceServerName(wtrRefresh.Url, user);

                string urlPath = wtrRefresh.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtrRefresh.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtrRefresh.Headers.Add(headerItem);
                wtrRefresh.Method = "Post";
            }
            yield return (wtrRefresh);

            this.EndTransaction("CreateList");
        }

        private const string newIdMarker = @"separator + ""id=";
        private CRMEntity user;
        private string newListId;
        private string gridXml;
        private string crmServer;
        private string tokenKey;
        private static string myListViewId = "823B9210-B82A-4B11-B0F6-B9E4C9C8403F";
        private Guid orgId;
        private Guid ownerId;
        private string referer;
        private WRPCInfo rpcInfo;
        private WebTestRequestHeader headerItem;
    }
}
