namespace CRM_Perf_BenchMark
{
    using System;
    using System.Collections.Generic;
    using Microsoft.VisualStudio.TestTools.WebTesting;
    using System.Net;
    using System.Text;
    using System.Xml;
    
    /// <summary>
    /// 1. Start at Sales homepage
    /// 2. Click Contacts
    /// 3. Insert name criteria in Find box and Select Find button
    /// 4. Double click a contact
    /// 5. Select the Notes Tab
    /// 6. Click to enter a new note
    /// 7. Enter NoteData  <date/time stamp>"
    /// 8. Save and close
    /// </summary>

    public class CreateNewNoteForContact : CrmWebTest
    {
        public void CreateNewNoteForContact_PreWebTest(object sender, PreWebTestEventArgs e)
        {
			EntityRequest er = new EntityRequest();
			er.Type = EntityNames.Users;
			er.ReturnAs = EntityNames.Users;
			er.ParentID = EntityManager.Instance.OrgId;

			System.Collections.Hashtable Props = new System.Collections.Hashtable();

            if (e.WebTest.Context.ContainsKey("UserRole"))
            {
                Props.Add("role", e.WebTest.Context["UserRole"]);
            }
            
			EntityRequest ContactEr = new EntityRequest();
			ContactEr.Type = EntityNames.Contacts;
			ContactEr.ParentID = Guid.NewGuid();
			ContactEr.ReturnAs = EntityNames.Contacts;

			Props.Add(EntityNames.Accounts, new EntityRequest[] { ContactEr });

			er.Props = Props;

			System.Collections.Hashtable Entities = EntityManager.Instance.GetEntities(er);

			user = (CRMEntity)Entities[EntityNames.Users];
			contact = (CRMEntity)Entities[EntityNames.Contacts];

            crmServer = user["CrmServer"];
            tokenKey = EntityManager.Instance.ReadTokenKey(user["organizationid"]);

            orgId = new Guid(user["organizationid"]);

            e.WebTest.UserName = user["domainName"];
            e.WebTest.Password = user["userpassword"];            
        }

        public void CreateNewNoteForContact_PostWebTest(object sender, PostWebTestEventArgs e)
        {
			try
			{
				// Don't delete the entity till end of test run as it is deleted from crm.
                if (newAnnotationId != null)
                {
                    newAnnotationId = new Guid(newAnnotationId).ToString();
                    Guid g = EntityManager.Instance.GetOwnerGuid(new Guid(user[EntityIDNames.User]), new Guid(user["OrganizationId"]));
					EntityManager.Instance.AddEntity(EntityNames.Notes, g, new string[] { "OwnerID", "AnnotationID", "ObjectID", "ObjectTypeCode" ,"EntityManagerOwningUser"}, new string[] { user["systemuserid"], newAnnotationId, contact["contactid"], "2", g.ToString() });
                    contact["has" + EntityNames.Notes] = true.ToString();
                }
			}
            catch (System.FormatException fe)
            {
                System.Diagnostics.Trace.WriteLine("AnnotationId should never be set to anything other than Guid. This should not happen. " + fe.ToString());
            }
			finally
			{
				EntityManager.Instance.FreeEntity(user);
				EntityManager.Instance.FreeEntity(contact);
			}
        }

		private void NewAnnotation_ValidateResponse(object sender, ValidationEventArgs e)
		{
			string ID = null;

			int idIdx = e.Response.BodyString.IndexOf(newIdMarker);
			ID = e.Response.BodyString.Substring(idIdx + (CreateNewNoteForContact.newIdMarker.Length), 38);
			try
			{
				ID = new Guid(ID).ToString();
			}
			catch (System.FormatException fe)
			{
				System.Diagnostics.Trace.WriteLine("Format exception: " + fe.ToString());
				e.IsValid = false;
				return;
			}
			e.IsValid = true;
		}

        public CreateNewNoteForContact()
        {
            WebRequest.RegisterPrefix("crm", new crmRequestFactory());
            string random = Utils.GetRandomString(2, 2);
            gridXml = GridXmlHelper.ContactSearchView(-1, searchContactsView, random, null);

            this.Proxy = null;            
            PreWebTest += new EventHandler<PreWebTestEventArgs>(CreateNewNoteForContact_PreWebTest);
            PostWebTest += new EventHandler<PostWebTestEventArgs>(CreateNewNoteForContact_PostWebTest);
        }

        public override IEnumerator<WebTestRequest> GetRequestEnumerator()
        {
			this.BeginTransaction("NavContactHomepage");

            // Get the home page
            _root_homepage_aspx hp = new _root_homepage_aspx(user);
            hp.etc = 2;
            yield return (hp);

			this.EndTransaction("NavContactHomepage");

            this.BeginTransaction("SearchContacts");

            // Search for the Contact name
            AppGridWebService.AppGridWebService ag = new AppGridWebService.AppGridWebService();
            CrmRequest wtr = null;

            try { ag.Reset(gridXml, gridId); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
            }
            yield return (wtr);

            this.EndTransaction("SearchContacts");

            this.BeginTransaction("OpenContact");

            // simulate double clicking of contact
            sfa_conts_edit_aspx dp = new sfa_conts_edit_aspx(user);
            dp.id = contact["contactid"];
            yield return (dp);

            referer = dp.lastResponse.ResponseUri.AbsoluteUri;

            _forms_FormScript_js_aspx f = new _forms_FormScript_js_aspx(user);
            f.etc = 2;
            f.FormType = "main";
            f.FormName = "crmForm";
            headerItem = new WebTestRequestHeader("referer", referer);
            f.Headers.Add(headerItem);
            yield return (f);
                        
            this.EndTransaction("OpenContact");

            this.BeginTransaction("CreateNoteForContact");

            _controls_notes_notesdata_aspx notes = new _controls_notes_notesdata_aspx(user);
            notes.id = new Guid(contact["contactid"]);
            notes.ParentEntity = 2;
            notes.EnableInlineEdit = true;
            notes.EnableInsert = true;
            yield return (notes);

            DateTimeService.DateTimeWebService dtws = new DateTimeService.DateTimeWebService();
            wtr = null;

            try { dtws.GetFormattedDate(DateTime.Today.ToString()); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
            }
            yield return (wtr);

            AnnotationWebservice.AnnotationWebService annotationWebService = new AnnotationWebservice.AnnotationWebService();
            wtr = null;
            dataXml = EntityXmlHelper.GetCreateAnnotationXml(new Guid(contact["contactid"]), 2, new Guid(user["systemuserid"]), 8); //a.Serialize();

            try { annotationWebService.SaveWithAttachment(null, false, notesType, dataXml); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
            }
			wtr.ValidateResponse += this.NewAnnotation_ValidateResponse;
            yield return (wtr);

			this.EndTransaction("CreateNoteForContact");

            //get the newly created annotation id out of the post reply
            int idIdx = wtr.lastResponse.BodyString.IndexOf(newIdMarker);
            string tempNewAnnotationId = wtr.lastResponse.BodyString.Substring(idIdx + (CreateNewNoteForContact.newIdMarker.Length), 38);
			newAnnotationId = new Guid(tempNewAnnotationId).ToString();
        }

        private string gridXml;
        private static string gridId = "crmGrid";
        private static string searchContactsView = "8DF19B44-A073-40C3-9D6D-EE1355D8C4BA";
        private string dataXml;
        private CRMEntity user;
        private CRMEntity contact;
        private static int notesType = 5;
        private static string newIdMarker = "&lt;annotationid&gt;";
        private string newAnnotationId;
        private string crmServer;
        private string tokenKey;

        private Guid orgId;
        private string referer;
        private WRPCInfo rpcInfo;
        private WebTestRequestHeader headerItem;
    }
}