namespace CRM_Perf_BenchMark
{
    using System;
    using System.Collections.Generic;
    using Microsoft.VisualStudio.TestTools.WebTesting;
    using System.Net;
    using System.Text;
    using System.Xml;
    
    /// <summary>
    /// 1. Start at Sales homepage
    /// 2. Click Leads
    /// 3. Insert name criteria in Find box and Select Find button
    /// 4. Double click a lead
    /// 5. Select the Notes Tab
    /// 6. Click to enter a new note
    /// 7. Enter NoteData  <date/time stamp>"
    /// 8. Save and close
    /// </summary>

    public class CreateNewNoteForLead : CrmWebTest
    {
        public void CreateNewNoteForLead_PreWebTest(object sender, PreWebTestEventArgs e)
        {
			EntityRequest er = new EntityRequest();
			er.Type = EntityNames.Users;
			er.ReturnAs = EntityNames.Users;
			er.ParentID = EntityManager.Instance.OrgId;

			System.Collections.Hashtable Props = new System.Collections.Hashtable();

            if (e.WebTest.Context.ContainsKey("UserRole"))
            {
                Props.Add("role", e.WebTest.Context["UserRole"]);
            }
            
			EntityRequest LeadEr = new EntityRequest();
			LeadEr.Type = EntityNames.Leads;
			LeadEr.ParentID = Guid.NewGuid();
			LeadEr.ReturnAs = EntityNames.Leads;

			Props.Add(EntityNames.Accounts, new EntityRequest[] { LeadEr });

			er.Props = Props;

			System.Collections.Hashtable Entities = EntityManager.Instance.GetEntities(er);

			user = (CRMEntity)Entities[EntityNames.Users];
			lead = (CRMEntity)Entities[EntityNames.Leads];
            crmServer = user["CrmServer"];
            tokenKey = EntityManager.Instance.ReadTokenKey(user["organizationid"]);

            orgId = new Guid(user["organizationid"]);

            e.WebTest.UserName = user["domainName"];
            e.WebTest.Password = user["userpassword"];            
        }

        public void CreateNewNoteForLead_PostWebTest(object sender, PostWebTestEventArgs e)
        {
			try
			{
				// Don't delete the entity till end of test run as it is deleted from crm.
                if (newAnnotationId != null)
                {
                    newAnnotationId = new Guid(newAnnotationId).ToString();
                    Guid g = EntityManager.Instance.GetOwnerGuid(new Guid(user[EntityIDNames.User]), new Guid(user["OrganizationId"]));
                    EntityManager.Instance.AddEntity(EntityNames.Notes, g, new string[] { "OwnerID", "AnnotationID", "ObjectID", "ObjectTypeCode", "EntityManagerOwningUser" }, new string[] { user["systemuserid"], newAnnotationId, lead["leadid"], "4", g.ToString() });
                    lead["has" + EntityNames.Notes] = true.ToString();
                }
			}
            catch (System.FormatException fe)
            {
                System.Diagnostics.Trace.WriteLine("AnnotationId should never be set to anything other than Guid. This should not happen. " + fe.ToString());
            }
			finally
			{
				EntityManager.Instance.FreeEntity(user);
				EntityManager.Instance.FreeEntity(lead);
			}
        }

		private void NewAnnotation_ValidateResponse(object sender, ValidationEventArgs e)
		{
			string ID = null;

			int idIdx = e.Response.BodyString.IndexOf(newIdMarker);
			ID = e.Response.BodyString.Substring(idIdx + (CreateNewNoteForLead.newIdMarker.Length), 38);
			try
			{
				ID = new Guid(ID).ToString();
			}
			catch (System.FormatException fe)
			{
				System.Diagnostics.Trace.WriteLine("Format exception: " + fe.ToString());
				e.IsValid = false;
				return;
			}
			e.IsValid = true;
		}

        public CreateNewNoteForLead()
        {
            WebRequest.RegisterPrefix("crm", new crmRequestFactory());
            string random = Utils.GetRandomString(2, 2);
            gridXml = GridXmlHelper.LeadSearchView(-1, searchLeadsView, random, null);

            this.Proxy = null;            
            PreWebTest += new EventHandler<PreWebTestEventArgs>(CreateNewNoteForLead_PreWebTest);
            PostWebTest += new EventHandler<PostWebTestEventArgs>(CreateNewNoteForLead_PostWebTest);
        }

        public override IEnumerator<WebTestRequest> GetRequestEnumerator()
        {
			this.BeginTransaction("NavLeadHomepage");

            // Get the home page
            _root_homepage_aspx hp = new _root_homepage_aspx(user);
            hp.etc = 4;
            yield return (hp);

			this.EndTransaction("NavLeadHomepage");
            
            this.BeginTransaction("SearchLeads");
            
			// Search for the lead name
            AppGridWebService.AppGridWebService ag = new AppGridWebService.AppGridWebService();
            CrmRequest wtr = null;

            try { ag.Reset(gridXml, gridId); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
            }
            yield return (wtr);

            this.EndTransaction("SearchLeads");
            
            this.BeginTransaction("OpenLead");

            // simulate double clicking of lead
            sfa_leads_edit_aspx dp = new sfa_leads_edit_aspx(user);
            dp.id = lead["leadid"];
            yield return (dp);

            referer = dp.lastResponse.ResponseUri.AbsoluteUri;

            _forms_FormScript_js_aspx f = new _forms_FormScript_js_aspx(user);
            f.etc = 4;
            f.FormType = "main";
            f.FormName = "crmForm";
            headerItem = new WebTestRequestHeader("referer", referer);
            f.Headers.Add(headerItem);
            yield return (f);

            this.EndTransaction("OpenLead");
            
            this.BeginTransaction("CreateNoteForLead");

            _controls_notes_notesdata_aspx notes = new _controls_notes_notesdata_aspx(user);
            notes.id = new Guid(lead["leadid"]);
            notes.ParentEntity = 4;
            notes.EnableInlineEdit = true;
            notes.EnableInsert = true;
            yield return (notes);

            DateTimeService.DateTimeWebService dtws = new DateTimeService.DateTimeWebService();
            wtr = null;

            try { dtws.GetFormattedDate(DateTime.Today.ToString()); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);
            }
            yield return (wtr);

            AnnotationWebservice.AnnotationWebService annotationWebService = new AnnotationWebservice.AnnotationWebService();
            wtr = null;
            dataXml = EntityXmlHelper.GetCreateAnnotationXml(new Guid(lead["leadid"]), 4, new Guid(user["systemuserid"]), 8); //a.Serialize();

            try { annotationWebService.SaveWithAttachment(null, false, notesType, dataXml); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
            }
			wtr.ValidateResponse += this.NewAnnotation_ValidateResponse;
            yield return (wtr);

			this.EndTransaction("CreateNoteForLead");

            //get the newly created annotation id out of the post reply
            int idIdx = wtr.lastResponse.BodyString.IndexOf(newIdMarker);
            string tempNewAnnotationId = wtr.lastResponse.BodyString.Substring(idIdx + (CreateNewNoteForLead.newIdMarker.Length), 38);
			newAnnotationId = new Guid(tempNewAnnotationId).ToString();
        }

        private string gridXml;
        private static string gridId = "crmGrid";
        private static string searchLeadsView = "F1A7C4D9-F439-46DA-8B69-ECC133214695";
        private string dataXml;
        private CRMEntity user;
        private CRMEntity lead;
        private static int notesType = 5;
        private static string newIdMarker = "&lt;annotationid&gt;";
        private string newAnnotationId;
        private string crmServer;
        private string tokenKey;

        Guid orgId;

        private string referer;
        private WRPCInfo rpcInfo;
        private WebTestRequestHeader headerItem;
    }
}