namespace CRM_Perf_BenchMark
{
    using System;
    using System.Collections.Generic;
    using Microsoft.VisualStudio.TestTools.WebTesting;
    using System.Net;
    using System.Text;
    using System.Xml;
    
    /// <summary>
    /// 1. Start at Sales homepage
    /// 2. Click Opportunities
    /// 3. Insert name criteria in Find box and Select Find button
    /// 4. Double click an Opportunity
    /// 5. Select the Notes Tab
    /// 6. Click to enter a new note
    /// 7. Enter NoteData  <date/time stamp>"
    /// 8. Save and close
    /// </summary>

    public class CreateNewNoteForOpp : CrmWebTest
    {
        public void CreateNewNoteForOpp_PreWebTest(object sender, PreWebTestEventArgs e)
        {
			EntityRequest er = new EntityRequest();
			er.Type = EntityNames.Users;
			er.ReturnAs = EntityNames.Users;
			er.ParentID = EntityManager.Instance.OrgId;

			System.Collections.Hashtable Props = new System.Collections.Hashtable();

            if (e.WebTest.Context.ContainsKey("UserRole"))
            {
                Props.Add("role", e.WebTest.Context["UserRole"]);
            }

			EntityRequest OpportunityEr = new EntityRequest();
			OpportunityEr.Type = EntityNames.Opportunities;
			OpportunityEr.ParentID = Guid.NewGuid();
			OpportunityEr.ReturnAs = EntityNames.Opportunities;

			Props.Add(EntityNames.Opportunities, new EntityRequest[] { OpportunityEr });

			er.Props = Props;

			System.Collections.Hashtable Entities = EntityManager.Instance.GetEntities(er);

			user = (CRMEntity)Entities[EntityNames.Users];
			opportunity = (CRMEntity)Entities[EntityNames.Opportunities];
            crmServer = user["CrmServer"];
            tokenKey = EntityManager.Instance.ReadTokenKey(user["organizationid"]);

            orgId = new Guid(user["organizationid"]);

            e.WebTest.UserName = user["domainName"];
            e.WebTest.Password = user["userpassword"];            
        }

        public void CreateNewNoteForOpp_PostWebTest(object sender, PostWebTestEventArgs e)
        {
			try
			{
				// Don't delete the entity till end of test run as it is deleted from crm.
                if (newAnnotationId != null)
                {
                    newAnnotationId = new Guid(newAnnotationId).ToString();
                    Guid g = EntityManager.Instance.GetOwnerGuid(new Guid(user[EntityIDNames.User]), new Guid(user["OrganizationId"]));
                    EntityManager.Instance.AddEntity(EntityNames.Notes, g, new string[] { "OwnerID", "AnnotationID", "ObjectID", "ObjectTypeCode", "EntityManagerOwningUser" }, new string[] { user["systemuserid"], newAnnotationId, opportunity["opportunityid"], "3" ,g.ToString()});
                    opportunity["has" + EntityNames.Notes] = true.ToString();
                }
			}
            catch (System.FormatException fe)
            {
                System.Diagnostics.Trace.WriteLine("AnnotationId should never be set to anything other than Guid. This should not happen. " + fe.ToString());
            }
			finally
			{
				EntityManager.Instance.FreeEntity(user);
				EntityManager.Instance.FreeEntity(opportunity);
			}
        }
		private void NewAnnotation_ValidateResponse(object sender, ValidationEventArgs e)
		{
			string ID = null;

			int idIdx = e.Response.BodyString.IndexOf(newIdMarker);
			ID = e.Response.BodyString.Substring(idIdx + (CreateNewNoteForOpp.newIdMarker.Length), 38);
			try
			{
				ID = new Guid(ID).ToString();
			}
			catch (System.FormatException fe)
			{
				System.Diagnostics.Trace.WriteLine("Format exception: " + fe.ToString());
				e.IsValid = false;
				return;
			}
			e.IsValid = true;
		}

        public CreateNewNoteForOpp()
        {
            WebRequest.RegisterPrefix("crm", new crmRequestFactory());
            string random = Utils.GetRandomString(2, 2);
            gridXml = GridXmlHelper.OpportunitySearchView(-1, searchOppView, random, null);

            this.Proxy = null;            
            PreWebTest += new EventHandler<PreWebTestEventArgs>(CreateNewNoteForOpp_PreWebTest);
            PostWebTest += new EventHandler<PostWebTestEventArgs>(CreateNewNoteForOpp_PostWebTest);
        }

        public override IEnumerator<WebTestRequest> GetRequestEnumerator()
        {
            Guid objectid = new Guid(opportunity["opportunityid"]);
            Guid ownerid = new Guid(user["systemuserid"]);

            this.BeginTransaction("NavOpportunityHomepage");

            // Get the home page
            _root_homepage_aspx hp = new _root_homepage_aspx(user);
            hp.etc = 3;
            yield return (hp);

            this.EndTransaction("NavOpportunityHomepage");
 
            this.BeginTransaction("SearchOpportunities");
            
			// Search for the opp name
            AppGridWebService.AppGridWebService ag = new AppGridWebService.AppGridWebService();
            CrmRequest wtr = null;

            try { ag.Reset(gridXml, gridId); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
            }
            yield return (wtr);

            this.EndTransaction("SearchOpportunities");

            this.BeginTransaction("OpenOpportunity");

            // simulate double clicking of opp
            sfa_opps_edit_aspx dp = new sfa_opps_edit_aspx(user);
            dp.id = opportunity["opportunityid"];
            yield return (dp);

            referer = dp.lastResponse.ResponseUri.AbsoluteUri;

            _forms_FormScript_js_aspx f = new _forms_FormScript_js_aspx(user);
            f.etc = 3;
            f.FormType = "main";
            f.FormName = "crmForm";
            headerItem = new WebTestRequestHeader("referer", referer);
            f.Headers.Add(headerItem);
            yield return (f);
            
            this.EndTransaction("OpenOpportunity");
        
            this.BeginTransaction("CreateNoteForOpportunity");

            _controls_notes_notesdata_aspx notes = new _controls_notes_notesdata_aspx(user);
            notes.id = new Guid(opportunity["opportunityid"]);
            notes.ParentEntity = 3;
            notes.EnableInlineEdit = true;
            notes.EnableInsert = true;
            yield return (notes);

            DateTimeService.DateTimeWebService dtws = new DateTimeService.DateTimeWebService();
            wtr = null;

            try { dtws.GetFormattedDate(DateTime.Today.ToString()); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
            }
            yield return (wtr);

            AnnotationWebservice.AnnotationWebService annotationWebService = new AnnotationWebservice.AnnotationWebService();
            wtr = null;
            dataXml = EntityXmlHelper.GetCreateAnnotationXml(objectid, 3, ownerid, 8);

            try { annotationWebService.SaveWithAttachment(null, false, notesType, dataXml); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
            }
			wtr.ValidateResponse += this.NewAnnotation_ValidateResponse;
            yield return (wtr);

			this.EndTransaction("CreateNoteForOpportunity");

            //get the newly created annotation id out of the post reply
            int idIdx = wtr.lastResponse.BodyString.IndexOf(newIdMarker);
            string tempNewAnnotationId = wtr.lastResponse.BodyString.Substring(idIdx + (CreateNewNoteForOpp.newIdMarker.Length), 38);
			newAnnotationId = new Guid(tempNewAnnotationId).ToString();
        }

        private string gridXml;
        private static string gridId = "crmGrid";
        private static string searchOppView = "3F6D889E-6549-4025-9CE0-4C4103EFB6A3";
        private string dataXml;
        private CRMEntity user;
        private CRMEntity opportunity;
        private static int notesType = 5;
        private static string newIdMarker = "&lt;annotationid&gt;";
        private string newAnnotationId;
        private string crmServer;
        private string tokenKey;

        private Guid orgId;
        private string referer;
        private WRPCInfo rpcInfo;
        private WebTestRequestHeader headerItem;
    }
}