namespace CRM_Perf_BenchMark
{
    using System;
    using System.Collections.Generic;
    using Microsoft.VisualStudio.TestTools.WebTesting;
    using System.Net;
    using System.Text;
    using System.Xml;
    
    /// <summary>
    /// 1. Start at Workplace homepage
    /// 2. Select Activities
    /// 3. Select Type Task (which generates a My Task view)
    /// 4. Select All Tasks from view (depends on config setting)
    /// 5. Double click an Task
    /// 6. Select the Notes Tab
    /// 7. Click to enter a new note
    /// 8. Enter NoteData  <date/time stamp>"
    /// 9. Save and close
    /// </summary>

    public class CreateNewNoteForTask : CrmWebTest
    {
        public void CreateNewNoteForTask_PreWebTest(object sender, PreWebTestEventArgs e)
        {
			EntityRequest er = new EntityRequest();
			er.Type = EntityNames.Users;
			er.ReturnAs = EntityNames.Users;
			er.ParentID = EntityManager.Instance.OrgId;

			System.Collections.Hashtable Props = new System.Collections.Hashtable();

            if (e.WebTest.Context.ContainsKey("UserRole"))
            {
                Props.Add("role", e.WebTest.Context["UserRole"]);
            }
            
			EntityRequest TaskEr = new EntityRequest();
			TaskEr.Type = EntityNames.Tasks;
			TaskEr.ParentID = Guid.NewGuid();
			TaskEr.ReturnAs = EntityNames.Tasks;

			Props.Add(EntityNames.Tasks, new EntityRequest[] { TaskEr });

			er.Props = Props;

			System.Collections.Hashtable Entities = EntityManager.Instance.GetEntities(er);

			user = (CRMEntity)Entities[EntityNames.Users];
			task = (CRMEntity)Entities[EntityNames.Tasks];

            crmServer = user["CrmServer"];
            tokenKey = EntityManager.Instance.ReadTokenKey(user["organizationid"]);

            orgId = new Guid(user["organizationid"]);

            e.WebTest.UserName = user["domainName"];
            e.WebTest.Password = user["userpassword"];            
        }

        public void CreateNewNoteForTask_PostWebTest(object sender, PostWebTestEventArgs e)
        {
			try
			{
				// Don't delete the entity till end of test run as it is deleted from crm.
                if (newAnnotationId != null)
                {
                    newAnnotationId = new Guid(newAnnotationId).ToString();
                    Guid g = EntityManager.Instance.GetOwnerGuid(new Guid(user[EntityIDNames.User]), new Guid(user["OrganizationId"]));
                    EntityManager.Instance.AddEntity(EntityNames.Notes, g, new string[] { "OwnerID", "AnnotationID", "ObjectID", "ObjectTypeCode", "EntityManagerOwningUser" }, new string[] { user["systemuserid"], newAnnotationId, task["activityid"], "4212", g.ToString() });
                    //Update the parent to have Notes
                    task["has" + EntityNames.Notes] = true.ToString();
                }
			}
            catch (System.FormatException fe)
            {
                System.Diagnostics.Trace.WriteLine("AnnotationId should never be set to anything other than Guid. This should not happen. " + fe.ToString());
            }
			finally
			{
				EntityManager.Instance.FreeEntity(user);
				EntityManager.Instance.FreeEntity(task);
			}
        }

		private void NewAnnotation_ValidateResponse(object sender, ValidationEventArgs e)
		{
			string ID = null;

			int idIdx = e.Response.BodyString.IndexOf(newIdMarker);
			ID = e.Response.BodyString.Substring(idIdx + (CreateNewNoteForTask.newIdMarker.Length), 38);
			try
			{
				ID = new Guid(ID).ToString();
			}
			catch (System.FormatException fe)
			{
				System.Diagnostics.Trace.WriteLine("Format exception: " + fe.ToString());
				e.IsValid = false;
				return;
			}
			e.IsValid = true;
		}

        public CreateNewNoteForTask()
        {
            WebRequest.RegisterPrefix("crm", new crmRequestFactory());
            gridXml1 = GridXmlHelper.TaskPageView(-1, myTasksViewId, null, null, null);
            gridXml2 = GridXmlHelper.TaskPageView(-1, allTasksViewId, null, "4242", "1&#x2C;1");

            this.Proxy = null;            
            PreWebTest += new EventHandler<PreWebTestEventArgs>(CreateNewNoteForTask_PreWebTest);
            PostWebTest += new EventHandler<PostWebTestEventArgs>(CreateNewNoteForTask_PostWebTest);
        }

        public override IEnumerator<WebTestRequest> GetRequestEnumerator()
        {
			this.BeginTransaction("NavActivityHomepage");

            // Get the home page
            workplace_home_activities_aspx hp = new workplace_home_activities_aspx(user);
            yield return (hp);

			this.EndTransaction("NavActivityHomepage");

            this.BeginTransaction("NavMyTasks");
            
			// Choose task from activities drop down menu
            ActivitiesWebService.ActivitiesWebService act = new ActivitiesWebService.ActivitiesWebService();
            CrmRequest wtr = null;

            try { act.GetQueryList(entityName); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
            }
            yield return (wtr);

            AppGridWebService.AppGridWebService ag = new AppGridWebService.AppGridWebService();
            wtr = null;

            try { ag.Reset(gridXml1, gridId); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
            }
            yield return (wtr);

            this.EndTransaction("NavMyTasks");

            if (ConfigSettings.Default.alltasks == true)
            {
                this.BeginTransaction("NavAllTasks");

                // Search for all tasks
                ag = new AppGridWebService.AppGridWebService();
                wtr = null;

                try { ag.Reset(gridXml2, gridId); }
                catch (crmException x)
                {
                    wtr = (CrmRequest)x.wtr;
                    wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                    string urlPath = wtr.Url.Remove(0, crmServer.Length);
                    urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                    rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                    headerItem = new WebTestRequestHeader();
                    headerItem.Name = "crmwrpctokentimestamp";
                    headerItem.Value = rpcInfo.timeStamp.ToString();
                    wtr.Headers.Add(headerItem);
                    headerItem = new WebTestRequestHeader();
                    headerItem.Name = "crmwrpctoken";
                    headerItem.Value = rpcInfo.token;
                    wtr.Headers.Add(headerItem);
                    wtr.Method = "Post";
                }
                yield return (wtr);

                this.EndTransaction("NavAllTasks");
            }
   
            this.BeginTransaction("OpenTask");

            // simulate double clicking of a task
            activities_task_edit_aspx dp = new activities_task_edit_aspx(user);
            dp.id = task["activityid"];
            yield return (dp);

            referer = dp.lastResponse.ResponseUri.AbsoluteUri;

            _forms_FormScript_js_aspx f = new _forms_FormScript_js_aspx(user);
            f.etc = 4212;
            f.FormType = "main";
            f.FormName = "crmForm";
            headerItem = new WebTestRequestHeader("referer", referer);
            f.Headers.Add(headerItem);
            yield return (f);
            
            this.EndTransaction("OpenTask");
         
            this.BeginTransaction("CreateNoteForTask");

            _controls_notes_notesdata_aspx notes = new _controls_notes_notesdata_aspx(user);
            notes.id = new Guid(task["activityid"]);
            notes.ParentEntity = 4212;
            notes.EnableInlineEdit = true;
            notes.EnableInsert = true;
            yield return (notes);

            DateTimeService.DateTimeWebService dtws = new DateTimeService.DateTimeWebService();
            wtr = null;

            try { dtws.GetFormattedDate(DateTime.Today.ToString()); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
            }
            yield return (wtr);

            AnnotationWebservice.AnnotationWebService annotationWebService = new AnnotationWebservice.AnnotationWebService();
            wtr = null;
            dataXml = EntityXmlHelper.GetCreateAnnotationXml(new Guid(task["activityid"]), 4212, new Guid(user["systemuserid"]), 8);  //a.Serialize();

            try { annotationWebService.SaveWithAttachment(null, false, notesType, dataXml); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
            }

			wtr.ValidateResponse += this.NewAnnotation_ValidateResponse;
            yield return (wtr);

			this.EndTransaction("CreateNoteForTask");

            //get the newly created annotation id out of the post reply
            int idIdx = wtr.lastResponse.BodyString.IndexOf(newIdMarker);
            string tempNewAnnotationId = wtr.lastResponse.BodyString.Substring(idIdx + (CreateNewNoteForTask.newIdMarker.Length), 38);
			newAnnotationId = new Guid(tempNewAnnotationId).ToString();
        }

        private string gridXml1;
        private string gridXml2;
        private static string entityName = "task";
        private static string gridId = "crmGrid";
        private string dataXml;
        private CRMEntity user;
        private CRMEntity task;
        private static int notesType = 5;
        private static string newIdMarker = "&lt;annotationid&gt;";
        private string newAnnotationId;
        private string crmServer;
        private string tokenKey;
        private static string myTasksViewId = "6CF285AA-EB20-4277-925A-3E9735411FF0";
        private static string allTasksViewId = "2B5F5A5D-2D23-4FE7-AA58-E77995368AE7";
        private Guid orgId;

        private string referer;
        private WRPCInfo rpcInfo;
        private WebTestRequestHeader headerItem;
    }
}