namespace CRM_Perf_BenchMark
{
    using System;
    using System.Collections.Generic;
    using Microsoft.VisualStudio.TestTools.WebTesting;
    using System.Net;
    using System.Text;
    using System.Xml;
    
    /// <summary>
    /// 1. Start at Marketing homepage
    /// 2. Select Marketing Lists
    /// 3. Click On New List
    /// 4. Enter Data and Click Save
    /// 5. Click Marketing List Members
    /// 6. Click Manage Members
    /// 7. Select Use Lookup to add members
    /// 8. Open Campaign HomePage
    /// 9. Click on New Campaign
    /// 10. Input information and Click Save
    /// 11. Click Target Marketing List
    /// 12. Add Marketing List created earlier
    /// 13. Click on Campaign Activities and click "New"
    /// 14. Select Fax
    /// 15. Fill data and click Save
    /// 16. Click on Distribute Campaign Activity
    /// 17. Fill data and click "Distribute"
    /// 18. Refresh Campaign Activity Grid till it changes to complete
    /// 19. Click on Action and Select "Delete Campaign"
    /// </summary>

    public class DeleteCampaignDistributed : CrmWebTest
    {
        public void DeleteCampaignDistributed_PreWebTest(object sender, PreWebTestEventArgs e)
        {
            EntityRequest er = new EntityRequest();
            er.Type = EntityNames.Users;
            er.ReturnAs = EntityNames.Users;
            er.ParentID = EntityManager.Instance.OrgId;

            System.Collections.Hashtable Props = new System.Collections.Hashtable();

            if (e.WebTest.Context.ContainsKey("UserRole"))
            {
                Props.Add("role", e.WebTest.Context["UserRole"]);
            }
            
            EntityRequest AccountEr = new EntityRequest();
            AccountEr.Type = EntityNames.Accounts;
            AccountEr.ParentID = Guid.NewGuid();
            AccountEr.ReturnAs = EntityNames.Accounts;

            Props.Add(EntityNames.Accounts, new EntityRequest[] { AccountEr });

            er.Props = Props;

            System.Collections.Hashtable Entities = EntityManager.Instance.GetEntities(er);

            user = (CRMEntity)Entities[EntityNames.Users];
            account = (CRMEntity)Entities[EntityNames.Accounts];
            crmServer = user["CrmServer"];
            tokenKey = EntityManager.Instance.ReadTokenKey(user["organizationid"]);

            orgId = new Guid(user["organizationid"]);

            e.WebTest.UserName = user["domainName"];
            e.WebTest.Password = user["userpassword"];            
        }

        public void DeleteCampaignDistributed_PostWebTest(object sender, PostWebTestEventArgs e)
        {
            try
            {
                if (newListId != null)
                {
                    string newListString = newListId.ToString();
                    Guid g = EntityManager.Instance.GetOwnerGuid(new Guid(user[EntityIDNames.User]), new Guid(user["OrganizationId"]));
                    EntityManager.Instance.AddEntity(EntityNames.Lists, g, new string[] { "OwningUser", "ListID", "EntityManagerOwningUser" }, new string[] { user["systemuserid"], newListString , g.ToString()});
                }
            }
            catch (System.FormatException fe)
            {
                System.Diagnostics.Trace.WriteLine("Failed to convert bogus data into Guid. " + fe.ToString());
            }
            finally
            {
                EntityManager.Instance.FreeEntity(user);
                EntityManager.Instance.FreeEntity(account);
            }
        }

        private void DeleteCampaignDistributed_ValidateResponse(object sender, ValidationEventArgs e)
        {
            string ID = null;

            int idIdx = e.Response.BodyString.IndexOf(newIdMarker);
            ID = e.Response.BodyString.Substring(idIdx + (DeleteCampaignDistributed.newIdMarker.Length), 36);
            try
            {
                ID = new Guid(ID).ToString();
            }
            catch (System.FormatException fe)
            {
                System.Diagnostics.Trace.WriteLine("Format exception: " + fe.ToString());
                e.IsValid = false;
                return;
            }
            e.IsValid = true;
        }

        public DeleteCampaignDistributed()
        {
            WebRequest.RegisterPrefix("crm", new crmRequestFactory());
            refreshGridList = GridXmlHelper.ListPageViewRefresh(-1, myListViewId, null);
            refreshGridCampaign = GridXmlHelper.CampaignPageViewRefresh(-1, myCampaignViewId, null);

            this.Proxy = null;            
            PreWebTest += new EventHandler<PreWebTestEventArgs>(DeleteCampaignDistributed_PreWebTest);
            PostWebTest += new EventHandler<PostWebTestEventArgs>(DeleteCampaignDistributed_PostWebTest);
        }

        public override IEnumerator<WebTestRequest> GetRequestEnumerator()
        {
            this.BeginTransaction("NavListHomepage");

            _root_homepage_aspx hp = new _root_homepage_aspx(user);
            hp.etc = 4300;
            yield return (hp);

            this.EndTransaction("NavListHomepage");

            this.BeginTransaction("ManageMembersAdd");

            //ma/lists/edit.aspx
            ma_lists_edit_aspx ma = new ma_lists_edit_aspx(user);
            yield return (ma);

            referer = ma.lastResponse.ResponseUri.AbsoluteUri;

            //_forms/FormScript.js.aspx?etc=4300&FormType=main&FormName=crmForm
            _forms_FormScript_js_aspx f = new _forms_FormScript_js_aspx(user);
            f.etc = 4300;
            f.FormType = "main";
            f.FormName = "crmForm";
            headerItem = new WebTestRequestHeader("referer", referer);
            f.Headers.Add(headerItem);
            yield return (f);

            //ma/lists/edit.aspx - POST
            ma = new ma_lists_edit_aspx(user);
            ma.crmFormSubmitXml = EntityXmlHelper.GetCreateListXml(new Guid(user["systemuserid"]), 8);
            ma.crmFormSubmitMode = 1;
            ma.crmFormSubmitId = null;
            ma.crmFormUserModified = true;
            ma.crmFormSubmitObjectType = 4300;
            ma.crmFormSubmitSecurity = 852023;
            ma.crmFormSubmitOnline = 1;
                        
            rpcInfo = Utils.GenerateWRPCToken(ma_lists_edit_aspx.url, user, tokenKey);
            ma.CRMWRPCToken = rpcInfo.token;
            ma.CRMWRPCTokenTimeStamp = rpcInfo.timeStamp;
            ma.ValidateResponse += this.DeleteCampaignDistributed_ValidateResponse;
            yield return (ma);

            //get the newly created list id out of the post reply
            int idIdx = ma.lastResponse.BodyString.IndexOf(newIdMarker);
            string tempNewListId = ma.lastResponse.BodyString.Substring(idIdx + (DeleteCampaignDistributed.newIdMarker.Length), 36);
            newListId = new Guid(tempNewListId);

            ma = new ma_lists_edit_aspx(user);
            ma.id = newListId;
            yield return (ma);

            referer = ma.lastResponse.ResponseUri.AbsoluteUri;

            AppGridWebService.AppGridWebService agws = new AppGridWebService.AppGridWebService();
            CrmRequest wtrRefresh = null;
            try { agws.Refresh(refreshGridList); }
            catch (crmException x)
            {
                wtrRefresh = (CrmRequest)x.wtr;
                wtrRefresh.Url = Utils.ReplaceServerName(wtrRefresh.Url, user);

                string urlPath = wtrRefresh.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtrRefresh.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtrRefresh.Headers.Add(headerItem);
                wtrRefresh.Method = "Post";
            }
            yield return (wtrRefresh);
            
            f = new _forms_FormScript_js_aspx(user);
            f.etc = 4300;
            f.FormType = "main";
            f.FormName = "crmForm";
            headerItem = new WebTestRequestHeader("referer", referer);
            f.Headers.Add(headerItem);
            yield return (f);
                        
            ma_lists_areas_aspx listArea = new ma_lists_areas_aspx(user);
            listArea.oId = newListId;
            listArea.oType = 4300;
            listArea.security = 852023;
            listArea.createdfromcode = 1;
            listArea.tabSet = "areaListMember";
            yield return (listArea);

            MA_lists_ListQualificationDlg_dlg_manage_members_aspx listQualifications = new MA_lists_ListQualificationDlg_dlg_manage_members_aspx(user);
            yield return (listQualifications);
                       

            MarketingAutomation.MarketingAutomationWebService maws = new MarketingAutomation.MarketingAutomationWebService();
            CrmRequest wtrRetrieveList = null;
            string[] columns = new string[] { "createdfromcode" };
            try { maws.RetrieveList(newListId, columns); }
            catch (crmException x)
            {
                wtrRetrieveList = (CrmRequest)x.wtr;
                wtrRetrieveList.Url = Utils.ReplaceServerName(wtrRetrieveList.Url, user);

                string urlPath = wtrRetrieveList.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtrRetrieveList.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtrRetrieveList.Headers.Add(headerItem);
                wtrRetrieveList.Method = "Post";
            }
            yield return (wtrRetrieveList);

            _controls_lookup_lookupmulti_aspx lookupMulti = new _controls_lookup_lookupmulti_aspx(user);
            lookupMulti.Class = "activityrelatedcustomer";
            lookupMulti.objecttypes = "1";
            lookupMulti.browse = 0;
            lookupMulti.ShowNewButton = 1;
            yield return (lookupMulti);

            _controls_lookup_lookupdata_aspx lookupData = new _controls_lookup_lookupdata_aspx(user);
            lookupData.ObjectType = 1;
            lookupData.Class = "activityrelatedcustomer";
            lookupData.objecttypes = "1";
            lookupData.browse = 0;
            lookupData.ShowNewButton = 1;
            lookupData.Mode = 4;
            lookupData.ColumnsOnly = 1;
            yield return (lookupData);

            lookupData = new _controls_lookup_lookupdata_aspx(user);
            lookupData.Class = "activityrelatedcustomer";
            lookupData.objecttypes = "1";
            lookupData.browse = 0;
            lookupData.ShowNewButton = 1;
            lookupData.Mode = 4;
            lookupData.ObjectType = 1;
            lookupData.searchvalue = Utils.GetRandomString(2, 2);
            yield return (lookupData);

            _grid_cmds_dlg_addtolist_aspx addToList = new _grid_cmds_dlg_addtolist_aspx(user);
            addToList.autoTrigger = 1;
            addToList.iObjType = 1;
            addToList.iTotal = 1;
            addToList.sIds = null;
            addToList.itemObjectId = newListId;
            yield return (addToList);

            addToList = new _grid_cmds_dlg_addtolist_aspx(user);
            addToList.iObjType = 1;
            addToList.iTotal = 1;
            addToList.iIndex = 0;
            addToList.itemObjectId = newListId;
            addToList.iId = new Guid(account[EntityIDNames.Account]);

            StringHttpBody httpBody = new StringHttpBody();
            httpBody.BodyString = "<node />";
            httpBody.ContentType = "text/xml";
            addToList.Body = httpBody;

            rpcInfo = Utils.GenerateWRPCToken(_grid_cmds_dlg_addtolist_aspx.url, user, tokenKey);

            headerItem = new WebTestRequestHeader();
            headerItem.Name = "CRMWRPCTokenTimeStamp";
            headerItem.Value = rpcInfo.timeStamp.ToString();
            addToList.Headers.Add(headerItem);
            headerItem = new WebTestRequestHeader();
            headerItem.Name = "CRMWRPCToken";
            headerItem.Value = rpcInfo.token;
            addToList.Headers.Add(headerItem);
            addToList.Method = "Post";
            yield return (addToList);

            maws = new MarketingAutomation.MarketingAutomationWebService();
            wtrRetrieveList = null;
            columns = new string[] { "membercount" };
            try { maws.RetrieveList(newListId, columns); }
            catch (crmException x)
            {
                wtrRetrieveList = (CrmRequest)x.wtr;
                wtrRetrieveList.Url = Utils.ReplaceServerName(wtrRetrieveList.Url, user);

                string urlPath = wtrRetrieveList.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtrRetrieveList.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtrRetrieveList.Headers.Add(headerItem);
                wtrRetrieveList.Method = "Post";
            }
            yield return (wtrRetrieveList);

            // Make the list Grid for members
            tempNewListId = newListId.ToString();
            memberGrid = GridXmlHelper.MemberGridForListViewRefresh(-1, memberViewId, null, tempNewListId, "4300");
            agws = new AppGridWebService.AppGridWebService();
            wtrRefresh = null;
            try { agws.Refresh(memberGrid); }
            catch (crmException x)
            {
                wtrRefresh = (CrmRequest)x.wtr;
                wtrRefresh.Url = Utils.ReplaceServerName(wtrRefresh.Url, user);
                string urlPath = wtrRefresh.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtrRefresh.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtrRefresh.Headers.Add(headerItem);
                wtrRefresh.Method = "Post";
            }
            yield return (wtrRefresh);

            this.EndTransaction("ManageMembersAdd");

            this.BeginTransaction("NavCampaignHomepage");

            MA_home_camps_aspx campHome = new MA_home_camps_aspx(user);
            yield return (campHome);

            this.EndTransaction("NavCampaignHomepage");

            this.BeginTransaction("CreateCampaign");

            ma_camps_edit_aspx campPage = new ma_camps_edit_aspx(user);
            yield return (campPage);

            referer = campPage.lastResponse.ResponseUri.AbsoluteUri;

            f = new _forms_FormScript_js_aspx(user);
            f.etc = 4400;
            f.FormType = "main";
            f.FormName = "crmForm";
            headerItem = new WebTestRequestHeader("referer", referer);
            f.Headers.Add(headerItem);
            yield return (f);

            _controls_number_formatNumber_xsl format = new _controls_number_formatNumber_xsl(crmServer);
            yield return (format);

            campPage = new ma_camps_edit_aspx(user);
            string campXml = "<campaign><name>" + Utils.GetRandomString(5, 10) + "</name><statuscode>0</statuscode><typecode>1</typecode><ownerid type=\"8\">{" + user[EntityIDNames.User] + "}</ownerid></campaign>";
            campPage.crmFormSubmitXml = campXml;
            campPage.crmFormSubmitMode = 1;
            campPage.crmFormUserModified = true;
            campPage.crmFormSubmitObjectType = 4400;
            campPage.crmFormSubmitSecurity = 852023;
            campPage.crmFormSubmitOnline = 1;

            rpcInfo = Utils.GenerateWRPCToken(ma_camps_edit_aspx.url, user, tokenKey);
            campPage.CRMWRPCToken = rpcInfo.token;
            campPage.CRMWRPCTokenTimeStamp = rpcInfo.timeStamp;
            campPage.ValidateResponse += this.DeleteCampaignDistributed_ValidateResponse;
            yield return (campPage);

            this.EndTransaction("CreateCampaign");

            //get the newly created campaign id out of the post reply
            idIdx = campPage.lastResponse.BodyString.IndexOf(newIdMarker);
            string tempNewCampaignId = campPage.lastResponse.BodyString.Substring(idIdx + (DeleteCampaignDistributed.newIdMarker.Length), 36);
            newCampaignId = new Guid(tempNewCampaignId);

            this.BeginTransaction("OpenCampaign");

            ma_camps_edit_aspx openCamp = new ma_camps_edit_aspx(user);
            openCamp.id = newCampaignId;
            yield return (openCamp);

            referer = openCamp.lastResponse.ResponseUri.AbsoluteUri;

            agws = new AppGridWebService.AppGridWebService();
            wtrRefresh = null;
            try { agws.Refresh(refreshGridCampaign); }
            catch (crmException x)
            {
                wtrRefresh = (CrmRequest)x.wtr;
                wtrRefresh.Url = Utils.ReplaceServerName(wtrRefresh.Url, user);

                string urlPath = wtrRefresh.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtrRefresh.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtrRefresh.Headers.Add(headerItem);
                wtrRefresh.Method = "Post";
            }
            yield return (wtrRefresh);

            f = new _forms_FormScript_js_aspx(user);
            f.etc = 4400;
            f.FormType = "main";
            f.FormName = "crmForm";
            headerItem = new WebTestRequestHeader("referer", referer);
            f.Headers.Add(headerItem);
            yield return (f);

            this.EndTransaction("OpenCampaign");

            this.BeginTransaction("AddListToCampaign");

            ma_camps_areas_aspx campArea = new ma_camps_areas_aspx(user);
            campArea.oId = newCampaignId;
            campArea.oType = 4400;
            campArea.security = 852023;
            campArea.tabSet = "areaTargetLists";
            yield return (campArea);

            _controls_lookup_lookupmulti_aspx lookupLists = new _controls_lookup_lookupmulti_aspx(user);
            lookupLists.Class = "BasicListForCamp";
            lookupLists.objecttypes = "4300";
            lookupLists.browse = 0;
            yield return (lookupLists);

            lookupData = new _controls_lookup_lookupdata_aspx(user);
            lookupData.ObjectType = 4300;
            lookupData.Class = "BasicListForCamp";
            lookupData.objecttypes = "4300";
            lookupData.browse = 0;
            lookupData.Mode = 4;
            lookupData.ColumnsOnly = 1;
            yield return (lookupData);

            lookupData = new _controls_lookup_lookupdata_aspx(user);
            lookupData.Class = "BasicListForCamp";
            lookupData.objecttypes = "4300";
            lookupData.browse = 0;
            lookupData.Mode = 4;
            lookupData.ObjectType = 4300;
            lookupData.searchvalue = "";
            yield return (lookupData);

            _grid_cmds_dlg_listcaassociation_aspx lassoc = new _grid_cmds_dlg_listcaassociation_aspx(user);
            lassoc.iObjType = 4300;
            yield return (lassoc);

            AssociateRecords.AssociateRecords ar = new AssociateRecords.AssociateRecords();
            wtrRefresh = null;
			string associationName = "";
            try { ar.Associate(4300, 4400, newListId, newCampaignId, "targetListsaddtoCA" , associationName); }
            catch (crmException x)
            {
                wtrRefresh = (CrmRequest)x.wtr;
                wtrRefresh.Url = Utils.ReplaceServerName(wtrRefresh.Url, user);

                string urlPath = wtrRefresh.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtrRefresh.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtrRefresh.Headers.Add(headerItem);
                wtrRefresh.Method = "Post";
            }
            yield return (wtrRefresh);

            // Make the list Grid for members
            tempNewCampaignId = newCampaignId.ToString();
            listGrid = GridXmlHelper.ListGridForCampaignViewRefresh(-1, listViewInCampaignViewId, null, tempNewCampaignId, "4400");
            agws = new AppGridWebService.AppGridWebService();
            wtrRefresh = null;
            try { agws.Refresh(listGrid); }
            catch (crmException x)
            {
                wtrRefresh = (CrmRequest)x.wtr;
                wtrRefresh.Url = Utils.ReplaceServerName(wtrRefresh.Url, user);

                string urlPath = wtrRefresh.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtrRefresh.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtrRefresh.Headers.Add(headerItem);
                wtrRefresh.Method = "Post";
            }
            yield return (wtrRefresh);

            this.EndTransaction("addListToCampaign");

            this.BeginTransaction("CreateCampaignActivity");

            ma_camps_areas_aspx createCampaignAct = new ma_camps_areas_aspx(user);
            createCampaignAct.oId = newCampaignId;
            createCampaignAct.oType = 4400;
            createCampaignAct.security = 852023;            
            createCampaignAct.tabSet = "areaCampaignActivities";
            yield return (createCampaignAct);

            ma_campaignactivity_edit_aspx campActEdit = new ma_campaignactivity_edit_aspx(user);
            campActEdit._CreateFromType = 4400;
            campActEdit._CreateFromId = newCampaignId;
            yield return (campActEdit);

            referer = campActEdit.lastResponse.ResponseUri.AbsoluteUri;

            f = new _forms_FormScript_js_aspx(user);
            f.etc = 4402;
            f.FormType = "main";
            f.FormName = "crmForm";
            headerItem = new WebTestRequestHeader("referer", referer);
            f.Headers.Add(headerItem);
            yield return (f);

            campActEdit = new ma_campaignactivity_edit_aspx(user);
            // QueryString Parameters
            campActEdit._CreateFromType = 4400;
            campActEdit._CreateFromId = newCampaignId;

            // Form Parameters
            campActEdit.crmFormSubmitXml = EntityXmlHelper.GetCreateCampaignActivityXml(new Guid(user[EntityIDNames.User]), 8, newCampaignId, 4400, 5);  //ca.Serialize();
            campActEdit.crmFormSubmitMode = 1;
            campActEdit.crmFormSubmitId = null;
            campActEdit.crmFormUserModified = true;
            campActEdit.crmFormSubmitObjectType = 4402;
            campActEdit.crmFormSubmitSecurity = 852023;
            campActEdit.crmFormSubmitOnline = 1;

            rpcInfo = Utils.GenerateWRPCToken(ma_campaignactivity_edit_aspx.url, user, tokenKey);
            campActEdit.CRMWRPCToken = rpcInfo.token;
            campActEdit.CRMWRPCTokenTimeStamp = rpcInfo.timeStamp;
            campActEdit.ValidateResponse += this.DeleteCampaignDistributed_ValidateResponse;
            yield return (campActEdit);

            //get the newly created list id out of the post reply
            idIdx = campActEdit.lastResponse.BodyString.IndexOf(newIdMarker);
            string tempNewCampaignActivityId = campActEdit.lastResponse.BodyString.Substring(idIdx + (DeleteCampaignDistributed.newIdMarker.Length), 36);
            newCampaignActivityId = new Guid(tempNewCampaignActivityId);

            campActEdit = new ma_campaignactivity_edit_aspx(user);
            campActEdit._CreateFromType = 4400;
            campActEdit._CreateFromId = newCampaignId;
            campActEdit.id = newCampaignActivityId;
            yield return (campActEdit);

            referer = campActEdit.lastResponse.ResponseUri.AbsoluteUri;
            
            // Make the CampaignActivity Grid for Campaign
            tempNewCampaignId = newCampaignId.ToString();
            listGrid = GridXmlHelper.CampaignActivityGridForCampaignViewRefresh(-1, CampActInCampaignViewId, null, tempNewCampaignId, "4400");
            agws = new AppGridWebService.AppGridWebService();
            wtrRefresh = null;
            try { agws.Refresh(listGrid); }
            catch (crmException x)
            {
                wtrRefresh = (CrmRequest)x.wtr;
                wtrRefresh.Url = Utils.ReplaceServerName(wtrRefresh.Url, user);

                string urlPath = wtrRefresh.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtrRefresh.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtrRefresh.Headers.Add(headerItem);
                wtrRefresh.Method = "Post";
            }
            yield return (wtrRefresh);

            f = new _forms_FormScript_js_aspx(user);
            f.etc = 4402;
            f.FormType = "main";
            f.FormName = "crmForm";
            headerItem = new WebTestRequestHeader("referer", referer);
            f.Headers.Add(headerItem);
            yield return (f);

            this.EndTransaction("CreateCampaignActivity");
                        
            this.BeginTransaction("PropogateCampaignActivity");

            _grid_cmds_dlg_listfax_aspx listFax = new _grid_cmds_dlg_listfax_aspx(user);
            listFax.iObjType = 4402;
            listFax.iTotal = 1;
            listFax.sIds = newCampaignActivityId.ToString("B");
            yield return (listFax);

            referer = listFax.lastResponse.ResponseUri.AbsoluteUri;

            f = new _forms_FormScript_js_aspx(user);
            f.etc = 4204;
            f.FormType = "main";
            f.FormName = "crmForm";
            headerItem = new WebTestRequestHeader("referer", referer);
            f.Headers.Add(headerItem);
            yield return (f);

            MA_CampaignActivity_Dialogs_confirm_propagation_aspx propConfirm = new MA_CampaignActivity_Dialogs_confirm_propagation_aspx(user);
            propConfirm.EntityTypeCode = 4204;
            yield return (propConfirm);

            listFax = new _grid_cmds_dlg_listfax_aspx(user);
            listFax.iObjType = 4402;
            listFax.iTotal = 1;
            listFax.iIndex = 0;
            listFax.SendEmail = false;
            listFax.ownerType = 8;
            listFax.OwnerOption = 3;
            listFax.iId = newCampaignActivityId;

            httpBody = new StringHttpBody();
            string faxXml = "<fax><directioncode>1</directioncode><subject>" + Utils.GetRandomString(10, 15) + "</subject><description>" + Utils.GetRandomString(100, 120) + "</description><actualdurationminutes>30</actualdurationminutes><prioritycode>1</prioritycode></fax>";
            httpBody.BodyString = faxXml;
            httpBody.ContentType = "text/xml";
            listFax.Body = httpBody;

            rpcInfo = Utils.GenerateWRPCToken(_grid_cmds_dlg_listfax_aspx.url, user, tokenKey);

            headerItem = new WebTestRequestHeader();
            headerItem.Name = "CRMWRPCTokenTimeStamp";
            headerItem.Value = rpcInfo.timeStamp.ToString();
            listFax.Headers.Add(headerItem);
            headerItem = new WebTestRequestHeader();
            headerItem.Name = "CRMWRPCToken";
            headerItem.Value = rpcInfo.token;
            listFax.Headers.Add(headerItem);
            listFax.Method = "Post";
            yield return (listFax);

            format = new _controls_number_formatNumber_xsl(crmServer);
            yield return (format);

            campActEdit = new ma_campaignactivity_edit_aspx(user);
            // QueryString Parameters
            campActEdit._CreateFromType = 4400;
            campActEdit._CreateFromId = newCampaignId;
            campActEdit.id = newCampaignActivityId;
            // Form Parameters
            campActEdit.crmFormSubmitXml = "<campaignactivity></campaignactivity>";
            campActEdit.crmFormSubmitMode = 4;
            campActEdit.crmFormSubmitId = newCampaignActivityId.ToString("B");
            campActEdit.crmFormSubmitObjectType = 4402;
            campActEdit.crmFormSubmitSecurity = 852023;
            campActEdit.crmFormSubmitOnline = 1;

            rpcInfo = Utils.GenerateWRPCToken(ma_campaignactivity_edit_aspx.url, user, tokenKey);
            campActEdit.CRMWRPCToken = rpcInfo.token;
            campActEdit.CRMWRPCTokenTimeStamp = rpcInfo.timeStamp;
            yield return (campActEdit);

            referer = campActEdit.lastResponse.ResponseUri.AbsoluteUri;

            f = new _forms_FormScript_js_aspx(user);
            f.etc = 4402;
            f.FormType = "main";
            f.FormName = "crmForm";
            headerItem = new WebTestRequestHeader("referer", referer);
            f.Headers.Add(headerItem);
            yield return (f);
            
            // Make the CampaignActivity Grid for Campaign
            tempNewCampaignId = newCampaignId.ToString();
            listGrid = GridXmlHelper.CampaignActivityGridForCampaignViewRefresh(-1, CampActInCampaignViewId, null, tempNewCampaignId, "4400");
            agws = new AppGridWebService.AppGridWebService();
            wtrRefresh = null;
            try { agws.Refresh(listGrid); }
            catch (crmException x)
            {
                wtrRefresh = (CrmRequest)x.wtr;
                wtrRefresh.Url = Utils.ReplaceServerName(wtrRefresh.Url, user);

                string urlPath = wtrRefresh.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtrRefresh.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtrRefresh.Headers.Add(headerItem);
                wtrRefresh.Method = "Post";
            }
            yield return (wtrRefresh);

            this.EndTransaction("PropogateCampaignActivity");

            this.BeginTransaction("DeleteCampaign");

            _grid_cmds_dlg_delete_aspx delCamp = new _grid_cmds_dlg_delete_aspx(user);
            delCamp.iObjType = 4400;
            delCamp.iTotal = 1;
            delCamp.sIds = newCampaignId.ToString("B");
            yield return (delCamp);

            delCamp = new _grid_cmds_dlg_delete_aspx(user);
            delCamp.iObjType = 4400;
            delCamp.iTotal = 1;
            delCamp.iIndex = 0;
            delCamp.sSubTypes = null;
            delCamp.iId = newCampaignId.ToString("B");

            httpBody = new StringHttpBody();
            httpBody.BodyString = "<node />";
            httpBody.ContentType = "text/xml";
            delCamp.Body = httpBody;

            rpcInfo = Utils.GenerateWRPCToken(_grid_cmds_dlg_delete_aspx.url, user, tokenKey);

            headerItem = new WebTestRequestHeader();
            headerItem.Name = "CRMWRPCTokenTimeStamp";
            headerItem.Value = rpcInfo.timeStamp.ToString();
            delCamp.Headers.Add(headerItem);
            headerItem = new WebTestRequestHeader();
            headerItem.Name = "CRMWRPCToken";
            headerItem.Value = rpcInfo.token;
            delCamp.Headers.Add(headerItem);
            delCamp.Method = "Post";
            yield return (delCamp);

            agws = new AppGridWebService.AppGridWebService();
            wtrRefresh = null;
            try { agws.Refresh(refreshGridCampaign); }
            catch (crmException x)
            {
                wtrRefresh = (CrmRequest)x.wtr;
                wtrRefresh.Url = Utils.ReplaceServerName(wtrRefresh.Url, user);

                string urlPath = wtrRefresh.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtrRefresh.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtrRefresh.Headers.Add(headerItem);
                wtrRefresh.Method = "Post";
            }
            yield return (wtrRefresh);

            this.EndTransaction("DeleteCampaign");
        }

        private const string newIdMarker = @"separator + ""id=";
        private const string miniCampIdMarker = @"<CreateMiniCampaignResult>";
        private CRMEntity user;
        private CRMEntity account;
        private string crmServer;
        private string tokenKey;
        private Guid newListId;
        private Guid newCampaignId;
        private Guid newCampaignActivityId;
        private string refreshGridList;
        private string refreshGridCampaign;
        private string memberGrid;
        private string listGrid;

        private static string myListViewId = "823B9210-B82A-4B11-B0F6-B9E4C9C8403F";
        private static string memberViewId = "51DE84DD-732F-440F-A506-26123A19AB50";
        private static string myCampaignViewId = "7F68A34B-26F1-430F-B3EF-7079F2925C69";
        private static string listViewInCampaignViewId = "BDD93547-53F6-4609-B591-9F48CE86295F";
        private static string CampActInCampaignViewId = "89D107AB-42F5-4164-B7D9-AA83CE170673";
        private Guid orgId;

        private string referer;
        private WRPCInfo rpcInfo;
        private WebTestRequestHeader headerItem;
    }
}
