namespace CRM_Perf_BenchMark
{
    using System;
    using System.Collections.Generic;
    using Microsoft.VisualStudio.TestTools.WebTesting;
    using System.Net;
    using System.Text;
    using System.Xml;
    
    /// <summary>
    /// 1. Start at Sales homepage
    /// 2. Open Contact homepage
    /// 3. Insert name criteria in Find box and Select Find button
    /// 4. Open Contact (double click)
    /// 5. Click on Action and select delete
    /// 6. Confirm the delete dialog
    /// </summary>
    
    public class DeleteContact : CrmWebTest
    {
        public void DeleteContact_PreWebTest(object sender, PreWebTestEventArgs e)
        {
			EntityRequest er = new EntityRequest();
			er.Type = EntityNames.Users;
			er.ReturnAs = EntityNames.Users;
			er.ParentID = EntityManager.Instance.OrgId;

			System.Collections.Hashtable Props = new System.Collections.Hashtable();

            if (e.WebTest.Context.ContainsKey("UserRole"))
            {
                Props.Add("role", e.WebTest.Context["UserRole"]);
            }
            
			EntityRequest ContactEr = new EntityRequest();
			ContactEr.Type = EntityNames.Contacts;
			ContactEr.ParentID = Guid.NewGuid();
			ContactEr.ReturnAs = EntityNames.Contacts;

            System.Collections.Hashtable contactProps = new System.Collections.Hashtable();
            contactProps.Add("lastname", "deletable");
            ContactEr.Props = contactProps;

			Props.Add(EntityNames.Accounts, new EntityRequest[] { ContactEr });

			er.Props = Props;

			System.Collections.Hashtable Entities = EntityManager.Instance.GetEntities(er);

			user = (CRMEntity)Entities[EntityNames.Users];
			contact = (CRMEntity)Entities[EntityNames.Contacts];
            crmServer = user["CrmServer"];
            tokenKey = EntityManager.Instance.ReadTokenKey(user["organizationid"]);

            e.WebTest.UserName = user["domainName"];
            e.WebTest.Password = user["userpassword"];            
        }

        public void DeleteContact_PostWebTest(object sender, PostWebTestEventArgs e)
        {
            // Don't free the entity till end of the test run as it is already deleted.
            EntityManager.Instance.DeleteEntity(contact);
 			EntityManager.Instance.FreeEntity(user);
        }

        public DeleteContact()
        {
            WebRequest.RegisterPrefix("crm", new crmRequestFactory());
            string random = Utils.GetRandomString(2, 2);
            gridXml = GridXmlHelper.ContactSearchView(-1, searchContactsView, random, null);
            gridRefreshXml = GridXmlHelper.ContactPageSearchViewRefresh(-1, searchContactsView, random, null);

            this.Proxy = null;           
            PreWebTest += new EventHandler<PreWebTestEventArgs>(DeleteContact_PreWebTest);
            PostWebTest += new EventHandler<PostWebTestEventArgs>(DeleteContact_PostWebTest);
        }

        public override IEnumerator<WebTestRequest> GetRequestEnumerator()
        {
			this.BeginTransaction("NavContactHomepage");

            // Get the home page
            _root_homepage_aspx hp = new _root_homepage_aspx(user);
            hp.etc = 2;
            yield return (hp);

			this.EndTransaction("NavContactHomepage");

            this.BeginTransaction("SearchContacts");

            // Search for the Contact name
            AppGridWebService.AppGridWebService ag = new AppGridWebService.AppGridWebService();
            WebTestRequest wtr = null;

            try { ag.Reset(gridXml, gridId); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
            }
            yield return (wtr);

            this.EndTransaction("SearchContacts");

			this.BeginTransaction("OpenContact");

            // simulate double clicking of contact
            sfa_conts_edit_aspx dp = new sfa_conts_edit_aspx(user);
            dp.id = contact["contactid"];
            yield return (dp);

            referer = dp.lastResponse.ResponseUri.AbsoluteUri;

            _forms_FormScript_js_aspx f = new _forms_FormScript_js_aspx(user);
            f.etc = 2;
            f.FormType = "main";
            f.FormName = "crmForm";
            headerItem = new WebTestRequestHeader("referer", referer);
            f.Headers.Add(headerItem);
            yield return (f);

			this.EndTransaction("OpenContact");

			this.BeginTransaction("DeleteContact");

            // Click Action and select delete
            _grid_cmds_dlg_delete_contact_aspx delPage = new _grid_cmds_dlg_delete_contact_aspx(user);
            delPage.iObjType = 2;
            delPage.iTotal = 1;
            delPage.sIds = contact["contactid"];
            yield return (delPage);

            _grid_cmds_dlg_confirm_delete_aspx confirmDel = new _grid_cmds_dlg_confirm_delete_aspx(user);
            confirmDel.ObjectTypeId = 2;
            confirmDel.iTotal = 1;
            yield return (confirmDel);

            _grid_cmds_dlg_delete_aspx delPage2 = new _grid_cmds_dlg_delete_aspx(user);
            rpcInfo = Utils.GenerateWRPCToken(_grid_cmds_dlg_delete_aspx.url, user, tokenKey);
            headerItem = new WebTestRequestHeader();
            headerItem.Name = "CRMWRPCTokenTimeStamp";
            headerItem.Value = rpcInfo.timeStamp.ToString();
            delPage2.Headers.Add(headerItem);
            headerItem = new WebTestRequestHeader();
            headerItem.Name = "CRMWRPCToken";
            headerItem.Value = rpcInfo.token;
            delPage2.Headers.Add(headerItem);
            delPage2.Method = "Post";
            
            delPage2.iObjType = 2;
            delPage2.iTotal = 1;
            delPage2.iIndex = 0;
            delPage2.iId = contact["contactid"];

            StringHttpBody httpBody = new StringHttpBody();
            httpBody.BodyString = "<node />";
            httpBody.ContentType = "text/xml";
            delPage2.Body = httpBody;

            yield return (delPage2);

			AppGridWebService.AppGridWebService agws = new AppGridWebService.AppGridWebService();
            WebTestRequest wtrRefresh = null;
            try { agws.Refresh(gridRefreshXml); }
            catch (crmException x)
            {
                wtrRefresh = x.wtr;
                wtrRefresh.Url = Utils.ReplaceServerName(wtrRefresh.Url, user);

                string urlPath = wtrRefresh.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtrRefresh.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtrRefresh.Headers.Add(headerItem);
                wtrRefresh.Method = "Post";
            }
            yield return (wtrRefresh);

 			this.EndTransaction("DeleteContact");
        }

        private string gridXml;
        private string gridRefreshXml;
        private static string gridId = "crmGrid";
        private static string searchContactsView = "8DF19B44-A073-40C3-9D6D-EE1355D8C4BA";

        private CRMEntity user;
        private CRMEntity contact;
        private string crmServer;
        private string tokenKey;
        private WRPCInfo rpcInfo;
        private WebTestRequestHeader headerItem;
        private string referer;        
    }
}