  namespace CRM_Perf_BenchMark
    {
        using System;
        using System.Collections.Generic;
        using Microsoft.VisualStudio.TestTools.WebTesting;
        using System.Net;
        using System.Text;
        using System.Xml; 
        
        /// <summary>
        /// 1.	Start at Marketing homepage
        /// 2.  Select Marketing Lists 
        /// 3.	Click on New
        /// 4.	Input information and click Save
        /// 5.	Click on Marketing List Members
        /// 6.	Click Manage Members
        /// 7.	Select Use Lookup to add members
        /// 8.	Click Find
        /// 9.	Select a list (double click) and click OK
        /// 10.	Click Create Quick Campaign
        /// 11. Enter name for Quick Camaign
        /// 11.	Select Fax
        /// 12.	Input information and click Create
        /// 13.	Click Quick Campaigns tab
        /// 14.	Click the Refresh till quick campaign is complete
        /// 15.	Click Actions and select Delete Marketing List
        /// 16.	Confirm deletion prompt.
        /// </summary>
    
        public class DeleteListWithQC : CrmWebTest
        {
            public void DeleteListWithQC_PreWebTest(object sender, PreWebTestEventArgs e)
            {
                EntityRequest er = new EntityRequest();
                er.Type = EntityNames.Users;
                er.ReturnAs = EntityNames.Users;
                er.ParentID = EntityManager.Instance.OrgId;
    
                System.Collections.Hashtable Props = new System.Collections.Hashtable();

                if (e.WebTest.Context.ContainsKey("UserRole"))
                {
                    Props.Add("role", e.WebTest.Context["UserRole"]);
                }
                    
                EntityRequest AccountEr = new EntityRequest();
                AccountEr.Type = EntityNames.Accounts;
                AccountEr.ParentID = Guid.NewGuid();
                AccountEr.ReturnAs = EntityNames.Accounts;
    
                Props.Add(EntityNames.Accounts, new EntityRequest[] { AccountEr });
    
                er.Props = Props;
    
                System.Collections.Hashtable Entities = EntityManager.Instance.GetEntities(er);
    
                user = (CRMEntity)Entities[EntityNames.Users];
                account = (CRMEntity)Entities[EntityNames.Accounts];
                crmServer = user["CrmServer"];
                tokenKey = EntityManager.Instance.ReadTokenKey(user["organizationid"]);

                orgId = new Guid(user["organizationid"]);
                ownerId = new Guid(user["systemuserid"]);

                e.WebTest.UserName = user["domainName"];
                e.WebTest.Password = user["userpassword"];                
            }
    
            public void DeleteListWithQC_PostWebTest(object sender, PostWebTestEventArgs e)
            {
                EntityManager.Instance.FreeEntity(user);
            }
    
            private void DeleteListWithQC_ValidateResponse(object sender, ValidationEventArgs e)
            {
                string ID = null;
    
                int idIdx = e.Response.BodyString.IndexOf(newIdMarker);
                ID = e.Response.BodyString.Substring(idIdx + (DeleteListWithQC.newIdMarker.Length), 36);
                try
                {
                    ID = new Guid(ID).ToString();
                }
                catch (System.FormatException fe)
                {
                    System.Diagnostics.Trace.WriteLine("Format exception: " + fe.ToString());
                    e.IsValid = false;
                    return;
                }
                e.IsValid = true;
            }
    
            public DeleteListWithQC()
            {
                WebRequest.RegisterPrefix("crm", new crmRequestFactory());
                refreshGridList = GridXmlHelper.ListPageViewRefresh(-1, myListViewId, null);
    
                this.Proxy = null;                
                PreWebTest += new EventHandler<PreWebTestEventArgs>(DeleteListWithQC_PreWebTest);
                PostWebTest += new EventHandler<PostWebTestEventArgs>(DeleteListWithQC_PostWebTest);
            }
                    
            public override IEnumerator<WebTestRequest> GetRequestEnumerator()
            {
                this.BeginTransaction("NavListHomepage");

                _root_homepage_aspx hp = new _root_homepage_aspx(user);
                hp.etc = 4300;
                yield return (hp);

                this.EndTransaction("NavListHomepage");

                this.BeginTransaction("ManageMembersAdd");

                ma_lists_edit_aspx ma = new ma_lists_edit_aspx(user);
                yield return (ma);

                referer = ma.lastResponse.ResponseUri.AbsoluteUri;

                _forms_FormScript_js_aspx f = new _forms_FormScript_js_aspx(user);
                f.etc = 4300;
                f.FormType = "main";
                f.FormName = "crmForm";
                headerItem = new WebTestRequestHeader("referer", referer);
                f.Headers.Add(headerItem);
                yield return (f);

                ma = new ma_lists_edit_aspx(user);
                ma.crmFormSubmitXml = EntityXmlHelper.GetCreateListXml(ownerId, 8);
                ma.crmFormSubmitMode = 1;
                ma.crmFormSubmitId = null;
                ma.crmFormUserModified = true;
                ma.crmFormSubmitObjectType = 4300;
                ma.crmFormSubmitSecurity = 852023;
                ma.crmFormSubmitOnline = 1;
                rpcInfo = Utils.GenerateWRPCToken(ma_lists_edit_aspx.url, user, tokenKey);
                ma.CRMWRPCToken = rpcInfo.token;
                ma.CRMWRPCTokenTimeStamp = rpcInfo.timeStamp;
                ma.ValidateResponse += this.DeleteListWithQC_ValidateResponse;
                yield return (ma);

                //get the newly created list id out of the post reply
                int idIdx = ma.lastResponse.BodyString.IndexOf(newIdMarker);
                string tempNewListId = ma.lastResponse.BodyString.Substring(idIdx + (DeleteListWithQC.newIdMarker.Length), 36);
                newListId = new Guid(tempNewListId);

                ma = new ma_lists_edit_aspx(user);
                ma.id = newListId;
                yield return (ma);

                referer = ma.lastResponse.ResponseUri.AbsoluteUri;

                AppGridWebService.AppGridWebService agws = new AppGridWebService.AppGridWebService();
                CrmRequest wtrRefresh = null;
                try { agws.Refresh(refreshGridList); }
                catch (crmException x)
                {
                    wtrRefresh = (CrmRequest)x.wtr;
                    wtrRefresh.Url = Utils.ReplaceServerName(wtrRefresh.Url, user);

                    string urlPath = wtrRefresh.Url.Remove(0, crmServer.Length);
                    urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                    rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                    headerItem = new WebTestRequestHeader();
                    headerItem.Name = "crmwrpctokentimestamp";
                    headerItem.Value = rpcInfo.timeStamp.ToString();
                    wtrRefresh.Headers.Add(headerItem);
                    headerItem = new WebTestRequestHeader();
                    headerItem.Name = "crmwrpctoken";
                    headerItem.Value = rpcInfo.token;
                    wtrRefresh.Headers.Add(headerItem);
                    wtrRefresh.Method = "Post";
                }
                yield return (wtrRefresh);

                f = new _forms_FormScript_js_aspx(user);
                f.etc = 4300;
                f.FormType = "main";
                f.FormName = "crmForm";
                headerItem = new WebTestRequestHeader("referer", referer);
                f.Headers.Add(headerItem);
                yield return (f);

                ma_lists_areas_aspx listArea = new ma_lists_areas_aspx(user);
                listArea.oId = newListId;
                listArea.oType = 4300;
                listArea.security = 852023;
                listArea.createdfromcode = 1;
                listArea.tabSet = "areaListMember";
                yield return (listArea);

                MA_lists_ListQualificationDlg_dlg_manage_members_aspx listQualifications = new MA_lists_ListQualificationDlg_dlg_manage_members_aspx(user);
                yield return (listQualifications);

                MarketingAutomation.MarketingAutomationWebService maws = new MarketingAutomation.MarketingAutomationWebService();
                CrmRequest wtrRetrieveList = null;
                string[] columns = new string[] { "createdfromcode" };
                try { maws.RetrieveList(newListId, columns); }
                catch (crmException x)
                {
                    wtrRetrieveList = (CrmRequest)x.wtr;
                    wtrRetrieveList.Url = Utils.ReplaceServerName(wtrRetrieveList.Url, user);

                    string urlPath = wtrRetrieveList.Url.Remove(0, crmServer.Length);
                    urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                    rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                    headerItem = new WebTestRequestHeader();
                    headerItem.Name = "crmwrpctokentimestamp";
                    headerItem.Value = rpcInfo.timeStamp.ToString();
                    wtrRetrieveList.Headers.Add(headerItem);
                    headerItem = new WebTestRequestHeader();
                    headerItem.Name = "crmwrpctoken";
                    headerItem.Value = rpcInfo.token;
                    wtrRetrieveList.Headers.Add(headerItem);
                    wtrRetrieveList.Method = "Post";
                }
                yield return (wtrRetrieveList);

                _controls_lookup_lookupmulti_aspx lookupMulti = new _controls_lookup_lookupmulti_aspx(user);
                lookupMulti.Class = "activityrelatedcustomer";
                lookupMulti.objecttypes = "1";
                lookupMulti.browse = 0;
                lookupMulti.ShowNewButton = 1;
                yield return (lookupMulti);

                _controls_lookup_lookupdata_aspx lookupData = new _controls_lookup_lookupdata_aspx(user);
                lookupData.ObjectType = 1;
                lookupData.Class = "activityrelatedcustomer";
                lookupData.objecttypes = "1";
                lookupData.browse = 0;
                lookupData.ShowNewButton = 1;
                lookupData.Mode = 4;
                lookupData.ColumnsOnly = 1;
                yield return (lookupData);

                lookupData = new _controls_lookup_lookupdata_aspx(user);
                lookupData.Class = "activityrelatedcustomer";
                lookupData.objecttypes = "1";
                lookupData.browse = 0;
                lookupData.ShowNewButton = 1;
                lookupData.Mode = 4;
                lookupData.ObjectType = 1;
                lookupData.searchvalue = Utils.GetRandomString(2, 2);
                yield return (lookupData);

                _grid_cmds_dlg_addtolist_aspx addToList = new _grid_cmds_dlg_addtolist_aspx(user);
                addToList.autoTrigger = 1;
                addToList.iObjType = 1;
                addToList.iTotal = 1;
                addToList.sIds = null;
                addToList.itemObjectId = newListId;
                yield return (addToList);

                addToList = new _grid_cmds_dlg_addtolist_aspx(user);
                addToList.iObjType = 1;
                addToList.iTotal = 1;
                addToList.iIndex = 0;
                addToList.itemObjectId = newListId;
                addToList.iId = new Guid(account[EntityIDNames.Account]);

                StringHttpBody httpBody = new StringHttpBody();
                httpBody.BodyString = "<node />";
                httpBody.ContentType = "text/xml";
                addToList.Body = httpBody;

                rpcInfo = Utils.GenerateWRPCToken(_grid_cmds_dlg_addtolist_aspx.url, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "CRMWRPCTokenTimeStamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                addToList.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "CRMWRPCToken";
                headerItem.Value = rpcInfo.token;
                addToList.Headers.Add(headerItem);
                addToList.Method = "Post";
                yield return (addToList);

                maws = new MarketingAutomation.MarketingAutomationWebService();
                wtrRetrieveList = null;
                columns = new string[] { "membercount" };
                try { maws.RetrieveList(newListId, columns); }
                catch (crmException x)
                {
                    wtrRetrieveList = (CrmRequest)x.wtr;
                    wtrRetrieveList.Url = Utils.ReplaceServerName(wtrRetrieveList.Url, user);

                    string urlPath = wtrRetrieveList.Url.Remove(0, crmServer.Length);
                    urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                    rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                    headerItem = new WebTestRequestHeader();
                    headerItem.Name = "crmwrpctokentimestamp";
                    headerItem.Value = rpcInfo.timeStamp.ToString();
                    wtrRetrieveList.Headers.Add(headerItem);
                    headerItem = new WebTestRequestHeader();
                    headerItem.Name = "crmwrpctoken";
                    headerItem.Value = rpcInfo.token;
                    wtrRetrieveList.Headers.Add(headerItem);
                    wtrRetrieveList.Method = "Post";
                }
                yield return (wtrRetrieveList);

                // Make the list Grid for members
                tempNewListId = newListId.ToString();
                memberGrid = GridXmlHelper.MemberGridForListViewRefresh(-1, memberViewId, null, tempNewListId, "4300");
                agws = new AppGridWebService.AppGridWebService();
                wtrRefresh = null;
                try { agws.Refresh(memberGrid); }
                catch (crmException x)
                {
                    wtrRefresh = (CrmRequest)x.wtr;
                    wtrRefresh.Url = Utils.ReplaceServerName(wtrRefresh.Url, user);

                    string urlPath = wtrRefresh.Url.Remove(0, crmServer.Length);
                    urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                    rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                    headerItem = new WebTestRequestHeader();
                    headerItem.Name = "crmwrpctokentimestamp";
                    headerItem.Value = rpcInfo.timeStamp.ToString();
                    wtrRefresh.Headers.Add(headerItem);
                    headerItem = new WebTestRequestHeader();
                    headerItem.Name = "crmwrpctoken";
                    headerItem.Value = rpcInfo.token;
                    wtrRefresh.Headers.Add(headerItem);
                    wtrRefresh.Method = "Post";
                }
                yield return (wtrRefresh);

                this.EndTransaction("ManageMembersAdd");

                this.BeginTransaction("ListQuickCampaign");

                MA_MiniCampaign_MiniCampaign_aspx miniCampgn = new MA_MiniCampaign_MiniCampaign_aspx(user);
                yield return (miniCampgn);
                
                MA_MiniCampaign_iframes_letterForm_aspx letterForm = new MA_MiniCampaign_iframes_letterForm_aspx(user);
                yield return (letterForm);
                string referer1 = letterForm.lastResponse.ResponseUri.AbsoluteUri;

                MA_MiniCampaign_iframes_phonecallForm_aspx phoneForm = new MA_MiniCampaign_iframes_phonecallForm_aspx(user);
                yield return (phoneForm);
                string referer2 = phoneForm.lastResponse.ResponseUri.AbsoluteUri;

                MA_MiniCampaign_iframes_appointmentForm_aspx apptForm = new MA_MiniCampaign_iframes_appointmentForm_aspx(user);
                yield return (apptForm);
                string referer3 = apptForm.lastResponse.ResponseUri.AbsoluteUri;

                MA_MiniCampaign_iframes_faxForm_aspx faxForm = new MA_MiniCampaign_iframes_faxForm_aspx(user);
                yield return (faxForm);
                string referer4 = faxForm.lastResponse.ResponseUri.AbsoluteUri;

                MA_MiniCampaign_iframes_emailForm_aspx emailForm = new MA_MiniCampaign_iframes_emailForm_aspx(user);
                yield return (emailForm);
                string referer5 = emailForm.lastResponse.ResponseUri.AbsoluteUri;

                f = new _forms_FormScript_js_aspx(user);
                f.etc = 4207;
                f.FormType = "main";
                f.FormName = "crmForm";
                headerItem = new WebTestRequestHeader("referer", referer1);
                f.Headers.Add(headerItem);
                yield return (f);

                f = new _forms_FormScript_js_aspx(user);
                f.etc = 4210;
                f.FormType = "main";
                f.FormName = "crmForm";
                headerItem = new WebTestRequestHeader("referer", referer2);
                f.Headers.Add(headerItem);
                yield return (f);

                f = new _forms_FormScript_js_aspx(user);
                f.etc = 4204;
                f.FormType = "main";
                f.FormName = "crmForm";
                headerItem = new WebTestRequestHeader("referer", referer4);
                f.Headers.Add(headerItem);
                yield return (f);

                f = new _forms_FormScript_js_aspx(user);
                f.etc = 4202;
                f.FormType = "main";
                f.FormName = "crmForm";
                headerItem = new WebTestRequestHeader("referer", referer5);
                f.Headers.Add(headerItem);
                yield return (f);

                f = new _forms_FormScript_js_aspx(user);
                f.etc = 4201;
                f.FormType = "main";
                f.FormName = "crmForm";
                headerItem = new WebTestRequestHeader("referer", referer3);
                f.Headers.Add(headerItem);
                yield return (f);

                _controls_htmlbar_htmlbar_css_aspx htmlbar = new _controls_htmlbar_htmlbar_css_aspx(user);
                htmlbar.lcid = 1033;
                yield return (htmlbar);

                DateTimeService.DateTimeWebService dtws = new DateTimeService.DateTimeWebService();
                CrmRequest wtr = null;
                try
                {
                    dtws.GetFormattedDate(DateTime.Today.ToString());
                }
                catch (crmException x)
                {
                    wtr = (CrmRequest)x.wtr;
                    wtr.Url = Utils.ReplaceServerName(wtr.Url, user);
                }
                yield return (wtr);

                yield return (wtr);

                _controls_emailbody_msgbody_aspx emailbody = new _controls_emailbody_msgbody_aspx(user);
                emailbody.entityType = "email";
                yield return (emailbody);

                _forms_contols_controls_css_aspx conts = new _forms_contols_controls_css_aspx(crmServer);
                yield return (conts);


                maws = new MarketingAutomation.MarketingAutomationWebService();
                wtrRetrieveList = null;
                string activityString = @"<fax><subject>" + Utils.GetRandomString(10, 20) + "</subject><description>" + Utils.GetRandomString(100, 120) + "</description></fax>";
                string miniCampaignName = Utils.GetRandomString(5, 10);
                try { maws.CreateMiniCampaign(4300, "", 1, newListId.ToString("B"), "", "4204", 2, activityString, miniCampaignName, true, 8, ownerId.ToString("B"), false); }
                catch (crmException x)
                {
                    wtrRetrieveList = (CrmRequest)x.wtr;
                    wtrRetrieveList.Url = Utils.ReplaceServerName(wtrRetrieveList.Url, user);

                    string urlPath = wtrRetrieveList.Url.Remove(0, crmServer.Length);
                    urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                    rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                    headerItem = new WebTestRequestHeader();
                    headerItem.Name = "crmwrpctokentimestamp";
                    headerItem.Value = rpcInfo.timeStamp.ToString();
                    wtrRetrieveList.Headers.Add(headerItem);
                    headerItem = new WebTestRequestHeader();
                    headerItem.Name = "crmwrpctoken";
                    headerItem.Value = rpcInfo.token;
                    wtrRetrieveList.Headers.Add(headerItem);
                    wtrRetrieveList.Method = "Post";
                }
                yield return (wtrRetrieveList);

                int guidMarker = wtrRetrieveList.lastResponse.BodyString.IndexOf(miniCampIdMarker);
                string miniCampIdString = wtrRetrieveList.lastResponse.BodyString.Substring(guidMarker + miniCampIdMarker.Length, 36);
                Guid miniCampGuid = new Guid(miniCampIdString);

                ma_lists_areas_aspx listMembers = new ma_lists_areas_aspx(user);
                listMembers.oId = newListId;
                listMembers.oType = 4300;
                listMembers.security = 852023;
                listMembers.tabSet = "areaMiniCampaignsForList";
                yield return (listMembers);

                // Make the list Grid for members
                tempNewListId = newListId.ToString();
                memberGrid = GridXmlHelper.CampaignGridForListViewRefresh(-1, quickCampaignViewId, null, tempNewListId, "4300");
                agws = new AppGridWebService.AppGridWebService();
                wtrRefresh = null;
                try { agws.Refresh(memberGrid); }
                catch (crmException x)
                {
                    wtrRefresh = (CrmRequest)x.wtr;
                    wtrRefresh.Url = Utils.ReplaceServerName(wtrRefresh.Url, user);
                    wtrRefresh.ThinkTime = 30;

                    string urlPath = wtrRefresh.Url.Remove(0, crmServer.Length);
                    urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                    rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                    headerItem = new WebTestRequestHeader();
                    headerItem.Name = "crmwrpctokentimestamp";
                    headerItem.Value = rpcInfo.timeStamp.ToString();
                    wtrRefresh.Headers.Add(headerItem);
                    headerItem = new WebTestRequestHeader();
                    headerItem.Name = "crmwrpctoken";
                    headerItem.Value = rpcInfo.token;
                    wtrRefresh.Headers.Add(headerItem);
                    wtrRefresh.Method = "Post";

                }
                yield return (wtrRefresh);

                int startIdx = wtrRefresh.lastResponse.BodyString.IndexOf(miniCampGuid.ToString().ToUpper());
                {
                    int endIdx = wtrRefresh.lastResponse.BodyString.IndexOf(@"oid=""", startIdx);
                    string part = wtrRefresh.lastResponse.BodyString.Substring(startIdx, endIdx - startIdx);

                    if (part.IndexOf("Completed") < 0)
                    {
                        agws = new AppGridWebService.AppGridWebService();
                        wtrRefresh = null;
                        try { agws.Refresh(memberGrid); }
                        catch (crmException x)
                        {
                            wtrRefresh = (CrmRequest)x.wtr;
                            wtrRefresh.Url = Utils.ReplaceServerName(wtrRefresh.Url, user);
                            wtrRefresh.ThinkTime = 10;

                            string urlPath = wtrRefresh.Url.Remove(0, crmServer.Length);
                            urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                            rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                            headerItem = new WebTestRequestHeader();
                            headerItem.Name = "crmwrpctokentimestamp";
                            headerItem.Value = rpcInfo.timeStamp.ToString();
                            wtrRefresh.Headers.Add(headerItem);
                            headerItem = new WebTestRequestHeader();
                            headerItem.Name = "crmwrpctoken";
                            headerItem.Value = rpcInfo.token;
                            wtrRefresh.Headers.Add(headerItem);
                            wtrRefresh.Method = "Post";
                        }
                        yield return (wtrRefresh);
                    }
                }
                this.EndTransaction("ListQuickCampaign");

                this.BeginTransaction("DeleteList");

                _grid_cmds_dlg_delete_aspx delList = new _grid_cmds_dlg_delete_aspx(user);
                delList.iObjType = 4300;
                delList.iTotal = 1;
                delList.sIds = newListId.ToString("B");
                yield return (delList);

                delList = new _grid_cmds_dlg_delete_aspx(user);
                delList.iObjType = 4300;
                delList.iTotal = 1;
                delList.iIndex = 0;
                delList.sSubTypes = null;
                delList.iId = newListId.ToString("B");

                httpBody = new StringHttpBody();
                httpBody.BodyString = "<node />";
                httpBody.ContentType = "text/xml";
                delList.Body = httpBody;

                rpcInfo = Utils.GenerateWRPCToken(_grid_cmds_dlg_delete_aspx.url, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "CRMWRPCTokenTimeStamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                delList.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "CRMWRPCToken";
                headerItem.Value = rpcInfo.token;
                delList.Headers.Add(headerItem);
                delList.Method = "Post";
                yield return (delList);

                agws = new AppGridWebService.AppGridWebService();
                wtrRefresh = null;
                try { agws.Refresh(refreshGridList); }
                catch (crmException x)
                {
                    wtrRefresh = (CrmRequest)x.wtr;
                    wtrRefresh.Url = Utils.ReplaceServerName(wtrRefresh.Url, user);

                    string urlPath = wtrRefresh.Url.Remove(0, crmServer.Length);
                    urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                    rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                    headerItem = new WebTestRequestHeader();
                    headerItem.Name = "crmwrpctokentimestamp";
                    headerItem.Value = rpcInfo.timeStamp.ToString();
                    wtrRefresh.Headers.Add(headerItem);
                    headerItem = new WebTestRequestHeader();
                    headerItem.Name = "crmwrpctoken";
                    headerItem.Value = rpcInfo.token;
                    wtrRefresh.Headers.Add(headerItem);
                    wtrRefresh.Method = "Post";
                }
                yield return (wtrRefresh);

                this.EndTransaction("DeleteList");
            }

            private const string newIdMarker = @"separator + ""id=";
            private const string miniCampIdMarker = @"<CreateMiniCampaignResult>";
            private CRMEntity user;
            private CRMEntity account;
            private string crmServer;
            private string tokenKey;
            private Guid newListId;
            private string refreshGridList;
            private string memberGrid;
    
            private static string myListViewId = "823B9210-B82A-4B11-B0F6-B9E4C9C8403F";
            private static string memberViewId = "51DE84DD-732F-440F-A506-26123A19AB50";
            private static string quickCampaignViewId = "83DC2389-0A4C-4249-A5D3-BA1AB401FF1C";

            private Guid orgId;
            private Guid ownerId;

            private WebTestRequestHeader headerItem;
            private string referer;
            private WRPCInfo rpcInfo;
        }
    }
