namespace CRM_Perf_BenchMark
{
    using System;
    using System.Collections.Generic;
    using Microsoft.VisualStudio.TestTools.WebTesting;
    using System.Net;
    using System.Text;
    using System.Xml;
    
    /// <summary>
    /// 1.	Start at Sales homepage
    /// 2.  Select Contacts
    /// 3.	Insert name criteria in Find box and Select Find button
    /// 4.	Open Contact (double click)
    /// 5.	Click on Notes
    /// 6.	Right Click on a note and select delete
    /// </summary>
    
    public class DeleteNoteOnContact : CrmWebTest
    {
        public void DeleteNoteOnContact_PreWebTest(object sender, PreWebTestEventArgs e)
        {
			EntityRequest er = new EntityRequest();
			er.Type = EntityNames.Users;
			er.ReturnAs = EntityNames.Users;
			er.ParentID = EntityManager.Instance.OrgId;

			System.Collections.Hashtable Props = new System.Collections.Hashtable();

            if (e.WebTest.Context.ContainsKey("UserRole"))
            {
                Props.Add("role", e.WebTest.Context["UserRole"]);
            }
            
			EntityRequest ContactEr = new EntityRequest();
			ContactEr.Type = EntityNames.Contacts;
			ContactEr.ParentID = Guid.NewGuid();
			ContactEr.ReturnAs = EntityNames.Contacts;

			Props.Add(EntityNames.Contacts, new EntityRequest[] { ContactEr });
			er.Props = Props;

			EntityRequest NoteEr = new EntityRequest();
			NoteEr.Type = EntityNames.Notes;
			NoteEr.ParentID = Guid.Empty;
			NoteEr.ReturnAs = EntityNames.Notes;

			Props = new System.Collections.Hashtable();
			Props.Add("objectid", new EntityRequest[] { NoteEr });

			ContactEr.Props = Props;

			System.Collections.Hashtable Entities = EntityManager.Instance.GetEntities(er);

			user = (CRMEntity)Entities[EntityNames.Users];
			annotation = (CRMEntity)Entities[EntityNames.Notes];
			EntityManager.Instance.FreeEntity((CRMEntity)Entities[EntityNames.Contacts]);
            crmServer = user["CrmServer"];
            tokenKey = EntityManager.Instance.ReadTokenKey(user["organizationid"]);

            e.WebTest.UserName = user["domainName"];
            e.WebTest.Password = user["userpassword"];
        }

        public void DeleteNoteOnContact_PostWebTest(object sender, PostWebTestEventArgs e)
        {
            // Don't delete the entity till end of test run as it is deleted from crm.
            EntityManager.Instance.DeleteEntity(annotation);
            EntityManager.Instance.FreeEntity(user);
        }

        public DeleteNoteOnContact()
        {
            WebRequest.RegisterPrefix("crm", new crmRequestFactory());
            string random = Utils.GetRandomString(2, 2);
            gridXml = GridXmlHelper.ContactSearchView(-1, searchContactsView, random, null);

            this.Proxy = null;            
            PreWebTest += new EventHandler<PreWebTestEventArgs>(DeleteNoteOnContact_PreWebTest);
            PostWebTest += new EventHandler<PostWebTestEventArgs>(DeleteNoteOnContact_PostWebTest);
        }

        public override IEnumerator<WebTestRequest> GetRequestEnumerator()
        {
			this.BeginTransaction("NavContactHomepage");

            // Get the home page
            _root_homepage_aspx hp = new _root_homepage_aspx(user);
            hp.etc = 2;
            yield return (hp);

			this.EndTransaction("NavContactHomepage");

            this.BeginTransaction("SearchContacts");

            AppGridWebService.AppGridWebService ag = new AppGridWebService.AppGridWebService();
            WebTestRequest wtr = null;

            try { ag.Reset(gridXml, gridId); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
            }
            yield return (wtr);

            this.EndTransaction("SearchContacts");

            this.BeginTransaction("OpenContact");

            // simulate double clicking of contact
            sfa_conts_edit_aspx dp = new sfa_conts_edit_aspx(user);
            dp.id = annotation["objectid"];
            yield return (dp);

            referer = dp.lastResponse.ResponseUri.AbsoluteUri;

            _forms_FormScript_js_aspx f = new _forms_FormScript_js_aspx(user);
            f.etc = 2;
            f.FormType = "main";
            f.FormName = "crmForm";
            headerItem = new WebTestRequestHeader("referer", referer);
            f.Headers.Add(headerItem);
            yield return (f);

            this.EndTransaction("OpenContact");

            this.BeginTransaction("DeleteNoteOnContact");

            _controls_notes_notesdata_aspx notes = new _controls_notes_notesdata_aspx(user);
            notes.id = new Guid(annotation["objectid"]);
            notes.ParentEntity = 2;
            notes.EnableInlineEdit = true;
            notes.EnableInsert = true;
            yield return (notes);

            _grid_cmds_dlg_delete_aspx dlgDel = new _grid_cmds_dlg_delete_aspx(user);
            dlgDel.iObjType = 5;
            dlgDel.iTotal = 1;
            dlgDel.sIds = annotation["annotationid"];
            //dlgDel.iObjSubType = "undefined";
            yield return (dlgDel);

            _grid_cmds_dlg_delete_aspx delPage2 = new _grid_cmds_dlg_delete_aspx(user);
            rpcInfo = Utils.GenerateWRPCToken(_grid_cmds_dlg_delete_aspx.url, user, tokenKey);
            headerItem = new WebTestRequestHeader();
            headerItem.Name = "CRMWRPCTokenTimeStamp";
            headerItem.Value = rpcInfo.timeStamp.ToString();
            delPage2.Headers.Add(headerItem);
            headerItem = new WebTestRequestHeader();
            headerItem.Name = "CRMWRPCToken";
            headerItem.Value = rpcInfo.token;
            delPage2.Headers.Add(headerItem);
            delPage2.Method = "Post"; 
            
            delPage2.iObjType = 5;
            delPage2.iTotal = 1;
            delPage2.iIndex = 0;
            delPage2.sSubTypes = null;
            delPage2.sSrcQueueId = null;
            delPage2.sParentId = null;
            delPage2.sCalendarId = null;
            delPage2.iId = annotation["annotationid"];

            StringHttpBody httpBody = new StringHttpBody();
            httpBody.BodyString = "<node />";
            httpBody.ContentType = "text/xml";
            delPage2.Body = httpBody;
            yield return (delPage2);

            notes = new _controls_notes_notesdata_aspx(user);
            notes.id = new Guid(annotation["objectid"]);
            notes.ParentEntity = 2;
            notes.EnableInlineEdit = true;
            notes.EnableInsert = true;
            yield return (notes);

            this.EndTransaction("DeleteNoteOnContact");
        }

        private string gridXml;
        private static string gridId = "crmGrid";
        private static string searchContactsView = "8DF19B44-A073-40C3-9D6D-EE1355D8C4BA";

        private CRMEntity user;
        private CRMEntity annotation;
        private string crmServer;
        private string tokenKey;

        private WebTestRequestHeader headerItem;
        private WRPCInfo rpcInfo;
        private string referer; 
    }
}