namespace CRM_Perf_BenchMark
{
	using System;
	using System.Collections.Generic;
	using Microsoft.VisualStudio.TestTools.WebTesting;
	using System.Net;
	using System.Text;
	using System.Xml;

    /// <summary>
    /// 1. Start at the Service Calendar
    /// 2. Click "1 Day"
    /// 3. Click "1 Week"
    /// </summary>
    /// 
	public class DisplaySchedule : CrmWebTest
	{
		public void DisplaySchedule_PreWebTest(object sender, PreWebTestEventArgs e)
		{
            EntityRequest er = new EntityRequest();
            er.Type = EntityNames.Users;
            er.ReturnAs = EntityNames.Users;
            er.ParentID = EntityManager.Instance.OrgId;

            System.Collections.Hashtable userProps = new System.Collections.Hashtable();
            if (e.WebTest.Context.ContainsKey("UserRole"))
            {
                userProps.Add("role", e.WebTest.Context["UserRole"]);
                er.Props = userProps;
            }
            
            System.Collections.Hashtable Entities = EntityManager.Instance.GetEntities(er);

            user = (CRMEntity)Entities[EntityNames.Users];

            e.WebTest.UserName = user["domainName"];
            e.WebTest.Password = user["userpassword"];
            
            crmServer = user["CrmServer"];            
		}

		public void DisplaySchedule_PostWebTest(object sender, PostWebTestEventArgs e)
		{
			EntityManager.Instance.FreeEntity(user);
		}


		public DisplaySchedule()
		{
			WebRequest.RegisterPrefix("crm", new crmRequestFactory());

			this.Proxy = null;
			PreWebTest += new EventHandler<PreWebTestEventArgs>(DisplaySchedule_PreWebTest);
			PostWebTest += new EventHandler<PostWebTestEventArgs>(DisplaySchedule_PostWebTest);
		}

		public override IEnumerator<WebTestRequest> GetRequestEnumerator()
		{
			this.BeginTransaction("DisplaySchedule - One Day");
			
            DateTimeService.DateTimeWebService dtws = new DateTimeService.DateTimeWebService();
            CrmRequest wtr = null;
            string currDate = DateTime.Today.ToString();
            try { dtws.GetFormattedDate(currDate); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);
            }
            yield return (wtr);

            sm_ganttcontrolframe_aspx smgantcontrol = new sm_ganttcontrolframe_aspx(user);
			smgantcontrol.startDate = DateTime.Today;
			smgantcontrol.endDate = DateTime.Today.AddDays(1);
			smgantcontrol.subareatype = "schedules";
			smgantcontrol.viewID = new Guid("7738C997-2681-43BE-BFCF-DB1D159B5C5A");
			smgantcontrol.zoomLevel = 4;
			yield return (smgantcontrol);

            int index = smgantcontrol.lastResponse.BodyString.IndexOf(indexPointer);
            string idString = smgantcontrol.lastResponse.BodyString.Substring(index + indexPointer.Length, 36);
            Guid id = new Guid(idString);

            sm_detailsappointmentpane_aspx smdetails = new sm_detailsappointmentpane_aspx(user);
            smdetails.subAreaType = "schedules";
            smdetails.rowType = 4000;
            smdetails.rowId = id;
            yield return (smdetails);

            index = smdetails.lastResponse.BodyString.IndexOf(indexPtr);
            string customizableFormQueryString = smdetails.lastResponse.BodyString.Substring(index + indexPtr.Length, smdetails.lastResponse.BodyString.IndexOf('"', index + indexPtr.Length) - (index + indexPtr.Length));
            string tabIDString = customizableFormQueryString.Substring(customizableFormQueryString.IndexOf("tabID={") + 7, 36);
            Guid tabID = new Guid(tabIDString);

            sm_customizableform_aspx smcustomform = new sm_customizableform_aspx(user);
            smcustomform.type = 4000;
            smcustomform.id = id;
            smcustomform.tabID = tabID;
            yield return (smcustomform);

            referer = smcustomform.lastResponse.ResponseUri.ToString();

            _forms_FormScript_js_aspx fs = new _forms_FormScript_js_aspx(user);
            fs.etc = 4000;
            fs.FormType = "appointmentBook";
            fs.FormName = "crmForm";
            headerItem = new WebTestRequestHeader();
            headerItem.Name = "referer";
            headerItem.Value = referer;
            fs.Headers.Add(headerItem);
            yield return (fs);

			this.EndTransaction("DisplaySchedule - One Day");

            this.BeginTransaction("DisplaySchedule - One Week");

            dtws = new DateTimeService.DateTimeWebService();
            
            wtr = null;
            
            currDate = DateTime.Today.ToString();
            try { dtws.GetFormattedDate(currDate); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);
            }
            yield return (wtr);
                        
            smgantcontrol = new sm_ganttcontrolframe_aspx(user);
            smgantcontrol.startDate = DateTime.Today;
            smgantcontrol.endDate = DateTime.Today.AddDays(7);
            smgantcontrol.subareatype = "schedules";
            smgantcontrol.viewID = new Guid("7738C997-2681-43BE-BFCF-DB1D159B5C5A");
            smgantcontrol.zoomLevel = 4;
            yield return (smgantcontrol);

            index = smgantcontrol.lastResponse.BodyString.IndexOf(indexPointer);
            idString = smgantcontrol.lastResponse.BodyString.Substring(index + indexPointer.Length, 36);
            id = new Guid(idString);

            smdetails = new sm_detailsappointmentpane_aspx(user);
            smdetails.subAreaType = "schedules";
            smdetails.rowType = 4000;
            smdetails.rowId = id;
            yield return (smdetails);

            smcustomform = new sm_customizableform_aspx(user);
            smcustomform.type = 4000;
            smcustomform.id = id;
            smcustomform.tabID = tabID;
            yield return (smcustomform);

            referer = smcustomform.lastResponse.ResponseUri.ToString();

            fs = new _forms_FormScript_js_aspx(user);
            fs.etc = 4000;
            fs.FormType = "appointmentBook";
            fs.FormName = "crmForm";
            headerItem = new WebTestRequestHeader();
            headerItem.Name = "referer";
            headerItem.Value = referer;
            fs.Headers.Add(headerItem);
            yield return (fs);

            this.EndTransaction("DisplaySchedule - One Week");
		}

		private CRMEntity user;
        private string crmServer;
        private string referer;
        private WebTestRequestHeader headerItem;
        private string indexPtr = @"var pagesNames = new Array(""customizableForm.aspx?";
        private string indexPointer = @"name=""businessunitid"" CLASS=""ms-crm-List-DataColumn""><col><tr class=""ms-crm-List-Row"" oid=""&#123;";
	}
}