namespace CRM_Perf_BenchMark
{
    using System;
    using System.Collections.Generic;
    using Microsoft.VisualStudio.TestTools.WebTesting;
    using System.Net;
    using System.Text;
    using System.Xml;
    
    /// <summary>
    /// 1. Start at Sales homepage
    /// 2. Select Quotes
    /// 3. Search for a quote with a string
    /// 4. Open a Quote (double click)
    /// 5. Click On Activities
    /// 6. Click New Activity
    /// 7. Select Email
    /// 8. Fill Email data
    /// 9. Click Save and Close
    /// </summary>

    public class EmailQuote : CrmWebTest
    {
        public void EmailQuote_PreWebTest(object sender, PreWebTestEventArgs e)
        {
			EntityRequest er = new EntityRequest();
			er.Type = EntityNames.Users;
			er.ReturnAs = EntityNames.Users;
			er.ParentID = EntityManager.Instance.OrgId;

			System.Collections.Hashtable Props = new System.Collections.Hashtable();

            if (e.WebTest.Context.ContainsKey("UserRole"))
            {
                Props.Add("role", e.WebTest.Context["UserRole"]);
            }
            
			EntityRequest QuoteEr = new EntityRequest();
			QuoteEr.Type = EntityNames.Quotes;
			QuoteEr.ParentID = Guid.NewGuid();
			QuoteEr.ReturnAs = EntityNames.Quotes;

			Props.Add(EntityNames.Accounts, new EntityRequest[] { QuoteEr });

			er.Props = Props;

			System.Collections.Hashtable Entities = EntityManager.Instance.GetEntities(er);

			user = (CRMEntity)Entities[EntityNames.Users];
			quote = (CRMEntity)Entities[EntityNames.Quotes];
            crmServer = user["CrmServer"];
            tokenKey = EntityManager.Instance.ReadTokenKey(user["organizationid"]);
            orgId = new Guid(user["organizationid"]);

            e.WebTest.UserName = user["domainName"];
            e.WebTest.Password = user["userpassword"];            
        }

        public void EmailQuote_PostWebTest(object sender, PostWebTestEventArgs e)
        {
            EntityManager.Instance.FreeEntity(user);
            EntityManager.Instance.FreeEntity(quote);
        }

        public EmailQuote()
        {
            WebRequest.RegisterPrefix("crm", new crmRequestFactory());
            gridXml = GridXmlHelper.QuoteSearchView(-1, searchQuoteView, null);

            this.Proxy = null;            
            PreWebTest += new EventHandler<PreWebTestEventArgs>(EmailQuote_PreWebTest);
            PostWebTest += new EventHandler<PostWebTestEventArgs>(EmailQuote_PostWebTest);
        }

        public override IEnumerator<WebTestRequest> GetRequestEnumerator()
        {
            Guid quoteId = new Guid(quote["quoteid"]);
            gridRefreshXml = GridXmlHelper.ActivityGridForEntityViewRefresh(-1, activityView, null, quote[EntityIDNames.Quote], "1084");

			this.BeginTransaction("NavQuoteHomepage");

            _root_homepage_aspx hp = new _root_homepage_aspx(user);
            hp.etc = 1084;
            yield return (hp);

			this.EndTransaction("NavQuoteHomepage");

            this.BeginTransaction("SearchQuotes");
            //Search for a quote with a string
			AppGridWebService.AppGridWebService agws = new AppGridWebService.AppGridWebService();
            CrmRequest wtrReset = null;
            try { agws.Reset(gridXml, crmGrid); }
            catch (crmException x)
            {
                wtrReset = (CrmRequest)x.wtr;
                wtrReset.Url = Utils.ReplaceServerName(wtrReset.Url, user);

                string urlPath = wtrReset.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtrReset.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtrReset.Headers.Add(headerItem);
                wtrReset.Method = "Post";
            }
            yield return (wtrReset);

            this.EndTransaction("SearchQuotes");

            this.BeginTransaction("OpenQuote");

            sfa_quotes_edit_aspx dp = new sfa_quotes_edit_aspx(user);
            dp.id = quoteId;
            yield return (dp);

            referer = dp.lastResponse.ResponseUri.AbsoluteUri;

            _forms_FormScript_js_aspx formScript = new _forms_FormScript_js_aspx(user);
            formScript.etc = 1084;
            formScript.FormType = "main";
            formScript.FormName = "crmForm";
            headerItem = new WebTestRequestHeader("referer", referer);
            formScript.Headers.Add(headerItem);
            yield return (formScript);

            this.EndTransaction("OpenQuote");
            
            this.BeginTransaction("EmailQuote");

            sfa_quotes_areas_aspx qa = new sfa_quotes_areas_aspx(user);
            qa.oId = quoteId;
            qa.oType = 1084;
            qa.security = 852023;
            qa.tabSet = "areaActivities";
            yield return (qa);

            activities_dlg_create_aspx dg = new activities_dlg_create_aspx(user);
            yield return (dg);

            activities_email_edit_aspx emailEdit = new activities_email_edit_aspx(user);
            emailEdit.pId = quoteId;
            emailEdit.pType = 1084;
            emailEdit.pName = quote["name"];
            emailEdit.partyid = new Guid(quote["customerid"]);
            emailEdit.partytype = int.Parse(quote["customeridtype"]);
            emailEdit.partyname = quote["customeridname"];
            yield return (emailEdit);

            referer = emailEdit.lastResponse.ResponseUri.AbsoluteUri;

            formScript = new _forms_FormScript_js_aspx(user);
            formScript.etc = 4202;
            formScript.FormType = "main";
            formScript.FormName = "crmForm";
            headerItem = new WebTestRequestHeader("referer", referer);
            formScript.Headers.Add(headerItem);
            yield return (formScript);


            _controls_emailbody_msgbody_aspx msgBody = new _controls_emailbody_msgbody_aspx(user);
            msgBody.entityType = "email";
            yield return (msgBody);

            emailEdit = new activities_email_edit_aspx(user);
            emailEdit.pId = quoteId;
            emailEdit.pType = 1084;
            emailEdit.pName = quote["name"];
            emailEdit.partyid = new Guid(quote["customerid"]);
            emailEdit.partytype = int.Parse(quote["customeridtype"]);
            emailEdit.partyname = quote["customeridname"];

            emailEdit.crmFormSubmitXml = EntityXmlHelper.GetCreateEmailXml(new Guid(user["systemuserid"]), 8, new Guid(quote["customerid"]), int.Parse(quote["customeridtype"]), quoteId, 1084, new Guid(user["systemuserid"]), 8); //e.Serialize();

            rpcInfo = Utils.GenerateWRPCToken(activities_email_edit_aspx.url, user, tokenKey);
            emailEdit.CRMWRPCToken = rpcInfo.token;
            emailEdit.CRMWRPCTokenTimeStamp = rpcInfo.timeStamp;
            emailEdit.crmFormSubmitMode = 2;            
            emailEdit.crmFormUserModified = true;
            emailEdit.crmFormSubmitObjectType = 4202;
            emailEdit.crmFormSubmitSecurity = 852023;
            emailEdit.crmFormSubmitOnline = 1;
            yield return (emailEdit);
                        

            agws = new AppGridWebService.AppGridWebService();
            CrmRequest wtrRefresh = null;
            try { agws.Refresh(gridRefreshXml); }
            catch (crmException x)
            {
                wtrRefresh = (CrmRequest)x.wtr;
                wtrRefresh.Url = Utils.ReplaceServerName(wtrRefresh.Url, user);

                string urlPath = wtrRefresh.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtrRefresh.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtrRefresh.Headers.Add(headerItem);
                wtrRefresh.Method = "Post";
            }
            yield return (wtrRefresh);

            this.EndTransaction("EmailQuote");
        }

        private string gridXml;
        private string gridRefreshXml;
        private static string crmGrid = "crmGrid";

        private CRMEntity user;        
        private CRMEntity quote;
        private string crmServer;
        private string tokenKey;
        private static string searchQuoteView = "E346BF9F-9527-4FFE-89D6-748F53C1078A";
        private static string activityView = "00000000-0000-0000-00AA-000010001903";
        private Guid orgId;

        private WRPCInfo rpcInfo;
        private WebTestRequestHeader headerItem;
        private string referer;
    }
}