namespace CRM_Perf_BenchMark
{
    using System;
    using System.Collections.Generic;
    using Microsoft.VisualStudio.TestTools.WebTesting;
    using System.Net;
    using System.Text;
    using System.Xml;
    
    /// <summary>
    /// 1.	Start at Workplace homepage
    /// 2.  Select Activities
    /// 3.	Select Appointments from the Type list
    /// 4.	Select My Appointments from the view type
    /// 5.	Open an appointment (Double Click)
    /// </summary>
    
    public class OpenAppointment : CrmWebTest
    {
        public void OpenAppointment_PreWebTest(object sender, PreWebTestEventArgs e)
        {
            EntityRequest er = new EntityRequest();
            er.Type = EntityNames.Users;
            er.ReturnAs = EntityNames.Users;
            er.ParentID = EntityManager.Instance.OrgId;

            System.Collections.Hashtable Props = new System.Collections.Hashtable();

            if (e.WebTest.Context.ContainsKey("UserRole"))
            {
                Props.Add("role", e.WebTest.Context["UserRole"]);
            }
            
            EntityRequest ApptEr = new EntityRequest();
            ApptEr.Type = EntityNames.Appointments;
            ApptEr.ParentID = Guid.NewGuid();
            ApptEr.ReturnAs = EntityNames.Appointments;

            Props.Add(EntityNames.Appointments, new EntityRequest[] { ApptEr });

            er.Props = Props;

            System.Collections.Hashtable Entities = EntityManager.Instance.GetEntities(er);

            user = (CRMEntity)Entities[EntityNames.Users];
            appointment = (CRMEntity)Entities[EntityNames.Appointments];

            e.WebTest.UserName = user["domainName"];
            e.WebTest.Password = user["userpassword"];
            
            crmServer = user["CrmServer"];
            tokenKey = EntityManager.Instance.ReadTokenKey(user["organizationid"]);
            
            user2 = EntityManager.Instance.GetNextUser(new Guid(user["organizationid"]));
        }
        
        public void OpenAppointment_PostWebTest(object sender, PostWebTestEventArgs e)
        {
            // Don't delete the entity till end of test run as it is deleted from crm.
            EntityManager.Instance.FreeEntity(user);
            EntityManager.Instance.FreeEntity(user2);
            EntityManager.Instance.FreeEntity(appointment);
        }

        public OpenAppointment()
        {
            WebRequest.RegisterPrefix("crm", new crmRequestFactory());
            AllAppointmentsXml = GridXmlHelper.AppointmentPageAllView(-1, allAppointmentsView, null);
            MyAppointmentsXml = GridXmlHelper.AppointmentPageMyView(-1, myAppointmentView, null);
            gridXml = GridXmlHelper.AppointmentPageViewRefresh(-1, myAppointmentView, null);

            this.Proxy = null;            
            PreWebTest += new EventHandler<PreWebTestEventArgs>(OpenAppointment_PreWebTest);
            PostWebTest += new EventHandler<PostWebTestEventArgs>(OpenAppointment_PostWebTest);
        }

        public override IEnumerator<WebTestRequest> GetRequestEnumerator()
        {
            this.BeginTransaction("NavActivityHomepage");

            // Get the home page
            workplace_home_activities_aspx hp = new workplace_home_activities_aspx(user);
            yield return (hp);

            this.EndTransaction("NavActivityHomepage");

            this.BeginTransaction("NavAllAppointments");

            ActivitiesWebService.ActivitiesWebService actWs = new ActivitiesWebService.ActivitiesWebService();
            CrmRequest wtr = null;
            try { actWs.GetQueryList("appointment"); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
            }
            yield return (wtr);

            AppGridWebService.AppGridWebService ag = new AppGridWebService.AppGridWebService();
            wtr = null;
            try { ag.Reset(AllAppointmentsXml, crmGrid); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);
                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
            }
            yield return (wtr);

            this.EndTransaction("NavAllAppointments");

            this.BeginTransaction("NavMyAppointments");

            ag = new AppGridWebService.AppGridWebService();
            wtr = null;
            try { ag.Reset(MyAppointmentsXml, crmGrid); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
            }
            yield return (wtr);

            this.EndTransaction("NavMyAppointments");

            this.BeginTransaction("OpenAppointment");

            activities_appointment_edit_aspx editPage = new activities_appointment_edit_aspx(user);
            editPage.id = appointment[EntityIDNames.Appointment];
            yield return (editPage);

            referer = editPage.lastResponse.ResponseUri.AbsoluteUri;

            _forms_FormScript_js_aspx f = new _forms_FormScript_js_aspx(user);
            f.etc = 4201;
            f.FormType = "main";
            f.FormName = "crmForm";
            headerItem = new WebTestRequestHeader("referer", referer);
            f.Headers.Add(headerItem);
            yield return (f);

            this.EndTransaction("OpenAppointment");
        }

        private const string newIdMarker = @"SetQueryString(sLocation,""id"",""";
        private string gridXml;
        private string AllAppointmentsXml;
        private string MyAppointmentsXml;
        private CRMEntity user;
        private CRMEntity user2;
        private CRMEntity appointment;
        private string crmServer;
        private string tokenKey;
        private static string crmGrid = "crmGrid";
        private static string allAppointmentsView = "5B5F75F6-B585-4212-952A-B4CBB64E2627";
        private static string myAppointmentView = "606EEA12-0C33-47D3-96E1-BA1529EE8205";
        private WRPCInfo rpcInfo;
        private WebTestRequestHeader headerItem;
        private string referer;
    }
}