namespace CRM_Perf_BenchMark
{
    using System;
    using System.Collections.Generic;
    using Microsoft.VisualStudio.TestTools.WebTesting;
    using System.Net;
    using System.Text;
    using System.Xml;
    
    /// <summary>
    /// 1.	Start at Workplace homepage
    /// 2.  Select Activities
    /// 2.	Select Emails from the Type list
    /// 3.	Select My draft E-mails from the view type
    /// 4.	Open an Email(Double Click)
    /// </summary>

    public class OpenEmail : CrmWebTest
    {
        public void OpenEmail_PreWebTest(object sender, PreWebTestEventArgs e)
        {
            EntityRequest er = new EntityRequest();
            er.Type = EntityNames.Users;
            er.ReturnAs = EntityNames.Users;
            er.ParentID = EntityManager.Instance.OrgId;

            System.Collections.Hashtable Props = new System.Collections.Hashtable();

            if (e.WebTest.Context.ContainsKey("UserRole"))
            {
                Props.Add("role", e.WebTest.Context["UserRole"]);
            }
            
            EntityRequest EmailEr = new EntityRequest();
            EmailEr.Type = EntityNames.Emails;
            EmailEr.ParentID = Guid.NewGuid();
            EmailEr.ReturnAs = EntityNames.Emails;

            Props.Add(EntityNames.Emails, new EntityRequest[] { EmailEr });

            er.Props = Props;

            System.Collections.Hashtable Entities = EntityManager.Instance.GetEntities(er);

            user = (CRMEntity)Entities[EntityNames.Users];
            email = (CRMEntity)Entities[EntityNames.Emails];

            e.WebTest.UserName = user["domainName"];
            e.WebTest.Password = user["userpassword"];
            
            crmServer = user["CrmServer"];
            tokenKey = EntityManager.Instance.ReadTokenKey(user["organizationid"]);
            
            user2 = EntityManager.Instance.GetNextUser(new Guid(user["organizationid"]));            
        }
      
        public void OpenEmail_PostWebTest(object sender, PostWebTestEventArgs e)
        {
            // Don't delete the entity till end of test run as it is deleted from crm.
            EntityManager.Instance.FreeEntity(user);
            EntityManager.Instance.FreeEntity(user2);
            EntityManager.Instance.FreeEntity(email);
        }

        public OpenEmail()
        {
            WebRequest.RegisterPrefix("crm", new crmRequestFactory());
            MyReceivedEmailsXml = GridXmlHelper.EmailMyReceivedView(-1, myReceivedEmailsView, null);
            MyDraftedEmailsXml = GridXmlHelper.EmailMyDraftedView(-1, myDraftedEmailsView, null);

            this.Proxy = null;            
            PreWebTest += new EventHandler<PreWebTestEventArgs>(OpenEmail_PreWebTest);
            PostWebTest += new EventHandler<PostWebTestEventArgs>(OpenEmail_PostWebTest);
        }

        public override IEnumerator<WebTestRequest> GetRequestEnumerator()
        {
            this.BeginTransaction("NavActivityHomepage");

            // Get the home page
            workplace_home_activities_aspx hp = new workplace_home_activities_aspx(user);
            yield return (hp);

            this.EndTransaction("NavActivityHomepage");

            this.BeginTransaction("NavMyReceivedEmails");

            ActivitiesWebService.ActivitiesWebService actWs = new ActivitiesWebService.ActivitiesWebService();
            CrmRequest wtr = null;
            try { actWs.GetQueryList("email"); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
            }
            yield return (wtr);

            AppGridWebService.AppGridWebService ag = new AppGridWebService.AppGridWebService();
            wtr = null;
            try { ag.Reset(MyReceivedEmailsXml, crmGrid); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
            }
            yield return (wtr);

            this.EndTransaction("NavMyReceivedEmails");

            this.BeginTransaction("NavMyDraftEmails");

            ag = new AppGridWebService.AppGridWebService();
            wtr = null;
            try { ag.Reset(MyDraftedEmailsXml, crmGrid); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
            }
            yield return (wtr);

            this.EndTransaction("NavMyDraftEmails");

            this.BeginTransaction("OpenEmail");

            activities_email_edit_aspx editPage = new activities_email_edit_aspx(user);
            editPage.id = email[EntityIDNames.Email];
            yield return (editPage);

            referer = editPage.lastResponse.ResponseUri.AbsoluteUri;

            _forms_FormScript_js_aspx f = new _forms_FormScript_js_aspx(user);
            f.etc = 4202;
            f.FormType = "main";
            f.FormName = "crmForm";
            headerItem = new WebTestRequestHeader("referer", referer);
            f.Headers.Add(headerItem);
            yield return (f);

            _controls_emailbody_msgbody_aspx msgBody = new _controls_emailbody_msgbody_aspx(user);
            msgBody.id = new Guid(email[EntityIDNames.Email]);
            msgBody.entityType = "email";
            yield return (msgBody);

            this.EndTransaction("OpenEmail");
        }

        private const string newIdMarker = @"SetQueryString(sLocation,""id"",""";
        private string MyReceivedEmailsXml;
        private string MyDraftedEmailsXml;
        private CRMEntity user;
        private CRMEntity user2;
        private CRMEntity email;
        private string crmServer;
        private string tokenKey;
        private static string crmGrid = "crmGrid";
        private static string myReceivedEmailsView = "7AFF64B0-5B8C-4AD7-89F8-976777EEE178";
        private static string myDraftedEmailsView = "6E8B4B8C-30F7-44EF-A958-BE72BAABC349";

        private WRPCInfo rpcInfo;
        private WebTestRequestHeader headerItem;
        private string referer;
    }
}