namespace CRM_Perf_BenchMark
{
    using System;
    using System.Collections.Generic;
    using Microsoft.VisualStudio.TestTools.WebTesting;
    using System.Net;
    using System.Text;
    using System.Xml;
    
    /// <summary>
    /// 1.  Start at Sales homepage
    /// 2.	Select Accounts
    /// 3.	Select a random view
    /// 4.	Click right button at bottom till last page
    /// </summary>
    
    public class RefreshGridAccount : CrmWebTest
    {
        public void RefreshGridAccount_PreWebTest(object sender, PreWebTestEventArgs e)
        {
            EntityRequest er = new EntityRequest();
            er.Type = EntityNames.Users;
            er.ReturnAs = EntityNames.Users;
            er.ParentID = EntityManager.Instance.OrgId;

            System.Collections.Hashtable Props = new System.Collections.Hashtable();

            if (e.WebTest.Context.ContainsKey("UserRole"))
            {
                Props.Add("role", e.WebTest.Context["UserRole"]);
            }
            
            EntityRequest AccountEr = new EntityRequest();
            AccountEr.Type = EntityNames.Accounts;
            AccountEr.ParentID = Guid.NewGuid();
            AccountEr.ReturnAs = EntityNames.Accounts;

            Props.Add(EntityNames.Accounts, new EntityRequest[] { AccountEr });

            er.Props = Props;

            System.Collections.Hashtable Entities = EntityManager.Instance.GetEntities(er);

            user = (CRMEntity)Entities[EntityNames.Users];
            crmServer = user["CrmServer"];
            tokenKey = EntityManager.Instance.ReadTokenKey(user["organizationid"]);

            e.WebTest.UserName = user["domainName"];
            e.WebTest.Password = user["userpassword"];            
        }
        
        public void RefreshGridAccount_PostWebTest(object sender, PostWebTestEventArgs e)
        {
            EntityManager.Instance.FreeEntity(user);
        }

        public RefreshGridAccount()
        {
            WebRequest.RegisterPrefix("crm", new crmRequestFactory());

            numGen = new Random();
            randomView = numGen.Next(5); // 3
            currentViewName = viewNames[randomView];
            currentViewId = viewIds[randomView];
            gridXml = GridXmlHelper.AccountPageView(-1, currentViewId, null);

            this.Proxy = null;            
            PreWebTest += new EventHandler<PreWebTestEventArgs>(RefreshGridAccount_PreWebTest);
            PostWebTest += new EventHandler<PostWebTestEventArgs>(RefreshGridAccount_PostWebTest);
        }
         
        public override IEnumerator<WebTestRequest> GetRequestEnumerator()
        {
            _root_homepage_aspx homepage = new _root_homepage_aspx(user);
            homepage.etc = 1;
            yield return (homepage);

            bool moreRecords;
            int pageNum = 1;
            
            do
            {
                // Initialize moreRecords to false for every test. If next page exists, 
                // moreRecords is set to true to continue.
                moreRecords = false;
                string pagingCookie = null;
                AppGridWebService.AppGridWebService ag = new AppGridWebService.AppGridWebService();
                CrmRequest wtr = null;

                this.BeginTransaction(currentViewName + " pNum " + pageNum.ToString());

                if (pageNum == 1)
                {                         
                    try { ag.Reset(gridXml, gridId); }
                    catch (crmException x)
                    {
                        wtr = (CrmRequest)x.wtr;
                        wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                        string urlPath = wtr.Url.Remove(0, crmServer.Length);
                        urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                        rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                        headerItem = new WebTestRequestHeader();
                        headerItem.Name = "crmwrpctokentimestamp";
                        headerItem.Value = rpcInfo.timeStamp.ToString();
                        wtr.Headers.Add(headerItem);
                        headerItem = new WebTestRequestHeader();
                        headerItem.Name = "crmwrpctoken";
                        headerItem.Value = rpcInfo.token;
                        wtr.Headers.Add(headerItem);
                        wtr.Method = "Post";
                    }
                    yield return (wtr);
                }
                else
                {   
                    try { ag.Refresh(gridXml); }
                    catch (crmException x)
                    {
                        wtr = (CrmRequest)x.wtr;
                        wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                        string urlPath = wtr.Url.Remove(0, crmServer.Length);
                        urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                        rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                        headerItem = new WebTestRequestHeader();
                        headerItem.Name = "crmwrpctokentimestamp";
                        headerItem.Value = rpcInfo.timeStamp.ToString();
                        wtr.Headers.Add(headerItem);
                        headerItem = new WebTestRequestHeader();
                        headerItem.Name = "crmwrpctoken";
                        headerItem.Value = rpcInfo.token;
                        wtr.Headers.Add(headerItem);
                        wtr.Method = "Post";
                    }
                    yield return (wtr);
                }

                this.EndTransaction(currentViewName + " pNum " + pageNum.ToString());

                int pos = wtr.lastResponse.BodyString.IndexOf(@"pagingCookie=""");
                if (pos > 0)
                {
                    int endPos = wtr.lastResponse.BodyString.IndexOf('"', pos + 15);
                    pagingCookie = wtr.lastResponse.BodyString.Substring(pos + 14, endPos - (pos + 14));
                }
                if (wtr.lastResponse.BodyString.Contains(@"morerecords=""1"""))
                {
                    moreRecords = true;
                    pageNum++;
                    gridXml = GridXmlHelper.AccountPageViewRefresh( pageNum, currentViewId, pagingCookie);
                }
            } while (moreRecords == true);   
        }

        private string gridXml;
        private string gridId = "crmGrid";

        private CRMEntity user;
        private string crmServer;
        private string tokenKey;
        
        private string[] viewNames = new string[] { "Accts no order in last 6 months ",     "Accts respd to camps in last 6 months ", "Accts no campaign sent in last 3 m ",  "Active accounts view" ,                "Inactive accounts view",               "My active accounts view"};
        private string[] viewIds   = new string[] { "C147F1F7-1D78-4D10-85BF-7E03B79F74FA", "49FB9771-09E1-4E70-B193-198752493577",               "CFBCD7AF-AEE5-4E45-8ECC-C040D4020581", "00000000-0000-0000-00AA-000010001002", "00000000-0000-0000-00AA-000010001031", "00000000-0000-0000-00AA-000010001001"};
        private Random numGen;
        private int randomView;

        private string currentViewName;
        private string currentViewId;

        private WRPCInfo rpcInfo;
        private WebTestRequestHeader headerItem;
    }
}