namespace CRM_Perf_BenchMark
{
    using System;
    using System.Collections.Generic;
    using Microsoft.VisualStudio.TestTools.WebTesting;
    using System.Net;
    using System.Text;
    using System.Xml;
    
    /// <summary>
    /// 1.  Start at Sales homepage
    /// 2.	Select Contacts
    /// 3.	Select a random view
    /// 4.	Click right button at bottom till last page
    /// </summary>
   
    public class RefreshGridContact : CrmWebTest
    {
        public void RefreshGridContact_PreWebTest(object sender, PreWebTestEventArgs e)
        {
            EntityRequest er = new EntityRequest();
            er.Type = EntityNames.Users;
            er.ReturnAs = EntityNames.Users;
            er.ParentID = EntityManager.Instance.OrgId;

            System.Collections.Hashtable Props = new System.Collections.Hashtable();
            if (e.WebTest.Context.ContainsKey("UserRole"))
            {
                Props.Add("role", e.WebTest.Context["UserRole"]);
            }
            
            EntityRequest ContactEr = new EntityRequest();
            ContactEr.Type = EntityNames.Contacts;
            ContactEr.ParentID = Guid.NewGuid();
            ContactEr.ReturnAs = EntityNames.Contacts;

            Props.Add(EntityNames.Accounts, new EntityRequest[] { ContactEr });

            er.Props = Props;

            System.Collections.Hashtable Entities = EntityManager.Instance.GetEntities(er);

            user = (CRMEntity)Entities[EntityNames.Users];
            crmServer = user["CrmServer"];
            tokenKey = EntityManager.Instance.ReadTokenKey(user["organizationid"]);

            e.WebTest.UserName = user["domainName"];
            e.WebTest.Password = user["userpassword"];            
        }
        
        public void RefreshGridContact_PostWebTest(object sender, PostWebTestEventArgs e)
        {
            EntityManager.Instance.FreeEntity(user);
        }

        public RefreshGridContact()
        {
            WebRequest.RegisterPrefix("crm", new crmRequestFactory());
            numGen = new Random();
            randomView = numGen.Next(5); // 0
            currentViewName = viewNames[randomView];
            currentViewId = viewIds[randomView];
            gridXml = GridXmlHelper.ContactPageView(-1, currentViewId, null);

            this.Proxy = null;            
            PreWebTest += new EventHandler<PreWebTestEventArgs>(RefreshGridContact_PreWebTest);
            PostWebTest += new EventHandler<PostWebTestEventArgs>(RefreshGridContact_PostWebTest);
        }

        static RefreshGridContact()
        {
        }

        public override IEnumerator<WebTestRequest> GetRequestEnumerator()
        {
            _root_homepage_aspx homepage = new _root_homepage_aspx(user);
            homepage.etc = 2;
            yield return (homepage);

            bool moreRecords;
            int pageNum = 1;

            do
            {
                // Initialize moreRecords to false for every test. If next page exists, 
                // moreRecords is set to true to continue.
                moreRecords = false;
                string pagingCookie = null;
                AppGridWebService.AppGridWebService ag = new AppGridWebService.AppGridWebService();
                CrmRequest wtr = null;

                this.BeginTransaction(currentViewName + " pNum " + pageNum.ToString());

                if (pageNum == 1)
                {
                    try { ag.Reset(gridXml, gridId); }
                    catch (crmException x)
                    {
                        wtr = (CrmRequest)x.wtr;
                        wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                        string urlPath = wtr.Url.Remove(0, crmServer.Length);
                        urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                        rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                        headerItem = new WebTestRequestHeader();
                        headerItem.Name = "crmwrpctokentimestamp";
                        headerItem.Value = rpcInfo.timeStamp.ToString();
                        wtr.Headers.Add(headerItem);
                        headerItem = new WebTestRequestHeader();
                        headerItem.Name = "crmwrpctoken";
                        headerItem.Value = rpcInfo.token;
                        wtr.Headers.Add(headerItem);
                        wtr.Method = "Post";
                    }
                    yield return (wtr);
                }
                else
                {
                    try { ag.Refresh(gridXml); }
                    catch (crmException x)
                    {
                        wtr = (CrmRequest)x.wtr;
                        wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                        string urlPath = wtr.Url.Remove(0, crmServer.Length);
                        urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                        rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                        headerItem = new WebTestRequestHeader();
                        headerItem.Name = "crmwrpctokentimestamp";
                        headerItem.Value = rpcInfo.timeStamp.ToString();
                        wtr.Headers.Add(headerItem);
                        headerItem = new WebTestRequestHeader();
                        headerItem.Name = "crmwrpctoken";
                        headerItem.Value = rpcInfo.token;
                        wtr.Headers.Add(headerItem);
                        wtr.Method = "Post";
                    }
                    yield return (wtr);
                }

                this.EndTransaction(currentViewName + " pNum " + pageNum.ToString());

                int pos = wtr.lastResponse.BodyString.IndexOf(@"pagingCookie=""");
                if (pos > 0)
                {
                    int endPos = wtr.lastResponse.BodyString.IndexOf('"', pos + 15);
                    string pagingCookieOld = pagingCookie;
                    pagingCookie = wtr.lastResponse.BodyString.Substring(pos + 14, endPos - (pos + 14));
                    if (pagingCookie == pagingCookieOld)
                        throw new Exception("PagingCookie returned is same as previous one. Can potentially lead to infinite loop, excepting out");
                }
                if (wtr.lastResponse.BodyString.Contains(@"morerecords=""1"""))
                {
                    moreRecords = true;
                    pageNum++;
                    gridXml = GridXmlHelper.ContactPageViewRefresh(pageNum, currentViewId, pagingCookie);
                }
            } while (moreRecords == true);
        }

        private string gridXml;
        private string gridId = "crmGrid";

        private CRMEntity user;

        private string crmServer;
        private string tokenKey;
        private string[] viewNames = new string[] { "Active Contacts view",                 "Conts not placed order in last 6 m view",    "Conts respd to campgns in 6 mnth view", "Conts no campaign sent in last 3 m view", "Inactive contacts view",               "My active contacts view" };
        private string[] viewIds   = new string[] { "00000000-0000-0000-00AA-000010001004", "9818766E-7172-4D59-9279-013835C3DECD",       "9C241A33-CA0B-4E50-AE92-DB780D5B2A12",  "927E6CD8-B3ED-4C20-A154-B8BD8A86D172",       "00000000-0000-0000-00AA-000010001033", "00000000-0000-0000-00AA-000010001003" };
        private Random numGen;
        private int randomView;

        private string currentViewName;
        private string currentViewId;

        private WRPCInfo rpcInfo;
        private WebTestRequestHeader headerItem;
    }
}