namespace CRM_Perf_BenchMark
{
    using System;
    using System.Collections.Generic;
    using Microsoft.VisualStudio.TestTools.WebTesting;
    using System.Net;
    using System.Text;
    using System.Xml;
    
    /// <summary>
    /// 1.	Start at Marketing homepage
    /// 2.  Select Marketing Lists
    /// 3.	Click on New
    /// 4.	Input information and click Save and Close
    /// 5.	Open the created List
    /// 6.	Click on Marketing List Members
    /// 7.	Click Manage Members
    /// 8.	Select Use Lookup to add members
    /// 9.	Click Find
    /// 10.	Select a list (double click) and click OK
    /// 11.	Click on Manage Members again
    /// 12.	Click on Use Advanced Find to Remove Members
    /// 13.	Click on Find button
    /// 14.	Select an Account and click Remove from List
    /// 15.	Click Save and Close
    /// </summary>
    
    public class RemoveFromList : CrmWebTest
    {
        public void RemoveFromList_PreWebTest(object sender, PreWebTestEventArgs e)
        {
            EntityRequest er = new EntityRequest();
            er.Type = EntityNames.Users;
            er.ReturnAs = EntityNames.Users;
            er.ParentID = EntityManager.Instance.OrgId;

            System.Collections.Hashtable Props = new System.Collections.Hashtable();

            if (e.WebTest.Context.ContainsKey("UserRole"))
            {
                Props.Add("role", e.WebTest.Context["UserRole"]);
            }
            
            EntityRequest AccountEr = new EntityRequest();
            AccountEr.Type = EntityNames.Accounts;
            AccountEr.ParentID = Guid.NewGuid();
            AccountEr.ReturnAs = EntityNames.Accounts;

            Props.Add(EntityNames.Accounts, new EntityRequest[] { AccountEr });

            er.Props = Props;

            System.Collections.Hashtable Entities = EntityManager.Instance.GetEntities(er);

            user = (CRMEntity)Entities[EntityNames.Users];
            account = (CRMEntity)Entities[EntityNames.Accounts];
            crmServer = user["CrmServer"];
            tokenKey = EntityManager.Instance.ReadTokenKey(user["organizationid"]);

            orgId = new Guid(user["organizationid"]);

            e.WebTest.UserName = user["domainName"];
            e.WebTest.Password = user["userpassword"];            
        }

        public void RemoveFromList_PostWebTest(object sender, PostWebTestEventArgs e)
        {
            try
            {
                if (newListId != null)
                {
                    string newListString = newListId.ToString();
                    Guid g = EntityManager.Instance.GetOwnerGuid(new Guid(user[EntityIDNames.User]), new Guid(user["OrganizationId"]));
                    EntityManager.Instance.AddEntity(EntityNames.Lists, g, new string[] { "OwningUser", "ListID", "EntityManagerOwningUser" }, new string[] { user["systemuserid"], newListString, g.ToString() });
                }
            }
            catch (System.FormatException fe)
            {
                System.Diagnostics.Trace.WriteLine("Failed to convert bogus data into Guid. " + fe.ToString());
            }
            finally
            {
                EntityManager.Instance.FreeEntity(user);
                EntityManager.Instance.FreeEntity(account);
            }
        }

        private void RemoveFromList_ValidateResponse(object sender, ValidationEventArgs e)
        {
            string ID = null;

            int idIdx = e.Response.BodyString.IndexOf(newIdMarker);
            ID = e.Response.BodyString.Substring(idIdx + (RemoveFromList.newIdMarker.Length), 36);
            try
            {
                ID = new Guid(ID).ToString();
            }
            catch (System.FormatException fe)
            {
                System.Diagnostics.Trace.WriteLine("Format exception: " + fe.ToString());
                e.IsValid = false;
                return;
            }
            e.IsValid = true;
        }

        public RemoveFromList()
        {
            WebRequest.RegisterPrefix("crm", new crmRequestFactory());

            refreshGridList = GridXmlHelper.ListPageViewRefresh(-1, myListViewId, null);
            searchGrid = GridXmlHelper.ListMemberSearchView(-1, viewId, null);

            this.Proxy = null;            
            PreWebTest += new EventHandler<PreWebTestEventArgs>(RemoveFromList_PreWebTest);
            PostWebTest += new EventHandler<PostWebTestEventArgs>(RemoveFromList_PostWebTest);
        }

        public override IEnumerator<WebTestRequest> GetRequestEnumerator()
        {
            this.BeginTransaction("NavListHomepage");

            _root_homepage_aspx hp = new _root_homepage_aspx(user);
            hp.etc = 4300;
            yield return (hp);

            this.EndTransaction("NavListHomepage");

            this.BeginTransaction("ManageMembersAdd");

            ma_lists_edit_aspx ma = new ma_lists_edit_aspx(user);
            yield return (ma);

            referer = ma.lastResponse.ResponseUri.AbsoluteUri;

            _forms_FormScript_js_aspx formScript = new _forms_FormScript_js_aspx(user);
            formScript.etc = 4300;
            formScript.FormType = "main";
            formScript.FormName = "crmForm";
            headerItem = new WebTestRequestHeader();
            headerItem.Name = "referer";
            headerItem.Value = referer;
            formScript.Headers.Add(headerItem);
            yield return (formScript);

            ma = new ma_lists_edit_aspx(user);
            ma.crmFormSubmitXml = EntityXmlHelper.GetCreateListXml(new Guid(user["systemuserid"]), 8);
            ma.crmFormSubmitMode = 1;
            ma.crmFormSubmitId = null;
            ma.crmFormSubmitObjectType = 4300;
            ma.crmFormSubmitSecurity = 852023;
            ma.crmFormUserModified = true;
            ma.crmFormSubmitOnline = 1;

            rpcInfo = Utils.GenerateWRPCToken(ma_lists_edit_aspx.url, user, tokenKey);
            ma.CRMWRPCToken = rpcInfo.token;
            ma.CRMWRPCTokenTimeStamp = rpcInfo.timeStamp;
            ma.ValidateResponse += this.RemoveFromList_ValidateResponse;

            yield return (ma);

            //get the newly created list id out of the post reply
            int idIdx = ma.lastResponse.BodyString.IndexOf(newIdMarker);
            string tempNewListId = ma.lastResponse.BodyString.Substring(idIdx + (RemoveFromList.newIdMarker.Length), 36);
            newListId = new Guid(tempNewListId);

            ma = new ma_lists_edit_aspx(user);
            ma.id = newListId;
            yield return (ma);

            referer = ma.lastResponse.ResponseUri.AbsoluteUri;

            AppGridWebService.AppGridWebService agws = new AppGridWebService.AppGridWebService();
            WebTestRequest wtrRefresh = null;
            try { agws.Refresh(refreshGridList); }
            catch (crmException x)
            {
                wtrRefresh = x.wtr;
                wtrRefresh.Url = Utils.ReplaceServerName(wtrRefresh.Url, user);

                string urlPath = wtrRefresh.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtrRefresh.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtrRefresh.Headers.Add(headerItem);
                wtrRefresh.Method = "Post";
            }
            yield return (wtrRefresh);

            formScript = new _forms_FormScript_js_aspx(user);
            formScript.etc = 4300;
            formScript.FormType = "main";
            formScript.FormName = "crmForm";
            headerItem = new WebTestRequestHeader();
            headerItem.Name = "referer";
            headerItem.Value = referer;
            formScript.Headers.Add(headerItem);
            yield return (formScript);

            ma_lists_areas_aspx listArea = new ma_lists_areas_aspx(user);
            listArea.oId = newListId;
            listArea.oType = 4300;
            listArea.security = 852023;
            listArea.createdfromcode = 1;
            listArea.tabSet = "areaListMember";
            yield return (listArea);

            MA_lists_ListQualificationDlg_dlg_manage_members_aspx listQualifications = new MA_lists_ListQualificationDlg_dlg_manage_members_aspx(user);
            yield return (listQualifications);

            MarketingAutomation.MarketingAutomationWebService maws = new MarketingAutomation.MarketingAutomationWebService();
            WebTestRequest wtrRetrieveList = null;
            string[] columns = new string[] { "createdfromcode" };
            try { maws.RetrieveList(newListId, columns); }
            catch (crmException x)
            {
                wtrRetrieveList = x.wtr;
                wtrRetrieveList.Url = Utils.ReplaceServerName(wtrRetrieveList.Url, user);

                string urlPath = wtrRetrieveList.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtrRetrieveList.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtrRetrieveList.Headers.Add(headerItem);
                wtrRetrieveList.Method = "Post";
            }
            yield return (wtrRetrieveList);

            _controls_lookup_lookupmulti_aspx lookupMulti = new _controls_lookup_lookupmulti_aspx(user);
            lookupMulti.Class = "activityrelatedcustomer";
            lookupMulti.objecttypes = "1";
            lookupMulti.browse = 0;
            lookupMulti.ShowNewButton = 1;
            yield return (lookupMulti);

            _controls_lookup_lookupdata_aspx lookupData = new _controls_lookup_lookupdata_aspx(user);
            lookupData.ObjectType = 1;
            lookupData.Class = "activityrelatedcustomer";
            lookupData.objecttypes = "1";
            lookupData.browse = 0;
            lookupData.ShowNewButton = 1;
            lookupData.Mode = 4;
            lookupData.ColumnsOnly = 1;
            yield return (lookupData);

            lookupData = new _controls_lookup_lookupdata_aspx(user);
            lookupData.Class = "activityrelatedcustomer";
            lookupData.objecttypes = "1";
            lookupData.browse = 0;
            lookupData.ShowNewButton = 1;
            lookupData.Mode = 4;
            lookupData.ObjectType = 1;
            lookupData.searchvalue = Utils.GetRandomString(2, 2);
            yield return (lookupData);

            _grid_cmds_dlg_addtolist_aspx addToList = new _grid_cmds_dlg_addtolist_aspx(user);
            addToList.autoTrigger = 1;
            addToList.iObjType = 1;
            addToList.iTotal = 1;
            addToList.sIds = null;
            addToList.itemObjectId = newListId;
            yield return (addToList);

            addToList = new _grid_cmds_dlg_addtolist_aspx(user);
            addToList.iObjType = 1;
            addToList.iTotal = 1;
            addToList.iIndex = 0;
            addToList.itemObjectId = newListId;
            addToList.iId = new Guid(account[EntityIDNames.Account]);

            StringHttpBody httpBody = new StringHttpBody();
            httpBody.BodyString = "<node />";
            httpBody.ContentType = "text/xml";
            addToList.Body = httpBody;

            rpcInfo = Utils.GenerateWRPCToken(_grid_cmds_dlg_addtolist_aspx.url, user, tokenKey);

            headerItem = new WebTestRequestHeader();
            headerItem.Name = "CRMWRPCTokenTimeStamp";
            headerItem.Value = rpcInfo.timeStamp.ToString();
            addToList.Headers.Add(headerItem);
            headerItem = new WebTestRequestHeader();
            headerItem.Name = "CRMWRPCToken";
            headerItem.Value = rpcInfo.token;
            addToList.Headers.Add(headerItem);
            addToList.Method = "Post";

            yield return (addToList);

            maws = new MarketingAutomation.MarketingAutomationWebService();
            wtrRetrieveList = null;
            columns = new string[] { "membercount" };
            try { maws.RetrieveList(newListId, columns); }
            catch (crmException x)
            {
                wtrRetrieveList = x.wtr;
                wtrRetrieveList.Url = Utils.ReplaceServerName(wtrRetrieveList.Url, user);

                string urlPath = wtrRetrieveList.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtrRetrieveList.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtrRetrieveList.Headers.Add(headerItem);
                wtrRetrieveList.Method = "Post";
            }
            yield return (wtrRetrieveList);

            // Make the list Grid for members
            tempNewListId = newListId.ToString();
            memberGrid = GridXmlHelper.MemberGridForListViewRefresh(-1, memberViewId, null, tempNewListId, "4300");
            agws = new AppGridWebService.AppGridWebService();
            wtrRefresh = null;
            try { agws.Refresh(memberGrid); }
            catch (crmException x)
            {
                wtrRefresh = x.wtr;
                wtrRefresh.Url = Utils.ReplaceServerName(wtrRefresh.Url, user);

                string urlPath = wtrRefresh.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtrRefresh.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtrRefresh.Headers.Add(headerItem);
                wtrRefresh.Method = "Post";
            }
            yield return (wtrRefresh);

            this.EndTransaction("ManageMembersAdd");

            this.BeginTransaction("ManageMembersDelete");

            MA_lists_ListQualificationDlg_dlg_manage_members_aspx dlgMan = new MA_lists_ListQualificationDlg_dlg_manage_members_aspx(user);
            yield return (dlgMan);

            MA_lists_ListQualificationDlg_dlg_query_build_aspx dlgQuery = new MA_lists_ListQualificationDlg_dlg_query_build_aspx(user);
            dlgQuery.ListMemberType = 1;
            dlgQuery.ListId = newListId;
            dlgQuery.InvokeType = "lqRemove";
            dlgQuery.invoker = "listmembers";
            dlgQuery.restrictOtherEntities = 1;
            yield return (dlgQuery);

            AdvancedFind.AdvancedFind af = new AdvancedFind.AdvancedFind();
            WebTestRequest wtr = null;

            try { af.GetQueryList("account",0, true, true, false); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);
            }
            yield return (wtr);

            try { af.GetSystemQuery("{00000000-0000-0000-00AA-000000666000}"); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);
            }
            yield return (wtr);

            try { af.GetDefaultAdvancedFindView("account", false); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);
            }
            yield return (wtr);

            agws = new AppGridWebService.AppGridWebService();
            wtrRefresh = null;
            try { agws.Reset(searchGrid,"crmGrid"); }
            catch (crmException x)
            {
                wtrRefresh = x.wtr;
                wtrRefresh.Url = Utils.ReplaceServerName(wtrRefresh.Url, user);
            }
            yield return (wtrRefresh);

            try { af.GetEntityFieldList("account"); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
            }
            yield return (wtr);

            try { af.GetLinkedEntityList("account"); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
            }
            yield return (wtr);

            AdvancedFind_fetchData_aspx fetchData = new AdvancedFind_fetchData_aspx(user);
            // QueryString Parameters
            fetchData.ListMemberType = 1;
            fetchData.ListId = newListId;
            fetchData.InvokeType = "lqRemove";
            fetchData.invoker = "listmembers";
            fetchData.restrictOtherEntities = 1;
            fetchData.dispActions = false;

            // Form Parameters
            string fetchString = @"<fetch version=""1.0"" output-format=""xml-platform"" mapping=""logical"" distinct=""false""><entity name=""account""><link-entity name=""listmember"" from=""entityid"" to=""accountid""><filter><condition attribute=""listid"" operator=""eq"" value=""_GUID_""/></filter></link-entity><attribute name=""name""/><attribute name=""primarycontactid""/><attribute name=""telephone1""/><attribute name=""accountid""/><order attribute=""name"" descending=""false""/></entity></fetch>";
            fetchString = fetchString.Replace("_GUID_", newListId.ToString("B"));
            fetchData.FetchXml = fetchString;
            string LayoutString = @"<grid name=""resultset"" object=""1"" jump=""name"" select=""1"" icon=""1"" preview=""1""><row name=""result"" id=""accountid""><cell name=""name"" width=""300"" /><cell name=""primarycontactid"" width=""150"" /><cell name=""telephone1"" width=""100"" /></row></grid>";
            
            fetchData.LayoutXml = LayoutString;
            fetchData.EntityName = "account";
            fetchData.DefaultAdvFindViewId = new Guid("00000000-0000-0000-00AA-000000666000");
            fetchData.ViewId = new Guid("00000000-0000-0000-00AA-000000666000");
            fetchData.ViewType = 1039;
            fetchData.SortCol = "name";
            fetchData.SortDescend = false;
            fetchData.Method = "Post";
            yield return (fetchData);

            MA_Lists_ListQualificationDlg_dlg_List_qualify_aspx listQualify = new MA_Lists_ListQualificationDlg_dlg_List_qualify_aspx(user);
            listQualify.itemObjectId = newListId;
            listQualify.iObjType = 1;
            listQualify.itemObjectTypeCode = 4300;
            listQualify.iTotal = 1;
            listQualify.savedQueryId = new Guid("00000000-0000-0000-00AA-000000666000");
            listQualify.savedQueryType = 1039;
            listQualify.isDirty = false;
            listQualify.sInvokeType = "lqRemove";
            listQualify.iOption = 1;
            listQualify.iViewType = 1039;

            httpBody = new StringHttpBody();
            string bodyString = @"<root><fetch version=""1.0"" output-format=""xml-platform"" mapping=""logical"" distinct=""false""><entity name=""account""><attribute name=""name""/><attribute name=""primarycontactid""/><attribute name=""telephone1""/><attribute name=""accountid""/><order attribute=""name"" descending=""false""/></entity></fetch><Ids><Id>{_LISTMEMBER_GUID_}</Id></Ids></root>";
            bodyString = bodyString.Replace("_LISTMEMBER_GUID_", account[EntityIDNames.Account]);
            httpBody.ContentType = "text/xml";
            httpBody.BodyString = bodyString;
            listQualify.Body = httpBody;
            listQualify.Method = "POST";

            rpcInfo = Utils.GenerateWRPCToken(MA_Lists_ListQualificationDlg_dlg_List_qualify_aspx.url, user, tokenKey);

            headerItem = new WebTestRequestHeader();
            headerItem.Name = "CRMWRPCTokenTimeStamp";
            headerItem.Value = rpcInfo.timeStamp.ToString();
            listQualify.Headers.Add(headerItem);
            headerItem = new WebTestRequestHeader();
            headerItem.Name = "CRMWRPCToken";
            headerItem.Value = rpcInfo.token;
            listQualify.Headers.Add(headerItem);
            
            yield return (listQualify);

            maws = new MarketingAutomation.MarketingAutomationWebService();
            wtrRetrieveList = null;
            columns = new string[] { "membercount" };
            try { maws.RetrieveList(newListId, columns); }
            catch (crmException x)
            {
                wtrRetrieveList = x.wtr;
                wtrRetrieveList.Url = Utils.ReplaceServerName(wtrRetrieveList.Url, user);

                string urlPath = wtrRetrieveList.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtrRetrieveList.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtrRetrieveList.Headers.Add(headerItem);
                wtrRetrieveList.Method = "Post";
            }
            yield return (wtrRetrieveList);

            // Make the list Grid for members
            tempNewListId = newListId.ToString();
            memberGrid = GridXmlHelper.MemberGridForListViewRefresh(-1, memberViewId, null, tempNewListId, "4300");
            
            agws = new AppGridWebService.AppGridWebService();
            wtrRefresh = null;
            try { agws.Refresh(memberGrid); }
            catch (crmException x)
            {
                wtrRefresh = x.wtr;
                wtrRefresh.Url = Utils.ReplaceServerName(wtrRefresh.Url, user);

                string urlPath = wtrRefresh.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtrRefresh.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtrRefresh.Headers.Add(headerItem);
                wtrRefresh.Method = "Post";
            }
            yield return (wtrRefresh);

            this.EndTransaction("ManageMembersDelete");
        }

        private const string newIdMarker = @"separator + ""id=";
        private CRMEntity user;
        private CRMEntity account;
        private string crmServer;
        private string tokenKey;
        private Guid newListId;
        private string refreshGridList;
        private string memberGrid;
        private string searchGrid;

        private static string myListViewId = "823B9210-B82A-4B11-B0F6-B9E4C9C8403F";
        private static string memberViewId = "51DE84DD-732F-440F-A506-26123A19AB50";
        private static string viewId = "E62FC2B4-D7C3-410E-ADAA-BB65FBF6F926";

        private Guid orgId;
        private string referer;
        private WebTestRequestHeader headerItem;
        private WRPCInfo rpcInfo;
    }
}
