namespace CRM_Perf_BenchMark
{
    using System;
    using System.Collections.Generic;
    using Microsoft.VisualStudio.TestTools.WebTesting;
    using System.Net;
    using System.Text;
    using System.Xml;
    
    /// <summary>
    /// 1.  Start at HomePage
    /// 2.	Select Settings 
    /// 3.	Select Administration in the Left menu
    /// 4.	Select Business Units
    /// 5.	Open Business Unit by double clicking on selected Business Unit
    /// 6.	Click Action and Change Parent Business
    /// 7.	In the Change Parent Business Dialog, select the Lookup button
    /// 8.	Select a new Business Unit and select OK
    /// 9.	Select OK in the Change Parent Business Dialog to re-parent.
    /// </summary>
    
    public class ReparentBU : CrmWebTest
    {
        public void ReparentBU_PreWebTest(object sender, PreWebTestEventArgs e)
        {
            EntityRequest er = new EntityRequest();
            er.Type = EntityNames.Users;
            er.ReturnAs = EntityNames.Users;
            er.ParentID = EntityManager.Instance.OrgId;

            System.Collections.Hashtable userProps = new System.Collections.Hashtable();
            if (e.WebTest.Context.ContainsKey("UserRole"))
            {
                userProps.Add("role", e.WebTest.Context["UserRole"]);
                er.Props = userProps;
            }
                        
            System.Collections.Hashtable Entities = EntityManager.Instance.GetEntities(er);

            user = (CRMEntity)Entities[EntityNames.Users];
            
            e.WebTest.UserName = user["domainName"];
            e.WebTest.Password = user["userpassword"];
            
            crmServer = user["CrmServer"];
            tokenKey = EntityManager.Instance.ReadTokenKey(user["organizationid"]);
           
            businessunit2 = EntityManager.Instance.GetNextORGEntity(EntityNames.BusinessUnits, new Guid(user["organizationid"]));
            businessunit = EntityManager.Instance.GetNextORGEntity(EntityNames.BusinessUnits, new Guid(user["organizationid"]));
            
        }
       
        public void ReparentBU_PostWebTest(object sender, PostWebTestEventArgs e)
        {
            EntityManager.Instance.FreeEntity(user);
            EntityManager.Instance.FreeEntity(businessunit2);
            EntityManager.Instance.FreeEntity(businessunit);
        }

        public ReparentBU()
        {
            WebRequest.RegisterPrefix("crm", new crmRequestFactory());
            this.Proxy = null;            
            PreWebTest += new EventHandler<PreWebTestEventArgs>(ReparentBU_PreWebTest);
            PostWebTest += new EventHandler<PostWebTestEventArgs>(ReparentBU_PostWebTest);
        }

        public override IEnumerator<WebTestRequest> GetRequestEnumerator()
        {
            Guid businessUnitId = new Guid(businessunit[EntityIDNames.BusinessUnit]);
            gridXml = GridXmlHelper.BusinessUnitPageView(-1, activeBusinessUnitView, null);

            this.BeginTransaction("NavSettingsAdministrationHomepage");

            tools_Admin_admin_aspx settingsHome = new tools_Admin_admin_aspx(user);
            yield return (settingsHome);

            this.EndTransaction("NavSettingsAdministrationHomepage");

            this.BeginTransaction("NavBusinessUnit");

            settingsHome = new tools_Admin_admin_aspx(user);
            settingsHome.pid = 03;
            yield return (settingsHome);

            tools_business_home_biz_aspx bh = new tools_business_home_biz_aspx(user);
            yield return (bh);            

            this.EndTransaction("NavBusinessUnit");

            this.BeginTransaction("OpenBusinessUnit");

            biz_business_edit_aspx be = new biz_business_edit_aspx(user);
            be.id = businessUnitId;
            yield return (be);

            referer = be.lastResponse.ResponseUri.AbsoluteUri;

            _forms_FormScript_js_aspx formScript = new _forms_FormScript_js_aspx(user);
            formScript.etc = 10;
            formScript.FormType = "main";
            formScript.FormName = "crmForm";
            headerItem = new WebTestRequestHeader("referer", referer);            
            formScript.Headers.Add(headerItem);
            yield return (formScript);

            this.EndTransaction("OpenBusinessUnit");

            this.BeginTransaction("AdminReparentBU");

            _grid_cmds_dlg_changeparent_aspx cp = new _grid_cmds_dlg_changeparent_aspx(user);
            cp.iObjType = 10;
            cp.iTotal = 1;
            cp.sIds = businessUnitId.ToString("B");            
            yield return (cp);

            _controls_lookup_lookupsingle_aspx lookBU = new _controls_lookup_lookupsingle_aspx(user);
            lookBU.Class = "BasicOwner";
            lookBU.objecttypes = "10";
            lookBU.browse = 0;
            lookBU.businessunitid = businessUnitId;
            lookBU.ShowNewButton = 1;
            lookBU.ShowPropButton = 1;
            lookBU.DefaultType = 0;
            yield return (lookBU);

            cp = new _grid_cmds_dlg_changeparent_aspx(user);
            cp.iObjType = 10;
            cp.iTotal = 1;
            cp.iIndex = 0;
            cp.ownerId = new Guid(businessunit2[EntityIDNames.BusinessUnit]);
            cp.ownerType = 10;
            cp.iId = businessUnitId.ToString("B").ToUpper();

            rpcInfo = Utils.GenerateWRPCToken(_grid_cmds_dlg_changeparent_aspx.url, user, tokenKey);

            headerItem = new WebTestRequestHeader();
            headerItem.Name = "CRMWRPCTokenTimeStamp";
            headerItem.Value = rpcInfo.timeStamp.ToString();
            cp.Headers.Add(headerItem);
            headerItem = new WebTestRequestHeader();
            headerItem.Name = "CRMWRPCToken";
            headerItem.Value = rpcInfo.token;
            cp.Headers.Add(headerItem);
            cp.Method = "Post";

            StringHttpBody body = new StringHttpBody();
            body.ContentType = "text/xml";
            body.BodyString = "<node/>";
            cp.Body = body;
            yield return (cp);

            be = new biz_business_edit_aspx(user);
            // QueryString Parameters
            be.id = businessUnitId;

            // Form Parameters
            be.crmFormSubmitMode = 4;
            be.crmFormSubmitId = businessUnitId;
            be.crmFormUserModified = false;
            be.crmFormSubmitObjectType = 10;
            be.crmFormSubmitSecurity = 852023;
            be.crmFormSubmitOnline = 1;
            rpcInfo = Utils.GenerateWRPCToken(biz_business_edit_aspx.url, user, tokenKey);
            be.CRMWRPCToken = rpcInfo.token;
            be.CRMWRPCTokenTimeStamp = rpcInfo.timeStamp;
            yield return (be);

            referer = be.lastResponse.ResponseUri.AbsoluteUri;

            AppGridWebService.AppGridWebService agws = new AppGridWebService.AppGridWebService();
            WebTestRequest wtrRefresh = null;
            try { agws.Refresh(gridXml); }
            catch (crmException x)
            {
                wtrRefresh = x.wtr;
                wtrRefresh.Url = Utils.ReplaceServerName(wtrRefresh.Url, user);

                string urlPath = wtrRefresh.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtrRefresh.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtrRefresh.Headers.Add(headerItem);
                wtrRefresh.Method = "Post";
            }
            yield return (wtrRefresh);

            formScript = new _forms_FormScript_js_aspx(user);
            formScript.etc = 10;
            formScript.FormType = "main";
            formScript.FormName = "crmForm";
            headerItem = new WebTestRequestHeader("referer", referer);
            formScript.Headers.Add(headerItem);
            yield return (formScript);

            this.EndTransaction("AdminReparentBU");
        }

        private string gridXml;
        private CRMEntity user;
        private string crmServer;
        private string tokenKey;
        private CRMEntity businessunit;
        private CRMEntity businessunit2;
        private static string activeBusinessUnitView = "00000000-0000-0000-00AA-000010001018";

        private string referer;
        private WebTestRequestHeader headerItem;
        private WRPCInfo rpcInfo;
    }
}