namespace CRM_Perf_BenchMark
{
    using System;
    using System.Collections.Generic;
    using Microsoft.VisualStudio.TestTools.WebTesting;
    using System.Net;
    using System.Text;
    using System.Xml;
    
    /// <summary>
    /// 1.	Start at Sales homepage
    /// 2.  Select Accounts
    /// 3.	Search for Accounts with a randomly selected string
    /// 4.	Highlight 5 accounts
    /// 5.	Click More Actions
    /// 6.	Select Sharing
    /// 7.	Click Add User/Team
    /// 8.	Lookup Users
    /// 9.	Select 5 users
    /// 10.	Click OK
    /// </summary>
    
    public class ShareAccounts : CrmWebTest
    {
        public void ShareAccounts_PreWebTest(object sender, PreWebTestEventArgs e)
        {
            EntityRequest er = new EntityRequest();
            er.Type = EntityNames.Users;
            er.ReturnAs = EntityNames.Users;
            er.ParentID = EntityManager.Instance.OrgId;

            System.Collections.Hashtable Props = new System.Collections.Hashtable();

            if (e.WebTest.Context.ContainsKey("UserRole"))
            {
                Props.Add("role", e.WebTest.Context["UserRole"]);
            }
            
            EntityRequest AccountEr1 = new EntityRequest();
            AccountEr1.Type = EntityNames.Accounts;
            AccountEr1.ParentID = Guid.NewGuid();
            AccountEr1.ReturnAs = "Account1";

            EntityRequest AccountEr2 = new EntityRequest();
            AccountEr2.Type = EntityNames.Accounts;
            AccountEr2.ParentID = Guid.NewGuid();
            AccountEr2.ReturnAs = "Account2";

            EntityRequest AccountEr3 = new EntityRequest();
            AccountEr3.Type = EntityNames.Accounts;
            AccountEr3.ParentID = Guid.NewGuid();
            AccountEr3.ReturnAs = "Account3";

            EntityRequest AccountEr4 = new EntityRequest();
            AccountEr4.Type = EntityNames.Accounts;
            AccountEr4.ParentID = Guid.NewGuid();
            AccountEr4.ReturnAs = "Account4";

            EntityRequest AccountEr5 = new EntityRequest();
            AccountEr5.Type = EntityNames.Accounts;
            AccountEr5.ParentID = Guid.NewGuid();
            AccountEr5.ReturnAs = "Account5";

            Props.Add("FOO", new EntityRequest[] { AccountEr1, AccountEr2, AccountEr3, AccountEr4, AccountEr5 });

            er.Props = Props;

            System.Collections.Hashtable Entities = EntityManager.Instance.GetEntities(er);

            user[0] = (CRMEntity)Entities[EntityNames.Users];
            account[0] = (CRMEntity)Entities["Account1"];
            account[1] = (CRMEntity)Entities["Account2"];
            account[2] = (CRMEntity)Entities["Account3"];
            account[3] = (CRMEntity)Entities["Account4"];
            account[4] = (CRMEntity)Entities["Account5"];

            crmServer = user[0]["CrmServer"];
            tokenKey = EntityManager.Instance.ReadTokenKey(user[0]["organizationid"]);

            e.WebTest.UserName = user[0]["domainName"];
            e.WebTest.Password = user[0]["userpassword"];
            
            user[1] = EntityManager.Instance.GetNextUser(new Guid(user[0]["organizationid"]));
            user[2] = EntityManager.Instance.GetNextUser(new Guid(user[0]["organizationid"]));
            user[3] = EntityManager.Instance.GetNextUser(new Guid(user[0]["organizationid"]));
            user[4] = EntityManager.Instance.GetNextUser(new Guid(user[0]["organizationid"]));
            user[5] = EntityManager.Instance.GetNextUser(new Guid(user[0]["organizationid"]));                        
        }

        public void ShareAccounts_PostWebTest(object sender, PostWebTestEventArgs e)
        {
            EntityManager.Instance.FreeEntity(user[0]);
            EntityManager.Instance.FreeEntity(account[0]);
            EntityManager.Instance.FreeEntity(account[1]);
            EntityManager.Instance.FreeEntity(account[2]);
            EntityManager.Instance.FreeEntity(account[3]);
            EntityManager.Instance.FreeEntity(account[4]);
            EntityManager.Instance.FreeEntity(user[1]);
            EntityManager.Instance.FreeEntity(user[2]);
            EntityManager.Instance.FreeEntity(user[3]);
            EntityManager.Instance.FreeEntity(user[4]);
            EntityManager.Instance.FreeEntity(user[5]);
        }

        public ShareAccounts()
        {
            WebRequest.RegisterPrefix("crm", new crmRequestFactory());
            string random = Utils.GetRandomString(2, 2);
            gridXml = GridXmlHelper.AccountSearchView(-1, searchAccountView, random, null);
            gridRefreshXml = GridXmlHelper.AccountPageSearchViewRefresh(-1, searchAccountView, random, null);
            CreateNodes();

            this.Proxy = null;            
            PreWebTest += new EventHandler<PreWebTestEventArgs>(ShareAccounts_PreWebTest);
            PostWebTest += new EventHandler<PostWebTestEventArgs>(ShareAccounts_PostWebTest);
        }

        private void CreateNodes()
        {
            xmlDoc = new XmlDocument();                                       
            string readerTxt = @"<share><ids></ids></share>";            
            xmlDoc.LoadXml(readerTxt);

            root = new XmlDocument();            
            root.LoadXml("<root></root>");
        }

        private void FillGuids()
        {
            for (int i = 1; i < 6; i++)
            {
                XmlNode r = root.DocumentElement;
                XmlElement elem = root.CreateElement("assignee");
                elem.InnerXml = assigneeStr;
                XmlNode assigneeId = elem.SelectSingleNode("assigneeid");
                assigneeId.InnerText = new Guid(user[i][EntityIDNames.User]).ToString("B");
                r.AppendChild(elem);
            }
        }

        public override IEnumerator<WebTestRequest> GetRequestEnumerator()
        {
            FillGuids();

            this.BeginTransaction("NavAccountHomepage");

            // Get the home page
            _root_homepage_aspx hp = new _root_homepage_aspx(user[0]);
            hp.etc = 1;
            yield return (hp);

            this.EndTransaction("NavAccountHomepage");

            this.BeginTransaction("SearchAccounts");

            // Search for the Account name
            AppGridWebService.AppGridWebService ag = new AppGridWebService.AppGridWebService();
            WebTestRequest wtr = null;

            try { ag.Reset(gridXml, gridId); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user[0]);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user[0]["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user[0], tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
            }
            yield return (wtr);

            this.EndTransaction("SearchAccounts");

            this.BeginTransaction("ShareAccounts");

            _grid_cmds_dlg_share_aspx share = new _grid_cmds_dlg_share_aspx(user[0]);
            share.iObjType = 1;
            share.iTotal = 5;
            share.sIds = null;
            yield return (share);

            share = new _grid_cmds_dlg_share_aspx(user[0]);
            // QueryString Parameters
            share.action = "retrieve";
            share.iObjType = 1;
            share.iTotal = 5;

            // Body
            XmlNode ids = xmlDoc.SelectSingleNode("share/ids");
            for (int i = 0; i < 5; i++)
            {
                ids.InnerText += account[i][EntityIDNames.Account] + ";";
            }
            StringHttpBody httpBody = new StringHttpBody();
            httpBody.BodyString = xmlDoc.OuterXml;
            httpBody.ContentType = "text/xml";
            share.Body = httpBody;
                        
            rpcInfo = Utils.GenerateWRPCToken(_grid_cmds_dlg_share_aspx.url, user[0], tokenKey);

            headerItem = new WebTestRequestHeader();
            headerItem.Name = "CRMWRPCTokenTimeStamp";
            headerItem.Value = rpcInfo.timeStamp.ToString();
            share.Headers.Add(headerItem);
            headerItem = new WebTestRequestHeader();
            headerItem.Name = "CRMWRPCToken";
            headerItem.Value = rpcInfo.token;
            share.Headers.Add(headerItem);
            share.Method = "Post";
            yield return (share);

            _grid_cmds_getshareaccess_xsl getShareAccess = new _grid_cmds_getshareaccess_xsl(crmServer);
            yield return (getShareAccess);

            // Lookup for users when assigning to other users
            _controls_lookup_lookupmulti_aspx lookup = new _controls_lookup_lookupmulti_aspx(user[0]);
            lookup.Class = "BasicOwner";
            lookup.objecttypes = "8,9";
            lookup.browse = 0;
            yield return (lookup);

            _controls_lookup_lookupdata_aspx lookForUsers = new _controls_lookup_lookupdata_aspx(user[0]);
            lookForUsers.ObjectType = 8;
            lookForUsers.Class = "BasicOwner";
            lookForUsers.objecttypes = "8,9";
            lookForUsers.browse = 0;
            lookForUsers.Mode = 4;
            lookForUsers.ColumnsOnly = 1;
            yield return (lookForUsers);

            // This page is called twice
            lookForUsers = new _controls_lookup_lookupdata_aspx(user[0]);            
            lookForUsers.Class = "BasicOwner";
            lookForUsers.objecttypes = "8,9";
            lookForUsers.browse = 0;
            lookForUsers.Mode = 4;
            lookForUsers.ObjectType = 8;
            lookForUsers.searchvalue = null;
            yield return (lookForUsers);                         

            for (int i = 0; i < 5; i++)
            {
                _grid_cmds_dlg_share_aspx share2 = new _grid_cmds_dlg_share_aspx(user[0]);
                share2.iObjType = 1;
                share2.iTotal = 1;
                share2.iIndex = i;
                share2.iId = new Guid(account[i][EntityIDNames.Account]);

                httpBody = new StringHttpBody();
                httpBody.BodyString = root.OuterXml;
                httpBody.ContentType = "text/xml";
                share2.Body = httpBody;

                rpcInfo = Utils.GenerateWRPCToken(_grid_cmds_dlg_share_aspx.url, user[0], tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "CRMWRPCTokenTimeStamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                share2.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "CRMWRPCToken";
                headerItem.Value = rpcInfo.token;
                share2.Headers.Add(headerItem);
                share2.Method = "Post";

                yield return (share2);
            }

            AppGridWebService.AppGridWebService agws = new AppGridWebService.AppGridWebService();
            WebTestRequest wtrRefresh = null;
            try { agws.Refresh(gridRefreshXml); }
            catch (crmException x)
            {
                wtrRefresh = x.wtr;
                wtrRefresh.Url = Utils.ReplaceServerName(wtrRefresh.Url, user[0]);

                string urlPath = wtrRefresh.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user[0]["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user[0], tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtrRefresh.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtrRefresh.Headers.Add(headerItem);
                wtrRefresh.Method = "Post";
            }
            yield return (wtrRefresh);

            this.EndTransaction("ShareAccounts");
        }

        private string gridXml;
        private string gridRefreshXml;
        private static string gridId = "crmGrid";
        private static string searchAccountView = "2D1187C4-23FE-4BB5-9647-43BB1C6DDBD1";
        private static string assigneeStr = "<assigneeid></assigneeid><type>8</type><accessrights>1</accessrights><originalrights>-1</originalrights>";
        private CRMEntity[] user = new CRMEntity[6];
        private CRMEntity[] account = new CRMEntity[5];
        private string crmServer;
        private string tokenKey;
        private XmlDocument xmlDoc;
        private XmlDocument root;

        private WRPCInfo rpcInfo;
        private WebTestRequestHeader headerItem;
    }
}