namespace CRM_Perf_BenchMark
{
    using System;
    using System.Collections.Generic;
    using Microsoft.VisualStudio.TestTools.WebTesting;
    using System.Net;
    using System.Text;
    using System.Xml;

    /// <summary>
    /// 1.	Start at Workplace homepage
    /// 2.  Select Activities
    /// 3.	Select Letter from Type drop down
    /// 4.	Open a Letter. (Double click)
    /// 5.	Change data and click Save and Close
    /// </summary>

    public class UpdateLetter : CrmWebTest
    {
        public void UpdateLetter_PreWebTest(object sender, PreWebTestEventArgs e)
        {
            EntityRequest er = new EntityRequest();
            er.Type = EntityNames.Users;
            er.ReturnAs = EntityNames.Users;
            er.ParentID = EntityManager.Instance.OrgId;

            System.Collections.Hashtable Props = new System.Collections.Hashtable();

            if (e.WebTest.Context.ContainsKey("UserRole"))
            {
                Props.Add("role", e.WebTest.Context["UserRole"]);
            }

            EntityRequest LetterEr = new EntityRequest();
            LetterEr.Type = EntityNames.Letters;
            LetterEr.ParentID = Guid.NewGuid();
            LetterEr.ReturnAs = EntityNames.Letters;

            Props.Add(EntityNames.Letters, new EntityRequest[] { LetterEr });

            er.Props = Props;

            System.Collections.Hashtable Entities = EntityManager.Instance.GetEntities(er);

            user = (CRMEntity)Entities[EntityNames.Users];
            letter = (CRMEntity)Entities[EntityNames.Letters];

            e.WebTest.UserName = user["domainName"];
            e.WebTest.Password = user["userpassword"];

            crmServer = user["CrmServer"];
            tokenKey = EntityManager.Instance.ReadTokenKey(user["organizationid"]);

            orgId = new Guid(user["organizationid"]);

            user2 = EntityManager.Instance.GetNextUser(new Guid(user["organizationid"]));
        }

        public void UpdateLetter_PostWebTest(object sender, PostWebTestEventArgs e)
        {
            // Don't delete the entity till end of test run as it is deleted from crm.
            EntityManager.Instance.FreeEntity(user);
            EntityManager.Instance.FreeEntity(user2);
            EntityManager.Instance.FreeEntity(letter);
        }

        public UpdateLetter()
        {
            WebRequest.RegisterPrefix("crm", new crmRequestFactory());
            MyLetterXml = GridXmlHelper.LetterPageView(-1, myLetterView, null);
            gridXml = GridXmlHelper.LetterPageViewRefresh(-1, myLetterView, null);

            this.Proxy = null;
            PreWebTest += new EventHandler<PreWebTestEventArgs>(UpdateLetter_PreWebTest);
            PostWebTest += new EventHandler<PostWebTestEventArgs>(UpdateLetter_PostWebTest);
        }

        public override IEnumerator<WebTestRequest> GetRequestEnumerator()
        {

            this.BeginTransaction("NavActivityHomepage");

            // Get the home page
            workplace_home_activities_aspx hp = new workplace_home_activities_aspx(user);
            yield return (hp);

            this.EndTransaction("NavActivityHomepage");

            this.BeginTransaction("NavMyLetters");

            ActivitiesWebService.ActivitiesWebService actWs = new ActivitiesWebService.ActivitiesWebService();
            CrmRequest wtr = null;
            try { actWs.GetQueryList("letter"); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);
                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
            }
            yield return (wtr);

            AppGridWebService.AppGridWebService ag = new AppGridWebService.AppGridWebService();
            wtr = null;
            try { ag.Reset(MyLetterXml, crmGrid); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
            }
            yield return (wtr);

            this.EndTransaction("NavMyLetters");

            this.BeginTransaction("OpenLetter");

            activities_letter_edit_aspx editPage = new activities_letter_edit_aspx(user);
            editPage.id = letter[EntityIDNames.Letter];
            yield return (editPage);

            referer = editPage.lastResponse.ResponseUri.AbsoluteUri;

            _forms_FormScript_js_aspx f = new _forms_FormScript_js_aspx(user);
            f.etc = 4207;
            f.FormType = "main";
            f.FormName = "crmForm";
            headerItem = new WebTestRequestHeader("referer", referer);
            f.Headers.Add(headerItem);
            yield return (f);

            this.EndTransaction("OpenLetter");

            this.BeginTransaction("UpdateLetter");

            editPage = new activities_letter_edit_aspx(user);
            // QueryString Parameters
            editPage.id = letter[EntityIDNames.Letter];
            // Form Parameters
            editPage.crmFormSubmitXml = EntityXmlHelper.GetUpdateLetterXml();

            rpcInfo = Utils.GenerateWRPCToken(activities_letter_edit_aspx.url, user, tokenKey);
            editPage.CRMWRPCToken = rpcInfo.token;
            editPage.CRMWRPCTokenTimeStamp = rpcInfo.timeStamp;
            editPage.crmFormSubmitMode = 2;
            editPage.crmFormSubmitId = new Guid(letter[EntityIDNames.Letter]);
            editPage.crmFormUserModified = true;
            editPage.crmFormSubmitObjectType = 4207;
            editPage.crmFormSubmitSecurity = 852023;
            editPage.crmFormSubmitOnline = 1;
            yield return (editPage);

            ag = new AppGridWebService.AppGridWebService();
            wtr = null;
            try { ag.Refresh(gridXml); }
            catch (crmException x)
            {
                wtr = (CrmRequest)x.wtr;
                wtr.Url = Utils.ReplaceServerName(wtr.Url, user);

                string urlPath = wtr.Url.Remove(0, crmServer.Length);
                urlPath = urlPath.Remove(0, user["organizationName"].Length + 1);

                rpcInfo = Utils.GenerateWRPCToken(urlPath, user, tokenKey);

                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctokentimestamp";
                headerItem.Value = rpcInfo.timeStamp.ToString();
                wtr.Headers.Add(headerItem);
                headerItem = new WebTestRequestHeader();
                headerItem.Name = "crmwrpctoken";
                headerItem.Value = rpcInfo.token;
                wtr.Headers.Add(headerItem);
                wtr.Method = "Post";
            }
            yield return (wtr);

            this.EndTransaction("UpdateLetter");
        }

        private const string newIdMarker = @"SetQueryString(sLocation,""id"",""";
        private string gridXml;
        private string MyLetterXml;
        private CRMEntity user;
        private CRMEntity user2;
        private CRMEntity letter;
        private string crmServer;
        private string tokenKey;
        private static string crmGrid = "crmGrid";
        private static string myLetterView = "ED36ABD8-7F54-4077-8B46-D6F9BF8E71DF";
        private Guid orgId;
        private WRPCInfo rpcInfo;
        private WebTestRequestHeader headerItem;
        private string referer;
    }
}