  namespace CRM_Perf_BenchMark
    {
    	using System;
    	using System.Text;
    	using System.Net;
    	using System.IO;
    	using System.Web;
    	using System.Security.Cryptography;
    	using System.Xml;
        using System.Collections.Generic;
        using System.Diagnostics;
        using Microsoft.VisualStudio.TestTools.WebTesting;
        using System.Globalization;
        using Microsoft.Crm.Authentication;
        using Microsoft.Crm;
        using System.Xml.Serialization;
        using System.Diagnostics.CodeAnalysis;

        using Microsoft.Win32;
        
      public abstract class CrmWebTest : WebTest
      {

          public CrmWebTest()
          {
              this.PreAuthenticate = false;
          }
          public override abstract IEnumerator<WebTestRequest> GetRequestEnumerator();          

          public new void BeginTransaction(string trac)
          {
              if (ConfigSettings.Default.BandWidthCapture != null)              {
                  if (ConfigSettings.Default.BandWidthCapture.ToLower() == "true")
                  {
                      string netCapDir = ConfigSettings.Default.NetCapDir;
                      string outputDir = ConfigSettings.Default.BandWidthOutputDir;
                      string initial = ConfigSettings.Default.ParseDependentRequests;
                      string capFile = outputDir + "\\" + trac + "_warm.cap";
                      if (initial == "true")
                          capFile = outputDir + "\\" + trac + "_cold.cap";

                      BandWidthUtilities.StartProgram(netCapDir + "\\netcap.exe ", " /F:" + netCapDir + "\\file.cf /C:" + capFile + " /n:2 /B:100 /T P 1e 01010101", string.Empty, false);
                      System.Threading.Thread.Sleep(1000);
                  }
              }
              base.BeginTransaction(trac);
          }

          public new void EndTransaction(string trac)
          {
              base.EndTransaction(trac);
              if (ConfigSettings.Default.BandWidthCapture != null)
              {
                  if (ConfigSettings.Default.BandWidthCapture.ToLower() == "true")
                  {

                      string outputDir = ConfigSettings.Default.BandWidthOutputDir;

                      string initial = ConfigSettings.Default.ParseDependentRequests;
                      string capFile = outputDir + "\\" + trac + "_warm.cap";
                      if (initial == "true")
                          capFile = outputDir + "\\" + trac + "_cold.cap";

                      BandWidthUtilities.StopCapture("Ping", "-n 1 1.1.1.1");

                      string title = trac + " (Warm) ";
                      if (initial == "true")
                          title = trac + " (Cold) ";

                      if (BandWidthUtilities.RunNetMonExpert("rta", capFile))
                      {

                          if (BandWidthUtilities.RunNetMonExpert("vrta", capFile))
                          {

                              if (BandWidthUtilities.RunNetMonExpert("genius", capFile))
                              {

                                  BandWidthUtilities.ParseRTAFile(capFile, title);
                              }
                          }
                      }
                  }
              }
          }
      }

      public class ConfigSettings
      {
          private static ConfigSettings m_instance = null;
        
          private string m_WebServiceNS;
          private string m_HelpAndAssistance;
          private string m_EMSQLCNN;
          private string m_alltasks;
          private string m_BulkImportDir;
          private string m_ReportServer;
          private string m_BandWidthCapture;
          private string m_NetCapDir;
          private string m_BandWidthOutputDir;
          private string m_ParseDependentRequests;
          private string m_RunAsUser;
          private string m_RunAsPassword;
          private string m_CRMDomain;
          //this is the server we are going to use for metabase
          private string m_MetabaseServer;
          //we will use this property to see if we need to reload the entitymanager database
          private string m_ReloadEntityManager = "false";

          private string m_ConfigSku = "OnPremise";
          private bool m_IgnoreTokenCheck = false;

          //these are to support multi server scenarios 
          public int m_numServers;
          public MultipleServersData[] m_MulServers = null;

          //this is where we are going to store all the Sync outlook xml files
          public string m_OutlookSyncDir = string.Empty;

          //this stores the option to wait for async operation completion
          private bool m_WaitForAsyncCompletion = true;

          // this stores the option whether to deactivate the rule at the end of the Unittest of to keep it.
          private bool m_deactivateRules = true;

          public static ConfigSettings Default
          {
              get
              {
                  return m_instance;
              }
          }

          public string CRMDomain
          {
              get
              {
                  return m_CRMDomain;
              }
          }

          public bool IgnoreTokenCheck
          {
              get
              {
                  return m_IgnoreTokenCheck;
              }
          }

          public string MetabaseServer
          {
              get
              {
                  return m_MetabaseServer;
              }
          }

          public string RunAsUser
          {
              get
              {
                  return m_RunAsUser;
              }
          }

          public string RunAsPassword
          {
              get
              {
                  return m_RunAsPassword;
              }
          }

          public string ReportServer
          {
              get
              {
                  return m_ReportServer;
              }
          }
          public string ReloadEntityManager
          {
              get
              {
                  return m_ReloadEntityManager;
              }
          }                   

          public string WebServiceNS
          {
              get
              {
                  return m_WebServiceNS;
              }
          }

          public string HelpAndAssistancePaneOn
          {
              get
              {
                  return (m_HelpAndAssistance);
              }
          }

       
          public string EMSQLCNN
          {
              get
              {
                  return m_EMSQLCNN;
              }
          }

          public bool alltasks
          {
              get
              {
                  return bool.Parse(m_alltasks);
              }
          }

          public string BulkImportDir
          {
              get
              {
                  return m_BulkImportDir;
              }
          }

          public string BandWidthCapture
          {
              get
              {
                  return m_BandWidthCapture;
              }
          }

          public string NetCapDir
          {
              get
              {
                  return m_NetCapDir;
              }
          }

          public string BandWidthOutputDir
          {
              get
              {
                  return m_BandWidthOutputDir;
              }
          }

          public string ParseDependentRequests
          {
              get
              {
                  return m_ParseDependentRequests;
              }
          }

          public string OutlookSyncDir
          {
              get
              {
                  return m_OutlookSyncDir;
              }
          }

          public bool WaitForAsyncCompletion
          {
              get
              {
                  return m_WaitForAsyncCompletion;
              }
          }

          public bool DeactivateRules
          {
              get
              {
                  return m_deactivateRules;
              }
          }

          public string ConfigSku
          {
              get
              {
                  return m_ConfigSku;
              }
          }

          static ConfigSettings()
          {
              m_instance = new ConfigSettings();
          }

          public ConfigSettings()
          {
              string configDirKeyPath = "SOFTWARE\\Microsoft\\MSCRM";
              string configDir = null;

              if (Registry.LocalMachine.OpenSubKey(configDirKeyPath) != null)
              {
                  RegistryKey regKey = Registry.LocalMachine.OpenSubKey(configDirKeyPath);
                  configDir = (string)regKey.GetValue("CRM_Perf_Toolkit_ConfigDir", "c:\\crmstress");
              }
              else
              {
                  //setting the default directory of configsettings.xml to c:\crmstress folder
                  configDir = "c:\\crmstress";
              }

              XmlDocument xmlDoc = new XmlDocument();
              try
              {
                  xmlDoc.Load(configDir + "\\ConfigSettings.xml");
                  XmlNode configsettings = xmlDoc.DocumentElement;

                  XmlNode currentNode = configsettings.SelectSingleNode("webservicens");
                  m_WebServiceNS = currentNode.Attributes["value"].Value;

                  currentNode = configsettings.SelectSingleNode("helpandassistance");
                  m_HelpAndAssistance = currentNode.Attributes["value"].Value;

                  currentNode = configsettings.SelectSingleNode("emsqlcnn");
                  m_EMSQLCNN = currentNode.Attributes["value"].Value;

                  currentNode = configsettings.SelectSingleNode("alltasks");
                  m_alltasks = currentNode.Attributes["value"].Value;

                  currentNode = configsettings.SelectSingleNode("bulkimportdir");
                  m_BulkImportDir = currentNode.Attributes["value"].Value;

                  currentNode = configsettings.SelectSingleNode("reportserver");
                  m_ReportServer = currentNode.Attributes["value"].Value;

                  if (configsettings.SelectSingleNode("parsedependentrequests") != null)
                  {
                      currentNode = configsettings.SelectSingleNode("parsedependentrequests");
                      m_ParseDependentRequests = currentNode.Attributes["value"].Value;
                  }

                  if (configsettings.SelectSingleNode("outlooksyncdir") != null)
                  {
                      currentNode = configsettings.SelectSingleNode("outlooksyncdir");
                      m_OutlookSyncDir = currentNode.Attributes["value"].Value;
                  }

                  if (configsettings.SelectSingleNode("bandwidthcapture") != null)
                  {
                      XmlNode bandwidthCapture = configsettings.SelectSingleNode("bandwidthcapture");
                      m_BandWidthCapture = bandwidthCapture.Attributes["value"].Value;

                      if (bandwidthCapture.SelectSingleNode("netcapdir") != null)
                      {
                          XmlNode iterator = bandwidthCapture.SelectSingleNode("netcapdir");
                          m_NetCapDir = iterator.Attributes["value"].Value;
                      }

                      if (bandwidthCapture.SelectSingleNode("bandwidthoutputdir") != null)
                      {
                          XmlNode iterator = bandwidthCapture.SelectSingleNode("bandwidthoutputdir");
                          m_BandWidthOutputDir = iterator.Attributes["value"].Value;
                      }

                  }

                  if (configsettings.SelectSingleNode("runas") != null)
                  {
                      currentNode = configsettings.SelectSingleNode("runas");
                      if (currentNode.Attributes["userid"] != null)
                          m_RunAsUser = currentNode.Attributes["userid"].Value;
                      else
                          System.Diagnostics.Trace.WriteLine("UserName missing of the run as user");

                      if(currentNode.Attributes["password"] != null)
                          m_RunAsPassword = currentNode.Attributes["password"].Value;
                      else
                          System.Diagnostics.Trace.WriteLine("Password of the run as user missing");                        
                  }

                  if (configsettings.SelectSingleNode("crmdomain") != null)
                  {
                      currentNode = configsettings.SelectSingleNode("crmdomain");
                      m_CRMDomain = currentNode.Attributes["value"].Value;
                  }
                  if (configsettings.SelectSingleNode("metabaseserver") != null)
                  {
                      currentNode = configsettings.SelectSingleNode("metabaseserver");
                      m_MetabaseServer = currentNode.Attributes["value"].Value;
                  }

                  if (configsettings.SelectSingleNode("reloadentitymanager") != null)
                  {
                      currentNode = configsettings.SelectSingleNode("reloadentitymanager");
                      m_ReloadEntityManager = currentNode.Attributes["value"].Value;
                  }

                  if (configsettings.SelectSingleNode("configsku") != null)
                  {
                      currentNode = configsettings.SelectSingleNode("configsku");
                      m_ConfigSku = currentNode.Attributes["value"].Value;
                  }

                  if (configsettings.SelectSingleNode("ignoretokencheck") != null)
                  {
                      currentNode = configsettings.SelectSingleNode("ignoretokencheck");
                      m_IgnoreTokenCheck = bool.Parse(currentNode.Attributes["value"].Value);
                  }

                  if (configsettings.SelectSingleNode("datamigration") != null)
                  {
                      currentNode = configsettings.SelectSingleNode("datamigration");
                      XmlNode waitForAsyncCompletion = currentNode.SelectSingleNode("waitforasynccompletion");
                      if (waitForAsyncCompletion != null)
                      {
                          m_WaitForAsyncCompletion = bool.Parse(waitForAsyncCompletion.Attributes["value"].Value);
                      }

                      XmlNode deactivateRules = currentNode.SelectSingleNode("deactivaterules");
                      if (deactivateRules != null)
                      {
                          m_deactivateRules = bool.Parse(deactivateRules.Attributes["value"].Value);
                      }
                  }

                  currentNode = configsettings.SelectSingleNode("crmservers");
                  XmlNodeList crmServerNodes = currentNode.SelectNodes("crmserver");
                  
                  m_MulServers = new MultipleServersData[crmServerNodes.Count];

                  for (int iter = 0; iter < crmServerNodes.Count; iter++)
                  {
                      m_MulServers[iter] = new MultipleServersData();
                      XmlNode crmServer = crmServerNodes[iter];

                      currentNode = crmServer.SelectSingleNode("sqlcnn");
                      m_MulServers[iter].SQLCNN = currentNode.Attributes["value"].Value;

                      currentNode = crmServer.SelectSingleNode("configsqlcnn");
                      m_MulServers[iter].ConfigSqlCNN = currentNode.Attributes["value"].Value;

                      currentNode = crmServer.SelectSingleNode("server");
                      m_MulServers[iter].CRMServer = currentNode.Attributes["value"].Value;

                      currentNode = crmServer.SelectSingleNode("discoveryserver");
                      if (currentNode != null)
                      {
                          m_MulServers[iter].DiscoveryServer = currentNode.Attributes["value"].Value;
                      }
                      else
                      {
                          m_MulServers[iter].DiscoveryServer = m_MulServers[iter].CRMServer;
                      }

                      currentNode = crmServer.SelectSingleNode("apiserver");
                      if (currentNode != null)
                      {
                          m_MulServers[iter].APIServer = currentNode.Attributes["value"].Value;
                      }
                      else
                      {
                          m_MulServers[iter].APIServer = m_MulServers[iter].CRMServer;
                      }

                      currentNode = crmServer.SelectSingleNode("organization");
                      m_MulServers[iter].OrgName = currentNode.Attributes["name"].Value;

                      XmlNode AD = crmServer.SelectSingleNode("AD");
                      if (AD != null)
                      {
                          currentNode = AD.SelectSingleNode("userpassword");
                          m_MulServers[iter].userPassword = currentNode.Attributes["value"].Value;

                          currentNode = AD.SelectSingleNode("userbase");
                          m_MulServers[iter].userbase = currentNode.Attributes["value"].Value;

                          currentNode = AD.SelectSingleNode("start");
                          m_MulServers[iter].start = currentNode.Attributes["value"].Value;

                          currentNode = AD.SelectSingleNode("count");
                          m_MulServers[iter].count = currentNode.Attributes["value"].Value;

                          currentNode = AD.SelectSingleNode("outlookstart");
                          if (currentNode != null)
                              m_MulServers[iter].outlookstart = currentNode.Attributes["value"].Value;
                          else
                              m_MulServers[iter].outlookstart = m_MulServers[iter].start;

                          currentNode = AD.SelectSingleNode("outlookcount");
                          if (currentNode != null)
                              m_MulServers[iter].outlookcount = currentNode.Attributes["value"].Value;
                          else
                              m_MulServers[iter].outlookcount = m_MulServers[iter].count;
                      }

                      XmlNode passport = crmServer.SelectSingleNode("passport");
                      if (passport != null)
                      {
                          currentNode = passport.SelectSingleNode("passportdomain");
                          m_MulServers[iter].passportdomain = currentNode.Attributes["value"].Value;

                          currentNode = passport.SelectSingleNode("partner");
                          m_MulServers[iter].partner = currentNode.Attributes["value"].Value;

                          currentNode = passport.SelectSingleNode("userbase");
                          m_MulServers[iter].passportuserbase = currentNode.Attributes["value"].Value;

                          currentNode = passport.SelectSingleNode("start");
                          m_MulServers[iter].passportuserstart = currentNode.Attributes["value"].Value;

                          currentNode = passport.SelectSingleNode("count");
                          m_MulServers[iter].passportusercount = currentNode.Attributes["value"].Value;

                          currentNode = passport.SelectSingleNode("password");
                          m_MulServers[iter].passportuserpassword = currentNode.Attributes["value"].Value;

                          currentNode = passport.SelectSingleNode("outlookstart");
                          if (currentNode != null)
                              m_MulServers[iter].passportoutlookuserstart = currentNode.Attributes["value"].Value;
                          else
                              m_MulServers[iter].passportoutlookuserstart = m_MulServers[iter].passportuserstart;

                          currentNode = passport.SelectSingleNode("outlookcount");
                          if (currentNode != null)
                              m_MulServers[iter].passportoutlookusercount = currentNode.Attributes["value"].Value;
                          else
                              m_MulServers[iter].passportoutlookusercount = m_MulServers[iter].passportusercount;

                      }
                  }
              }
              catch (XmlException e)
              {
                  System.Diagnostics.Trace.WriteLine("XML Load Error:" + e.Message);
              }
              catch (Exception E)
              {
                  System.Diagnostics.Trace.WriteLine(E.Message);
              }
          }
      }

      public class MultipleServersData
      {
          private string m_orgName;
          private string m_CRMServer;
          private string m_APIServer;
          private string m_discoveryServer;          
          private string m_SQLCNN;
          private string m_userpassword;
          private string m_userbase;
          private string m_start;
          private string m_count;
          private string m_outlookstart;
          private string m_outlookcount;

          private string m_configSqlCnn;

          private string m_passportuserpassword;
          private string m_passportuserbase;
          private string m_passportuserstart;
          private string m_passportusercount;
          private string m_passportoutlookstart;
          private string m_passportoutlookcount;
          private string m_passportdomain;
          private string m_partner;

          public string OrgName
          {
              get
              {
                  return m_orgName;
              }
              set
              {
                  m_orgName = value;
              }
          }

          public string CRMServer
          {
              get
              {
                  return m_CRMServer;
              }
              set
              {
                  m_CRMServer = value;
              }
          }

          public string APIServer
          {
              get
              {
                  return m_APIServer;
              }
              set
              {
                  m_APIServer = value;
              }
          }

          public string DiscoveryServer
          {
              get
              {
                  return m_discoveryServer;
              }
              set
              {
                  m_discoveryServer = value;
              }
          }          

          public string SQLCNN
          {
              get
              {
                  return m_SQLCNN;
              }
              set
              {
                  m_SQLCNN = value;
              }
          }

          public string ConfigSqlCNN
          {
              get
              {
                  return m_configSqlCnn;
              }
              set
              {
                  m_configSqlCnn = value;
              }
          }

          public string userPassword
          {
              get
              {
                  return m_userpassword;
              }
              set
              {
                  m_userpassword = value;
              }
          }


          public string userbase
          {
              get
              {
                  return m_userbase;
              }
              set
              {
                  m_userbase = value;
              }
          }

          public string start
          {
              get
              {
                  return m_start;
              }
              set
              {
                  m_start = value;
              }
          }

          public string count
          {
              get
              {
                  return m_count;
              }
              set
              {
                  m_count = value;
              }
          }

          public string outlookstart
          {
              get
              {
                  return m_outlookstart;
              }
              set
              {
                  m_outlookstart = value;
              }
          }

          public string outlookcount
          {
              get
              {
                  return m_outlookcount;
              }
              set
              {
                  m_outlookcount = value;
              }
          }

          public string passportuserpassword
          {
              get
              {
                  return m_passportuserpassword;
              }
              set
              {
                  m_passportuserpassword = value;
              }
          }

          public string passportuserbase
          {
              get
              {
                  return m_passportuserbase;
              }
              set
              {
                  m_passportuserbase = value;
              }
          }

          public string passportusercount
          {
              get
              {
                  return m_passportusercount;
              }
              set
              {
                  m_passportusercount = value;
              }
          }

          public string passportuserstart
          {
              get
              {
                  return m_passportuserstart;
              }
              set
              {
                  m_passportuserstart = value;
              }
          }

          public string passportoutlookuserstart
          {
              get
              {
                  return m_passportoutlookstart;
              }
              set
              {
                  m_passportoutlookstart = value;
              }
          }

          public string passportoutlookusercount
          {
              get
              {
                  return m_passportoutlookcount;
              }
              set
              {
                  m_passportoutlookcount = value;
              }
          }

          public string passportdomain
          {
              get
              {
                  return m_passportdomain;
              }
              set
              {
                  m_passportdomain = value;
              }
          }

          public string partner
          {
              get
              {
                  return m_partner;
              }
              set
              {
                  m_partner = value;
              }
          }
      }
    
    	public class WRPCInfo
    	{
    		public string token;
    		public long timeStamp;
    	}
    
        public class CrmDelRequest : CrmRequest
        {
            public CrmDelRequest(string url)
                : base(url)
            {
            }
            
            public CrmDelRequest(string url, CRMEntity user)
                : base(url, user)
            {
            }
            public CrmDelRequest(Uri uri)
                : base(uri)
            {
            }
    
            public override void CrmRequest_ValidateResponse(object sender, ValidationEventArgs e)
            {
                e.IsValid = true;
    
                if (e.Response.StatusCode != HttpStatusCode.OK)
                {
                    e.IsValid = false;
                    lastResponse = e.Response;
                    return;
                }
                if (e.Response.BodyString == null)
                {
                    lastResponse = e.Response;
                    return;
                }
    
                if (e.Response.BodyString.Contains("<title>Microsoft CRM Application Error Report</title>"))
                {
                    e.IsValid = false;
                    lastResponse = e.Response;
                    return;
                }
                if (e.Response.BodyString.Contains("<title>Microsoft CRM Platform Error Report</title>"))
                {
                    e.IsValid = false;
                    lastResponse = e.Response;
                    return;
                }
    
                // redirects in CRM indicate an error.  "Found" is the code you get after being redirected.
                if (e.Response.StatusCode == HttpStatusCode.Found)
                {
                    e.IsValid = false;
                    lastResponse = e.Response;
                    return;
                }
    
    			if (e.Response.ResponseUri.AbsolutePath.Contains("errorhandler.aspx"))
    			{
    				e.IsValid = false;
    				lastResponse = e.Response;
    				return;
    			}
    			
                if (e.Response.BodyString.Contains("<details>Exception from HRESULT:"))
                {
                    e.IsValid = false;
                    lastResponse = e.Response;
                    return;
                }
                
                lastResponse = e.Response;
                return;
            }
        }
    
    	public class Utils
    	{   

            //added another function to get WPRCTokento support multi server scenario
            public static WRPCInfo GenerateWRPCToken(string baseurl, CRMEntity user, string tokenKey)
            {
                if (ConfigSettings.Default.IgnoreTokenCheck == false)
                {
                    string userId = "{" + user["SystemUserId"].ToUpper() + "}";
                    string organizationName = user["organizationName"];
                    string url = "/" + organizationName + baseurl;

                    WRPCInfo rpcInfo = new WRPCInfo();
                    rpcInfo.timeStamp = DateTime.UtcNow.Ticks;

                    Guid ScaleGroupId = LocatorService.Instance.GetOrganizationScaleGroupId(new Guid(user["OrganizationId"]));

                    // Construct token data in online and web
                    CrmKey wrpcKey = CrmKey.GetActiveKey(CrmKeyType.CrmWRPCTokenKey, ScaleGroupId, true);

					byte[] hash = CrmKeyService.ComputeHash(wrpcKey, new StringHashParameter(url.ToUpper(CultureInfo.InvariantCulture)), new CharHashParameter('T'), new LongHashParameter(rpcInfo.timeStamp), new StringHashParameter(userId));

					rpcInfo.token = CryptoRandom.ConvertKeyToString(hash);

                    return (rpcInfo);
                }
                else
                {
                    WRPCInfo rpcInfo = new WRPCInfo();
                    rpcInfo.timeStamp = DateTime.UtcNow.Ticks;
                    rpcInfo.token = "";
                    return rpcInfo;
                }
            }

            //this functions replace the server name with the server name from crm server
            //we are assumung url is like http://perf1web:5555/test.asmx and crmserver is like http://perf5web
            public static string ReplaceServerName(string url, CRMEntity user)
            {
                string crmServer = user["CrmServer"];
                string organizationName = user["OrganizationName"];

                string[] strs = url.Split(new char[] { '/' }, System.StringSplitOptions.RemoveEmptyEntries);

                string[] crmServerStrs = crmServer.Split(new char[] { '/' }, System.StringSplitOptions.RemoveEmptyEntries);
                //last string should be server name
                string CrmServerWithOrg = crmServerStrs[1] + "/" + organizationName;

                url = url.Replace(strs[0], crmServerStrs[0]);
                if (user["crmticket"] != "")
                {
                    url = url.Replace(strs[1], crmServerStrs[1]);
                }
                else
                {
                    url = url.Replace(strs[1], CrmServerWithOrg);
                }

                return url;
            }

    		public static string GetRandomString(int minLen, int maxLen)
    		{
    
    			int alphabetLength = Alphabet.Length;
    			int stringLength;
    			lock (m_randLock) { stringLength = m_randomInstance.Next(minLen, maxLen); }
    			char[] str = new char[stringLength];
    
    			// max length of the randomizer array is 5
    			int randomizerLength = (stringLength > 5) ? 5 : stringLength;
    
    			int[] rndInts = new int[randomizerLength];
    			int[] rndIncrements = new int[randomizerLength];
    
    			// Prepare a "randomizing" array
    			for (int i = 0; i < randomizerLength; i++)
    			{
    				int rnd = m_randomInstance.Next(alphabetLength);
    				rndInts[i] = rnd;
    				rndIncrements[i] = rnd;
    			}
    
    			// Generate "random" string out of the alphabet used
    			for (int i = 0; i < stringLength; i++)
    			{
    				int indexRnd = i % randomizerLength;
    				int indexAlphabet = rndInts[indexRnd] % alphabetLength;
    				str[i] = Alphabet[indexAlphabet];
    
    				// Each rndInt "cycles" characters from the array, 
    				// so we have more or less random string as a result
    				rndInts[indexRnd] += rndIncrements[indexRnd];
    			}
    			return (new string(str));
    		}

            public static string GetRandomNumber(int min, int max)
            {
                return (m_randomInstance.Next(min,max)).ToString();
            }
    
    		public static string GetRandomPhoneNumber()
    		{
    			int area_code, first3, last4;
    
    			lock (m_randLock)
    			{
    				area_code = 100 + m_randomInstance.Next(899);
    				first3 = 100 + m_randomInstance.Next(899);
    				last4 = m_randomInstance.Next(9999);
    			}
    
    			return string.Format("({0:D3}) {1:D3}-{2:D4}", area_code, first3, last4);
    		}             
    
    		private static char[] Alphabet = ("ABCDEFGHIJKLMNOPQRSTUVWXYZabcefghijklmnopqrstuvwxyz0123456789").ToCharArray();
    		private static Random m_randomInstance = new Random();
    		private static Object m_randLock = new object();
    
    	}
           

      public class SqlRequest : WebTestRequest
      {
          public SqlRequest(string url)
              : base(url)
          {
              this.Url = ConfigSettings.Default.ReportServer + this.Url;
              this.ParseDependentRequests = false;
              this.ValidateResponse += new EventHandler<ValidationEventArgs>(CrmRequest_ValidateResponse);
          }
          public SqlRequest(Uri uri)
              : base(uri)
          {
              // IMPORTANT:  if we come in through this constructor (typically used in the WSDL hijacking (crmrequest.GetResponse())
              // check to see if the uri prefix is crm (again, this is used to enable the WSDL hijacking).  If so, convert it to http              
              Url = Url.Replace("crm://", "http://");

              this.ParseDependentRequests = false;
              this.ValidateResponse += new EventHandler<ValidationEventArgs>(CrmRequest_ValidateResponse);

          }

          public void RequestTypePost()
          {
              formBody = new FormPostHttpBody();
              Method = "POST";
              Body = formBody;
          }

          public QueryStringParameter AddQueryStringParameter(string name, string value)
          {
              QueryStringParameter p = new QueryStringParameter();
              QueryStringParameters.Add(p);
              p.Name = name;
              p.Value = value;
              return p;
          }

          public void AddFormParameter(string name, string value)
          {
              // if we haven't been here yet, allocate the FormPostHttpBody
              // ASSUME:  if we set a form parameter, we must be doing a POST
              if (formBody == null)
              {
                  formBody = new FormPostHttpBody();
                  Method = "POST";
                  Body = formBody;
              }
              formBody.FormPostParameters.Add(new FormPostParameter(name, value));
          }


          public virtual void CrmRequest_ValidateResponse(object sender, ValidationEventArgs e)
          {
              e.IsValid = true;

              if (e.Response.StatusCode != HttpStatusCode.OK)
              {
                  e.IsValid = false;
                  lastResponse = e.Response;
                  return;
              }

              if (e.Response.BodyString == null)
              {
                  lastResponse = e.Response;
                  return;
              }

              if (e.Response.BodyString.Contains("<title>Microsoft CRM Application Error Report</title>"))
              {
                  e.IsValid = false;
                  lastResponse = e.Response;
                  return;
              }
              if (e.Response.BodyString.Contains("<title>Microsoft CRM Platform Error Report</title>"))
              {
                  e.IsValid = false;
                  lastResponse = e.Response;
                  return;
              }

              // redirects in CRM indicate an error.  "Found" is the code you get after being redirected.
              if (e.Response.StatusCode == HttpStatusCode.Found || e.Response.StatusCode == HttpStatusCode.Redirect)
              {
                  e.IsValid = false;
                  lastResponse = e.Response;
                  return;
              }

              if (e.Response.ResponseUri.AbsolutePath.Contains("errorhandler.aspx"))
              {
                  e.IsValid = false;
                  lastResponse = e.Response;
                  return;
              }

              lastResponse = e.Response;
          }

          public WebTestResponse lastResponse = null;
          protected FormPostHttpBody formBody = null;
      }
    
    	public class CrmRequest : WebTestRequest
    	{
            //we should not call into this after we move to multi server
            public CrmRequest(string url)
                : base(url)
            {
                //this.Url = ConfigSettings.Default.MetabaseServer + this.Url;
                this.ParseDependentRequests = false;
                if (ConfigSettings.Default.ParseDependentRequests != null)
                {
                    try
                    {
                        this.ParseDependentRequests = bool.Parse(ConfigSettings.Default.ParseDependentRequests);
                    }
                    catch (Exception e)
                    {
                        System.Diagnostics.Debug.WriteLine(e.ToString());
                    }
                }
                this.ValidateResponse += new EventHandler<ValidationEventArgs>(CrmRequest_ValidateResponse);
            }

            public CrmRequest(string url, CRMEntity user)
                : base(url)
            {
                string crmServer = user["CrmServer"];
                string organizationName = user["OrganizationName"];

                // check if on_premise
                if (user["crmticket"] != "")
                {
                    this.Url = crmServer + url;                   
                }
                else
                {
                    this.Url = crmServer + "/" + organizationName + url;
                }
                
                this.ParseDependentRequests = false;
                if (ConfigSettings.Default.ParseDependentRequests != null)
                {
                    try
                    {
                        this.ParseDependentRequests = bool.Parse(ConfigSettings.Default.ParseDependentRequests);
                    }
                    catch (Exception e)
                    {
                        System.Diagnostics.Debug.WriteLine(e.ToString());
                    }
                }
                this.ValidateResponse += new EventHandler<ValidationEventArgs>(CrmRequest_ValidateResponse);
            }

    		public CrmRequest(Uri uri)
    			: base(uri)
    		{
    			// IMPORTANT:  if we come in through this constructor (typically used in the WSDL hijacking (crmrequest.GetResponse())
    			// check to see if the uri prefix is crm (again, this is used to enable the WSDL hijacking).  If so, convert it to http
                Url = Url.Replace("crm://", "http://");
    
  			    this.ParseDependentRequests = false;
    			this.ValidateResponse += new EventHandler<ValidationEventArgs>(CrmRequest_ValidateResponse);
    		}
    
            public void RequestTypePost()
            {
                formBody = new FormPostHttpBody();
               Method = "POST";
               Body = formBody;
            }
    
    		public QueryStringParameter AddQueryStringParameter (string name, string value)
    		{
    			QueryStringParameter p = new QueryStringParameter();
    			QueryStringParameters.Add(p);
    			p.Name = name;
    			p.Value = value;
    			return p;
    		}
    
    		public void AddFormParameter(string name, string value)
    		{
    			// if we haven't been here yet, allocate the FormPostHttpBody
    			// ASSUME:  if we set a form parameter, we must be doing a POST
    			if (formBody == null)
    			{
    				formBody = new FormPostHttpBody();
    				Method = "POST";
    				Body = formBody;
    			}
    			formBody.FormPostParameters.Add(new FormPostParameter(name, value));
    		}
    
    
    		public virtual void CrmRequest_ValidateResponse(object sender, ValidationEventArgs e)
            {
                e.IsValid = true;

                if (e.Response.StatusCode != HttpStatusCode.OK)
                {
                    e.IsValid = false;
                    lastResponse = e.Response;
                    return;
                }

                if (e.Response.BodyString == null)
                {
                    lastResponse = e.Response;
                    return;
                }

                if (e.Response.BodyString.Contains("<title>Microsoft CRM Application Error Report</title>"))
                {
                    e.IsValid = false;
                    lastResponse = e.Response;
                    return;
                }
                if (e.Response.BodyString.Contains("<title>Microsoft CRM Platform Error Report</title>"))
                {
                    e.IsValid = false;
                    lastResponse = e.Response;
                    return;
                }

                // redirects in CRM indicate an error.  "Found" is the code you get after being redirected.
                if (e.Response.StatusCode == HttpStatusCode.Found || e.Response.StatusCode == HttpStatusCode.Redirect)
                {
                    e.IsValid = false;
                    lastResponse = e.Response;
                    return;
                }

                if (e.Response.ResponseUri.AbsolutePath.Contains("errorhandler.aspx"))
                {
                    e.IsValid = false;
                    lastResponse = e.Response;
                    return;
                }

                lastResponse = e.Response;
            }
    
    		public WebTestResponse lastResponse = null;
    		protected FormPostHttpBody formBody = null;
    	}
    
    	public class crmRequestFactory : System.Net.IWebRequestCreate
    	{
            public WebRequest Create(Uri uri)
            {
               return (new crmrequest(uri));
            }

            public crmRequestFactory()
            {
               
            }
           
    	}
    
    	public class crmrequest : WebRequest
    	{
    		public crmrequest(Uri uri)
    		{
    			m_uri = uri;
    			m_cocStream = new copyoncloseStream();
    			m_whc = new WebHeaderCollection();
    		}
            
    
    
    		public override string Method
    		{
    			get
    			{
    				return (m_method);
    			}
    			set
    			{
    				m_method = value;
    			}
    		}
    
    		public override Uri RequestUri
    		{
    			get
    			{
    				return (m_uri);
    			}
    		}
    
    		public override WebHeaderCollection Headers
    		{
    			get
    			{
    				return m_whc;
    			}
    			set
    			{
    				m_whc = value;
    			}
    		}
    
    		public override long ContentLength
    		{
    			get
    			{
    				return (m_cl);
    			}
    			set
    			{
    				m_cl = value;
    			}
    		}
    
    		public override string ContentType
    		{
    			get
    			{
    				return (m_ct);
    			}
    			set
    			{
    				m_ct = value;
    			}
    		}
    
    		public override ICredentials Credentials
    		{
    			get
    			{
    				return (m_cred);
    			}
    			set
    			{
    				m_cred = value;
    			}
    		}
    
    		public override bool PreAuthenticate
    		{
    			get
    			{
    				return (m_preauth);
    			}
    			set
    			{
    				m_preauth = value;
    			}
    		}
    
    		public override System.IO.Stream GetRequestStream()
    		{
    			return (m_cocStream);
    		}
    
    		//begingetrequeststream and endgetrequeststream are not implemented.
    
    		public override WebResponse GetResponse()
    		{
    			// Copy on close stream snaps the SOAP body off the stream on close.  Access it here to setup a WebTestRequest that we'll
    			// throw back to the caller (we get here from SoapHttpClientProtocol::Invoke).
    			// Didn't necessarily like to use exceptions here, but it was the only straightforward way I could find.
    
    			WebTestRequest wtr = new CrmRequest(RequestUri);
    			wtr.Method = Method;
    
    			// TODO: find out more about this setting
    			wtr.Cache = false;
    			for (int i = 0; i < Headers.Count; i++)
    			{
    				wtr.Headers.Add(new WebTestRequestHeader(Headers.Keys[i], Headers[i]));
    			}
    
    			UTF8Encoding utf8 = new UTF8Encoding();
    			string foo = utf8.GetString(m_cocStream.ba, 0, (int)m_cocStream.length);
    
    			StringHttpBody body = new StringHttpBody();
    			//body.BodyString = HttpUtility.UrlEncode (foo);
    			body.BodyString = foo;
    			body.ContentType = ContentType;
    			wtr.Body = body;
    			throw new crmException(wtr);
    		}
    
    		public override int Timeout
    		{
    			get
    			{
    				return (m_to);
    			}
    			set
    			{
    				m_to = value;
    			}
    		}
    
    		public override string ConnectionGroupName
    		{
    			get
    			{
    				return (m_cgn);
    			}
    			set
    			{
    				m_cgn = value;
    			}
    		}
    		private string m_method;
    		private Uri m_uri;
    		private WebHeaderCollection m_whc;
    		private long m_cl;
    		private string m_ct;
    		private ICredentials m_cred;
    		private bool m_preauth;
    		private copyoncloseStream m_cocStream;
    		private int m_to;
    		private string m_cgn;
          
    
    	}
    
    	public class copyoncloseStream : MemoryStream
    	{
    		public override void Close()
    		{
    			ba = GetBuffer();
    			length = Length;
    			base.Close();
    		}
    		public byte[] ba;
    		public long length;
    	}
    
    	public class crmException : Exception
    	{
    		public crmException(WebTestRequest wtr)
    		{
    			this.wtr = wtr;
    		}
    		public WebTestRequest wtr;
    	}
    
    
    	//<summary>
    	//Currently implemented as a set of simple text operations, based off of
    	//the code in remotecommand.js
    	//Should there be a need to move this to a proper XmlDocument, etc. I don't see
    	//any difficulties
    	//</summary>
    
    	public class CrmSoapBody
    	{
    		private const string soapHead =
    @"<?xml version=""1.0"" encoding=""utf-8"" ?>
    <soap:Envelope xmlns:soap=""http://schemas.xmlsoap.org/soap/envelope/"" xmlns:xsi=""http://www.w3.org/2001/XMLSchema-instance"" xmlns:xsd=""http://www.w3.org/2001/XMLSchema"">
    <soap:Body>";
    
    		// need that space before the xmlns attribute!
    		private const string cmdNS = @" xmlns=""http://www.microsoft.com/BusinessSolutions/Crm/2.0/WebServices""";
    
    		public CrmSoapBody() 
    		{
    			sbParams = new StringBuilder();
    		}
    		
    		public string Method
    		{
    			get
    			{
    				return m_cmdName;
    			}
    		}
    
    		public void Clear()
    		{
    			m_cmdName = null;
    			sbParams.Length = 0;
    		}
    		public void SetCommand(string cmd)
    		{
    			m_cmdName = cmd;
    		}
    		public void AddParam(string paramName, string paramVal)
    		{
    			sbParams.Append("<");
    			sbParams.Append(paramName);
    			sbParams.Append(">");
    			sbParams.Append(paramVal);
    			sbParams.Append("</");
    			sbParams.Append(paramName);
    			sbParams.Append(">");
    		}
    
    		public string GetBody()
    		{
    			StringBuilder body = new StringBuilder();
    			body.Append(soapHead);
    			body.Append("<");
    			body.Append(m_cmdName);
    			body.Append(cmdNS);
    			body.Append(">");
    			body.Append(sbParams.ToString());
    			body.Append("</");
    			body.Append(m_cmdName);
    			body.Append(">");
    			body.Append("</soap:Body>");
    			body.Append("</soap:Envelope>");
    			return body.ToString();
    		}
    
    		private string m_cmdName;
    		private StringBuilder sbParams;
    	}
    
    	public class GetContextHtml_Helper
    	{
    		public static void PrimaryContact(out RelatedInformation.Context ctxt, out string formXml, out string formSubmitId)
    		{
    			ctxt = new RelatedInformation.Context();
    			ctxt.TabIndex = 1690;
    			ctxt.FormObjectType = 1;
    			ctxt.FormId = new Guid("b053a39a-041a-4356-acef-ddf00182762b");
    
    			RelatedInformation.TextCategory tc = new RelatedInformation.TextCategory();
    			tc.CategoryId = "Description";
    			tc.CssClass = "CategoryData";
    			tc.DisplayName = "Description";
    			tc.OrderIndex = 0;
    			tc.DisplayText = "Unique identifier for the primary contact for the account.";
    
    			RelatedInformation.LookupList ll = new RelatedInformation.LookupList();
    			ll.CategoryId = "Category_Lookup";
    			ll.CssClass = "CategoryData_Lookup";
    			ll.DisplayName = "Lookup Records";
    			ll.OrderIndex = 20;
    
    			int[] lookuptypes = new int[1];
    			lookuptypes[0] = 2;
    
    			ll.LookupTypes = lookuptypes;
    
    			
    			object[] objects = new object[2];
    			objects[0] = (object)tc;
    			objects[1] = (object)ll;
    
    			ctxt.Items = objects;
    
    			ctxt.ContextId = "primarycontactid";
    			ctxt.ControlId = "primarycontactid";
    			ctxt.DisplayName = "Primary Contact";
    			ctxt.OrderIndex = 1030;
    
    			formXml = "<account><name>_AccountName_</name><ownerid type=\"8\">{REPLACE_GUID}</ownerid><creditonhold>0</creditonhold><preferredcontactmethodcode>1</preferredcontactmethodcode><donotemail>0</donotemail><donotbulkemail>0</donotbulkemail><donotphone>0</donotphone><donotfax>0</donotfax><donotpostalmail>0</donotpostalmail><donotsendmarketingmaterial>0</donotsendmarketingmaterial></account>";
    			formSubmitId = null;
    		}
    
            public static void Subject(out RelatedInformation.Context ctxt, out string formXml, out string formSubmitId)
            {
                ctxt = new RelatedInformation.Context();
                ctxt.TabIndex = 1250;
                ctxt.FormObjectType = 112;
                ctxt.FormId = new Guid("f658c846-f74c-459b-b591-b97558958106");
    
                RelatedInformation.SubjectTreeCategory stc = new RelatedInformation.SubjectTreeCategory();
                stc.CategoryId = "Category_subject";
                stc.CssClass = "CategoryData_Subject";
                stc.DisplayName = "Subjects";
                stc.OrderIndex = 10;
                stc.ObjectType = 129;
    
                RelatedInformation.TipsCategory tc = new RelatedInformation.TipsCategory();
                tc.CategoryId = "CategoryTips";
                tc.CssClass = "CategoryData";
                tc.DisplayName = "Tips";
                tc.OrderIndex = 100;
                tc.DisplayText = "Choose a subject from the tree";
    
                object[] objects = new object[2];
                objects[0] = (object)stc;
                objects[1] = (object)tc;
    
                ctxt.Items = objects;
    
                ctxt.ContextId = "subjectid";
                ctxt.ControlId = "subjectid";
                ctxt.DisplayName = "Subject";
                ctxt.OrderIndex = 1020;
    
                formXml = "<formXml><incident><title>_CASE_</title><customerid type=\"1\">{CUSTOMER_GUID}</customerid><subjectid type=\"129\">{SUBJECT_GUID}</subjectid><ownerid type=\"8\">{USER_GUID}</ownerid><statuscode>1</statuscode><prioritycode>2</prioritycode></incident></formXml>";
                formSubmitId = null;
            }

            public static void PrimaryCustomer(out RelatedInformation.Context ctxt, out string formXml, out string formSubmitId)
            {
                ctxt = new RelatedInformation.Context();
                ctxt.TabIndex = 1690;
                ctxt.FormObjectType = 1084;
                ctxt.FormId = new Guid("00000000-0000-0000-0000-000000000000");
                ctxt.FormType = "main";

                RelatedInformation.LookupList tc = new RelatedInformation.LookupList();
                tc.CategoryId = "Category_Lookup";
                tc.CssClass = "CategoryData_Lookup";
                tc.DisplayName = "Look Up Records";
                tc.OrderIndex = 20;

                int[] lookuptypes = new int[2];
                lookuptypes[0] = 1;
                lookuptypes[1] = 2;
                tc.LookupTypes = lookuptypes;

                object[] objects = new object[1];
                objects[0] = (object)tc;
                ctxt.Items = objects;

                RelatedInformation.TipsCategory tipsCategory = new RelatedInformation.TipsCategory();
                tipsCategory.CategoryId = "CategoryTips";
                tipsCategory.CssClass = "CategoryData";
                tipsCategory.DisplayName = "Tips";
                tipsCategory.OrderIndex = 100;
                tipsCategory.DisplayText = "This customer will be either the account or contact associated with this quote. The Potential Customer column appears in the Quotes list in the Sales pane.";

                ctxt.ContextId = "customerid";
                ctxt.ControlId = "customerid";
                ctxt.DisplayName = "Potential Customer";
                ctxt.OrderIndex = 1020;

                formXml = "<formXml><quote><customerid type=\"1\">ACCOUNT_GUID</customerid><willcall>0</willcall><ownerid type=\"8\">USER_GUID</ownerid><statuscode name=\"In&#32;Progress\">1</statuscode><statecode name=\"Draft\">Draft</statecode><pricingerrorcode>1</pricingerrorcode></quote></formXml>";
                formSubmitId = null;
            }
    
            public static void PrimaryPricelevel(out RelatedInformation.Context ctxt, out string formXml, out string formSubmitId)
            {
                ctxt = new RelatedInformation.Context();
                ctxt.TabIndex = 1630;
                ctxt.FormObjectType = 1084;
                ctxt.FormId = new Guid("a7152a8b-afbf-44f0-99c7-fa9e44dd73f5");
    
                RelatedInformation.LookupList tc = new RelatedInformation.LookupList();
                tc.CategoryId = "Category_Lookup";
                tc.CssClass = "CategoryData_Lookup";
                tc.DisplayName = "Lookup Records";
                tc.OrderIndex = 20;
    
                int[] lookuptypes = new int[1];
                lookuptypes[0] = 1022;
                tc.LookupTypes = lookuptypes;
    
                object[] objects = new object[1];
                objects[0] = (object)tc;
                ctxt.Items = objects;
    
                ctxt.ContextId = "pricelevelid";
                ctxt.ControlId = "pricelevelid";
                ctxt.DisplayName = "Price List";
                ctxt.OrderIndex = 1050;
    
                formXml = "<formXml><quote><name>_QuoteName_</name><customerid type=\"1\">{ACCOUNT_GUID}</customerid><pricelevelid type=\"1022\">{PRICELEVEL_GUID}</pricelevelid><willcall>0</willcall><ownerid type=\"8\">{USER_GUID}</ownerid><statuscode>1</statuscode></quote></formXml>";
                formSubmitId = null;
            }
    		
    		public static void PriceList(out RelatedInformation.Context ctxt, out string formXml, out string formSubmitId)
    		{
    			ctxt = new RelatedInformation.Context();
    			ctxt.FormObjectType = 3;
    			ctxt.FormId = new Guid("a2a56263-ec37-4991-a3b0-e7b37a71d818");
    
    			RelatedInformation.TextCategory tc = new RelatedInformation.TextCategory();
    			tc.CategoryId = "Description";
    			tc.CssClass = "CategoryData";
    			tc.DisplayName = "Description";
    			tc.OrderIndex = 0;
    			tc.DisplayText = "Unique identifier for the price list associated with the opportunity.";
    
    			RelatedInformation.LookupList ll = new RelatedInformation.LookupList();
    			ll.CategoryId = "Category_Lookup";
    			ll.CssClass = "CategoryData_Lookup";
    			ll.DisplayName = "Lookup Records";
    			ll.OrderIndex = 20;
                ll.LookupTypes = new int[] {1022};
    
    			object[] objects = new object[2];
    			objects[0] = (object)tc;
    			objects[1] = (object)ll;
    			ctxt.Items = objects;
    
    			ctxt.ContextId = "pricelevelid";
    			ctxt.ControlId = "pricelevelid";
    			ctxt.DisplayName = "Price List";
    			ctxt.OrderIndex = 1030;
    
    			formXml = @"<opportunity><name>newopp</name><customerid type=""1"">{188A0EF2-A71B-4C03-AE3C-8F7AB0D34CB3}</customerid><isrevenuesystemcalculated>1</isrevenuesystemcalculated><opportunityratingcode>2</opportunityratingcode><ownerid type=""8"">{50D8FF3D-5938-4B36-B7BD-A240A22C2A59}</ownerid><statuscode>1</statuscode></opportunity>";
    			formSubmitId = null;
    			
    		}
    	}

      public class EntityXmlHelper
      {
          public static string GetCreateAccountWithCustomAttributesXml(Guid ownerid, int owneridtype, string name)
          {
              try
              {
                  string accountStr = "<account><name>" + name + "</name><ownerid type=\"" + owneridtype.ToString() + "\">" + ownerid.ToString("B") + "</ownerid><creditonhold>0</creditonhold><preferredcontactmethodcode>1</preferredcontactmethodcode><donotemail>0</donotemail><donotbulkemail>0</donotbulkemail><donotphone>0</donotphone><donotfax>0</donotfax><donotpostalmail>0</donotpostalmail><donotsendmm>0</donotsendmm><new_attribute1>" + Utils.GetRandomString(5, 10) + "</new_attribute1><new_attribute10>" + Utils.GetRandomString(5, 10) + "</new_attribute10><new_attribute2>1</new_attribute2><new_attribute3>0</new_attribute3><new_attribute4>" + Utils.GetRandomNumber(10, 100).ToString() + "</new_attribute4><new_attribute5>0</new_attribute5></account>";
                  XmlDocument test = new XmlDocument();
                  test.LoadXml(accountStr);
                  return accountStr;
              }
              catch (Exception e)
              {
                  System.Diagnostics.Trace.WriteLine(e.Message);
                  return "";
              }
          }

          public static string GetUpdateAccountWithCustomAttributesXml()
          {
              try
              {
                  string accountStr = "<account><new_attribute1>" + Utils.GetRandomString(5, 10) + "</new_attribute1><new_attribute10>" + Utils.GetRandomString(10, 15) + "</new_attribute10><new_attribute2>" + Utils.GetRandomNumber(0, 4).ToString() + "</new_attribute2><new_attribute4>" + Utils.GetRandomNumber(20, 100).ToString() + "</new_attribute4><new_attribute5>" + Utils.GetRandomNumber(10, 30).ToString() + "</new_attribute5><new_attribute6>" + Utils.GetRandomNumber(10, 30).ToString() + "</new_attribute6><new_attribute8>" + DateTime.Now.ToString() + "</new_attribute8><new_attribute7>" + Utils.GetRandomString(20, 30) + "</new_attribute7><new_attribute9>" + DateTime.Today.ToString() + "</new_attribute9></account>";
                  XmlDocument test = new XmlDocument();
                  test.LoadXml(accountStr);
                  return accountStr;
              }
              catch (Exception e)
              {
                  System.Diagnostics.Trace.WriteLine(e.Message);
                  return "";
              }
          }

          public static string GetCreateCompetitorXml(string name)
          {
              try
              {
                  string accountStr = "<competitor><name>" + name + "</name></competitor>";
                  XmlDocument test = new XmlDocument();
                  test.LoadXml(accountStr);
                  return accountStr;
              }
              catch (Exception e)
              {
                  System.Diagnostics.Trace.WriteLine(e.Message);
                  return "";
              }
          }

          public static string GetCreateAccountXml(Guid ownerid, int owneridtype, string name)
          {
              try
              {
                  string accountStr = "<account><name>" + name + "</name><ownerid type=\"" + owneridtype.ToString() + "\">" + ownerid.ToString() + "</ownerid><creditonhold>0</creditonhold><preferredcontactmethodcode name=\"Any\">1</preferredcontactmethodcode><donotemail>0</donotemail><donotbulkemail>0</donotbulkemail><donotphone>0</donotphone><donotfax>0</donotfax><donotpostalmail>0</donotpostalmail><donotsendmm>0</donotsendmm></account>";
                  XmlDocument test = new XmlDocument();
                  test.LoadXml(accountStr);
                  return accountStr;
              }
              catch (Exception e)
              {
                  System.Diagnostics.Trace.WriteLine(e.Message);
                  return "";
              }
          }

          public static string GetUpdateAccountXml()
          {
              try
              {
                  string accountStr = "<account><telephone1>" + Utils.GetRandomPhoneNumber() + "</telephone1></account>";
                  XmlDocument test = new XmlDocument();
                  test.LoadXml(accountStr);
                  return accountStr;
              }
              catch (Exception e)
              {
                  System.Diagnostics.Trace.WriteLine(e.Message);
                  return "";
              }
          }

          public static string GetCreateAnnotationXml(Guid objectid, int objectidtypecode, Guid ownerid , int owneridtypecode)
          {
              try
              {
                  string annotationStr = "<annotation><isdocument>0</isdocument><notetext>" + Utils.GetRandomString(10,15) + "</notetext><objectid>" +  objectid.ToString("B") + "</objectid><objecttypecode>" + objectidtypecode.ToString() + "</objecttypecode><ownerid type='" + owneridtypecode.ToString() + "'>" + ownerid.ToString("B") + "</ownerid></annotation>";
                  XmlDocument test = new XmlDocument();
                  test.LoadXml(annotationStr);
                  return annotationStr;
              }
              catch (Exception e)
              {
                  System.Diagnostics.Trace.WriteLine(e.Message);
                  return "";
              }
          }

          public static string GetCreateAppointmentXml(Guid requiredattendeeid, int requiredattendeetypecode, Guid ownerid , int owneridtypecode)
          {
              try
              {
                  DateTime sysEnd = System.DateTime.Now + System.TimeSpan.FromMinutes(30);

                  string appointmentStr = "<appointment><subject>" + Utils.GetRandomString(5,10)+ "</subject><requiredattendees><activityparty><partyid type=\"" + requiredattendeetypecode.ToString() + "\">" + requiredattendeeid.ToString("B") + "</partyid></activityparty></requiredattendees><statuscode name=\"Busy\">5</statuscode><scheduledstart>" + DateTime.Now.ToString("s")  + "</scheduledstart><scheduleddurationminutes>30</scheduleddurationminutes><scheduledend>" + sysEnd.ToString("s") + "</scheduledend><isalldayevent>0</isalldayevent><description>" + Utils.GetRandomString(10,30) + "</description><ownerid type=\"" + owneridtypecode.ToString() + "\">" + ownerid.ToString("B") + "</ownerid><organizer><activityparty><partyid type=\"" + owneridtypecode.ToString() + "\">" + ownerid.ToString("B") + "</partyid></activityparty></organizer></appointment>";
                  XmlDocument test = new XmlDocument();
                  test.LoadXml(appointmentStr);
                  return appointmentStr;
              }
              catch (Exception e)
              {
                  System.Diagnostics.Trace.WriteLine(e.Message);
                  return "";
              }
          }

          public static string GetUpdateAppointmentXml()
          {
              try
              {
                  DateTime sysEnd = System.DateTime.Now + System.TimeSpan.FromMinutes(30);

                  string appointmentStr = "<appointment><subject>" + Utils.GetRandomString(10, 15) + "</subject><scheduledstart>" + System.DateTime.Now.ToString() + "</scheduledstart><scheduleddurationminutes>30</scheduleddurationminutes><scheduledend>" + sysEnd.ToString() + "</scheduledend><description>" + Utils.GetRandomString(30, 40) + "</description></appointment>";
                  XmlDocument test = new XmlDocument();
                  test.LoadXml(appointmentStr);
                  return appointmentStr;
              }
              catch (Exception e)
              {
                  System.Diagnostics.Trace.WriteLine(e.Message);
                  return "";
              }
          }


          public static string GetCreateCampaignXml(Guid ownerid, int owneridtype)
          {
              try
              {
                  string campaignStr = "<campaign><name>" + Utils.GetRandomString(5,10) + "-" + System.DateTime.Now.ToString() + "</name><statuscode>0</statuscode><codename>" + Utils.GetRandomString(5,10) + "</codename><typecode>1</typecode><expectedresponse>" + Utils.GetRandomNumber(1,100) + "</expectedresponse><objective>" + Utils.GetRandomString(10,15) + "</objective><description>" + Utils.GetRandomString(50,60) + "</description><ownerid type=\"" + owneridtype.ToString() + "\">" + ownerid.ToString("B") + "</ownerid></campaign>";
                  XmlDocument test = new XmlDocument();
                  test.LoadXml(campaignStr);
                  return campaignStr;
              }
              catch (Exception e)
              {
                  System.Diagnostics.Trace.WriteLine(e.Message);
                  return "";
              }
          }

          public static string GetCreateCampaignActivityXml(Guid ownerid, int owneridtype, Guid regardingobjectid, int regardingobjectidtype, int channeltypecode)
          {
              try
              {
                  string campaignactStr = "<campaignactivity><regardingobjectid type=\"" + regardingobjectidtype.ToString() + "\">" + regardingobjectid.ToString("B") + "</regardingobjectid><statuscode>1</statuscode><channeltypecode>" + channeltypecode.ToString() + "</channeltypecode><typecode>1</typecode><subject>" + Utils.GetRandomString(10,15) + "</subject><description>" + Utils.GetRandomString(50,60) + "</description><ownerid type=\"" + owneridtype.ToString() + "\">" + ownerid.ToString("B") + "</ownerid><prioritycode>1</prioritycode><isbilled name=\"\">false</isbilled><isworkflowcreated name=\"\">false</isworkflowcreated><donotsendonoptout name=\"\">true</donotsendonoptout><ignoreinactivelistmembers name=\"\">true</ignoreinactivelistmembers><statecode name=\"Open\">Open</statecode><actualdurationminutes formattedvalue=\"\">30</actualdurationminutes></campaignactivity>";
                  XmlDocument test = new XmlDocument();
                  test.LoadXml(campaignactStr);
                  return campaignactStr;
              }
              catch (Exception e)
              {
                  System.Diagnostics.Trace.WriteLine(e.Message);
                  return "";
              }
          }

          public static string GetCreateCampaignResponseXml(Guid ownerid, int owneridtype, Guid regardingobjectid, int regardingobjectidtype)
          {
              try
              {
                  DateTime receivedOn = DateTime.Now;
                  string campaignrespStr = "<campaignresponse><regardingobjectid type=\"" + regardingobjectidtype.ToString() + "\">" + regardingobjectid.ToString("B") + "</regardingobjectid><responsecode>1</responsecode><promotioncodename>12</promotioncodename><receivedon>" + receivedOn.ToString() + "</receivedon><subject>" + Utils.GetRandomString(10,15) + "</subject><description>" + Utils.GetRandomString(30,40) + "</description><ownerid type=\"" + owneridtype.ToString() + "\">" + ownerid.ToString("B") + "</ownerid><prioritycode>1</prioritycode><isworkflowcreated>false</isworkflowcreated><actualdurationminutes formattedvalue=\"\">30</actualdurationminutes><statecode name=\"Open\">Open</statecode><isbilled name=\"\">false</isbilled></campaignresponse>";
                  XmlDocument test = new XmlDocument();
                  test.LoadXml(campaignrespStr);
                  return campaignrespStr;
              }
              catch (Exception e)
              {
                  System.Diagnostics.Trace.WriteLine(e.Message);
                  return "";
              }
          }

          public static string GetCreateContactWithCustomAttributesXml(Guid ownerid, int owneridtype, string lastname, string firstname)
          {
              try
              {
                  string contactStr = "<contact><firstname>" + firstname + "</firstname><lastname>" + lastname + "</lastname><ownerid type=\"" + owneridtype.ToString() + "\">" + ownerid.ToString("B") + "</ownerid><creditonhold>0</creditonhold><preferredcontactmethodcode>1</preferredcontactmethodcode><donotemail>0</donotemail><donotbulkemail>0</donotbulkemail><donotphone>0</donotphone><donotfax>0</donotfax><donotpostalmail>0</donotpostalmail><donotsendmm>0</donotsendmm><new_attribute1>" + Utils.GetRandomString(5, 10) + "</new_attribute1><new_attribute10>" + Utils.GetRandomString(3, 10) + "</new_attribute10><new_attribute2>" + Utils.GetRandomNumber(1, 4).ToString() + "</new_attribute2><new_attribute3>" + Utils.GetRandomNumber(0, 1).ToString() + "</new_attribute3><new_attribute4>" + Utils.GetRandomNumber(10, 100).ToString() + "</new_attribute4><new_attribute5>" + Utils.GetRandomNumber(10, 100).ToString() + "</new_attribute5><new_attribute6>" + Utils.GetRandomNumber(10, 100).ToString() + "</new_attribute6><new_attribute8>" + DateTime.Now.ToString() + "</new_attribute8><new_attribute7>" + Utils.GetRandomString(30, 100) + "</new_attribute7><new_attribute9>" + DateTime.Today.ToString() + "</new_attribute9></contact>";
                  XmlDocument test = new XmlDocument();
                  test.LoadXml(contactStr);
                  return contactStr;
              }
              catch (Exception e)
              {
                  System.Diagnostics.Trace.WriteLine(e.Message);
                  return "";
              }
          }

          public static string GetUpdateContactWithCustomAttributesXml()
          {
              try
              {
                  string contactStr = "<contact><new_attribute1>" + Utils.GetRandomString(5, 10) + "</new_attribute1><new_attribute10>" + Utils.GetRandomString(3, 10) + "</new_attribute10><new_attribute2>" + Utils.GetRandomNumber(1, 4).ToString() + "</new_attribute2><new_attribute3>" + Utils.GetRandomNumber(0, 1).ToString() + "</new_attribute3><new_attribute4>" + Utils.GetRandomNumber(10, 100).ToString() + "</new_attribute4><new_attribute5>" + Utils.GetRandomNumber(10, 100).ToString() + "</new_attribute5><new_attribute6>" + Utils.GetRandomNumber(10, 100).ToString() + "</new_attribute6><new_attribute8>" + DateTime.Now.ToString() + "</new_attribute8><new_attribute7>" + Utils.GetRandomString(30, 100) + "</new_attribute7><new_attribute9>" + DateTime.Today.ToString() + "</new_attribute9></contact>";
                  XmlDocument test = new XmlDocument();
                  test.LoadXml(contactStr);
                  return contactStr;
              }
              catch (Exception e)
              {
                  System.Diagnostics.Trace.WriteLine(e.Message);
                  return "";
              }
          }

          public static string GetCreateContactXml(Guid ownerid, int owneridtype, string lastname, string firstname)
          {
              try
              {
                  string contactStr = "<contact><telephone1>" + Utils.GetRandomPhoneNumber() + "</telephone1><firstname>" + Utils.GetRandomString(5,10) + "</firstname><lastname>" + Utils.GetRandomString(5,10) + "</lastname><ownerid type=\"" + owneridtype.ToString() + "\">" + ownerid.ToString("B") + "</ownerid><creditonhold>0</creditonhold><preferredcontactmethodcode>1</preferredcontactmethodcode><donotemail>0</donotemail><donotbulkemail>0</donotbulkemail><donotphone>0</donotphone><donotfax>0</donotfax><donotpostalmail>0</donotpostalmail><donotsendmm>0</donotsendmm></contact>";
                  XmlDocument test = new XmlDocument();
                  test.LoadXml(contactStr);
                  return contactStr;
              }
              catch (Exception e)
              {
                  System.Diagnostics.Trace.WriteLine(e.Message);
                  return "";
              }
          }

          public static string GetUpdateContactXml()
          {
              try
              {
                  string contactStr = "<contact><telephone1>" + Utils.GetRandomPhoneNumber() + "</telephone1></contact>";
                  XmlDocument test = new XmlDocument();
                  test.LoadXml(contactStr);
                  return contactStr;
              }
              catch (Exception e)
              {
                  System.Diagnostics.Trace.WriteLine(e.Message);
                  return "";
              }
          }

          

          public static string GetCreateEmailXml(Guid fromid, int fromidtype, Guid toid, int toidtype, Guid regardingobjectid, int regardingobjectidtype, Guid ownerid, int owneridtype)
          {
              try
              {
                  string emailStr = "<email><directioncode>1</directioncode><from><activityparty><partyid type=\"" + fromidtype.ToString() + "\">" + fromid.ToString("B") + "</partyid></activityparty></from><to><activityparty><partyid type=\"" + toidtype.ToString() + "\">" + toid.ToString("B") + "</partyid></activityparty></to><subject>" + Utils.GetRandomString(10,15) + "</subject><description>" + Utils.GetRandomString(20,40) + "</description><regardingobjectid type=\"" + regardingobjectidtype.ToString() + "\">" + regardingobjectid.ToString("B") + "</regardingobjectid><ownerid type=\"" + owneridtype.ToString() + "\">" + ownerid.ToString("B") + "</ownerid><actualdurationminutes>30</actualdurationminutes><prioritycode>1</prioritycode></email>";
                  XmlDocument test = new XmlDocument();
                  test.LoadXml(emailStr);
                  return emailStr;
              }
              catch (Exception e)
              {
                  System.Diagnostics.Trace.WriteLine(e.Message);
                  return "";
              }
          }

          public static string GetUpdateEmailXml()
          {
              try
              {
                  string emailStr = "<email><subject>" + Utils.GetRandomString(10, 20) + "</subject><description>" + Utils.GetRandomString(50, 80) + "</description></email>";
                  XmlDocument test = new XmlDocument();
                  test.LoadXml(emailStr);
                  return emailStr;
              }
              catch (Exception e)
              {
                  System.Diagnostics.Trace.WriteLine(e.Message);
                  return "";
              }
          }


          public static string GetCreateFaxXml(Guid fromid, int fromidtype, Guid toid, int toidtype, Guid regardingobjectid, int regardingobjectidtype, Guid ownerid, int owneridtype)
          {
              try
              {
                  string faxStr = "=<fax><from><activityparty><partyid type=\"" + fromidtype.ToString() + "\">" + fromid.ToString("B") + "</partyid></activityparty></from><faxnumber>" + Utils.GetRandomPhoneNumber() + "</faxnumber><to><activityparty><partyid type=\"" + toidtype.ToString() + "\">" + toid.ToString("B") + "</partyid></activityparty></to><directioncode>1</directioncode><subject>" + Utils.GetRandomString(10,15) + "</subject><description>" + Utils.GetRandomString(20,30) + "</description><regardingobjectid type=\"" + regardingobjectidtype.ToString() + "\">" + regardingobjectid.ToString("B") + "</regardingobjectid><ownerid type=\"" + owneridtype.ToString() + "\">" + ownerid.ToString("B") + "</ownerid><actualdurationminutes>30</actualdurationminutes><prioritycode>1</prioritycode></fax>";
                  XmlDocument test = new XmlDocument();
                  test.LoadXml(faxStr);
                  return faxStr;
              }
              catch (Exception e)
              {
                  System.Diagnostics.Trace.WriteLine(e.Message);
                  return "";
              }
          }

          public static string GetCreateFaxXml(Guid ownerid, int owneridtype)
          {
              try
              {
                  string faxStr = "<fax><from><activityparty><partyid type=\"" + owneridtype.ToString() + "\">" + ownerid.ToString("B") + "</partyid></activityparty></from><directioncode>1</directioncode><subject>" + Utils.GetRandomString(10,20) + "</subject><description>" + Utils.GetRandomString(50,80) +"</description><ownerid type=\"" + owneridtype.ToString() + "\">" + ownerid.ToString("B") + "</ownerid><actualdurationminutes>30</actualdurationminutes><prioritycode>1</prioritycode></fax>";
                  XmlDocument test = new XmlDocument();
                  test.LoadXml(faxStr);
                  return faxStr;
              }
              catch (Exception e)
              {
                  System.Diagnostics.Trace.WriteLine(e.Message);
                  return "";
              }
          }

          public static string GetUpdateFaxXml()
          {
              try
              {
                  string faxStr = "<fax><subject>" + Utils.GetRandomString(10, 20) + "</subject><description>" + Utils.GetRandomString(50, 80) + "</description></fax>";
                  XmlDocument test = new XmlDocument();
                  test.LoadXml(faxStr);
                  return faxStr;
              }
              catch (Exception e)
              {
                  System.Diagnostics.Trace.WriteLine(e.Message);
                  return "";
              }
          }

          public static string GetCreateIncidentXml(Guid ownerid, int owneridtype, Guid customerid, int customeridtype, Guid subjectid, int subjectidtype)
          {
              try
              {
                  string incidentStr = "<incident><title>" + Utils.GetRandomString(5, 10) + "</title><customerid type=\"" + customeridtype.ToString() + "\">" + customerid.ToString("B") + "</customerid><subjectid type=\"" + subjectidtype.ToString() + "\">" + subjectid.ToString("B") + "</subjectid><ownerid type=\"" + owneridtype.ToString() + "\">" + ownerid.ToString("B") + "</ownerid><statuscode name=\"In Progress\">1</statuscode><prioritycode name=\"Normal\">2</prioritycode></incident>";
                  XmlDocument test = new XmlDocument();
                  test.LoadXml(incidentStr);
                  return incidentStr;
              }
              catch (Exception e)
              {
                  System.Diagnostics.Trace.WriteLine(e.Message);
                  return "";
              }
          }

          public static string GetCreateLeadXml(Guid ownerid, int owneridtype, string firstname, string lastname)
          {
              try
              {
                  //string leadStr = "<lead><subject>" + Utils.GetRandomString(10,20) + "</subject><firstname>" + firstname + "</firstname><lastname>" + lastname + "</lastname><companyname>" + Utils.GetRandomString(5,7) + "</companyname><leadqualitycode>1</leadqualitycode><telephone1>" + Utils.GetRandomPhoneNumber() + "</telephone1><emailaddress1>testemail@email.com</emailaddress1><description>" + Utils.GetRandomString(60,70) + "</description><ownerid type=\"" + owneridtype.ToString() +"\">" + ownerid.ToString("B") + "</ownerid><statuscode>1</statuscode><preferredcontactmethodcode>1</preferredcontactmethodcode><donotemail>0</donotemail><donotbulkemail>0</donotbulkemail><donotphone>0</donotphone><donotfax>0</donotfax><donotpostalmail>0</donotpostalmail><donotsendmm>0</donotsendmm></lead>";
                  string leadStr = @"<lead><subject>" + Utils.GetRandomString(10,20) +  @"</subject><firstname>" + firstname + @"</firstname><lastname>" + lastname + @"</lastname><companyname>" + Utils.GetRandomString(5,10) + @"</companyname><leadqualitycode name=""Warm"">2</leadqualitycode><description>"+ Utils.GetRandomString(50,60) + @"</description><ownerid type=" + owneridtype.ToString() + @""">" + ownerid.ToString("B") + @"</ownerid><statuscode name=""New"">1</statuscode><preferredcontactmethodcode name=""Any"">1</preferredcontactmethodcode><donotemail>0</donotemail><donotbulkemail>0</donotbulkemail><donotphone>0</donotphone><donotfax>0</donotfax><donotpostalmail>0</donotpostalmail><donotsendmm>0</donotsendmm><salesstagecode>1</salesstagecode><statecode name=""Open"">Open</statecode><address2_shippingmethodcode>1</address2_shippingmethodcode><prioritycode>1</prioritycode><address1_shippingmethodcode>1</address1_shippingmethodcode><isprivate name="""">false</isprivate><address1_addresstypecode>1</address1_addresstypecode><participatesinworkflow name="""">false</participatesinworkflow><merged name="""">false</merged><address2_addresstypecode>1</address2_addresstypecode></lead>";
                  XmlDocument test = new XmlDocument();
                  test.LoadXml(leadStr);
                  return leadStr;
              }
              catch (Exception e)
              {
                  System.Diagnostics.Trace.WriteLine(e.Message);
                  return "";
              }
          }

          public static string GetUpdateLeadXml()
          {
              try
              {
                  string leadStr = "<lead><telephone1>" + Utils.GetRandomPhoneNumber() + "</telephone1></lead>";
                  XmlDocument test = new XmlDocument();
                  test.LoadXml(leadStr);
                  return leadStr;
              }
              catch (Exception e)
              {
                  System.Diagnostics.Trace.WriteLine(e.Message);
                  return "";
              }
          }

          public static string GetCreateLetterXml(Guid fromid, int fromidtype, Guid toid, int toidtype, Guid regardingobjectid, int regardingobjectidtype, Guid ownerid, int owneridtype)
          {
              try
              {
                  string letterStr = "<letter><from><activityparty><partyid type=\""+ fromidtype.ToString() + "\">"+ fromid.ToString("B") + "</partyid></activityparty></from><to><activityparty><partyid type=\"" + toidtype.ToString() + "\">" + toid.ToString("B") + "</partyid></activityparty></to><directioncode>1</directioncode><subject>" + Utils.GetRandomString(10,15) + "</subject><description>" + Utils.GetRandomString(15,20) + "</description><regardingobjectid type=\"" + regardingobjectidtype.ToString() +"\">" + regardingobjectid.ToString("B") + "</regardingobjectid><ownerid type=\"" + owneridtype.ToString() + "\">" + ownerid.ToString("B") + "</ownerid><actualdurationminutes>30</actualdurationminutes><prioritycode>1</prioritycode></letter>";
                  XmlDocument test = new XmlDocument();
                  test.LoadXml(letterStr);
                  return letterStr;
              }
              catch (Exception e)
              {
                  System.Diagnostics.Trace.WriteLine(e.Message);
                  return "";
              }
          }

          public static string GetCreateListXml(Guid ownerid, int owneridtype)
          {
              try
              {
                  string listStr = "<list><listname>" + Utils.GetRandomString(5,10) + "</listname><createdfromcode>1</createdfromcode><purpose>" + Utils.GetRandomString(5,10) + "</purpose><lockstatus>0</lockstatus><ownerid type=\"" + owneridtype.ToString() + "\">" + ownerid.ToString("B") + "</ownerid><description>" + Utils.GetRandomString(50,60) + "</description></list>";
                  XmlDocument test = new XmlDocument();
                  test.LoadXml(listStr);
                  return listStr;
              }
              catch (Exception e)
              {
                  System.Diagnostics.Trace.WriteLine(e.Message);
                  return "";
              }
          }

          public static string GetCreateOpportunityXml(Guid ownerid, int owneridtype, string name, Guid customerid, int customeridtype)
          {
              try
              {
                  string oppStr = "<opportunity><name>" + name + "testopp</name><customerid type=\"" + customeridtype.ToString() + "\">" + customerid.ToString("B") + "</customerid><isrevenuesystemcalculated>1</isrevenuesystemcalculated><opportunityratingcode>2</opportunityratingcode><description>" + Utils.GetRandomString(20,30) + "</description><ownerid type=\"" + owneridtype.ToString() + "\">" + ownerid.ToString("B") + "</ownerid><statuscode>1</statuscode></opportunity>";
                  XmlDocument test = new XmlDocument();
                  test.LoadXml(oppStr);
                  return oppStr;
              }
              catch (Exception e)
              {
                  System.Diagnostics.Trace.WriteLine(e.Message);
                  return "";
              }
          }

          public static string GetUpdateOpportunityXml()
          {
              try
              {
                  string oppStr = "<opportunity><description>" + Utils.GetRandomString(20, 30) + "</description></opportunity>";
                  XmlDocument test = new XmlDocument();
                  test.LoadXml(oppStr);
                  return oppStr;
              }
              catch (Exception e)
              {
                  System.Diagnostics.Trace.WriteLine(e.Message);
                  return "";
              }
          }

          public static string GetCreatePhonecallXml(Guid ownerid, int owneridtype, Guid toUser, int toUsertype, Guid regardingobjectid, int regardingobjectidtype)
          {
              try
              {
                  string phonecallStr = "<phonecall><from><activityparty><partyid type=\"" + owneridtype.ToString() + "\">" + ownerid.ToString("B") + "</partyid></activityparty></from><phonenumber>" + Utils.GetRandomPhoneNumber() + "</phonenumber><to><activityparty><partyid type=\"" + toUsertype.ToString() + "\">" + toUser.ToString("B") + "</partyid></activityparty></to><directioncode>1</directioncode><subject>" + Utils.GetRandomString(10,15) + "</subject><description>" + Utils.GetRandomString(50,60) + "</description><regardingobjectid type=\"" + regardingobjectidtype.ToString() + "\">" + regardingobjectid.ToString("B") + "</regardingobjectid><ownerid type=\"" + owneridtype.ToString() + "\">" + ownerid.ToString("B") + "</ownerid><actualdurationminutes>30</actualdurationminutes><prioritycode>1</prioritycode></phonecall>";
                  XmlDocument test = new XmlDocument();
                  test.LoadXml(phonecallStr);
                  return phonecallStr;
              }
              catch (Exception e)
              {
                  System.Diagnostics.Trace.WriteLine(e.Message);
                  return "";
              }
          }

          public static string GetUpdatePhonecallXml()
          {
              try
              {
                  string phonecallStr = "<phonecall><subject>" + Utils.GetRandomString(10,15) + "</subject><description>" + Utils.GetRandomString(50, 60) + "</description></phonecall>";
                  XmlDocument test = new XmlDocument();
                  test.LoadXml(phonecallStr);
                  return phonecallStr;
              }
              catch (Exception e)
              {
                  System.Diagnostics.Trace.WriteLine(e.Message);
                  return "";
              }
          }

          public static string GetUpdateLetterXml()
          {
              try
              {
                  string letterStr = "<letter><description>" + Utils.GetRandomString(50,80) + "</description></letter>";
                  XmlDocument test = new XmlDocument();
                  test.LoadXml(letterStr);
                  return letterStr;
              }
              catch (Exception e)
              {
                  System.Diagnostics.Trace.WriteLine(e.Message);
                  return "";
              }
          }

          public static string GetCreateQuoteXml(Guid ownerid, int owneridtype, Guid pricelevelid, int pricelevelidtype, Guid customerid, int customeridtype)
          {
              try
              {
                  string quoteStr = "<quote><name>" + Utils.GetRandomString(5,10) + "</name><customerid type=\"" + customeridtype.ToString() + "\">" + customerid.ToString("B") + "</customerid><pricelevelid type=\"" + pricelevelidtype.ToString() + "\">" + pricelevelid.ToString("B") + "</pricelevelid><willcall>0</willcall><ownerid type=\"" + owneridtype.ToString() + "\">" + ownerid.ToString("B") + "</ownerid><statuscode>1</statuscode></quote>";
                  XmlDocument test = new XmlDocument();
                  test.LoadXml(quoteStr);
                  return quoteStr;
              }
              catch (Exception e)
              {
                  System.Diagnostics.Trace.WriteLine(e.Message);
                  return "";
              }
          }

          public static string GetCreateRelationshipRoleXml(string name)
          {
              try
              {
                  string relroleStr = "<relationshiprole><name>" + name + "asdf</name></relationshiprole>";
                  XmlDocument test = new XmlDocument();
                  test.LoadXml(relroleStr);
                  return relroleStr;
              }
              catch (Exception e)
              {
                  System.Diagnostics.Trace.WriteLine(e.Message);
                  return "";
              }
          }

          public static string GetCreateQuoteDetailXml(Guid quoteid, int priceperunit, int quantity)
          {
              try
              {
                  string quotedetailStr = "<quotedetail><quoteid>" + quoteid.ToString("B") + "</quoteid><isproductoverridden>1</isproductoverridden><productdescription>" + Utils.GetRandomString(20,30) +"</productdescription><ispriceoverridden>1</ispriceoverridden><priceperunit>" + priceperunit.ToString() + "</priceperunit><quantity>" + quantity.ToString() + "</quantity><willcall>0</willcall><pricingerrorcode>1</pricingerrorcode><quotestatecode>-1</quotestatecode></quotedetail>";
                  XmlDocument test = new XmlDocument();
                  test.LoadXml(quotedetailStr);
                  return quotedetailStr;
              }
              catch (Exception e)
              {
                  System.Diagnostics.Trace.WriteLine(e.Message);
                  return "";
              }
          }

          public static string GetCreateTaskXml(Guid ownerid, int owneridtype, Guid regardingobjectid, int regardingobjectidtype)
          {
              try
              {
                  string taskStr = "<task><subject>" + Utils.GetRandomString(5, 10) + "</subject><description>" + Utils.GetRandomString(20, 30) + "</description><regardingobjectid type=\"" + regardingobjectidtype.ToString() + "\">" + regardingobjectid.ToString("B") + "</regardingobjectid><ownerid type=\"" + owneridtype.ToString() + "\">" + ownerid.ToString("B") + "</ownerid><actualdurationminutes>30</actualdurationminutes><prioritycode name=\"Normal\">1</prioritycode></task>";
                  XmlDocument test = new XmlDocument();
                  test.LoadXml(taskStr);
                  return taskStr;
              }
              catch (Exception e)
              {
                  System.Diagnostics.Trace.WriteLine(e.Message);
                  return "";
              }
          }

          public static string GetCreateTaskWithCustomAttributesXml(Guid ownerid, int owneridtype)
          {
              try
              {
                  string taskStr = "<task><subject>" + Utils.GetRandomString(10,15) + "</subject><description>" + Utils.GetRandomString(20,50) + "</description><ownerid type=\"" + owneridtype.ToString() + "\">" + ownerid.ToString("B") + "</ownerid><actualdurationminutes>30</actualdurationminutes><prioritycode name=\"Normal\">1</prioritycode><new_attribute1>" + Utils.GetRandomString(5,10) + "</new_attribute1><new_attribute10>" + Utils.GetRandomString(5,10) + "</new_attribute10><new_attribute2>" + Utils.GetRandomNumber(1,5).ToString() + "</new_attribute2><new_attribute3>" + Utils.GetRandomNumber(0,1).ToString() + "</new_attribute3><new_attribute4>" + Utils.GetRandomNumber(10,90).ToString() + "</new_attribute4><new_attribute5>" + Utils.GetRandomNumber(0,10).ToString() + "</new_attribute5><new_attribute6>" + Utils.GetRandomNumber(5,10).ToString() + "</new_attribute6><new_attribute8>" + System.DateTime.Now.ToString() + "</new_attribute8><new_attribute7>" + Utils.GetRandomString(10,50) + "</new_attribute7><new_attribute9>" + System.DateTime.Today.ToString() + "</new_attribute9></task>";
                  XmlDocument test = new XmlDocument();
                  test.LoadXml(taskStr);
                  return taskStr;
              }
              catch (Exception e)
              {
                  System.Diagnostics.Trace.WriteLine(e.Message);
                  return "";
              }
          }

          public static string GetUpdateTaskWithCustomAttributesXml()
          {
              try
              {
                  string taskStr = "<task><new_attribute1>" + Utils.GetRandomString(5, 10) + "</new_attribute1><new_attribute10>" + Utils.GetRandomString(5, 10) + "</new_attribute10><new_attribute2>" + Utils.GetRandomNumber(1, 5).ToString() + "</new_attribute2><new_attribute3>" + Utils.GetRandomNumber(0, 1).ToString() + "</new_attribute3><new_attribute4>" + Utils.GetRandomNumber(10, 90).ToString() + "</new_attribute4><new_attribute5>" + Utils.GetRandomNumber(0, 10).ToString() + "</new_attribute5><new_attribute6>" + Utils.GetRandomNumber(5, 10).ToString() + "</new_attribute6><new_attribute8>" + System.DateTime.Now.ToString() + "</new_attribute8><new_attribute7>" + Utils.GetRandomString(10, 50) + "</new_attribute7><new_attribute9>" + System.DateTime.Today.ToString() + "</new_attribute9></task>";
                  XmlDocument test = new XmlDocument();
                  test.LoadXml(taskStr);
                  return taskStr;
              }
              catch (Exception e)
              {
                  System.Diagnostics.Trace.WriteLine(e.Message);
                  return "";
              }
          }

          public static string GetCreateTaskXml(Guid ownerid, int owneridtype)
          {
              try
              {
                  string taskStr = "<task><subject>" + Utils.GetRandomString(10, 15) + "</subject><description>" + Utils.GetRandomString(60, 80) + "</description><ownerid type=\"" + owneridtype.ToString() + "\">" + ownerid.ToString("B") + "</ownerid></task>";
                  XmlDocument test = new XmlDocument();
                  test.LoadXml(taskStr);
                  return taskStr;
              }
              catch (Exception e)
              {
                  System.Diagnostics.Trace.WriteLine(e.Message);
                  return "";
              }
          }

          public static string GetUpdateTaskXml()
          {
              try
              {
                  string taskStr = "<task><subject>" + Utils.GetRandomString(10, 15) + "</subject><description>" + Utils.GetRandomString(60, 80) + "</description></task>";
                  XmlDocument test = new XmlDocument();
                  test.LoadXml(taskStr);
                  return taskStr;
              }
              catch (Exception e)
              {
                  System.Diagnostics.Trace.WriteLine(e.Message);
                  return "";
              }
          }
          

      }
    
    	public class GridXmlHelper
    	{    
            public static string AccountListInFaxView(int pageNum, string guid, string pagingCookie)
            {
                string grid = @"&#x3C;grid&#x3E;&#x3C;sortColumns&#x3E;name&#x26;&#x23;x3A&#x3B;1&#x3C;/sortColumns&#x3E;&#x3C;pageNum&#x3E;1&#x3C;/pageNum&#x3E;&#x3C;recsPerPage&#x3E;50&#x3C;/recsPerPage&#x3E;&#x3C;dataProvider&#x3E;Microsoft.Crm.Application.Controls.LookupGridDataProvider&#x3C;/dataProvider&#x3E;&#x3C;uiProvider&#x3E;Microsoft.Crm.Application.Controls.AppGridUIProvider&#x3C;/uiProvider&#x3E;&#x3C;cols/&#x3E;&#x3C;max&#x3E;1&#x3C;/max&#x3E;&#x3C;refreshAsync&#x3E;False&#x3C;/refreshAsync&#x3E;&#x3C;pagingCookie/&#x3E;&#x3C;enableMultiSort&#x3E;true&#x3C;/enableMultiSort&#x3E;&#x3C;enablePagingWhenOnePage&#x3E;true&#x3C;/enablePagingWhenOnePage&#x3E;&#x3C;parameters&#x3E;&#x3C;autorefresh&#x3E;1&#x3C;/autorefresh&#x3E;&#x3C;disableDblClick&#x3E;0&#x3C;/disableDblClick&#x3E;&#x3C;viewid&#x3E;&#x26;&#x23;x7B&#x3B;E88CA999-0B16-4AE9-B6A9-9EDC840D42D8&#x26;&#x23;x7D&#x3B;&#x3C;/viewid&#x3E;&#x3C;viewtype&#x3E;1039&#x3C;/viewtype&#x3E;&#x3C;otc&#x3E;1&#x3C;/otc&#x3E;&#x3C;ObjectType&#x3E;8&#x3C;/ObjectType&#x3E;&#x3C;class&#x3E;ActivityRecipient&#x3C;/class&#x3E;&#x3C;objecttypes&#x3E;1&#x26;&#x23;x2C&#x3B;2&#x26;&#x23;x2C&#x3B;4&#x26;&#x23;x2C&#x3B;8&#x26;&#x23;x2C&#x3B;2020&#x3C;/objecttypes&#x3E;&#x3C;browse&#x3E;0&#x3C;/browse&#x3E;&#x3C;DefaultType&#x3E;0&#x3C;/DefaultType&#x3E;&#x3C;Mode&#x3E;2&#x3C;/Mode&#x3E;&#x3C;searchvalue/&#x3E;&#x3C;BindingColumns/&#x3E;&#x3C;showjumpbar&#x3E;0&#x3C;/showjumpbar&#x3E;&#x3C;DataProviderOverride&#x3E;Microsoft.Crm.Application.Controls.LookupGridDataProvider&#x3C;/DataProviderOverride&#x3E;&#x3C;/parameters&#x3E;&#x3C;/grid&#x3E;";
                grid = HttpUtility.HtmlDecode(grid);
                grid = grid.Replace("<pagingCookie/>", "<pagingCookie></pagingCookie>");
                grid = grid.Remove(grid.IndexOf("<viewid>") + 14, 36);
                grid = grid.Insert(grid.IndexOf("<viewid>") + 14, guid);
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }

                return grid;
            }
    
            public static string AccountPageView(int pageNum, string guid, string pagingCookie)
            {
                string grid = @"<grid><sortColumns>name&#x3A;1</sortColumns><pageNum>1</pageNum><recsPerPage>50</recsPerPage><dataProvider>Microsoft.Crm.Application.Controls.AppGridDataProvider</dataProvider><uiProvider>Microsoft.Crm.Application.Controls.AppGridUIProvider</uiProvider><cols/><max>-1</max><refreshAsync>False</refreshAsync><pagingCookie></pagingCookie><enableMultiSort>true</enableMultiSort><enablePagingWhenOnePage>true</enablePagingWhenOnePage><parameters><autorefresh>1</autorefresh><viewid>&#x7B;VIEW_GUID&#x7D;</viewid><viewtype>1039</viewtype><otc>1</otc><preview>1</preview><quickfind/><filter/><filterDisplay/></parameters></grid>";
                grid = grid.Replace("VIEW_GUID", guid);
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }

                return grid;
            }

            

            public static string CompetitorPageViewRefresh(int pageNum, string guid, string pagingCookie)
            {
                string grid = @"&#x3C;grid&#x3E;&#x3C;sortColumns&#x3E;name&#x26;&#x23;x3A&#x3B;1&#x3C;/sortColumns&#x3E;&#x3C;pageNum&#x3E;1&#x3C;/pageNum&#x3E;&#x3C;recsPerPage&#x3E;50&#x3C;/recsPerPage&#x3E;&#x3C;dataProvider&#x3E;Microsoft.Crm.Application.Controls.AppGridDataProvider&#x3C;/dataProvider&#x3E;&#x3C;uiProvider&#x3E;Microsoft.Crm.Application.Controls.AppGridUIProvider&#x3C;/uiProvider&#x3E;&#x3C;cols/&#x3E;&#x3C;max&#x3E;-1&#x3C;/max&#x3E;&#x3C;refreshAsync&#x3E;False&#x3C;/refreshAsync&#x3E;&#x3C;pagingCookie/&#x3E;&#x3C;enableMultiSort&#x3E;true&#x3C;/enableMultiSort&#x3E;&#x3C;enablePagingWhenOnePage&#x3E;true&#x3C;/enablePagingWhenOnePage&#x3E;&#x3C;parameters&#x3E;&#x3C;autorefresh&#x3E;1&#x3C;/autorefresh&#x3E;&#x3C;viewid&#x3E;&#x26;&#x23;x7B&#x3B;00000000-0000-0000-00AA-000010001110&#x26;&#x23;x7D&#x3B;&#x3C;/viewid&#x3E;&#x3C;viewtype&#x3E;1039&#x3C;/viewtype&#x3E;&#x3C;otc&#x3E;123&#x3C;/otc&#x3E;&#x3C;preview&#x3E;1&#x3C;/preview&#x3E;&#x3C;/parameters&#x3E;&#x3C;columns&#x3E;&#x3C;column width&#x3D;&#x22;300&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Name&#x22; fieldname&#x3D;&#x22;name&#x22; entityname&#x3D;&#x22;competitor&#x22;&#x3E;name&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;230&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Web Site&#x22; fieldname&#x3D;&#x22;websiteurl&#x22; entityname&#x3D;&#x22;competitor&#x22;&#x3E;websiteurl&#x3C;/column&#x3E;&#x3C;/columns&#x3E;&#x3C;/grid&#x3E;";

                grid = HttpUtility.HtmlDecode(grid);
                grid = grid.Replace("<pagingCookie/>", "<pagingCookie></pagingCookie>");
                grid = grid.Remove(grid.IndexOf("<viewid>") + 14, 36);
                grid = grid.Insert(grid.IndexOf("<viewid>") + 14, guid);

                grid = grid.Replace("VIEW_GUID", guid);
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }

                return grid;                
            }

            public static string AccountPageViewRefresh(int pageNum, string guid, string pagingCookie)
            {                
                string grid = @"<grid><sortColumns>name&#x3A;1</sortColumns><pageNum>1</pageNum><recsPerPage>50</recsPerPage><dataProvider>Microsoft.Crm.Application.Controls.AppGridDataProvider</dataProvider><uiProvider>Microsoft.Crm.Application.Controls.AppGridUIProvider</uiProvider><cols/><max>-1</max><refreshAsync>False</refreshAsync><pagingCookie></pagingCookie><enableMultiSort>true</enableMultiSort><enablePagingWhenOnePage>true</enablePagingWhenOnePage><parameters><autorefresh>1</autorefresh><viewid>&#x7B;VIEW_GUID&#x7D;</viewid><viewtype>1039</viewtype><otc>1</otc><preview>1</preview></parameters><columns><column width=""300"" isHidden=""false"" isMetadataBound=""true"" isSortable=""true"" label=""Account Name"" fieldname=""name"" entityname=""account"">name</column><column width=""150"" isHidden=""false"" isMetadataBound=""true"" isSortable=""true"" label=""Primary Contact"" fieldname=""primarycontactid"" entityname=""account"">primarycontactid</column><column width=""100"" isHidden=""false"" isMetadataBound=""true"" isSortable=""true"" label=""Main Phone"" fieldname=""telephone1"" entityname=""account"">telephone1</column><column width=""100"" isHidden=""false"" isMetadataBound=""true"" isSortable=""true"" label=""Address 1&#x3A; City"" fieldname=""address1_city"" entityname=""account"">address1_city</column></columns></grid>";
                grid = grid.Replace("VIEW_GUID", guid);
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }

                return grid;                
            }

            public static string AccountPageSearchViewRefresh(int pageNum, string guid, string random, string pagingCookie)
            {
                string grid = @"<grid><sortColumns>name&#x3A;1</sortColumns><pageNum>1</pageNum><recsPerPage>50</recsPerPage><dataProvider>Microsoft.Crm.Application.Controls.AppGridDataProvider</dataProvider><uiProvider>Microsoft.Crm.Application.Controls.AppGridUIProvider</uiProvider><cols/><max>-1</max><refreshAsync>False</refreshAsync><pagingCookie></pagingCookie><enableMultiSort>true</enableMultiSort><enablePagingWhenOnePage>true</enablePagingWhenOnePage><parameters><autorefresh>1</autorefresh><viewid>&#x7B;VIEW_GUID&#x7D;</viewid><viewtype>1039</viewtype><otc>1</otc><preview>1&#x2C;1</preview><quickfind>RAMDOM_STRING</quickfind><filter/><filterDisplay/></parameters><columns><column width=""300"" isHidden=""false"" isMetadataBound=""true"" isSortable=""true"" label=""Account Name"" fieldname=""name"" entityname=""account"">name</column><column width=""100"" isHidden=""false"" isMetadataBound=""true"" isSortable=""true"" label=""Account Number"" fieldname=""accountnumber"" entityname=""account"">accountnumber</column><column width=""150"" isHidden=""false"" isMetadataBound=""true"" isSortable=""true"" label=""Primary Contact"" fieldname=""primarycontactid"" entityname=""account"">primarycontactid</column><column width=""100"" isHidden=""false"" isMetadataBound=""true"" isSortable=""true"" label=""Address 1&#x3A; City"" fieldname=""address1_city"" entityname=""account"">address1_city</column><column width=""100"" isHidden=""false"" isMetadataBound=""true"" isSortable=""true"" label=""Main Phone"" fieldname=""telephone1"" entityname=""account"">telephone1</column><column width=""200"" isHidden=""false"" isMetadataBound=""true"" isSortable=""true"" label=""E-mail"" fieldname=""emailaddress1"" entityname=""account"">emailaddress1</column></columns></grid>";
                grid = grid.Replace("VIEW_GUID", guid);
                grid = grid.Replace("RANDOM_STRING", random);
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }
                return grid;
            }

            public static string AccountSearchView(int pageNum, string guid, string random, string pagingCookie)
            {
                string grid = @"<grid><sortColumns>name&#x3A;1</sortColumns><pageNum>1</pageNum><recsPerPage>50</recsPerPage><dataProvider>Microsoft.Crm.Application.Controls.AppGridDataProvider</dataProvider><uiProvider>Microsoft.Crm.Application.Controls.AppGridUIProvider</uiProvider><cols/><max>-1</max><refreshAsync>False</refreshAsync><pagingCookie></pagingCookie><enableMultiSort>true</enableMultiSort><enablePagingWhenOnePage>true</enablePagingWhenOnePage><parameters><autorefresh>1</autorefresh><viewid>&#x7B;VIEW_GUID&#x7D;</viewid><viewtype>1039</viewtype><otc>1</otc><preview>1&#x2C;1&#x2C;1&#x2C;1</preview><quickfind>RAMDOM_STRING</quickfind><filter/><filterDisplay/></parameters><columns><column width=""300"" isHidden=""false"" isMetadataBound=""true"" isSortable=""true"" label=""Account Name"" fieldname=""name"" entityname=""account"">name</column><column width=""100"" isHidden=""false"" isMetadataBound=""true"" isSortable=""true"" label=""Account Number"" fieldname=""accountnumber"" entityname=""account"">accountnumber</column><column width=""150"" isHidden=""false"" isMetadataBound=""true"" isSortable=""true"" label=""Primary Contact"" fieldname=""primarycontactid"" entityname=""account"">primarycontactid</column><column width=""100"" isHidden=""false"" isMetadataBound=""true"" isSortable=""true"" label=""Address 1&#x3A; City"" fieldname=""address1_city"" entityname=""account"">address1_city</column><column width=""100"" isHidden=""false"" isMetadataBound=""true"" isSortable=""true"" label=""Main Phone"" fieldname=""telephone1"" entityname=""account"">telephone1</column><column width=""200"" isHidden=""false"" isMetadataBound=""true"" isSortable=""true"" label=""E-mail"" fieldname=""emailaddress1"" entityname=""account"">emailaddress1</column></columns></grid>";
                grid = grid.Replace("VIEW_GUID", guid);
                grid = grid.Replace("RANDOM_STRING", random);
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }

                return grid;
            }

            public static string CustomAccountPageViewRefresh(int pageNum, string guid, string pagingCookie)
            {
                string grid = @"<grid><sortColumns>new_name&#x3A;1</sortColumns><pageNum>1</pageNum><recsPerPage>50</recsPerPage><dataProvider>Microsoft.Crm.Application.Controls.AppGridDataProvider</dataProvider><uiProvider>Microsoft.Crm.Application.Controls.AppGridUIProvider</uiProvider><cols/><max>-1</max><refreshAsync>False</refreshAsync><pagingCookie></pagingCookie><enableMultiSort>true</enableMultiSort><enablePagingWhenOnePage>true</enablePagingWhenOnePage><parameters><autorefresh>1</autorefresh><viewid>&#x7B;VIEW_GUID&#x7D;</viewid><viewtype>1039</viewtype><otc>10000</otc><preview>1</preview></parameters><columns><column width=""300"" isHidden=""false"" isMetadataBound=""true"" isSortable=""true"" label=""Name"">new_name</column><column width=""100"" isHidden=""false"" isMetadataBound=""true"" isSortable=""true"" label=""ParentAccount"">new_parentaccountid</column><column width=""100"" isHidden=""false"" isMetadataBound=""true"" isSortable=""true"" label=""monthcreated"">new_monthcreated</column><column width=""100"" isHidden=""false"" isMetadataBound=""true"" isSortable=""true"" label=""Owner"">ownerid</column><column width=""125"" isHidden=""false"" isMetadataBound=""true"" isSortable=""true"" label=""Created On"">createdon</column></columns></grid>";
                grid = grid.Replace("VIEW_GUID", guid);
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }

                return grid;
            }

            public static string NewSimpleCustomAccountSearchView(int pageNum, string guid, string random, string pagingCookie)
            {
                string grid = @"<grid><sortColumns>new_name&#x3A;1</sortColumns><pageNum>1</pageNum><recsPerPage>50</recsPerPage><dataProvider>Microsoft.Crm.Application.Controls.AppGridDataProvider</dataProvider><uiProvider>Microsoft.Crm.Application.Controls.AppGridUIProvider</uiProvider><cols/><max>-1</max><refreshAsync>False</refreshAsync><pagingCookie></pagingCookie><enableMultiSort>true</enableMultiSort><enablePagingWhenOnePage>true</enablePagingWhenOnePage><parameters><autorefresh>1</autorefresh><viewid>&#x7B;VIEW_GUID&#x7D;</viewid><viewtype>1039</viewtype><otc>10000</otc><preview>1</preview><quickfind>RANDOM_STRING</quickfind><filter/><filterDisplay/></parameters></grid>";
                grid = grid.Replace("VIEW_GUID", guid);
                grid = grid.Replace("RANDOM_STRING", random);
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }

                return grid;
            }

            public static string ActivitiesPageViewRefresh(int pageNum, string guid, string pagingCookie)
            {
                string grid = @"&#x3C;grid&#x3E;&#x3C;sortColumns&#x3E;scheduledend&#x26;&#x23;x3A&#x3B;1&#x3C;/sortColumns&#x3E;&#x3C;pageNum&#x3E;1&#x3C;/pageNum&#x3E;&#x3C;recsPerPage&#x3E;50&#x3C;/recsPerPage&#x3E;&#x3C;dataProvider&#x3E;Microsoft.Crm.Application.Controls.ActivitiesGridDataProvider&#x3C;/dataProvider&#x3E;&#x3C;uiProvider&#x3E;Microsoft.Crm.Application.Controls.AppGridUIProvider&#x3C;/uiProvider&#x3E;&#x3C;cols/&#x3E;&#x3C;max&#x3E;-1&#x3C;/max&#x3E;&#x3C;refreshAsync&#x3E;False&#x3C;/refreshAsync&#x3E;&#x3C;pagingCookie/&#x3E;&#x3C;enableMultiSort&#x3E;true&#x3C;/enableMultiSort&#x3E;&#x3C;enablePagingWhenOnePage&#x3E;true&#x3C;/enablePagingWhenOnePage&#x3E;&#x3C;parameters&#x3E;&#x3C;autorefresh&#x3E;1&#x3C;/autorefresh&#x3E;&#x3C;viewid&#x3E;&#x26;&#x23;x7B&#x3B;00000000-0000-0000-00AA-000010001899&#x26;&#x23;x7D&#x3B;&#x3C;/viewid&#x3E;&#x3C;viewtype&#x3E;1039&#x3C;/viewtype&#x3E;&#x3C;otc&#x3E;4200&#x3C;/otc&#x3E;&#x3C;preview&#x3E;1&#x3C;/preview&#x3E;&#x3C;/parameters&#x3E;&#x3C;columns&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Activity Type&#x22; fieldname&#x3D;&#x22;activitytypecode&#x22; entityname&#x3D;&#x22;activitypointer&#x22;&#x3E;activitytypecode&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;300&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Subject&#x22; fieldname&#x3D;&#x22;subject&#x22; entityname&#x3D;&#x22;activitypointer&#x22;&#x3E;subject&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Regarding&#x22; fieldname&#x3D;&#x22;regardingobjectid&#x22; entityname&#x3D;&#x22;activitypointer&#x22;&#x3E;regardingobjectid&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Priority&#x22; fieldname&#x3D;&#x22;prioritycode&#x22; entityname&#x3D;&#x22;activitypointer&#x22;&#x3E;prioritycode&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;140&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Start Date&#x22; fieldname&#x3D;&#x22;scheduledstart&#x22; entityname&#x3D;&#x22;activitypointer&#x22;&#x3E;scheduledstart&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;140&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Due Date&#x22; fieldname&#x3D;&#x22;scheduledend&#x22; entityname&#x3D;&#x22;activitypointer&#x22;&#x3E;scheduledend&#x3C;/column&#x3E;&#x3C;/columns&#x3E;&#x3C;/grid&#x3E;";
                grid = HttpUtility.HtmlDecode(grid);
                grid = grid.Replace("<pagingCookie/>", "<pagingCookie></pagingCookie>");
                grid = grid.Remove(grid.IndexOf("<viewid>") + 14, 36);
                grid = grid.Insert(grid.IndexOf("<viewid>") + 14, guid);

                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }

                return grid;
            }

            public static string ActivityGridForEntityViewRefresh(int pageNum, string guid, string pagingCookie, string objectGuid, string objectType)
            {                
                string grid = @"&#x3C;grid&#x3E;&#x3C;sortColumns&#x3E;subject&#x26;&#x23;x3A&#x3B;1&#x3C;/sortColumns&#x3E;&#x3C;pageNum&#x3E;1&#x3C;/pageNum&#x3E;&#x3C;recsPerPage&#x3E;50&#x3C;/recsPerPage&#x3E;&#x3C;dataProvider&#x3E;Microsoft.Crm.Application.Controls.ActivitiesGridDataProvider&#x3C;/dataProvider&#x3E;&#x3C;uiProvider&#x3E;Microsoft.Crm.Application.Controls.AppGridUIProvider&#x3C;/uiProvider&#x3E;&#x3C;cols/&#x3E;&#x3C;max&#x3E;-1&#x3C;/max&#x3E;&#x3C;refreshAsync&#x3E;False&#x3C;/refreshAsync&#x3E;&#x3C;pagingCookie/&#x3E;&#x3C;enableMultiSort&#x3E;true&#x3C;/enableMultiSort&#x3E;&#x3C;enablePagingWhenOnePage&#x3E;true&#x3C;/enablePagingWhenOnePage&#x3E;&#x3C;parameters&#x3E;&#x3C;autorefresh&#x3E;1&#x3C;/autorefresh&#x3E;&#x3C;viewid&#x3E;&#x26;&#x23;x7B&#x3B;00000000-0000-0000-00AA-000010001903&#x26;&#x23;x7D&#x3B;&#x3C;/viewid&#x3E;&#x3C;viewtype&#x3E;1039&#x3C;/viewtype&#x3E;&#x3C;otc&#x3E;4200&#x3C;/otc&#x3E;&#x3C;preview&#x3E;1&#x3C;/preview&#x3E;&#x3C;queryapi&#x3E;CRMActivity.RetrieveByParty&#x3C;/queryapi&#x3E;&#x3C;showjumpbar&#x3E;0&#x3C;/showjumpbar&#x3E;&#x3C;enablepaging&#x3E;1&#x3C;/enablepaging&#x3E;&#x3C;oId&#x3E;&#x26;&#x23;x7B&#x3B;92bdabd4-24f9-db11-9929-003048305b3a&#x26;&#x23;x7D&#x3B;&#x3C;/oId&#x3E;&#x3C;oType&#x3E;1084&#x3C;/oType&#x3E;&#x3C;stategroup&#x3E;active&#x3C;/stategroup&#x3E;&#x3C;/parameters&#x3E;&#x3C;columns&#x3E;&#x3C;column width&#x3D;&#x22;250&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Subject&#x22; fieldname&#x3D;&#x22;subject&#x22; entityname&#x3D;&#x22;activitypointer&#x22;&#x3E;subject&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;110&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Activity Type&#x22; fieldname&#x3D;&#x22;activitytypecode&#x22; entityname&#x3D;&#x22;activitypointer&#x22;&#x3E;activitytypecode&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Activity Status&#x22; fieldname&#x3D;&#x22;statecode&#x22; entityname&#x3D;&#x22;activitypointer&#x22;&#x3E;statecode&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Priority&#x22; fieldname&#x3D;&#x22;prioritycode&#x22; entityname&#x3D;&#x22;activitypointer&#x22;&#x3E;prioritycode&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;140&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Due Date&#x22; fieldname&#x3D;&#x22;scheduledend&#x22; entityname&#x3D;&#x22;activitypointer&#x22;&#x3E;scheduledend&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Created By&#x22; fieldname&#x3D;&#x22;createdby&#x22; entityname&#x3D;&#x22;activitypointer&#x22;&#x3E;createdby&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Regarding&#x22; fieldname&#x3D;&#x22;regardingobjectid&#x22; entityname&#x3D;&#x22;activitypointer&#x22;&#x3E;regardingobjectid&#x3C;/column&#x3E;&#x3C;/columns&#x3E;&#x3C;/grid&#x3E;";
                grid = HttpUtility.HtmlDecode(grid);
                grid = grid.Replace("<pagingCookie/>", "<pagingCookie></pagingCookie>");
                grid = grid.Remove(grid.IndexOf("<viewid>") + 14, 36);
                grid = grid.Insert(grid.IndexOf("<viewid>") + 14, guid);

                if (objectGuid != null)
                {
                    grid = grid.Remove(grid.IndexOf("<oId>") + 11, 36);
                    grid = grid.Insert(grid.IndexOf("<oId>") + 11, objectGuid);
                }

                if (objectType != null)
                {
                    grid = grid.Remove(grid.IndexOf("<oType>") + 7, grid.IndexOf("</oType>") - (grid.IndexOf("<oType>") + 7));
                    grid = grid.Insert(grid.IndexOf("<oType>") + 7, objectType);
                }
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }
                return grid;
            }           
            
            public static string AppointmentPageAllView(int pageNum, string guid, string pagingCookie)
            {
                string grid = @"<grid><sortColumns>scheduledend&#x3A;1</sortColumns><pageNum>1</pageNum><recsPerPage>50</recsPerPage><dataProvider>Microsoft.Crm.Application.Controls.ActivitiesGridDataProvider</dataProvider><uiProvider>Microsoft.Crm.Application.Controls.AppGridUIProvider</uiProvider><cols/><max>-1</max><refreshAsync>False</refreshAsync><pagingCookie></pagingCookie><enableMultiSort>true</enableMultiSort><enablePagingWhenOnePage>true</enablePagingWhenOnePage><parameters><autorefresh>1</autorefresh><viewid>&#x7B;VIEW_GUID&#x7D;</viewid><viewtype>1039</viewtype><otc>4200</otc><preview>1</preview><quickfind/><filter/><filterDisplay/></parameters></grid>";
                grid = grid.Replace("VIEW_GUID", guid);
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }

                return grid;
            }

            public static string AppointmentPageMyView(int pageNum, string guid, string pagingCookie)
            {

                string grid = @"<grid><sortColumns>subject&#x3A;1</sortColumns><pageNum>1</pageNum><recsPerPage>50</recsPerPage><dataProvider>Microsoft.Crm.Application.Controls.ActivitiesGridDataProvider</dataProvider><uiProvider>Microsoft.Crm.Application.Controls.AppGridUIProvider</uiProvider><cols/><max>-1</max><refreshAsync>False</refreshAsync><pagingCookie></pagingCookie><enableMultiSort>true</enableMultiSort><enablePagingWhenOnePage>true</enablePagingWhenOnePage><parameters><autorefresh>1</autorefresh><viewid>&#x7B;VIEW_GUID&#x7D;</viewid><viewtype>1039</viewtype><otc>4201</otc><preview>1&#x2C;1</preview><quickfind/><filter/><filterDisplay/></parameters></grid>";
                grid = grid.Replace("VIEW_GUID", guid);
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }

                return grid;
            }

            public static string AppointmentPageViewRefresh(int pageNum, string guid, string pagingCookie)
            {
                string grid = @"<gridXml>&#60;grid&#62;&#60;sortColumns&#62;scheduledend&#38;&#35;58&#59;1&#60;&#47;sortColumns&#62;&#60;pageNum&#62;1&#60;&#47;pageNum&#62;&#60;recsPerPage&#62;50&#60;&#47;recsPerPage&#62;&#60;dataProvider&#62;Microsoft.Crm.Application.Controls.ActivitiesGridDataProvider&#60;&#47;dataProvider&#62;&#60;uiProvider&#62;Microsoft.Crm.Application.Controls.AppGridUIProvider&#60;&#47;uiProvider&#62;&#60;cols&#47;&#62;&#60;max&#62;-1&#60;&#47;max&#62;&#60;refreshAsync&#62;False&#60;&#47;refreshAsync&#62;&#60;pagingCookie&#47;&#62;&#60;enableMultiSort&#62;true&#60;&#47;enableMultiSort&#62;&#60;enablePagingWhenOnePage&#62;true&#60;&#47;enablePagingWhenOnePage&#62;&#60;parameters&#62;&#60;autorefresh&#62;1&#60;&#47;autorefresh&#62;&#60;viewid&#62;&#38;&#35;123&#59;00000000-0000-0000-00AA-000010001899&#38;&#35;125&#59;&#60;&#47;viewid&#62;&#60;viewtype&#62;1039&#60;&#47;viewtype&#62;&#60;otc&#62;4200&#60;&#47;otc&#62;&#60;preview&#62;1&#60;&#47;preview&#62;&#60;&#47;parameters&#62;&#60;columns&#62;&#60;column width&#61;&#34;100&#34; isHidden&#61;&#34;false&#34; isMetadataBound&#61;&#34;true&#34; isSortable&#61;&#34;true&#34; label&#61;&#34;Activity Type&#34; fieldname&#61;&#34;activitytypecode&#34; entityname&#61;&#34;activitypointer&#34;&#62;activitytypecode&#60;&#47;column&#62;&#60;column width&#61;&#34;300&#34; isHidden&#61;&#34;false&#34; isMetadataBound&#61;&#34;true&#34; isSortable&#61;&#34;true&#34; label&#61;&#34;Subject&#34; fieldname&#61;&#34;subject&#34; entityname&#61;&#34;activitypointer&#34;&#62;subject&#60;&#47;column&#62;&#60;column width&#61;&#34;100&#34; isHidden&#61;&#34;false&#34; isMetadataBound&#61;&#34;true&#34; isSortable&#61;&#34;true&#34; label&#61;&#34;Regarding&#34; fieldname&#61;&#34;regardingobjectid&#34; entityname&#61;&#34;activitypointer&#34;&#62;regardingobjectid&#60;&#47;column&#62;&#60;column width&#61;&#34;100&#34; isHidden&#61;&#34;false&#34; isMetadataBound&#61;&#34;true&#34; isSortable&#61;&#34;true&#34; label&#61;&#34;Priority&#34; fieldname&#61;&#34;prioritycode&#34; entityname&#61;&#34;activitypointer&#34;&#62;prioritycode&#60;&#47;column&#62;&#60;column width&#61;&#34;140&#34; isHidden&#61;&#34;false&#34; isMetadataBound&#61;&#34;true&#34; isSortable&#61;&#34;true&#34; label&#61;&#34;Start Date&#34; fieldname&#61;&#34;scheduledstart&#34; entityname&#61;&#34;activitypointer&#34;&#62;scheduledstart&#60;&#47;column&#62;&#60;column width&#61;&#34;140&#34; isHidden&#61;&#34;false&#34; isMetadataBound&#61;&#34;true&#34; isSortable&#61;&#34;true&#34; label&#61;&#34;Due Date&#34; fieldname&#61;&#34;scheduledend&#34; entityname&#61;&#34;activitypointer&#34;&#62;scheduledend&#60;&#47;column&#62;&#60;&#47;columns&#62;&#60;&#47;grid&#62;</gridXml>";

                grid = HttpUtility.HtmlDecode(grid);
                grid = grid.Replace("<pagingCookie/>", "<pagingCookie></pagingCookie>");
                grid = grid.Remove(grid.IndexOf("<viewid>") + 14, 36);
                grid = grid.Insert(grid.IndexOf("<viewid>") + 14, guid);
                
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }

                return grid;
            }

            public static string MyAppointmentsPageViewRefresh(int pageNum, string guid, string pagingCookie)
            {
                string grid = @"<gridXml>&#60;grid&#62;&#60;sortColumns&#62;scheduledstart&#38;&#35;58&#59;1&#60;&#47;sortColumns&#62;&#60;pageNum&#62;1&#60;&#47;pageNum&#62;&#60;recsPerPage&#62;50&#60;&#47;recsPerPage&#62;&#60;dataProvider&#62;Microsoft.Crm.Application.Controls.ActivitiesGridDataProvider&#60;&#47;dataProvider&#62;&#60;uiProvider&#62;Microsoft.Crm.Application.Controls.AppGridUIProvider&#60;&#47;uiProvider&#62;&#60;cols&#47;&#62;&#60;max&#62;-1&#60;&#47;max&#62;&#60;refreshAsync&#62;False&#60;&#47;refreshAsync&#62;&#60;pagingCookie&#47;&#62;&#60;enableMultiSort&#62;true&#60;&#47;enableMultiSort&#62;&#60;enablePagingWhenOnePage&#62;true&#60;&#47;enablePagingWhenOnePage&#62;&#60;parameters&#62;&#60;autorefresh&#62;1&#60;&#47;autorefresh&#62;&#60;viewid&#62;&#38;&#35;123&#59;606EEA12-0C33-47D3-96E1-BA1529EE8205&#38;&#35;125&#59;&#60;&#47;viewid&#62;&#60;viewtype&#62;1039&#60;&#47;viewtype&#62;&#60;otc&#62;4201&#60;&#47;otc&#62;&#60;preview&#62;1,1,1&#60;&#47;preview&#62;&#60;quickfind&#47;&#62;&#60;filter&#47;&#62;&#60;filterDisplay&#47;&#62;&#60;&#47;parameters&#62;&#60;columns&#62;&#60;column width&#61;&#34;250&#34; isHidden&#61;&#34;false&#34; isMetadataBound&#61;&#34;true&#34; isSortable&#61;&#34;true&#34; label&#61;&#34;Subject&#34; fieldname&#61;&#34;subject&#34; entityname&#61;&#34;appointment&#34;&#62;subject&#60;&#47;column&#62;&#60;column width&#61;&#34;100&#34; isHidden&#61;&#34;false&#34; isMetadataBound&#61;&#34;true&#34; isSortable&#61;&#34;true&#34; label&#61;&#34;Regarding&#34; fieldname&#61;&#34;regardingobjectid&#34; entityname&#61;&#34;appointment&#34;&#62;regardingobjectid&#60;&#47;column&#62;&#60;column width&#61;&#34;100&#34; isHidden&#61;&#34;false&#34; isMetadataBound&#61;&#34;true&#34; isSortable&#61;&#34;true&#34; label&#61;&#34;Location&#34; fieldname&#61;&#34;location&#34; entityname&#61;&#34;appointment&#34;&#62;location&#60;&#47;column&#62;&#60;column width&#61;&#34;100&#34; isHidden&#61;&#34;false&#34; isMetadataBound&#61;&#34;true&#34; isSortable&#61;&#34;true&#34; label&#61;&#34;Priority&#34; fieldname&#61;&#34;prioritycode&#34; entityname&#61;&#34;appointment&#34;&#62;prioritycode&#60;&#47;column&#62;&#60;column width&#61;&#34;100&#34; isHidden&#61;&#34;false&#34; isMetadataBound&#61;&#34;true&#34; isSortable&#61;&#34;true&#34; label&#61;&#34;Start Time&#34; fieldname&#61;&#34;scheduledstart&#34; entityname&#61;&#34;appointment&#34;&#62;scheduledstart&#60;&#47;column&#62;&#60;column width&#61;&#34;100&#34; isHidden&#61;&#34;false&#34; isMetadataBound&#61;&#34;true&#34; isSortable&#61;&#34;true&#34; label&#61;&#34;End Time&#34; fieldname&#61;&#34;scheduledend&#34; entityname&#61;&#34;appointment&#34;&#62;scheduledend&#60;&#47;column&#62;&#60;&#47;columns&#62;&#60;&#47;grid&#62;</gridXml>";

                grid = HttpUtility.HtmlDecode(grid);
                grid = grid.Replace("<pagingCookie/>", "<pagingCookie></pagingCookie>");
                grid = grid.Remove(grid.IndexOf("<viewid>") + 14, 36);
                grid = grid.Insert(grid.IndexOf("<viewid>") + 14, guid);

                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }

                return grid;
            }

            public static string AttributePageView(Guid entityId, int pageNum, string guid, string pagingCookie)
            {
                string grid = @"<grid><sortColumns>schemaname&#x3A;1</sortColumns><pageNum>1</pageNum><recsPerPage>50</recsPerPage><dataProvider>Microsoft.Crm.Application.Controls.SystemCustomization.AttributeGridDataProvider</dataProvider><uiProvider>Microsoft.Crm.Application.Controls.SystemCustomization.SysCustGridUIProvider</uiProvider><cols/><max>1</max><refreshAsync>False</refreshAsync><pagingCookie></pagingCookie><enableMultiSort>true</enableMultiSort><enablePagingWhenOnePage>true</enablePagingWhenOnePage><parameters><autorefresh>1</autorefresh><viewid>&#x7B;VIEW_GUID&#x7D;</viewid><viewtype>1039</viewtype><otc>0</otc><showjumpbar>0</showjumpbar><enablepaging>0</enablepaging><attributeFilter>All</attributeFilter><entityId>&#x7B;ENTITY_GUID&#x7D;</entityId><disableDblClick>0</disableDblClick></parameters><columns><column width=""150"" isHidden=""false"" isMetadataBound=""false"" isSortable=""true"" label=""Schema Name"" fieldname=""schemaname"" entityname="""">schemaname</column><column width=""100"" isHidden=""false"" isMetadataBound=""false"" isSortable=""true"" label=""Display Name"" fieldname=""displayname"" entityname="""">displayname</column><column width=""100"" isHidden=""false"" isMetadataBound=""false"" isSortable=""true"" label=""Type"" fieldname=""type"" entityname="""">type</column><column width=""400"" isHidden=""false"" isMetadataBound=""false"" isSortable=""true"" label=""Description"" fieldname=""description"" entityname="""">description</column></columns></grid>";
                grid = grid.Replace("ENTITY_GUID", entityId.ToString().ToUpper());
                grid = grid.Replace("VIEW_GUID", guid);
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }

                return grid;
            }

            public static string BusinessUnitPageView(int pageNum, string guid, string pagingCookie)
            {
                string grid = @"&#60;grid&#62;&#60;sortColumns&#62;name&#38;&#35;58&#59;1&#60;&#47;sortColumns&#62;&#60;pageNum&#62;1&#60;&#47;pageNum&#62;&#60;recsPerPage&#62;50&#60;&#47;recsPerPage&#62;&#60;dataProvider&#62;Microsoft.Crm.Application.Controls.AppGridDataProvider&#60;&#47;dataProvider&#62;&#60;uiProvider&#62;Microsoft.Crm.Application.Controls.AppGridUIProvider&#60;&#47;uiProvider&#62;&#60;cols&#47;&#62;&#60;max&#62;-1&#60;&#47;max&#62;&#60;refreshAsync&#62;False&#60;&#47;refreshAsync&#62;&#60;pagingCookie&#47;&#62;&#60;enableMultiSort&#62;true&#60;&#47;enableMultiSort&#62;&#60;enablePagingWhenOnePage&#62;true&#60;&#47;enablePagingWhenOnePage&#62;&#60;parameters&#62;&#60;autorefresh&#62;1&#60;&#47;autorefresh&#62;&#60;viewid&#62;&#38;&#35;123&#59;00000000-0000-0000-00AA-000010001018&#38;&#35;125&#59;&#60;&#47;viewid&#62;&#60;viewtype&#62;1039&#60;&#47;viewtype&#62;&#60;otc&#62;10&#60;&#47;otc&#62;&#60;preview&#62;1&#60;&#47;preview&#62;&#60;&#47;parameters&#62;&#60;columns&#62;&#60;column width&#61;&#34;300&#34; isHidden&#61;&#34;false&#34; isMetadataBound&#61;&#34;true&#34; isSortable&#61;&#34;true&#34; label&#61;&#34;Name&#34; fieldname&#61;&#34;name&#34; entityname&#61;&#34;businessunit&#34;&#62;name&#60;&#47;column&#62;&#60;column width&#61;&#34;150&#34; isHidden&#61;&#34;false&#34; isMetadataBound&#61;&#34;true&#34; isSortable&#61;&#34;true&#34; label&#61;&#34;Main Phone&#34; fieldname&#61;&#34;address1_telephone1&#34; entityname&#61;&#34;businessunit&#34;&#62;address1_telephone1&#60;&#47;column&#62;&#60;column width&#61;&#34;150&#34; isHidden&#61;&#34;false&#34; isMetadataBound&#61;&#34;true&#34; isSortable&#61;&#34;true&#34; label&#61;&#34;Web Site&#34; fieldname&#61;&#34;websiteurl&#34; entityname&#61;&#34;businessunit&#34;&#62;websiteurl&#60;&#47;column&#62;&#60;column width&#61;&#34;300&#34; isHidden&#61;&#34;false&#34; isMetadataBound&#61;&#34;true&#34; isSortable&#61;&#34;true&#34; label&#61;&#34;Parent Business&#34; fieldname&#61;&#34;parentbusinessunitid&#34; entityname&#61;&#34;businessunit&#34;&#62;parentbusinessunitid&#60;&#47;column&#62;&#60;&#47;columns&#62;&#60;&#47;grid&#62;";
                
                grid = HttpUtility.HtmlDecode(grid);
                grid = grid.Replace("<pagingCookie/>", "<pagingCookie></pagingCookie>");
                grid = grid.Remove(grid.IndexOf("<viewid>") + 14, 36);
                grid = grid.Insert(grid.IndexOf("<viewid>") + 14, guid);

                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }

                return grid;
            }

            public static string CampaignActivityGridForCampaignViewRefresh(int pageNum, string guid, string pagingCookie, string objectGuid, string objectType)
            {
                string grid = @"&#x3C;grid&#x3E;&#x3C;sortColumns&#x3E;subject&#x26;&#x23;x3A&#x3B;0&#x3C;/sortColumns&#x3E;&#x3C;pageNum&#x3E;1&#x3C;/pageNum&#x3E;&#x3C;recsPerPage&#x3E;50&#x3C;/recsPerPage&#x3E;&#x3C;dataProvider&#x3E;Microsoft.Crm.Application.Controls.ActivitiesGridDataProvider&#x3C;/dataProvider&#x3E;&#x3C;uiProvider&#x3E;Microsoft.Crm.Application.Controls.AppGridUIProvider&#x3C;/uiProvider&#x3E;&#x3C;cols/&#x3E;&#x3C;max&#x3E;-1&#x3C;/max&#x3E;&#x3C;refreshAsync&#x3E;False&#x3C;/refreshAsync&#x3E;&#x3C;pagingCookie/&#x3E;&#x3C;enableMultiSort&#x3E;true&#x3C;/enableMultiSort&#x3E;&#x3C;enablePagingWhenOnePage&#x3E;true&#x3C;/enablePagingWhenOnePage&#x3E;&#x3C;parameters&#x3E;&#x3C;autorefresh&#x3E;1&#x3C;/autorefresh&#x3E;&#x3C;viewid&#x3E;&#x26;&#x23;x7B&#x3B;89D107AB-42F5-4164-B7D9-AA83CE170673&#x26;&#x23;x7D&#x3B;&#x3C;/viewid&#x3E;&#x3C;viewtype&#x3E;1039&#x3C;/viewtype&#x3E;&#x3C;otc&#x3E;4402&#x3C;/otc&#x3E;&#x3C;preview&#x3E;1&#x3C;/preview&#x3E;&#x3C;queryapi&#x3E;Campaign.RetrieveCampaignActivitiesForCampaign&#x3C;/queryapi&#x3E;&#x3C;showjumpbar&#x3E;0&#x3C;/showjumpbar&#x3E;&#x3C;enablepaging&#x3E;1&#x3C;/enablepaging&#x3E;&#x3C;oId&#x3E;&#x26;&#x23;x7B&#x3B;c1a609d5-cff2-db11-8d7d-001143dcb18d&#x26;&#x23;x7D&#x3B;&#x3C;/oId&#x3E;&#x3C;oType&#x3E;4400&#x3C;/oType&#x3E;&#x3C;/parameters&#x3E;&#x3C;columns&#x3E;&#x3C;column width&#x3D;&#x22;250&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Subject&#x22; fieldname&#x3D;&#x22;subject&#x22; entityname&#x3D;&#x22;campaignactivity&#x22;&#x3E;subject&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;150&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Owner&#x22; fieldname&#x3D;&#x22;ownerid&#x22; entityname&#x3D;&#x22;campaignactivity&#x22;&#x3E;ownerid&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;150&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;false&#x22; label&#x3D;&#x22;Outsource Vendors&#x22; fieldname&#x3D;&#x22;partners&#x22; entityname&#x3D;&#x22;campaignactivity&#x22;&#x3E;partners&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Channel&#x22; fieldname&#x3D;&#x22;channeltypecode&#x22; entityname&#x3D;&#x22;campaignactivity&#x22;&#x3E;channeltypecode&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Status Reason&#x22; fieldname&#x3D;&#x22;statuscode&#x22; entityname&#x3D;&#x22;campaignactivity&#x22;&#x3E;statuscode&#x3C;/column&#x3E;&#x3C;/columns&#x3E;&#x3C;/grid&#x3E;";

                grid = HttpUtility.HtmlDecode(grid);
                grid = grid.Replace("<pagingCookie/>", "<pagingCookie></pagingCookie>");
                grid = grid.Remove(grid.IndexOf("<viewid>") + 14, 36);
                grid = grid.Insert(grid.IndexOf("<viewid>") + 14, guid);

                if (objectGuid != null)
                {
                    grid = grid.Remove(grid.IndexOf("<oId>") + 11, 36);
                    grid = grid.Insert(grid.IndexOf("<oId>") + 11, objectGuid);
                }

                if (objectType != null)
                {
                    grid = grid.Remove(grid.IndexOf("<oType>") + 7, grid.IndexOf("</oType>") - (grid.IndexOf("<oType>") + 7));
                    grid = grid.Insert(grid.IndexOf("<oType>") + 7, objectType);
                }
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }

                return grid;
            }

            public static string CampaignGridForListViewRefresh(int pageNum, string guid, string pagingCookie, string objectGuid, string objectType)
            {
                string grid = @"&#x3C;grid&#x3E;&#x3C;sortColumns&#x3E;createdon&#x26;&#x23;x3A&#x3B;0&#x3C;/sortColumns&#x3E;&#x3C;pageNum&#x3E;1&#x3C;/pageNum&#x3E;&#x3C;recsPerPage&#x3E;50&#x3C;/recsPerPage&#x3E;&#x3C;dataProvider&#x3E;Microsoft.Crm.Application.Controls.AppGridDataProvider&#x3C;/dataProvider&#x3E;&#x3C;uiProvider&#x3E;Microsoft.Crm.Application.Controls.BulkOperationLogUIProvider&#x3C;/uiProvider&#x3E;&#x3C;cols/&#x3E;&#x3C;max&#x3E;-1&#x3C;/max&#x3E;&#x3C;refreshAsync&#x3E;False&#x3C;/refreshAsync&#x3E;&#x3C;pagingCookie/&#x3E;&#x3C;enableMultiSort&#x3E;true&#x3C;/enableMultiSort&#x3E;&#x3C;enablePagingWhenOnePage&#x3E;true&#x3C;/enablePagingWhenOnePage&#x3E;&#x3C;parameters&#x3E;&#x3C;autorefresh&#x3E;1&#x3C;/autorefresh&#x3E;&#x3C;viewid&#x3E;&#x26;&#x23;x7B&#x3B;83DC2389-0A4C-4249-A5D3-BA1AB401FF1C&#x26;&#x23;x7D&#x3B;&#x3C;/viewid&#x3E;&#x3C;viewtype&#x3E;1039&#x3C;/viewtype&#x3E;&#x3C;otc&#x3E;4406&#x3C;/otc&#x3E;&#x3C;queryapi&#x3E;BulkOperation.RetrieveMiniCampaignsForList&#x3C;/queryapi&#x3E;&#x3C;showjumpbar&#x3E;0&#x3C;/showjumpbar&#x3E;&#x3C;enablepaging&#x3E;1&#x3C;/enablepaging&#x3E;&#x3C;oId&#x3E;&#x26;&#x23;x7B&#x3B;dd1af052-2ff4-db11-8d7d-001143dcb18d&#x26;&#x23;x7D&#x3B;&#x3C;/oId&#x3E;&#x3C;oType&#x3E;4300&#x3C;/oType&#x3E;&#x3C;/parameters&#x3E;&#x3C;columns&#x3E;&#x3C;column width&#x3D;&#x22;250&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Subject&#x22; fieldname&#x3D;&#x22;subject&#x22; entityname&#x3D;&#x22;bulkoperation&#x22;&#x3E;subject&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;150&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Activity Type&#x22; fieldname&#x3D;&#x22;createdrecordtypecode&#x22; entityname&#x3D;&#x22;bulkoperation&#x22;&#x3E;createdrecordtypecode&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;No. of Successes&#x22; fieldname&#x3D;&#x22;successcount&#x22; entityname&#x3D;&#x22;bulkoperation&#x22;&#x3E;successcount&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;No. of Failures&#x22; fieldname&#x3D;&#x22;failurecount&#x22; entityname&#x3D;&#x22;bulkoperation&#x22;&#x3E;failurecount&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Status Reason&#x22; fieldname&#x3D;&#x22;statuscode&#x22; entityname&#x3D;&#x22;bulkoperation&#x22;&#x3E;statuscode&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Created On&#x22; fieldname&#x3D;&#x22;createdon&#x22; entityname&#x3D;&#x22;bulkoperation&#x22;&#x3E;createdon&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Created By&#x22; fieldname&#x3D;&#x22;ownerid&#x22; entityname&#x3D;&#x22;bulkoperation&#x22;&#x3E;ownerid&#x3C;/column&#x3E;&#x3C;/columns&#x3E;&#x3C;/grid&#x3E;";

                grid = HttpUtility.HtmlDecode(grid);
                grid = grid.Replace("<pagingCookie/>", "<pagingCookie></pagingCookie>");
                grid = grid.Remove(grid.IndexOf("<viewid>") + 14, 36);
                grid = grid.Insert(grid.IndexOf("<viewid>") + 14, guid);

                if (objectGuid != null)
                {
                    grid = grid.Remove(grid.IndexOf("<oId>") + 11, 36);
                    grid = grid.Insert(grid.IndexOf("<oId>") + 11, objectGuid);
                }
                if (objectType != null)
                {
                    grid = grid.Remove(grid.IndexOf("<oType>") + 7, grid.IndexOf("</oType>") - (grid.IndexOf("<oType>") + 7));
                    grid = grid.Insert(grid.IndexOf("<oType>") + 7, objectType);
                }
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }
                return grid;
            }

            public static string CampaignPageViewRefresh(int pageNum, string guid, string pagingCookie)
            {
                string grid = @"&#x3C;grid&#x3E;&#x3C;sortColumns&#x3E;name&#x26;&#x23;x3A&#x3B;1&#x3C;/sortColumns&#x3E;&#x3C;pageNum&#x3E;1&#x3C;/pageNum&#x3E;&#x3C;recsPerPage&#x3E;50&#x3C;/recsPerPage&#x3E;&#x3C;dataProvider&#x3E;Microsoft.Crm.Application.Controls.AppGridDataProvider&#x3C;/dataProvider&#x3E;&#x3C;uiProvider&#x3E;Microsoft.Crm.Application.Controls.AppGridUIProvider&#x3C;/uiProvider&#x3E;&#x3C;cols/&#x3E;&#x3C;max&#x3E;-1&#x3C;/max&#x3E;&#x3C;refreshAsync&#x3E;False&#x3C;/refreshAsync&#x3E;&#x3C;pagingCookie/&#x3E;&#x3C;enableMultiSort&#x3E;true&#x3C;/enableMultiSort&#x3E;&#x3C;enablePagingWhenOnePage&#x3E;true&#x3C;/enablePagingWhenOnePage&#x3E;&#x3C;parameters&#x3E;&#x3C;autorefresh&#x3E;1&#x3C;/autorefresh&#x3E;&#x3C;viewid&#x3E;&#x26;&#x23;x7B&#x3B;7F68A34B-26F1-430F-B3EF-7079F2925C69&#x26;&#x23;x7D&#x3B;&#x3C;/viewid&#x3E;&#x3C;viewtype&#x3E;1039&#x3C;/viewtype&#x3E;&#x3C;otc&#x3E;4400&#x3C;/otc&#x3E;&#x3C;preview&#x3E;1&#x3C;/preview&#x3E;&#x3C;/parameters&#x3E;&#x3C;columns&#x3E;&#x3C;column width&#x3D;&#x22;300&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Name&#x22; fieldname&#x3D;&#x22;name&#x22; entityname&#x3D;&#x22;campaign&#x22;&#x3E;name&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Template&#x22; fieldname&#x3D;&#x22;istemplate&#x22; entityname&#x3D;&#x22;campaign&#x22;&#x3E;istemplate&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Status Reason&#x22; fieldname&#x3D;&#x22;statuscode&#x22; entityname&#x3D;&#x22;campaign&#x22;&#x3E;statuscode&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Created On&#x22; fieldname&#x3D;&#x22;createdon&#x22; entityname&#x3D;&#x22;campaign&#x22;&#x3E;createdon&#x3C;/column&#x3E;&#x3C;/columns&#x3E;&#x3C;/grid&#x3E;";

                grid = HttpUtility.HtmlDecode(grid);
                grid = grid.Replace("<pagingCookie/>", "<pagingCookie></pagingCookie>");
                grid = grid.Remove(grid.IndexOf("<viewid>") + 14, 36);
                grid = grid.Insert(grid.IndexOf("<viewid>") + 14, guid);

                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }

                return grid;                                
            }

            public static string CampaignResponseGridForCampaignViewRefresh(int pageNum, string guid, string pagingCookie, string objectGuid, string objectType)
            {
                string grid = @"<grid><sortColumns>subject&#x3A;0</sortColumns><pageNum>1</pageNum><recsPerPage>50</recsPerPage><dataProvider>Microsoft.Crm.Application.Controls.AppGridDataProvider</dataProvider><uiProvider>Microsoft.Crm.Application.Controls.AppGridUIProvider</uiProvider><cols/><max>-1</max><refreshAsync>False</refreshAsync><pagingCookie></pagingCookie><enableMultiSort>true</enableMultiSort><enablePagingWhenOnePage>true</enablePagingWhenOnePage><parameters><autorefresh>1</autorefresh><viewid>&#x7B;VIEW_GUID&#x7D;</viewid><viewtype>1039</viewtype><otc>4401</otc><preview>1</preview><queryapi>Campaign.RetrieveCampaignResponsesForCampaign</queryapi><showjumpbar>0</showjumpbar><enablepaging>1</enablepaging><oId>&#x7B;OBJECT_GUID&#x7D;</oId><oType>OBJECT_TYPE</oType></parameters><columns><column width=""250"" isHidden=""false"" isMetadataBound=""true"" isSortable=""true"" label=""Subject"" fieldname=""subject"" entityname=""campaignresponse"">subject</column><column width=""150"" isHidden=""false"" isMetadataBound=""true"" isSortable=""false"" label=""Customer"" fieldname=""customer"" entityname=""campaignresponse"">customer</column><column width=""100"" isHidden=""false"" isMetadataBound=""true"" isSortable=""true"" label=""Received On"" fieldname=""receivedon"" entityname=""campaignresponse"">receivedon</column><column width=""100"" isHidden=""false"" isMetadataBound=""true"" isSortable=""true"" label=""Response Code"" fieldname=""responsecode"" entityname=""campaignresponse"">responsecode</column><column width=""100"" isHidden=""false"" isMetadataBound=""true"" isSortable=""true"" label=""Status Reason"" fieldname=""statuscode"" entityname=""campaignresponse"">statuscode</column></columns></grid>";
                grid = grid.Replace("VIEW_GUID", guid);
                if (objectGuid != null)
                {
                    grid = grid.Replace("OBJECT_GUID", objectGuid);
                }
                if (objectType != null)
                {
                    grid = grid.Replace("OBJECT_TYPE", objectType);
                }
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }

                return grid;
            }

            public static string ContactPageView(int pageNum, string guid, string pagingCookie)
            {
                string grid = @"<grid><sortColumns>fullname&#x3A;1</sortColumns><pageNum>1</pageNum><recsPerPage>50</recsPerPage><dataProvider>Microsoft.Crm.Application.Controls.AppGridDataProvider</dataProvider><uiProvider>Microsoft.Crm.Application.Controls.AppGridUIProvider</uiProvider><cols/><max>-1</max><refreshAsync>False</refreshAsync><pagingCookie></pagingCookie><enableMultiSort>true</enableMultiSort><enablePagingWhenOnePage>true</enablePagingWhenOnePage><parameters><autorefresh>1</autorefresh><viewid>&#x7B;VIEW_GUID&#x7D;</viewid><viewtype>1039</viewtype><otc>2</otc><preview>1</preview><quickfind/><filter/><filterDisplay/></parameters></grid>";
                grid = grid.Replace("VIEW_GUID", guid);
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }

                return grid;
            }

            public static string ContactPageViewRefresh(int pageNum, string guid, string pagingCookie)
            {                
                string grid = @"&#x3C;grid&#x3E;&#x3C;sortColumns&#x3E;fullname&#x26;&#x23;x3A&#x3B;1&#x3C;/sortColumns&#x3E;&#x3C;pageNum&#x3E;1&#x3C;/pageNum&#x3E;&#x3C;recsPerPage&#x3E;50&#x3C;/recsPerPage&#x3E;&#x3C;dataProvider&#x3E;Microsoft.Crm.Application.Controls.AppGridDataProvider&#x3C;/dataProvider&#x3E;&#x3C;uiProvider&#x3E;Microsoft.Crm.Application.Controls.AppGridUIProvider&#x3C;/uiProvider&#x3E;&#x3C;cols/&#x3E;&#x3C;max&#x3E;-1&#x3C;/max&#x3E;&#x3C;refreshAsync&#x3E;False&#x3C;/refreshAsync&#x3E;&#x3C;pagingCookie/&#x3E;&#x3C;enableMultiSort&#x3E;true&#x3C;/enableMultiSort&#x3E;&#x3C;enablePagingWhenOnePage&#x3E;true&#x3C;/enablePagingWhenOnePage&#x3E;&#x3C;parameters&#x3E;&#x3C;autorefresh&#x3E;1&#x3C;/autorefresh&#x3E;&#x3C;viewid&#x3E;&#x26;&#x23;x7B&#x3B;00000000-0000-0000-00AA-000010001003&#x26;&#x23;x7D&#x3B;&#x3C;/viewid&#x3E;&#x3C;viewtype&#x3E;1039&#x3C;/viewtype&#x3E;&#x3C;otc&#x3E;2&#x3C;/otc&#x3E;&#x3C;preview&#x3E;1&#x3C;/preview&#x3E;&#x3C;/parameters&#x3E;&#x3C;columns&#x3E;&#x3C;column width&#x3D;&#x22;300&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Full Name&#x22; fieldname&#x3D;&#x22;fullname&#x22; entityname&#x3D;&#x22;contact&#x22;&#x3E;fullname&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;150&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Parent Customer&#x22; fieldname&#x3D;&#x22;parentcustomerid&#x22; entityname&#x3D;&#x22;contact&#x22;&#x3E;parentcustomerid&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;125&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Business Phone&#x22; fieldname&#x3D;&#x22;telephone1&#x22; entityname&#x3D;&#x22;contact&#x22;&#x3E;telephone1&#x3C;/column&#x3E;&#x3C;/columns&#x3E;&#x3C;/grid&#x3E;";
                
                grid = HttpUtility.HtmlDecode(grid);
                grid = grid.Replace("<pagingCookie/>", "<pagingCookie></pagingCookie>");
                grid = grid.Remove(grid.IndexOf("<viewid>") + 14, 36);  //36 is length of a guid string
                grid = grid.Insert(grid.IndexOf("<viewid>") + 14, guid); 
                
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }

                return grid;                
            }

            public static string ContactPageSearchViewRefresh(int pageNum, string guid, string random, string pagingCookie)
            {
                string grid = @"&#x3C;grid&#x3E;&#x3C;sortColumns&#x3E;fullname&#x26;&#x23;x3A&#x3B;1&#x3C;/sortColumns&#x3E;&#x3C;pageNum&#x3E;1&#x3C;/pageNum&#x3E;&#x3C;recsPerPage&#x3E;50&#x3C;/recsPerPage&#x3E;&#x3C;dataProvider&#x3E;Microsoft.Crm.Application.Controls.AppGridDataProvider&#x3C;/dataProvider&#x3E;&#x3C;uiProvider&#x3E;Microsoft.Crm.Application.Controls.AppGridUIProvider&#x3C;/uiProvider&#x3E;&#x3C;cols/&#x3E;&#x3C;max&#x3E;-1&#x3C;/max&#x3E;&#x3C;refreshAsync&#x3E;False&#x3C;/refreshAsync&#x3E;&#x3C;pagingCookie/&#x3E;&#x3C;enableMultiSort&#x3E;true&#x3C;/enableMultiSort&#x3E;&#x3C;enablePagingWhenOnePage&#x3E;true&#x3C;/enablePagingWhenOnePage&#x3E;&#x3C;parameters&#x3E;&#x3C;autorefresh&#x3E;1&#x3C;/autorefresh&#x3E;&#x3C;viewid&#x3E;&#x26;&#x23;x7B&#x3B;8DF19B44-A073-40C3-9D6D-EE1355D8C4BA&#x26;&#x23;x7D&#x3B;&#x3C;/viewid&#x3E;&#x3C;viewtype&#x3E;1039&#x3C;/viewtype&#x3E;&#x3C;otc&#x3E;2&#x3C;/otc&#x3E;&#x3C;preview&#x3E;1&#x26;&#x23;x2C&#x3B;1&#x3C;/preview&#x3E;&#x3C;quickfind&#x3E;a&#x3C;/quickfind&#x3E;&#x3C;filter/&#x3E;&#x3C;filterDisplay/&#x3E;&#x3C;/parameters&#x3E;&#x3C;columns&#x3E;&#x3C;column width&#x3D;&#x22;300&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Full Name&#x22; fieldname&#x3D;&#x22;fullname&#x22; entityname&#x3D;&#x22;contact&#x22;&#x3E;fullname&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;150&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Parent Customer&#x22; fieldname&#x3D;&#x22;parentcustomerid&#x22; entityname&#x3D;&#x22;contact&#x22;&#x3E;parentcustomerid&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Address 1&#x26;&#x23;x3A&#x3B; City&#x22; fieldname&#x3D;&#x22;address1_city&#x22; entityname&#x3D;&#x22;contact&#x22;&#x3E;address1_city&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;125&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Address 1&#x26;&#x23;x3A&#x3B; Phone&#x22; fieldname&#x3D;&#x22;address1_telephone1&#x22; entityname&#x3D;&#x22;contact&#x22;&#x3E;address1_telephone1&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;125&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Business Phone&#x22; fieldname&#x3D;&#x22;telephone1&#x22; entityname&#x3D;&#x22;contact&#x22;&#x3E;telephone1&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;200&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;E-mail&#x22; fieldname&#x3D;&#x22;emailaddress1&#x22; entityname&#x3D;&#x22;contact&#x22;&#x3E;emailaddress1&#x3C;/column&#x3E;&#x3C;/columns&#x3E;&#x3C;/grid&#x3E;";

                grid = HttpUtility.HtmlDecode(grid);
                grid = grid.Replace("<pagingCookie/>", "<pagingCookie></pagingCookie>");
                grid = grid.Remove(grid.IndexOf("<viewid>") + 14, 36);
                grid = grid.Insert(grid.IndexOf("<viewid>") + 14, guid);

                grid = grid.Remove(grid.IndexOf("<quickfind>") + 11, grid.IndexOf("</quickfind>") - (grid.IndexOf("<quickfind>") + 11));
                grid = grid.Insert(grid.IndexOf("<quickfind>") + 11, random);
                
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }

                return grid;
            }

            public static string ContactSearchView(int pageNum, string guid, string random, string pagingCookie)
            {
                string grid = @"<grid><sortColumns>fullname&#x3A;1</sortColumns><pageNum>1</pageNum><recsPerPage>50</recsPerPage><dataProvider>Microsoft.Crm.Application.Controls.AppGridDataProvider</dataProvider><uiProvider>Microsoft.Crm.Application.Controls.AppGridUIProvider</uiProvider><cols/><max>-1</max><refreshAsync>False</refreshAsync><pagingCookie></pagingCookie><enableMultiSort>true</enableMultiSort><enablePagingWhenOnePage>true</enablePagingWhenOnePage><parameters><autorefresh>1</autorefresh><viewid>&#x7B;VIEW_GUID&#x7D;</viewid><viewtype>1039</viewtype><otc>2</otc><preview>1&#x2C;1</preview><quickfind>RANDOM_STRING</quickfind><filter/><filterDisplay/></parameters></grid>";
                grid = grid.Replace("VIEW_GUID", guid);
                grid = grid.Replace("RANDOM_STRING", random);
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }

                return grid;
            }

            public static string EmailMyDraftedView(int pageNum, string guid, string pagingCookie)
            {
                string grid = @"<grid><sortColumns>actualend&#x3A;0</sortColumns><pageNum>1</pageNum><recsPerPage>50</recsPerPage><dataProvider>Microsoft.Crm.Application.Controls.ActivitiesGridDataProvider</dataProvider><uiProvider>Microsoft.Crm.Application.Controls.AppGridUIProvider</uiProvider><cols/><max>-1</max><refreshAsync>False</refreshAsync><pagingCookie></pagingCookie><enableMultiSort>true</enableMultiSort><enablePagingWhenOnePage>true</enablePagingWhenOnePage><parameters><autorefresh>1</autorefresh><viewid>&#x7B;VIEW_GUID&#x7D;</viewid><viewtype>1039</viewtype><otc>4202</otc><preview>1&#x2C;1</preview><quickfind/><filter/><filterDisplay/></parameters></grid>";
                grid = grid.Replace("VIEW_GUID", guid);
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }

                return grid;
            }

            public static string EmailMyReceivedView(int pageNum, string guid, string pagingCookie)
            {
                string grid = @"<grid><sortColumns>scheduledend&#x3A;1</sortColumns><pageNum>1</pageNum><recsPerPage>50</recsPerPage><dataProvider>Microsoft.Crm.Application.Controls.ActivitiesGridDataProvider</dataProvider><uiProvider>Microsoft.Crm.Application.Controls.AppGridUIProvider</uiProvider><cols/><max>-1</max><refreshAsync>False</refreshAsync><pagingCookie></pagingCookie><enableMultiSort>true</enableMultiSort><enablePagingWhenOnePage>true</enablePagingWhenOnePage><parameters><autorefresh>1</autorefresh><viewid>&#x7B;VIEW_GUID&#x7D;</viewid><viewtype>1039</viewtype><otc>4200</otc><preview>1</preview><quickfind/><filter/><filterDisplay/></parameters></grid>";
                grid = grid.Replace("VIEW_GUID", guid);
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }

                return grid;
            }

            public static string EmailPageViewRefresh(int pageNum, string guid, string pagingCookie)
            {
                string grid = @"&#x3C;grid&#x3E;&#x3C;sortColumns&#x3E;modifiedon&#x26;&#x23;x3A&#x3B;1&#x3C;/sortColumns&#x3E;&#x3C;pageNum&#x3E;1&#x3C;/pageNum&#x3E;&#x3C;recsPerPage&#x3E;50&#x3C;/recsPerPage&#x3E;&#x3C;dataProvider&#x3E;Microsoft.Crm.Application.Controls.ActivitiesGridDataProvider&#x3C;/dataProvider&#x3E;&#x3C;uiProvider&#x3E;Microsoft.Crm.Application.Controls.AppGridUIProvider&#x3C;/uiProvider&#x3E;&#x3C;cols/&#x3E;&#x3C;max&#x3E;-1&#x3C;/max&#x3E;&#x3C;refreshAsync&#x3E;False&#x3C;/refreshAsync&#x3E;&#x3C;pagingCookie/&#x3E;&#x3C;enableMultiSort&#x3E;true&#x3C;/enableMultiSort&#x3E;&#x3C;enablePagingWhenOnePage&#x3E;true&#x3C;/enablePagingWhenOnePage&#x3E;&#x3C;parameters&#x3E;&#x3C;autorefresh&#x3E;1&#x3C;/autorefresh&#x3E;&#x3C;viewid&#x3E;&#x26;&#x23;x7B&#x3B;6E8B4B8C-30F7-44EF-A958-BE72BAABC349&#x26;&#x23;x7D&#x3B;&#x3C;/viewid&#x3E;&#x3C;viewtype&#x3E;1039&#x3C;/viewtype&#x3E;&#x3C;otc&#x3E;4202&#x3C;/otc&#x3E;&#x3C;preview&#x3E;1&#x26;&#x23;x2C&#x3B;1&#x26;&#x23;x2C&#x3B;1&#x3C;/preview&#x3E;&#x3C;quickfind/&#x3E;&#x3C;filter/&#x3E;&#x3C;filterDisplay/&#x3E;&#x3C;/parameters&#x3E;&#x3C;columns&#x3E;&#x3C;column width&#x3D;&#x22;250&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Subject&#x22; fieldname&#x3D;&#x22;subject&#x22; entityname&#x3D;&#x22;email&#x22;&#x3E;subject&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Regarding&#x22; fieldname&#x3D;&#x22;regardingobjectid&#x22; entityname&#x3D;&#x22;email&#x22;&#x3E;regardingobjectid&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Modified On&#x22; fieldname&#x3D;&#x22;modifiedon&#x22; entityname&#x3D;&#x22;email&#x22;&#x3E;modifiedon&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;150&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;false&#x22; label&#x3D;&#x22;To&#x22; fieldname&#x3D;&#x22;to&#x22; entityname&#x3D;&#x22;email&#x22;&#x3E;to&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Priority&#x22; fieldname&#x3D;&#x22;prioritycode&#x22; entityname&#x3D;&#x22;email&#x22;&#x3E;prioritycode&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Due Date&#x22; fieldname&#x3D;&#x22;scheduledend&#x22; entityname&#x3D;&#x22;email&#x22;&#x3E;scheduledend&#x3C;/column&#x3E;&#x3C;/columns&#x3E;&#x3C;/grid&#x3E;";

                grid = HttpUtility.HtmlDecode(grid);
                grid = grid.Replace("<pagingCookie/>", "<pagingCookie></pagingCookie>");
                grid = grid.Remove(grid.IndexOf("<viewid>") + 14, 36);
                grid = grid.Insert(grid.IndexOf("<viewid>") + 14, guid);
                
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }

                return grid;
            }

            public static string EntityPageView(int pageNum, string guid, string pagingCookie)
            {
                string grid = @"<grid><sortColumns>name&#x3A;1</sortColumns><pageNum>1</pageNum><recsPerPage>50</recsPerPage><dataProvider>Microsoft.Crm.Application.Controls.SystemCustomization.EntityGridDataProvider</dataProvider><uiProvider>Microsoft.Crm.Application.Controls.SystemCustomization.SysCustGridUIProvider</uiProvider><cols/><max>-1</max><refreshAsync>False</refreshAsync><pagingCookie></pagingCookie><enableMultiSort>true</enableMultiSort><enablePagingWhenOnePage>true</enablePagingWhenOnePage><parameters><autorefresh>1</autorefresh><viewid>&#x7B;VIEW_GUID&#x7D;</viewid><viewtype>1039</viewtype><otc>0</otc><showjumpbar>0</showjumpbar><enablepaging>0</enablepaging><entityFilter>CustomizableAndMappable</entityFilter><exportFilter>All</exportFilter><disableDblClick>0</disableDblClick></parameters><columns><column width=""150"" isHidden=""false"" isMetadataBound=""false"" isSortable=""true"" label=""Name"" fieldname=""name"" entityname="""">name</column><column width=""150"" isHidden=""false"" isMetadataBound=""false"" isSortable=""true"" label=""Schema Name"" fieldname=""schemaname"" entityname="""">schemaname</column><column width=""100"" isHidden=""false"" isMetadataBound=""false"" isSortable=""true"" label=""Type"" fieldname=""type"" entityname="""">type</column><column width=""350"" isHidden=""false"" isMetadataBound=""false"" isSortable=""true"" label=""Description"" fieldname=""description"" entityname="""">description</column></columns></grid>";
                grid = grid.Replace("VIEW_GUID", guid);
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }

                return grid;
            }

            public static string FaxPageView(int pageNum, string guid, string pagingCookie)
            {
                string grid = @"<grid><sortColumns>scheduledend&#x3A;1</sortColumns><pageNum>1</pageNum><recsPerPage>50</recsPerPage><dataProvider>Microsoft.Crm.Application.Controls.ActivitiesGridDataProvider</dataProvider><uiProvider>Microsoft.Crm.Application.Controls.AppGridUIProvider</uiProvider><cols/><max>-1</max><refreshAsync>False</refreshAsync><pagingCookie></pagingCookie><enableMultiSort>true</enableMultiSort><enablePagingWhenOnePage>true</enablePagingWhenOnePage><parameters><autorefresh>1</autorefresh><viewid>&#x7B;VIEW_GUID&#x7D;</viewid><viewtype>1039</viewtype><otc>4200</otc><preview>1</preview><quickfind/><filter/><filterDisplay/></parameters></grid>";
                grid = grid.Replace("VIEW_GUID", guid);
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }

                return grid;
            }

            public static string FaxPageViewRefresh(int pageNum, string guid, string pagingCookie)
            {
                string grid = @"&#x3C;grid&#x3E;&#x3C;sortColumns&#x3E;createdon&#x26;&#x23;x3A&#x3B;0&#x3C;/sortColumns&#x3E;&#x3C;pageNum&#x3E;1&#x3C;/pageNum&#x3E;&#x3C;recsPerPage&#x3E;50&#x3C;/recsPerPage&#x3E;&#x3C;dataProvider&#x3E;Microsoft.Crm.Application.Controls.ActivitiesGridDataProvider&#x3C;/dataProvider&#x3E;&#x3C;uiProvider&#x3E;Microsoft.Crm.Application.Controls.AppGridUIProvider&#x3C;/uiProvider&#x3E;&#x3C;cols/&#x3E;&#x3C;max&#x3E;-1&#x3C;/max&#x3E;&#x3C;refreshAsync&#x3E;False&#x3C;/refreshAsync&#x3E;&#x3C;pagingCookie/&#x3E;&#x3C;enableMultiSort&#x3E;true&#x3C;/enableMultiSort&#x3E;&#x3C;enablePagingWhenOnePage&#x3E;true&#x3C;/enablePagingWhenOnePage&#x3E;&#x3C;parameters&#x3E;&#x3C;autorefresh&#x3E;1&#x3C;/autorefresh&#x3E;&#x3C;viewid&#x3E;&#x26;&#x23;x7B&#x3B;616D0480-0A0D-4753-8617-CAD4A04101AA&#x26;&#x23;x7D&#x3B;&#x3C;/viewid&#x3E;&#x3C;viewtype&#x3E;1039&#x3C;/viewtype&#x3E;&#x3C;otc&#x3E;4204&#x3C;/otc&#x3E;&#x3C;preview&#x3E;1&#x26;&#x23;x2C&#x3B;1&#x3C;/preview&#x3E;&#x3C;quickfind/&#x3E;&#x3C;filter/&#x3E;&#x3C;filterDisplay/&#x3E;&#x3C;/parameters&#x3E;&#x3C;columns&#x3E;&#x3C;column width&#x3D;&#x22;250&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Subject&#x22; fieldname&#x3D;&#x22;subject&#x22; entityname&#x3D;&#x22;fax&#x22;&#x3E;subject&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;150&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;false&#x22; label&#x3D;&#x22;Sender&#x22; fieldname&#x3D;&#x22;from&#x22; entityname&#x3D;&#x22;fax&#x22;&#x3E;from&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;150&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;false&#x22; label&#x3D;&#x22;Recipient&#x22; fieldname&#x3D;&#x22;to&#x22; entityname&#x3D;&#x22;fax&#x22;&#x3E;to&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Regarding&#x22; fieldname&#x3D;&#x22;regardingobjectid&#x22; entityname&#x3D;&#x22;fax&#x22;&#x3E;regardingobjectid&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Activity Status&#x22; fieldname&#x3D;&#x22;statecode&#x22; entityname&#x3D;&#x22;fax&#x22;&#x3E;statecode&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Created On&#x22; fieldname&#x3D;&#x22;createdon&#x22; entityname&#x3D;&#x22;fax&#x22;&#x3E;createdon&#x3C;/column&#x3E;&#x3C;/columns&#x3E;&#x3C;/grid&#x3E;";

                grid = HttpUtility.HtmlDecode(grid);
                grid = grid.Replace("<pagingCookie/>", "<pagingCookie></pagingCookie>");
                grid = grid.Remove(grid.IndexOf("<viewid>") + 14, 36);
                grid = grid.Insert(grid.IndexOf("<viewid>") + 14, guid);
                
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }

                return grid;
            }

            public static string IncidentPageView(int pageNum, string guid, string pagingCookie)
    		{
    			string grid = @"<grid><sortColumns>title&#x3A;1</sortColumns><pageNum>1</pageNum><recsPerPage>50</recsPerPage><dataProvider>Microsoft.Crm.Application.Controls.AppGridDataProvider</dataProvider><uiProvider>Microsoft.Crm.Application.Controls.AppGridUIProvider</uiProvider><cols/><max>-1</max><refreshAsync>False</refreshAsync><pagingCookie></pagingCookie><enableMultiSort>true</enableMultiSort><enablePagingWhenOnePage>true</enablePagingWhenOnePage><parameters><autorefresh>1</autorefresh><viewid>&#x7B;VIEW_GUID&#x7D;</viewid><viewtype>1039</viewtype><otc>112</otc><preview>1</preview><quickfind/><filter/><filterDisplay/></parameters></grid>";
    			grid = grid.Replace("VIEW_GUID", guid);
    			if (pagingCookie != null)
    			{
    				grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
    			}
    			if (pageNum != -1)
    			{
    				grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
    			}

                return grid;
    		}
    
    		public static string IncidentPageViewRefresh(int pageNum, string guid, string pagingCookie)
    		{
                string grid = @"&#x3C;grid&#x3E;&#x3C;sortColumns&#x3E;title&#x26;&#x23;x3A&#x3B;1&#x3C;/sortColumns&#x3E;&#x3C;pageNum&#x3E;1&#x3C;/pageNum&#x3E;&#x3C;recsPerPage&#x3E;50&#x3C;/recsPerPage&#x3E;&#x3C;dataProvider&#x3E;Microsoft.Crm.Application.Controls.AppGridDataProvider&#x3C;/dataProvider&#x3E;&#x3C;uiProvider&#x3E;Microsoft.Crm.Application.Controls.AppGridUIProvider&#x3C;/uiProvider&#x3E;&#x3C;cols/&#x3E;&#x3C;max&#x3E;-1&#x3C;/max&#x3E;&#x3C;refreshAsync&#x3E;False&#x3C;/refreshAsync&#x3E;&#x3C;pagingCookie/&#x3E;&#x3C;enableMultiSort&#x3E;true&#x3C;/enableMultiSort&#x3E;&#x3C;enablePagingWhenOnePage&#x3E;true&#x3C;/enablePagingWhenOnePage&#x3E;&#x3C;parameters&#x3E;&#x3C;autorefresh&#x3E;1&#x3C;/autorefresh&#x3E;&#x3C;viewid&#x3E;&#x26;&#x23;x7B&#x3B;00000000-0000-0000-00AA-000010001028&#x26;&#x23;x7D&#x3B;&#x3C;/viewid&#x3E;&#x3C;viewtype&#x3E;1039&#x3C;/viewtype&#x3E;&#x3C;otc&#x3E;112&#x3C;/otc&#x3E;&#x3C;preview&#x3E;1&#x3C;/preview&#x3E;&#x3C;/parameters&#x3E;&#x3C;columns&#x3E;&#x3C;column width&#x3D;&#x22;300&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Title&#x22; fieldname&#x3D;&#x22;title&#x22; entityname&#x3D;&#x22;incident&#x22;&#x3E;title&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;150&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Case Number&#x22; fieldname&#x3D;&#x22;ticketnumber&#x22; entityname&#x3D;&#x22;incident&#x22;&#x3E;ticketnumber&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;150&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Priority&#x22; fieldname&#x3D;&#x22;prioritycode&#x22; entityname&#x3D;&#x22;incident&#x22;&#x3E;prioritycode&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;150&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Created On&#x22; fieldname&#x3D;&#x22;createdon&#x22; entityname&#x3D;&#x22;incident&#x22;&#x3E;createdon&#x3C;/column&#x3E;&#x3C;/columns&#x3E;&#x3C;/grid&#x3E;";

                grid = HttpUtility.HtmlDecode(grid);
                grid = grid.Replace("<pagingCookie/>", "<pagingCookie></pagingCookie>");
                grid = grid.Remove(grid.IndexOf("<viewid>") + 14, 36);
                grid = grid.Insert(grid.IndexOf("<viewid>") + 14, guid);
                
                if (pagingCookie != null)
    			{
    				grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
    			}
    			if (pageNum != -1)
    			{
    				grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
    			}

                return grid;
    		}

            public static string IncidentPageSearchViewRefresh(int pageNum, string guid, string random, string pagingCookie)
    		{
                string grid = @"<gridXml>&#60;grid&#62;&#60;sortColumns&#62;title&#38;&#35;58&#59;1&#60;&#47;sortColumns&#62;&#60;pageNum&#62;1&#60;&#47;pageNum&#62;&#60;recsPerPage&#62;50&#60;&#47;recsPerPage&#62;&#60;dataProvider&#62;Microsoft.Crm.Application.Controls.AppGridDataProvider&#60;&#47;dataProvider&#62;&#60;uiProvider&#62;Microsoft.Crm.Application.Controls.AppGridUIProvider&#60;&#47;uiProvider&#62;&#60;cols&#47;&#62;&#60;max&#62;-1&#60;&#47;max&#62;&#60;refreshAsync&#62;False&#60;&#47;refreshAsync&#62;&#60;pagingCookie&#47;&#62;&#60;enableMultiSort&#62;true&#60;&#47;enableMultiSort&#62;&#60;enablePagingWhenOnePage&#62;true&#60;&#47;enablePagingWhenOnePage&#62;&#60;parameters&#62;&#60;autorefresh&#62;1&#60;&#47;autorefresh&#62;&#60;viewid&#62;&#38;&#35;123&#59;D95D47EF-4A1D-479E-9065-B9CEA0089048&#38;&#35;125&#59;&#60;&#47;viewid&#62;&#60;viewtype&#62;1039&#60;&#47;viewtype&#62;&#60;otc&#62;112&#60;&#47;otc&#62;&#60;preview&#62;1,1,1&#60;&#47;preview&#62;&#60;quickfind&#62;co&#60;&#47;quickfind&#62;&#60;filter&#47;&#62;&#60;filterDisplay&#47;&#62;&#60;&#47;parameters&#62;&#60;columns&#62;&#60;column width&#61;&#34;300&#34; isHidden&#61;&#34;false&#34; isMetadataBound&#61;&#34;true&#34; isSortable&#61;&#34;true&#34; label&#61;&#34;Title&#34; fieldname&#61;&#34;title&#34; entityname&#61;&#34;incident&#34;&#62;title&#60;&#47;column&#62;&#60;column width&#61;&#34;150&#34; isHidden&#61;&#34;false&#34; isMetadataBound&#61;&#34;true&#34; isSortable&#61;&#34;true&#34; label&#61;&#34;Case Number&#34; fieldname&#61;&#34;ticketnumber&#34; entityname&#61;&#34;incident&#34;&#62;ticketnumber&#60;&#47;column&#62;&#60;column width&#61;&#34;150&#34; isHidden&#61;&#34;false&#34; isMetadataBound&#61;&#34;true&#34; isSortable&#61;&#34;true&#34; label&#61;&#34;Priority&#34; fieldname&#61;&#34;prioritycode&#34; entityname&#61;&#34;incident&#34;&#62;prioritycode&#60;&#47;column&#62;&#60;column width&#61;&#34;150&#34; isHidden&#61;&#34;false&#34; isMetadataBound&#61;&#34;true&#34; isSortable&#61;&#34;true&#34; label&#61;&#34;Created On&#34; fieldname&#61;&#34;createdon&#34; entityname&#61;&#34;incident&#34;&#62;createdon&#60;&#47;column&#62;&#60;&#47;columns&#62;&#60;&#47;grid&#62;</gridXml>";
                
                grid = HttpUtility.HtmlDecode(grid);
                grid = grid.Replace("<pagingCookie/>", "<pagingCookie></pagingCookie>");
                grid = grid.Remove(grid.IndexOf("<viewid>") + 14, 36);
                grid = grid.Insert(grid.IndexOf("<viewid>") + 14, guid);

                grid = grid.Remove(grid.IndexOf("<quickfind>") + 11, grid.IndexOf("</quickfind>") - (grid.IndexOf("<quickfind>") + 11));
                grid = grid.Insert(grid.IndexOf("<quickfind>") + 11, random);

    			if (pagingCookie != null)
    			{
    				grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
    			}
    			if (pageNum != -1)
    			{
    				grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
    			}

                return grid;
    		}

            public static string IncidentSearchView(int pageNum, string guid, string random, string pagingCookie)
    		{
    			string grid = @"<grid><sortColumns>title&#x3A;1</sortColumns><pageNum>1</pageNum><recsPerPage>50</recsPerPage><dataProvider>Microsoft.Crm.Application.Controls.AppGridDataProvider</dataProvider><uiProvider>Microsoft.Crm.Application.Controls.AppGridUIProvider</uiProvider><cols/><max>-1</max><refreshAsync>False</refreshAsync><pagingCookie></pagingCookie><enableMultiSort>true</enableMultiSort><enablePagingWhenOnePage>true</enablePagingWhenOnePage><parameters><autorefresh>1</autorefresh><viewid>&#x7B;VIEW_GUID&#x7D;</viewid><viewtype>1039</viewtype><otc>112</otc><preview>1&#x2C;1</preview><quickfind>RANDOM_STRING</quickfind><filter/><filterDisplay/></parameters></grid>";	
    			grid = grid.Replace("VIEW_GUID", guid);
                grid = grid.Replace("RANDOM_STRING", random);
    			if (pagingCookie != null)
    			{
    				grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
    			}
    			if (pageNum != -1)
    			{
    				grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
    			}

                return grid;
    		}

            public static string LeadPageView(int pageNum, string guid, string pagingCookie)
            {
                string grid = @"<grid><sortColumns>createdon&#x3A;0</sortColumns><pageNum>1</pageNum><recsPerPage>50</recsPerPage><dataProvider>Microsoft.Crm.Application.Controls.AppGridDataProvider</dataProvider><uiProvider>Microsoft.Crm.Application.Controls.AppGridUIProvider</uiProvider><cols/><max>-1</max><refreshAsync>False</refreshAsync><pagingCookie></pagingCookie><enableMultiSort>true</enableMultiSort><enablePagingWhenOnePage>true</enablePagingWhenOnePage><parameters><autorefresh>1</autorefresh><viewid>&#x7B;VIEW_GUID&#x7D;</viewid><viewtype>1039</viewtype><otc>4</otc><preview>1</preview><quickfind/><filter/><filterDisplay/></parameters></grid>";
                grid = grid.Replace("VIEW_GUID", guid);
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }
                return grid;
            }

            public static string LeadPageViewRefresh(int pageNum, string guid, string pagingCookie)
            {
                string grid = @"&#x3C;grid&#x3E;&#x3C;sortColumns&#x3E;createdon&#x26;&#x23;x3A&#x3B;0&#x3C;/sortColumns&#x3E;&#x3C;pageNum&#x3E;1&#x3C;/pageNum&#x3E;&#x3C;recsPerPage&#x3E;50&#x3C;/recsPerPage&#x3E;&#x3C;dataProvider&#x3E;Microsoft.Crm.Application.Controls.AppGridDataProvider&#x3C;/dataProvider&#x3E;&#x3C;uiProvider&#x3E;Microsoft.Crm.Application.Controls.AppGridUIProvider&#x3C;/uiProvider&#x3E;&#x3C;cols/&#x3E;&#x3C;max&#x3E;-1&#x3C;/max&#x3E;&#x3C;refreshAsync&#x3E;False&#x3C;/refreshAsync&#x3E;&#x3C;pagingCookie/&#x3E;&#x3C;enableMultiSort&#x3E;true&#x3C;/enableMultiSort&#x3E;&#x3C;enablePagingWhenOnePage&#x3E;true&#x3C;/enablePagingWhenOnePage&#x3E;&#x3C;parameters&#x3E;&#x3C;autorefresh&#x3E;1&#x3C;/autorefresh&#x3E;&#x3C;viewid&#x3E;&#x26;&#x23;x7B&#x3B;00000000-0000-0000-00AA-000010001005&#x26;&#x23;x7D&#x3B;&#x3C;/viewid&#x3E;&#x3C;viewtype&#x3E;1039&#x3C;/viewtype&#x3E;&#x3C;otc&#x3E;4&#x3C;/otc&#x3E;&#x3C;preview&#x3E;1&#x3C;/preview&#x3E;&#x3C;/parameters&#x3E;&#x3C;columns&#x3E;&#x3C;column width&#x3D;&#x22;300&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Name&#x22; fieldname&#x3D;&#x22;fullname&#x22; entityname&#x3D;&#x22;lead&#x22;&#x3E;fullname&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;200&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Topic&#x22; fieldname&#x3D;&#x22;subject&#x22; entityname&#x3D;&#x22;lead&#x22;&#x3E;subject&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Status Reason&#x22; fieldname&#x3D;&#x22;statuscode&#x22; entityname&#x3D;&#x22;lead&#x22;&#x3E;statuscode&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;125&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Created On&#x22; fieldname&#x3D;&#x22;createdon&#x22; entityname&#x3D;&#x22;lead&#x22;&#x3E;createdon&#x3C;/column&#x3E;&#x3C;/columns&#x3E;&#x3C;/grid&#x3E;";

                grid = HttpUtility.HtmlDecode(grid);
                grid = grid.Replace("<pagingCookie/>", "<pagingCookie></pagingCookie>");
                grid = grid.Remove(grid.IndexOf("<viewid>") + 14, 36);
                grid = grid.Insert(grid.IndexOf("<viewid>") + 14, guid);

                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }

                return grid;
            }

            public static string LeadPageSearchViewRefresh(int pageNum, string guid, string random, string pagingCookie)
            {
                string grid = @"&#x3C;grid&#x3E;&#x3C;sortColumns&#x3E;subject&#x26;&#x23;x3A&#x3B;1&#x3C;/sortColumns&#x3E;&#x3C;pageNum&#x3E;1&#x3C;/pageNum&#x3E;&#x3C;recsPerPage&#x3E;50&#x3C;/recsPerPage&#x3E;&#x3C;dataProvider&#x3E;Microsoft.Crm.Application.Controls.AppGridDataProvider&#x3C;/dataProvider&#x3E;&#x3C;uiProvider&#x3E;Microsoft.Crm.Application.Controls.AppGridUIProvider&#x3C;/uiProvider&#x3E;&#x3C;cols/&#x3E;&#x3C;max&#x3E;-1&#x3C;/max&#x3E;&#x3C;refreshAsync&#x3E;False&#x3C;/refreshAsync&#x3E;&#x3C;pagingCookie/&#x3E;&#x3C;enableMultiSort&#x3E;true&#x3C;/enableMultiSort&#x3E;&#x3C;enablePagingWhenOnePage&#x3E;true&#x3C;/enablePagingWhenOnePage&#x3E;&#x3C;parameters&#x3E;&#x3C;autorefresh&#x3E;1&#x3C;/autorefresh&#x3E;&#x3C;viewid&#x3E;&#x26;&#x23;x7B&#x3B;F1A7C4D9-F439-46DA-8B69-ECC133214695&#x26;&#x23;x7D&#x3B;&#x3C;/viewid&#x3E;&#x3C;viewtype&#x3E;1039&#x3C;/viewtype&#x3E;&#x3C;otc&#x3E;4&#x3C;/otc&#x3E;&#x3C;preview&#x3E;1&#x26;&#x23;x2C&#x3B;1&#x3C;/preview&#x3E;&#x3C;quickfind&#x3E;a&#x3C;/quickfind&#x3E;&#x3C;filter/&#x3E;&#x3C;filterDisplay/&#x3E;&#x3C;/parameters&#x3E;&#x3C;columns&#x3E;&#x3C;column width&#x3D;&#x22;200&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Topic&#x22; fieldname&#x3D;&#x22;subject&#x22; entityname&#x3D;&#x22;lead&#x22;&#x3E;subject&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;300&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Name&#x22; fieldname&#x3D;&#x22;fullname&#x22; entityname&#x3D;&#x22;lead&#x22;&#x3E;fullname&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Company Name&#x22; fieldname&#x3D;&#x22;companyname&#x22; entityname&#x3D;&#x22;lead&#x22;&#x3E;companyname&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Status Reason&#x22; fieldname&#x3D;&#x22;statuscode&#x22; entityname&#x3D;&#x22;lead&#x22;&#x3E;statuscode&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Business Phone&#x22; fieldname&#x3D;&#x22;telephone1&#x22; entityname&#x3D;&#x22;lead&#x22;&#x3E;telephone1&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;200&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;E-mail&#x22; fieldname&#x3D;&#x22;emailaddress1&#x22; entityname&#x3D;&#x22;lead&#x22;&#x3E;emailaddress1&#x3C;/column&#x3E;&#x3C;/columns&#x3E;&#x3C;/grid&#x3E;";

                grid = HttpUtility.HtmlDecode(grid);
                grid = grid.Replace("<pagingCookie/>", "<pagingCookie></pagingCookie>");
                grid = grid.Remove(grid.IndexOf("<viewid>") + 14, 36);
                grid = grid.Insert(grid.IndexOf("<viewid>") + 14, guid);

                grid = grid.Remove(grid.IndexOf("<quickfind>") + 11, grid.IndexOf("</quickfind>") - (grid.IndexOf("<quickfind>") + 11));
                grid = grid.Insert(grid.IndexOf("<quickfind>") + 11, random);
                
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }
                return grid;
            }

            public static string LeadSearchView(int pageNum, string guid, string random, string pagingCookie)
            {
                string grid = @"<grid><sortColumns>createdon&#x3A;0</sortColumns><pageNum>1</pageNum><recsPerPage>50</recsPerPage><dataProvider>Microsoft.Crm.Application.Controls.AppGridDataProvider</dataProvider><uiProvider>Microsoft.Crm.Application.Controls.AppGridUIProvider</uiProvider><cols/><max>-1</max><refreshAsync>False</refreshAsync><pagingCookie></pagingCookie><enableMultiSort>true</enableMultiSort><enablePagingWhenOnePage>true</enablePagingWhenOnePage><parameters><autorefresh>1</autorefresh><viewid>&#x7B;VIEW_GUID&#x7D;</viewid><viewtype>1039</viewtype><otc>4</otc><preview>1&#x2C;1</preview><quickfind>RANDOM_STRING</quickfind><filter/><filterDisplay/></parameters></grid>";
                grid = grid.Replace("VIEW_GUID", guid);
                grid = grid.Replace("RANDOM_STRING", random);
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }
                return grid;
            }

            public static string LetterPageView(int pageNum, string guid, string pagingCookie)
            {
                string grid = @"<grid><sortColumns>scheduledend&#x3A;1</sortColumns><pageNum>1</pageNum><recsPerPage>50</recsPerPage><dataProvider>Microsoft.Crm.Application.Controls.ActivitiesGridDataProvider</dataProvider><uiProvider>Microsoft.Crm.Application.Controls.AppGridUIProvider</uiProvider><cols/><max>-1</max><refreshAsync>False</refreshAsync><pagingCookie></pagingCookie><enableMultiSort>true</enableMultiSort><enablePagingWhenOnePage>true</enablePagingWhenOnePage><parameters><autorefresh>1</autorefresh><viewid>&#x7B;VIEW_GUID&#x7D;</viewid><viewtype>1039</viewtype><otc>4200</otc><preview>1</preview><quickfind/><filter/><filterDisplay/></parameters></grid>";
                grid = grid.Replace("VIEW_GUID", guid);
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }
                return grid;
            }

            public static string LetterPageViewRefresh(int pageNum, string guid, string pagingCookie)
            {
                string grid = @"<gridXml>&#60;grid&#62;&#60;sortColumns&#62;createdon&#38;&#35;58&#59;0&#60;&#47;sortColumns&#62;&#60;pageNum&#62;1&#60;&#47;pageNum&#62;&#60;recsPerPage&#62;50&#60;&#47;recsPerPage&#62;&#60;dataProvider&#62;Microsoft.Crm.Application.Controls.ActivitiesGridDataProvider&#60;&#47;dataProvider&#62;&#60;uiProvider&#62;Microsoft.Crm.Application.Controls.AppGridUIProvider&#60;&#47;uiProvider&#62;&#60;cols&#47;&#62;&#60;max&#62;-1&#60;&#47;max&#62;&#60;refreshAsync&#62;False&#60;&#47;refreshAsync&#62;&#60;pagingCookie&#47;&#62;&#60;enableMultiSort&#62;true&#60;&#47;enableMultiSort&#62;&#60;enablePagingWhenOnePage&#62;true&#60;&#47;enablePagingWhenOnePage&#62;&#60;parameters&#62;&#60;autorefresh&#62;1&#60;&#47;autorefresh&#62;&#60;viewid&#62;&#38;&#35;123&#59;ED36ABD8-7F54-4077-8B46-D6F9BF8E71DF&#38;&#35;125&#59;&#60;&#47;viewid&#62;&#60;viewtype&#62;1039&#60;&#47;viewtype&#62;&#60;otc&#62;4207&#60;&#47;otc&#62;&#60;preview&#62;1,1&#60;&#47;preview&#62;&#60;quickfind&#47;&#62;&#60;filter&#47;&#62;&#60;filterDisplay&#47;&#62;&#60;&#47;parameters&#62;&#60;columns&#62;&#60;column width&#61;&#34;250&#34; isHidden&#61;&#34;false&#34; isMetadataBound&#61;&#34;true&#34; isSortable&#61;&#34;true&#34; label&#61;&#34;Subject&#34; fieldname&#61;&#34;subject&#34; entityname&#61;&#34;letter&#34;&#62;subject&#60;&#47;column&#62;&#60;column width&#61;&#34;150&#34; isHidden&#61;&#34;false&#34; isMetadataBound&#61;&#34;true&#34; isSortable&#61;&#34;false&#34; label&#61;&#34;Sender&#34; fieldname&#61;&#34;from&#34; entityname&#61;&#34;letter&#34;&#62;from&#60;&#47;column&#62;&#60;column width&#61;&#34;150&#34; isHidden&#61;&#34;false&#34; isMetadataBound&#61;&#34;true&#34; isSortable&#61;&#34;false&#34; label&#61;&#34;Recipient&#34; fieldname&#61;&#34;to&#34; entityname&#61;&#34;letter&#34;&#62;to&#60;&#47;column&#62;&#60;column width&#61;&#34;100&#34; isHidden&#61;&#34;false&#34; isMetadataBound&#61;&#34;true&#34; isSortable&#61;&#34;true&#34; label&#61;&#34;Regarding&#34; fieldname&#61;&#34;regardingobjectid&#34; entityname&#61;&#34;letter&#34;&#62;regardingobjectid&#60;&#47;column&#62;&#60;column width&#61;&#34;100&#34; isHidden&#61;&#34;false&#34; isMetadataBound&#61;&#34;true&#34; isSortable&#61;&#34;true&#34; label&#61;&#34;Activity Status&#34; fieldname&#61;&#34;statecode&#34; entityname&#61;&#34;letter&#34;&#62;statecode&#60;&#47;column&#62;&#60;column width&#61;&#34;100&#34; isHidden&#61;&#34;false&#34; isMetadataBound&#61;&#34;true&#34; isSortable&#61;&#34;true&#34; label&#61;&#34;Created On&#34; fieldname&#61;&#34;createdon&#34; entityname&#61;&#34;letter&#34;&#62;createdon&#60;&#47;column&#62;&#60;&#47;columns&#62;&#60;&#47;grid&#62;</gridXml>";

                grid = HttpUtility.HtmlDecode(grid);
                grid = grid.Replace("<pagingCookie/>", "<pagingCookie></pagingCookie>");
                grid = grid.Remove(grid.IndexOf("<viewid>") + 14, 36);
                grid = grid.Insert(grid.IndexOf("<viewid>") + 14, guid);
                
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }
                return grid;
            }

            public static string ListGridForCampaignViewRefresh(int pageNum, string guid, string pagingCookie, string objectGuid, string objectType)
            {
                string grid = @"&#x3C;grid&#x3E;&#x3C;sortColumns&#x3E;listname&#x26;&#x23;x3A&#x3B;0&#x3C;/sortColumns&#x3E;&#x3C;pageNum&#x3E;1&#x3C;/pageNum&#x3E;&#x3C;recsPerPage&#x3E;50&#x3C;/recsPerPage&#x3E;&#x3C;dataProvider&#x3E;Microsoft.Crm.Application.Controls.AppGridDataProvider&#x3C;/dataProvider&#x3E;&#x3C;uiProvider&#x3E;Microsoft.Crm.Application.Controls.AppGridUIProvider&#x3C;/uiProvider&#x3E;&#x3C;cols/&#x3E;&#x3C;max&#x3E;-1&#x3C;/max&#x3E;&#x3C;refreshAsync&#x3E;False&#x3C;/refreshAsync&#x3E;&#x3C;pagingCookie/&#x3E;&#x3C;enableMultiSort&#x3E;true&#x3C;/enableMultiSort&#x3E;&#x3C;enablePagingWhenOnePage&#x3E;true&#x3C;/enablePagingWhenOnePage&#x3E;&#x3C;parameters&#x3E;&#x3C;autorefresh&#x3E;1&#x3C;/autorefresh&#x3E;&#x3C;viewid&#x3E;&#x26;&#x23;x7B&#x3B;BDD93547-53F6-4609-B591-9F48CE86295F&#x26;&#x23;x7D&#x3B;&#x3C;/viewid&#x3E;&#x3C;viewtype&#x3E;1039&#x3C;/viewtype&#x3E;&#x3C;otc&#x3E;4300&#x3C;/otc&#x3E;&#x3C;preview&#x3E;1&#x3C;/preview&#x3E;&#x3C;queryapi&#x3E;Campaign.RetrieveRelatedEntities&#x3C;/queryapi&#x3E;&#x3C;showjumpbar&#x3E;0&#x3C;/showjumpbar&#x3E;&#x3C;enablepaging&#x3E;1&#x3C;/enablepaging&#x3E;&#x3C;oId&#x3E;&#x26;&#x23;x7B&#x3B;c1a609d5-cff2-db11-8d7d-001143dcb18d&#x26;&#x23;x7D&#x3B;&#x3C;/oId&#x3E;&#x3C;oType&#x3E;4400&#x3C;/oType&#x3E;&#x3C;/parameters&#x3E;&#x3C;columns&#x3E;&#x3C;column width&#x3D;&#x22;300&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Name&#x22; fieldname&#x3D;&#x22;listname&#x22; entityname&#x3D;&#x22;list&#x22;&#x3E;listname&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Marketing List Member Type&#x22; fieldname&#x3D;&#x22;createdfromcode&#x22; entityname&#x3D;&#x22;list&#x22;&#x3E;createdfromcode&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Last Used On&#x22; fieldname&#x3D;&#x22;lastusedon&#x22; entityname&#x3D;&#x22;list&#x22;&#x3E;lastusedon&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Purpose&#x22; fieldname&#x3D;&#x22;purpose&#x22; entityname&#x3D;&#x22;list&#x22;&#x3E;purpose&#x3C;/column&#x3E;&#x3C;/columns&#x3E;&#x3C;/grid&#x3E;";

                grid = HttpUtility.HtmlDecode(grid);
                grid = grid.Replace("<pagingCookie/>", "<pagingCookie></pagingCookie>");
                grid = grid.Remove(grid.IndexOf("<viewid>") + 14, 36);
                grid = grid.Insert(grid.IndexOf("<viewid>") + 14, guid);


                if (objectGuid != null)
                {
                    grid = grid.Remove(grid.IndexOf("<oId>") + 11, 36);
                    grid = grid.Insert(grid.IndexOf("<oId>") + 11, objectGuid);
                }

                if (objectType != null)
                {
                    grid = grid.Remove(grid.IndexOf("<oType>") + 7, grid.IndexOf("</oType>") - (grid.IndexOf("<oType>") + 7));
                    grid = grid.Insert(grid.IndexOf("<oType>") + 7, objectType);
                }
                
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }
                return grid;
            }

            public static string ListMemberSearchView(int pageNum, string guid, string pagingCookie)
            {    
                string grid = @"<grid><sortColumns>name&#x3A;1</sortColumns><pageNum>1</pageNum><recsPerPage>50</recsPerPage><dataProvider>Microsoft.Crm.Application.Controls.AppGridDataProvider</dataProvider><uiProvider>Microsoft.Crm.Application.Controls.AppGridUIProvider</uiProvider><cols/><max>-1</max><refreshAsync>False</refreshAsync><pagingCookie></pagingCookie><enableMultiSort>true</enableMultiSort><enablePagingWhenOnePage>true</enablePagingWhenOnePage><parameters><autorefresh>1</autorefresh><viewid>&#x7B;VIEW_GUID&#x7D;</viewid><viewtype>1039</viewtype><otc>4230</otc><preview>1</preview><returnedtypecode>1</returnedtypecode><customquery>FilterByObjectType</customquery></parameters></grid>";
                grid = grid.Replace("VIEW_GUID", guid);
    
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }
                return grid;
            }

            public static string ListPageViewRefresh(int pageNum, string guid, string pagingCookie)
            {
                string grid = @"&#x3C;grid&#x3E;&#x3C;sortColumns&#x3E;listname&#x26;&#x23;x3A&#x3B;1&#x3C;/sortColumns&#x3E;&#x3C;pageNum&#x3E;1&#x3C;/pageNum&#x3E;&#x3C;recsPerPage&#x3E;50&#x3C;/recsPerPage&#x3E;&#x3C;dataProvider&#x3E;Microsoft.Crm.Application.Controls.AppGridDataProvider&#x3C;/dataProvider&#x3E;&#x3C;uiProvider&#x3E;Microsoft.Crm.Application.Controls.AppGridUIProvider&#x3C;/uiProvider&#x3E;&#x3C;cols/&#x3E;&#x3C;max&#x3E;-1&#x3C;/max&#x3E;&#x3C;refreshAsync&#x3E;False&#x3C;/refreshAsync&#x3E;&#x3C;pagingCookie/&#x3E;&#x3C;enableMultiSort&#x3E;true&#x3C;/enableMultiSort&#x3E;&#x3C;enablePagingWhenOnePage&#x3E;true&#x3C;/enablePagingWhenOnePage&#x3E;&#x3C;parameters&#x3E;&#x3C;autorefresh&#x3E;1&#x3C;/autorefresh&#x3E;&#x3C;viewid&#x3E;&#x26;&#x23;x7B&#x3B;823B9210-B82A-4B11-B0F6-B9E4C9C8403F&#x26;&#x23;x7D&#x3B;&#x3C;/viewid&#x3E;&#x3C;viewtype&#x3E;1039&#x3C;/viewtype&#x3E;&#x3C;otc&#x3E;4300&#x3C;/otc&#x3E;&#x3C;preview&#x3E;1&#x3C;/preview&#x3E;&#x3C;/parameters&#x3E;&#x3C;columns&#x3E;&#x3C;column width&#x3D;&#x22;300&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Name&#x22; fieldname&#x3D;&#x22;listname&#x22; entityname&#x3D;&#x22;list&#x22;&#x3E;listname&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Marketing List Member Type&#x22; fieldname&#x3D;&#x22;createdfromcode&#x22; entityname&#x3D;&#x22;list&#x22;&#x3E;createdfromcode&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Last Used On&#x22; fieldname&#x3D;&#x22;lastusedon&#x22; entityname&#x3D;&#x22;list&#x22;&#x3E;lastusedon&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Purpose&#x22; fieldname&#x3D;&#x22;purpose&#x22; entityname&#x3D;&#x22;list&#x22;&#x3E;purpose&#x3C;/column&#x3E;&#x3C;/columns&#x3E;&#x3C;/grid&#x3E;";

                grid = HttpUtility.HtmlDecode(grid);
                grid = grid.Replace("<pagingCookie/>", "<pagingCookie></pagingCookie>");
                grid = grid.Remove(grid.IndexOf("<viewid>") + 14, 36);
                grid = grid.Insert(grid.IndexOf("<viewid>") + 14, guid);
    
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }
                return grid;
            }

            public static string MemberGridForListViewRefresh(int pageNum, string guid, string pagingCookie, string objectGuid, string objectType)
            {
                string grid = @"<grid><sortColumns>name&#x3A;1</sortColumns><pageNum>1</pageNum><recsPerPage>50</recsPerPage><dataProvider>Microsoft.Crm.Application.Controls.AppGridDataProvider</dataProvider><uiProvider>Microsoft.Crm.Application.Controls.AppGridUIProvider</uiProvider><cols/><max>-1</max><refreshAsync>False</refreshAsync><pagingCookie></pagingCookie><enableMultiSort>true</enableMultiSort><enablePagingWhenOnePage>true</enablePagingWhenOnePage><parameters><autorefresh>1</autorefresh><viewid>&#x7B;VIEW_GUID&#x7D;</viewid><viewtype>1039</viewtype><otc>1</otc><preview>1</preview><queryapi>ListMember.RetrieveByObject</queryapi><showjumpbar>1</showjumpbar><enablepaging>1</enablepaging><oId>&#x7B;OBJECT_GUID&#x7D;</oId><oType>OBJECT_TYPE</oType><otcmb>4301</otcmb></parameters><columns><column width=""300"" isHidden=""false"" isMetadataBound=""true"" isSortable=""true"" label=""Account Name"" fieldname=""name"" entityname=""account"">name</column><column width=""100"" isHidden=""false"" isMetadataBound=""true"" isSortable=""true"" label=""Main Phone"" fieldname=""telephone1"" entityname=""account"">telephone1</column><column width=""100"" isHidden=""false"" isMetadataBound=""true"" isSortable=""true"" label=""Address 1&#x3A; City"" fieldname=""address1_city"" entityname=""account"">address1_city</column></columns></grid>";
                grid = grid.Replace("VIEW_GUID", guid);
                if (objectGuid != null)
                {
                    grid = grid.Replace("OBJECT_GUID", objectGuid);
                }
                if (objectType != null)
                {
                    grid = grid.Replace("OBJECT_TYPE", objectType);
                }
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }
                return grid;
            }

            public static string OpportunityPageView(int pageNum, string guid, string pagingCookie)
            {
                string grid = @"<grid><sortColumns>name&#x3A;1</sortColumns><pageNum>1</pageNum><recsPerPage>50</recsPerPage><dataProvider>Microsoft.Crm.Application.Controls.AppGridDataProvider</dataProvider><uiProvider>Microsoft.Crm.Application.Controls.AppGridUIProvider</uiProvider><cols/><max>-1</max><refreshAsync>False</refreshAsync><pagingCookie></pagingCookie><enableMultiSort>true</enableMultiSort><enablePagingWhenOnePage>true</enablePagingWhenOnePage><parameters><autorefresh>1</autorefresh><viewid>&#x7B;VIEW_GUID&#x7D;</viewid><viewtype>1039</viewtype><otc>3</otc><preview>1</preview><quickfind/><filter/><filterDisplay/></parameters></grid>";
                grid = grid.Replace("VIEW_GUID", guid);
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }
                return grid;
            }

            public static string OpportunityPageViewRefresh(int pageNum, string guid, string pagingCookie)
            {                
                string grid = @"&#x3C;grid&#x3E;&#x3C;sortColumns&#x3E;name&#x26;&#x23;x3A&#x3B;1&#x3C;/sortColumns&#x3E;&#x3C;pageNum&#x3E;1&#x3C;/pageNum&#x3E;&#x3C;recsPerPage&#x3E;50&#x3C;/recsPerPage&#x3E;&#x3C;dataProvider&#x3E;Microsoft.Crm.Application.Controls.AppGridDataProvider&#x3C;/dataProvider&#x3E;&#x3C;uiProvider&#x3E;Microsoft.Crm.Application.Controls.AppGridUIProvider&#x3C;/uiProvider&#x3E;&#x3C;cols/&#x3E;&#x3C;max&#x3E;-1&#x3C;/max&#x3E;&#x3C;refreshAsync&#x3E;False&#x3C;/refreshAsync&#x3E;&#x3C;pagingCookie/&#x3E;&#x3C;enableMultiSort&#x3E;true&#x3C;/enableMultiSort&#x3E;&#x3C;enablePagingWhenOnePage&#x3E;true&#x3C;/enablePagingWhenOnePage&#x3E;&#x3C;parameters&#x3E;&#x3C;autorefresh&#x3E;1&#x3C;/autorefresh&#x3E;&#x3C;viewid&#x3E;&#x26;&#x23;x7B&#x3B;00000000-0000-0000-00AA-000010003000&#x26;&#x23;x7D&#x3B;&#x3C;/viewid&#x3E;&#x3C;viewtype&#x3E;1039&#x3C;/viewtype&#x3E;&#x3C;otc&#x3E;3&#x3C;/otc&#x3E;&#x3C;preview&#x3E;1&#x3C;/preview&#x3E;&#x3C;/parameters&#x3E;&#x3C;columns&#x3E;&#x3C;column width&#x3D;&#x22;300&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Topic&#x22; fieldname&#x3D;&#x22;name&#x22; entityname&#x3D;&#x22;opportunity&#x22;&#x3E;name&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;150&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Potential Customer&#x22; fieldname&#x3D;&#x22;customerid&#x22; entityname&#x3D;&#x22;opportunity&#x22;&#x3E;customerid&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Est. Revenue&#x22; fieldname&#x3D;&#x22;estimatedvalue&#x22; entityname&#x3D;&#x22;opportunity&#x22;&#x3E;estimatedvalue&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;125&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Est. Close Date&#x22; fieldname&#x3D;&#x22;estimatedclosedate&#x22; entityname&#x3D;&#x22;opportunity&#x22;&#x3E;estimatedclosedate&#x3C;/column&#x3E;&#x3C;/columns&#x3E;&#x3C;/grid&#x3E;";

                grid = HttpUtility.HtmlDecode(grid);
                grid = grid.Replace("<pagingCookie/>", "<pagingCookie></pagingCookie>");
                grid = grid.Remove(grid.IndexOf("<viewid>") + 14, 36);
                grid = grid.Insert(grid.IndexOf("<viewid>") + 14, guid);

                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }
                return grid;
            }

            public static string OpportunityPageSearchViewRefresh(int pageNum, string guid, string random, string pagingCookie)
            {
                string grid = @"&#x3C;grid&#x3E;&#x3C;sortColumns&#x3E;name&#x26;&#x23;x3A&#x3B;1&#x3C;/sortColumns&#x3E;&#x3C;pageNum&#x3E;1&#x3C;/pageNum&#x3E;&#x3C;recsPerPage&#x3E;50&#x3C;/recsPerPage&#x3E;&#x3C;dataProvider&#x3E;Microsoft.Crm.Application.Controls.AppGridDataProvider&#x3C;/dataProvider&#x3E;&#x3C;uiProvider&#x3E;Microsoft.Crm.Application.Controls.AppGridUIProvider&#x3C;/uiProvider&#x3E;&#x3C;cols/&#x3E;&#x3C;max&#x3E;-1&#x3C;/max&#x3E;&#x3C;refreshAsync&#x3E;False&#x3C;/refreshAsync&#x3E;&#x3C;pagingCookie/&#x3E;&#x3C;enableMultiSort&#x3E;true&#x3C;/enableMultiSort&#x3E;&#x3C;enablePagingWhenOnePage&#x3E;true&#x3C;/enablePagingWhenOnePage&#x3E;&#x3C;parameters&#x3E;&#x3C;autorefresh&#x3E;1&#x3C;/autorefresh&#x3E;&#x3C;viewid&#x3E;&#x26;&#x23;x7B&#x3B;3F6D889E-6549-4025-9CE0-4C4103EFB6A3&#x26;&#x23;x7D&#x3B;&#x3C;/viewid&#x3E;&#x3C;viewtype&#x3E;1039&#x3C;/viewtype&#x3E;&#x3C;otc&#x3E;3&#x3C;/otc&#x3E;&#x3C;preview&#x3E;1&#x26;&#x23;x2C&#x3B;1&#x26;&#x23;x2C&#x3B;1&#x3C;/preview&#x3E;&#x3C;quickfind&#x3E;a&#x3C;/quickfind&#x3E;&#x3C;filter/&#x3E;&#x3C;filterDisplay/&#x3E;&#x3C;/parameters&#x3E;&#x3C;columns&#x3E;&#x3C;column width&#x3D;&#x22;300&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Topic&#x22; fieldname&#x3D;&#x22;name&#x22; entityname&#x3D;&#x22;opportunity&#x22;&#x3E;name&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;150&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Potential Customer&#x22; fieldname&#x3D;&#x22;customerid&#x22; entityname&#x3D;&#x22;opportunity&#x22;&#x3E;customerid&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Est. Revenue&#x22; fieldname&#x3D;&#x22;estimatedvalue&#x22; entityname&#x3D;&#x22;opportunity&#x22;&#x3E;estimatedvalue&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;125&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Est. Close Date&#x22; fieldname&#x3D;&#x22;estimatedclosedate&#x22; entityname&#x3D;&#x22;opportunity&#x22;&#x3E;estimatedclosedate&#x3C;/column&#x3E;&#x3C;/columns&#x3E;&#x3C;/grid&#x3E;";

                grid = HttpUtility.HtmlDecode(grid);
                grid = grid.Replace("<pagingCookie/>", "<pagingCookie></pagingCookie>");
                grid = grid.Remove(grid.IndexOf("<viewid>") + 14, 36);
                grid = grid.Insert(grid.IndexOf("<viewid>") + 14, guid);

                grid = grid.Remove(grid.IndexOf("<quickfind>") + 11, grid.IndexOf("</quickfind>") - (grid.IndexOf("<quickfind>") + 11));
                grid = grid.Insert(grid.IndexOf("<quickfind>") + 11, random);
                                
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }
                return grid;
            }

            public static string OpportunitySearchView(int pageNum, string guid, string random, string pagingCookie)
            {   
                string grid = @"<grid><sortColumns>name&#x3A;1</sortColumns><pageNum>1</pageNum><recsPerPage>50</recsPerPage><dataProvider>Microsoft.Crm.Application.Controls.AppGridDataProvider</dataProvider><uiProvider>Microsoft.Crm.Application.Controls.AppGridUIProvider</uiProvider><cols/><max>-1</max><refreshAsync>False</refreshAsync><pagingCookie></pagingCookie><enableMultiSort>true</enableMultiSort><enablePagingWhenOnePage>true</enablePagingWhenOnePage><parameters><autorefresh>1</autorefresh><viewid>&#x7B;VIEW_GUID&#x7D;</viewid><viewtype>1039</viewtype><otc>3</otc><preview>1&#x2C;1</preview><quickfind>RANDOM_STRING</quickfind><filter/><filterDisplay/></parameters></grid>";
                grid = grid.Replace("VIEW_GUID", guid);
                grid = grid.Replace("RANDOM_STRING", random);
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }
                return grid;
            }

            public static string PhonecallPageView(int pageNum, string guid, string pagingCookie)
            {
                string grid = @"<grid><sortColumns>scheduledend&#x3A;1</sortColumns><pageNum>1</pageNum><recsPerPage>50</recsPerPage><dataProvider>Microsoft.Crm.Application.Controls.ActivitiesGridDataProvider</dataProvider><uiProvider>Microsoft.Crm.Application.Controls.AppGridUIProvider</uiProvider><cols/><max>-1</max><refreshAsync>False</refreshAsync><pagingCookie></pagingCookie><enableMultiSort>true</enableMultiSort><enablePagingWhenOnePage>true</enablePagingWhenOnePage><parameters><autorefresh>1</autorefresh><viewid>&#x7B;VIEW_GUID&#x7D;</viewid><viewtype>1039</viewtype><otc>4200</otc><preview>1</preview><quickfind/><filter/><filterDisplay/></parameters></grid>";
                grid = grid.Replace("VIEW_GUID", guid);
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }
                return grid;
            }

            public static string PhonecallPageViewRefresh(int pageNum, string guid, string pagingCookie)
            {
                string grid = @"&#x3C;grid&#x3E;&#x3C;sortColumns&#x3E;createdon&#x26;&#x23;x3A&#x3B;0&#x3C;/sortColumns&#x3E;&#x3C;pageNum&#x3E;1&#x3C;/pageNum&#x3E;&#x3C;recsPerPage&#x3E;50&#x3C;/recsPerPage&#x3E;&#x3C;dataProvider&#x3E;Microsoft.Crm.Application.Controls.ActivitiesGridDataProvider&#x3C;/dataProvider&#x3E;&#x3C;uiProvider&#x3E;Microsoft.Crm.Application.Controls.AppGridUIProvider&#x3C;/uiProvider&#x3E;&#x3C;cols/&#x3E;&#x3C;max&#x3E;-1&#x3C;/max&#x3E;&#x3C;refreshAsync&#x3E;False&#x3C;/refreshAsync&#x3E;&#x3C;pagingCookie/&#x3E;&#x3C;enableMultiSort&#x3E;true&#x3C;/enableMultiSort&#x3E;&#x3C;enablePagingWhenOnePage&#x3E;true&#x3C;/enablePagingWhenOnePage&#x3E;&#x3C;parameters&#x3E;&#x3C;autorefresh&#x3E;1&#x3C;/autorefresh&#x3E;&#x3C;viewid&#x3E;&#x26;&#x23;x7B&#x3B;616D0480-0A0D-4753-8617-CAD4A04101AA&#x26;&#x23;x7D&#x3B;&#x3C;/viewid&#x3E;&#x3C;viewtype&#x3E;1039&#x3C;/viewtype&#x3E;&#x3C;otc&#x3E;4204&#x3C;/otc&#x3E;&#x3C;preview&#x3E;1&#x26;&#x23;x2C&#x3B;1&#x3C;/preview&#x3E;&#x3C;quickfind/&#x3E;&#x3C;filter/&#x3E;&#x3C;filterDisplay/&#x3E;&#x3C;/parameters&#x3E;&#x3C;columns&#x3E;&#x3C;column width&#x3D;&#x22;250&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Subject&#x22; fieldname&#x3D;&#x22;subject&#x22; entityname&#x3D;&#x22;fax&#x22;&#x3E;subject&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;150&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;false&#x22; label&#x3D;&#x22;Sender&#x22; fieldname&#x3D;&#x22;from&#x22; entityname&#x3D;&#x22;fax&#x22;&#x3E;from&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;150&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;false&#x22; label&#x3D;&#x22;Recipient&#x22; fieldname&#x3D;&#x22;to&#x22; entityname&#x3D;&#x22;fax&#x22;&#x3E;to&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Regarding&#x22; fieldname&#x3D;&#x22;regardingobjectid&#x22; entityname&#x3D;&#x22;fax&#x22;&#x3E;regardingobjectid&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Activity Status&#x22; fieldname&#x3D;&#x22;statecode&#x22; entityname&#x3D;&#x22;fax&#x22;&#x3E;statecode&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Created On&#x22; fieldname&#x3D;&#x22;createdon&#x22; entityname&#x3D;&#x22;fax&#x22;&#x3E;createdon&#x3C;/column&#x3E;&#x3C;/columns&#x3E;&#x3C;/grid&#x3E;";

                grid = HttpUtility.HtmlDecode(grid);
                grid = grid.Replace("<pagingCookie/>", "<pagingCookie></pagingCookie>");
                grid = grid.Remove(grid.IndexOf("<viewid>") + 14, 36);
                grid = grid.Insert(grid.IndexOf("<viewid>") + 14, guid);
                
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }
                return grid;
            }

            public static string QuoteDetailPageRefresh(int pageNum, string guid, string quoteGuid, string pagingCookie)
            {
                string grid = @"&#60;grid&#62;&#60;sortColumns&#62;productid&#38;&#35;58&#59;1&#60;&#47;sortColumns&#62;&#60;pageNum&#62;1&#60;&#47;pageNum&#62;&#60;recsPerPage&#62;50&#60;&#47;recsPerPage&#62;&#60;dataProvider&#62;Microsoft.Crm.Application.Controls.AppGridDataProvider&#60;&#47;dataProvider&#62;&#60;uiProvider&#62;Microsoft.Crm.Application.Controls.AppGridUIProvider&#60;&#47;uiProvider&#62;&#60;cols&#47;&#62;&#60;max&#62;-1&#60;&#47;max&#62;&#60;refreshAsync&#62;False&#60;&#47;refreshAsync&#62;&#60;pagingCookie&#47;&#62;&#60;enableMultiSort&#62;true&#60;&#47;enableMultiSort&#62;&#60;enablePagingWhenOnePage&#62;true&#60;&#47;enablePagingWhenOnePage&#62;&#60;parameters&#62;&#60;autorefresh&#62;1&#60;&#47;autorefresh&#62;&#60;viewid&#62;&#38;&#35;123&#59;4EC5BD8A-5580-49AA-BD6E-1368AD611B6A&#38;&#35;125&#59;&#60;&#47;viewid&#62;&#60;viewtype&#62;1039&#60;&#47;viewtype&#62;&#60;otc&#62;1085&#60;&#47;otc&#62;&#60;preview&#62;1&#60;&#47;preview&#62;&#60;queryapi&#62;CRMQuoteDetail.RetrieveByObject&#60;&#47;queryapi&#62;&#60;showjumpbar&#62;0&#60;&#47;showjumpbar&#62;&#60;enablepaging&#62;1&#60;&#47;enablepaging&#62;&#60;oId&#62;&#38;&#35;123&#59;ac72a37b-b28d-dc11-8da0-0018fe33e1ee&#38;&#35;125&#59;&#60;&#47;oId&#62;&#60;oType&#62;1084&#60;&#47;oType&#62;&#60;relName&#47;&#62;&#60;roleOrd&#62;-1&#60;&#47;roleOrd&#62;&#60;isCustomRelationship&#62;false&#60;&#47;isCustomRelationship&#62;&#60;&#47;parameters&#62;&#60;columns&#62;&#60;column width&#61;&#34;300&#34; isHidden&#61;&#34;false&#34; isMetadataBound&#61;&#34;true&#34; isSortable&#61;&#34;true&#34; label&#61;&#34;Product&#34; fieldname&#61;&#34;productid&#34; entityname&#61;&#34;quotedetail&#34;&#62;productid&#60;&#47;column&#62;&#60;column width&#61;&#34;100&#34; isHidden&#61;&#34;false&#34; isMetadataBound&#61;&#34;true&#34; isSortable&#61;&#34;true&#34; label&#61;&#34;Price Per Unit&#34; fieldname&#61;&#34;priceperunit&#34; entityname&#61;&#34;quotedetail&#34;&#62;priceperunit&#60;&#47;column&#62;&#60;column width&#61;&#34;100&#34; isHidden&#61;&#34;false&#34; isMetadataBound&#61;&#34;true&#34; isSortable&#61;&#34;true&#34; label&#61;&#34;Quantity&#34; fieldname&#61;&#34;quantity&#34; entityname&#61;&#34;quotedetail&#34;&#62;quantity&#60;&#47;column&#62;&#60;column width&#61;&#34;125&#34; isHidden&#61;&#34;false&#34; isMetadataBound&#61;&#34;true&#34; isSortable&#61;&#34;true&#34; label&#61;&#34;Extended Amount&#34; fieldname&#61;&#34;extendedamount&#34; entityname&#61;&#34;quotedetail&#34;&#62;extendedamount&#60;&#47;column&#62;&#60;&#47;columns&#62;&#60;&#47;grid&#62;";

                grid = HttpUtility.HtmlDecode(grid);
                grid = grid.Replace("<pagingCookie/>", "<pagingCookie></pagingCookie>");
                grid = grid.Remove(grid.IndexOf("<viewid>") + 14, 36);
                grid = grid.Insert(grid.IndexOf("<viewid>") + 14, guid);

                grid = grid.Remove(grid.IndexOf("<oId>") + 11, 36);
                grid = grid.Insert(grid.IndexOf("<oId>") + 11, quoteGuid);
                                
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }
                return grid;                                
            }

            public static string QuotePageView(int pageNum, string guid, string pagingCookie)
            {
                string grid = @"&#x3C;grid&#x3E;&#x3C;sortColumns&#x3E;name&#x26;&#x23;x3A&#x3B;1&#x3C;/sortColumns&#x3E;&#x3C;pageNum&#x3E;1&#x3C;/pageNum&#x3E;&#x3C;recsPerPage&#x3E;50&#x3C;/recsPerPage&#x3E;&#x3C;dataProvider&#x3E;Microsoft.Crm.Application.Controls.AppGridDataProvider&#x3C;/dataProvider&#x3E;&#x3C;uiProvider&#x3E;Microsoft.Crm.Application.Controls.AppGridUIProvider&#x3C;/uiProvider&#x3E;&#x3C;cols/&#x3E;&#x3C;max&#x3E;-1&#x3C;/max&#x3E;&#x3C;refreshAsync&#x3E;False&#x3C;/refreshAsync&#x3E;&#x3C;pagingCookie/&#x3E;&#x3C;enableMultiSort&#x3E;true&#x3C;/enableMultiSort&#x3E;&#x3C;enablePagingWhenOnePage&#x3E;true&#x3C;/enablePagingWhenOnePage&#x3E;&#x3C;parameters&#x3E;&#x3C;autorefresh&#x3E;1&#x3C;/autorefresh&#x3E;&#x3C;viewid&#x3E;&#x26;&#x23;x7B&#x3B;008FBB15-98E9-4C5D-A81A-E685FF8CCC2C&#x26;&#x23;x7D&#x3B;&#x3C;/viewid&#x3E;&#x3C;viewtype&#x3E;1039&#x3C;/viewtype&#x3E;&#x3C;otc&#x3E;1084&#x3C;/otc&#x3E;&#x3C;preview&#x3E;1&#x3C;/preview&#x3E;&#x3C;/parameters&#x3E;&#x3C;columns&#x3E;&#x3C;column width&#x3D;&#x22;300&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Name&#x22; fieldname&#x3D;&#x22;name&#x22; entityname&#x3D;&#x22;quote&#x22;&#x3E;name&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;150&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Potential Customer&#x22; fieldname&#x3D;&#x22;customerid&#x22; entityname&#x3D;&#x22;quote&#x22;&#x3E;customerid&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Status&#x22; fieldname&#x3D;&#x22;statecode&#x22; entityname&#x3D;&#x22;quote&#x22;&#x3E;statecode&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Total Amount&#x22; fieldname&#x3D;&#x22;totalamount&#x22; entityname&#x3D;&#x22;quote&#x22;&#x3E;totalamount&#x3C;/column&#x3E;&#x3C;/columns&#x3E;&#x3C;/grid&#x3E;";

                grid = HttpUtility.HtmlDecode(grid);
                grid = grid.Replace("<pagingCookie/>", "<pagingCookie></pagingCookie>");
                grid = grid.Remove(grid.IndexOf("<viewid>") + 14, 36);
                grid = grid.Insert(grid.IndexOf("<viewid>") + 14, guid);

                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }
                return grid;
            }

            public static string QuotePageViewRefresh(int pageNum, string guid, string pagingCookie)
            {
                string grid = @"<grid><sortColumns>name&#x3A;1</sortColumns><pageNum>1</pageNum><recsPerPage>50</recsPerPage><dataProvider>Microsoft.Crm.Application.Controls.AppGridDataProvider</dataProvider><uiProvider>Microsoft.Crm.Application.Controls.AppGridUIProvider</uiProvider><cols/><max>-1</max><refreshAsync>False</refreshAsync><pagingCookie></pagingCookie><enableMultiSort>true</enableMultiSort><enablePagingWhenOnePage>true</enablePagingWhenOnePage><parameters><autorefresh>1</autorefresh><viewid>&#x7B;VIEW_GUID&#x7D;</viewid><viewtype>1039</viewtype><otc>1084</otc><preview>1&#x2C;1</preview><quickfind></quickfind><filter/><filterDisplay/></parameters><columns><column width=""300"" isHidden=""false"" isMetadataBound=""true"" isSortable=""true"" label=""Name"">name</column><column width=""150"" isHidden=""false"" isMetadataBound=""true"" isSortable=""true"" label=""Potential Customer"">customerid</column><column width=""100"" isHidden=""false"" isMetadataBound=""true"" isSortable=""true"" label=""Status"">statecode</column><column width=""100"" isHidden=""false"" isMetadataBound=""true"" isSortable=""true"" label=""Total Amount"">totalamount</column></columns></grid>";
                grid = grid.Replace("VIEW_GUID", guid);
                
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }
                return grid;
            }

            public static string QuoteSearchView(int pageNum, string guid, string pagingCookie)
            {
                string grid = @"<grid><sortColumns>name&#x3A;1</sortColumns><pageNum>1</pageNum><recsPerPage>50</recsPerPage><dataProvider>Microsoft.Crm.Application.Controls.AppGridDataProvider</dataProvider><uiProvider>Microsoft.Crm.Application.Controls.AppGridUIProvider</uiProvider><cols/><max>-1</max><refreshAsync>False</refreshAsync><pagingCookie></pagingCookie><enableMultiSort>true</enableMultiSort><enablePagingWhenOnePage>true</enablePagingWhenOnePage><parameters><autorefresh>1</autorefresh><viewid>&#x7B;VIEW_GUID&#x7D;</viewid><viewtype>1039</viewtype><otc>1084</otc><preview>1</preview><quickfind>" + Utils.GetRandomString(2, 2) + "</quickfind><filter/><filterDisplay/></parameters></grid>";
                grid = grid.Replace("VIEW_GUID", guid);
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }
                return grid;
            }

            public static string RelationshipPageView(int pageNum, string guid, string pagingCookie)
            {
                string grid = @"&#x3C;grid&#x3E;&#x3C;sortColumns&#x3E;name&#x26;&#x23;x3A&#x3B;1&#x3C;/sortColumns&#x3E;&#x3C;pageNum&#x3E;1&#x3C;/pageNum&#x3E;&#x3C;recsPerPage&#x3E;50&#x3C;/recsPerPage&#x3E;&#x3C;dataProvider&#x3E;Microsoft.Crm.Application.Controls.AppGridDataProvider&#x3C;/dataProvider&#x3E;&#x3C;uiProvider&#x3E;Microsoft.Crm.Application.Controls.AppGridUIProvider&#x3C;/uiProvider&#x3E;&#x3C;cols/&#x3E;&#x3C;max&#x3E;-1&#x3C;/max&#x3E;&#x3C;refreshAsync&#x3E;False&#x3C;/refreshAsync&#x3E;&#x3C;pagingCookie/&#x3E;&#x3C;enableMultiSort&#x3E;true&#x3C;/enableMultiSort&#x3E;&#x3C;enablePagingWhenOnePage&#x3E;true&#x3C;/enablePagingWhenOnePage&#x3E;&#x3C;parameters&#x3E;&#x3C;autorefresh&#x3E;1&#x3C;/autorefresh&#x3E;&#x3C;viewid&#x3E;&#x26;&#x23;x7B&#x3B;938894EC-71AA-4937-A56B-262957B44A02&#x26;&#x23;x7D&#x3B;&#x3C;/viewid&#x3E;&#x3C;viewtype&#x3E;1039&#x3C;/viewtype&#x3E;&#x3C;otc&#x3E;4500&#x3C;/otc&#x3E;&#x3C;/parameters&#x3E;&#x3C;columns&#x3E;&#x3C;column width&#x3D;&#x22;300&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Name&#x22; fieldname&#x3D;&#x22;name&#x22; entityname&#x3D;&#x22;relationshiprole&#x22;&#x3E;name&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;150&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Status&#x22; fieldname&#x3D;&#x22;statecode&#x22; entityname&#x3D;&#x22;relationshiprole&#x22;&#x3E;statecode&#x3C;/column&#x3E;&#x3C;/columns&#x3E;&#x3C;/grid&#x3E;";

                grid = HttpUtility.HtmlDecode(grid);
                grid = grid.Replace("<pagingCookie/>", "<pagingCookie></pagingCookie>");
                grid = grid.Remove(grid.IndexOf("<viewid>") + 14, 36);
                grid = grid.Insert(grid.IndexOf("<viewid>") + 14, guid);
                
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }
                return grid;
            }

            public static string RolePageViewRefresh(int pageNum, string guid, string pagingCookie)
            {
                string grid = @"<gridXml><grid><sortColumns>name&#58;1</sortColumns><pageNum>1</pageNum><recsPerPage>50</recsPerPage><dataProvider>Microsoft.Crm.Application.Controls.AppGridDataProvider</dataProvider><uiProvider>Microsoft.Crm.Application.Controls.AppGridUIProvider</uiProvider><cols/><max>-1</max><refreshAsync>False</refreshAsync><pagingCookie></pagingCookie><enableMultiSort>true</enableMultiSort><enablePagingWhenOnePage>true</enablePagingWhenOnePage><parameters><autorefresh>1</autorefresh><viewid>VIEW_GUID</viewid><viewtype>1039</viewtype><otc>1036</otc></parameters><columns><column width=""300"" isHidden=""false"" isMetadataBound=""true"" isSortable=""true"" label=""Name"" fieldname=""name"" entityname=""role"">name</column><column width=""300"" isHidden=""false"" isMetadataBound=""true"" isSortable=""true"" label=""Business Unit"" fieldname=""businessunitid"" entityname=""role"">businessunitid</column></columns></grid></gridXml>";
                grid = grid.Replace("VIEW_GUID", guid);

                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }
                return grid;
            }

            public static string ServicePage(string serviceID)
            {
                string grid = @"&#x3C;grid&#x3E;&#x3C;sortColumns&#x3E;&#x26;&#x23;x3A&#x3B;1&#x3C;/sortColumns&#x3E;&#x3C;pageNum&#x3E;1&#x3C;/pageNum&#x3E;&#x3C;recsPerPage&#x3E;50&#x3C;/recsPerPage&#x3E;&#x3C;dataProvider&#x3E;Microsoft.Crm.Application.Controls.ScheduleSearchDataProvider&#x3C;/dataProvider&#x3E;&#x3C;uiProvider&#x3E;Microsoft.Crm.Application.Controls.ScheduleSearchGridUIProvider&#x3C;/uiProvider&#x3E;&#x3C;cols/&#x3E;&#x3C;max&#x3E;1&#x3C;/max&#x3E;&#x3C;refreshAsync&#x3E;True&#x3C;/refreshAsync&#x3E;&#x3C;pagingCookie/&#x3E;&#x3C;enableMultiSort&#x3E;false&#x3C;/enableMultiSort&#x3E;&#x3C;enablePagingWhenOnePage&#x3E;true&#x3C;/enablePagingWhenOnePage&#x3E;&#x3C;parameters&#x3E;&#x3C;autorefresh&#x3E;0&#x3C;/autorefresh&#x3E;&#x3C;showjumpbar&#x3E;0&#x3C;/showjumpbar&#x3E;&#x3C;enablepaging&#x3E;1&#x3C;/enablepaging&#x3E;&#x3C;enablerefresh&#x3E;0&#x3C;/enablerefresh&#x3E;&#x3C;refreshasynchronous&#x3E;1&#x3C;/refreshasynchronous&#x3E;&#x3C;viewid&#x3E;&#x26;&#x23;x7B&#x3B;22520C00-3506-4670-89E8-E51C45D6DBA2&#x26;&#x23;x7D&#x3B;&#x3C;/viewid&#x3E;&#x3C;viewtype&#x3E;1039&#x3C;/viewtype&#x3E;&#x3C;otc&#x3E;0&#x3C;/otc&#x3E;&#x3C;disableDblClick&#x3E;0&#x3C;/disableDblClick&#x3E;&#x3C;AppointmentRequest&#x3E;&#x26;&#x23;x3C&#x3B;AppointmentRequest&#x26;&#x23;x3E&#x3B;&#x26;&#x23;x3C&#x3B;ServiceId xmlns&#x26;&#x23;x3D&#x3B;&#x26;&#x23;x22&#x3B;http&#x26;&#x23;x3A&#x3B;//schemas.microsoft.com/crm/2006/Scheduling&#x26;&#x23;x22&#x3B;&#x26;&#x23;x3E&#x3B;&#x26;&#x23;x7B&#x3B;SERVICE_GUID&#x26;&#x23;x7D&#x3B;&#x26;&#x23;x3C&#x3B;/ServiceId&#x26;&#x23;x3E&#x3B;&#x26;&#x23;x3C&#x3B;Direction xmlns&#x26;&#x23;x3D&#x3B;&#x26;&#x23;x22&#x3B;http&#x26;&#x23;x3A&#x3B;//schemas.microsoft.com/crm/2006/Scheduling&#x26;&#x23;x22&#x3B;&#x26;&#x23;x3E&#x3B;Forward&#x26;&#x23;x3C&#x3B;/Direction&#x26;&#x23;x3E&#x3B;&#x26;&#x23;x3C&#x3B;NumberOfResults xmlns&#x26;&#x23;x3D&#x3B;&#x26;&#x23;x22&#x3B;http&#x26;&#x23;x3A&#x3B;//schemas.microsoft.com/crm/2006/Scheduling&#x26;&#x23;x22&#x3B;&#x26;&#x23;x3E&#x3B;50&#x26;&#x23;x3C&#x3B;/NumberOfResults&#x26;&#x23;x3E&#x3B;&#x26;&#x23;x3C&#x3B;SearchWindowStart xmlns&#x26;&#x23;x3D&#x3B;&#x26;&#x23;x22&#x3B;http&#x26;&#x23;x3A&#x3B;//schemas.microsoft.com/crm/2006/Scheduling&#x26;&#x23;x22&#x3B;&#x26;&#x23;x3E&#x3B;2007-04-26T16&#x26;&#x23;x3A&#x3B;19&#x26;&#x23;x3A&#x3B;11&#x26;&#x23;x3C&#x3B;/SearchWindowStart&#x26;&#x23;x3E&#x3B;&#x26;&#x23;x3C&#x3B;SearchWindowEnd xmlns&#x26;&#x23;x3D&#x3B;&#x26;&#x23;x22&#x3B;http&#x26;&#x23;x3A&#x3B;//schemas.microsoft.com/crm/2006/Scheduling&#x26;&#x23;x22&#x3B;&#x26;&#x23;x3E&#x3B;2008-04-26T00&#x26;&#x23;x3A&#x3B;00&#x26;&#x23;x3A&#x3B;00&#x26;&#x23;x3C&#x3B;/SearchWindowEnd&#x26;&#x23;x3E&#x3B;&#x26;&#x23;x3C&#x3B;AnchorOffset xmlns&#x26;&#x23;x3D&#x3B;&#x26;&#x23;x22&#x3B;http&#x26;&#x23;x3A&#x3B;//schemas.microsoft.com/crm/2006/Scheduling&#x26;&#x23;x22&#x3B;&#x26;&#x23;x3E&#x3B;480&#x26;&#x23;x3C&#x3B;/AnchorOffset&#x26;&#x23;x3E&#x3B;&#x26;&#x23;x3C&#x3B;UserTimeZoneCode xmlns&#x26;&#x23;x3D&#x3B;&#x26;&#x23;x22&#x3B;http&#x26;&#x23;x3A&#x3B;//schemas.microsoft.com/crm/2006/Scheduling&#x26;&#x23;x22&#x3B;&#x26;&#x23;x3E&#x3B;4&#x26;&#x23;x3C&#x3B;/UserTimeZoneCode&#x26;&#x23;x3E&#x3B;&#x26;&#x23;x3C&#x3B;SearchRecurrenceRule xmlns&#x26;&#x23;x3D&#x3B;&#x26;&#x23;x22&#x3B;http&#x26;&#x23;x3A&#x3B;//schemas.microsoft.com/crm/2006/Scheduling&#x26;&#x23;x22&#x3B;&#x26;&#x23;x3E&#x3B;FREQ&#x26;&#x23;x3D&#x3B;DAILY&#x26;&#x23;x3B&#x3B;INTERVAL&#x26;&#x23;x3D&#x3B;1&#x26;&#x23;x3C&#x3B;/SearchRecurrenceRule&#x26;&#x23;x3E&#x3B;&#x26;&#x23;x3C&#x3B;SearchRecurrenceStart xmlns&#x26;&#x23;x3D&#x3B;&#x26;&#x23;x22&#x3B;http&#x26;&#x23;x3A&#x3B;//schemas.microsoft.com/crm/2006/Scheduling&#x26;&#x23;x22&#x3B;&#x26;&#x23;x3E&#x3B;2007-04-26T08&#x26;&#x23;x3A&#x3B;00&#x26;&#x23;x3A&#x3B;00Z&#x26;&#x23;x3C&#x3B;/SearchRecurrenceStart&#x26;&#x23;x3E&#x3B;&#x26;&#x23;x3C&#x3B;RecurrenceDuration xmlns&#x26;&#x23;x3D&#x3B;&#x26;&#x23;x22&#x3B;http&#x26;&#x23;x3A&#x3B;//schemas.microsoft.com/crm/2006/Scheduling&#x26;&#x23;x22&#x3B;&#x26;&#x23;x3E&#x3B;541&#x26;&#x23;x3C&#x3B;/RecurrenceDuration&#x26;&#x23;x3E&#x3B;&#x26;&#x23;x3C&#x3B;RecurrenceTimeZoneCode xmlns&#x26;&#x23;x3D&#x3B;&#x26;&#x23;x22&#x3B;http&#x26;&#x23;x3A&#x3B;//schemas.microsoft.com/crm/2006/Scheduling&#x26;&#x23;x22&#x3B;&#x26;&#x23;x3E&#x3B;4&#x26;&#x23;x3C&#x3B;/RecurrenceTimeZoneCode&#x26;&#x23;x3E&#x3B;&#x26;&#x23;x3C&#x3B;/AppointmentRequest&#x26;&#x23;x3E&#x3B;&#x3C;/AppointmentRequest&#x3E;&#x3C;ServiceName&#x3E;Branch &#x26;&#x23;x3A&#x3B; 879 - 11726407201914 - Lots of Bikes Storehouse&#x3C;/ServiceName&#x3E;&#x3C;SearchWindowStart/&#x3E;&#x3C;SearchWindowEnd/&#x3E;&#x3C;/parameters&#x3E;&#x3C;columns&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;false&#x22; isSortable&#x3D;&#x22;false&#x22; label&#x3D;&#x22;Service&#x22; fieldname&#x3D;&#x22;Service&#x22; entityname&#x3D;&#x22;&#x22;&#x3E;Service&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;240&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;false&#x22; isSortable&#x3D;&#x22;false&#x22; label&#x3D;&#x22;Resources&#x22; fieldname&#x3D;&#x22;ProposalParties&#x22; entityname&#x3D;&#x22;&#x22;&#x3E;ProposalParties&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;false&#x22; isSortable&#x3D;&#x22;false&#x22; label&#x3D;&#x22;Site&#x22; fieldname&#x3D;&#x22;Site&#x22; entityname&#x3D;&#x22;&#x22;&#x3E;Site&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;130&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;false&#x22; isSortable&#x3D;&#x22;false&#x22; label&#x3D;&#x22;Scheduled Start&#x22; fieldname&#x3D;&#x22;Start&#x22; entityname&#x3D;&#x22;&#x22;&#x3E;Start&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;130&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;false&#x22; isSortable&#x3D;&#x22;false&#x22; label&#x3D;&#x22;Scheduled End&#x22; fieldname&#x3D;&#x22;End&#x22; entityname&#x3D;&#x22;&#x22;&#x3E;End&#x3C;/column&#x3E;&#x3C;/columns&#x3E;&#x3C;/grid&#x3E;";
                grid = HttpUtility.HtmlDecode(grid);

                grid = grid.Replace("SERVICE_GUID", serviceID);

                return grid;                
            }

            public static string TaskGridForCampaignViewRefresh(int pageNum, string guid, string pagingCookie, string objectGuid, string objectType)
            {            

                string grid = @"&#x3C;grid&#x3E;&#x3C;sortColumns&#x3E;subject&#x26;&#x23;x3A&#x3B;1&#x3C;/sortColumns&#x3E;&#x3C;pageNum&#x3E;1&#x3C;/pageNum&#x3E;&#x3C;recsPerPage&#x3E;50&#x3C;/recsPerPage&#x3E;&#x3C;dataProvider&#x3E;Microsoft.Crm.Application.Controls.ActivitiesGridDataProvider&#x3C;/dataProvider&#x3E;&#x3C;uiProvider&#x3E;Microsoft.Crm.Application.Controls.AppGridUIProvider&#x3C;/uiProvider&#x3E;&#x3C;cols/&#x3E;&#x3C;max&#x3E;-1&#x3C;/max&#x3E;&#x3C;refreshAsync&#x3E;False&#x3C;/refreshAsync&#x3E;&#x3C;pagingCookie/&#x3E;&#x3C;enableMultiSort&#x3E;true&#x3C;/enableMultiSort&#x3E;&#x3C;enablePagingWhenOnePage&#x3E;true&#x3C;/enablePagingWhenOnePage&#x3E;&#x3C;parameters&#x3E;&#x3C;autorefresh&#x3E;1&#x3C;/autorefresh&#x3E;&#x3C;viewid&#x3E;&#x26;&#x23;x7B&#x3B;2B5F5A5D-2D23-4FE7-AA58-E77995368AE7&#x26;&#x23;x7D&#x3B;&#x3C;/viewid&#x3E;&#x3C;viewtype&#x3E;1039&#x3C;/viewtype&#x3E;&#x3C;otc&#x3E;4212&#x3C;/otc&#x3E;&#x3C;preview&#x3E;1&#x3C;/preview&#x3E;&#x3C;queryapi&#x3E;CRMTask.RetrieveByObject&#x3C;/queryapi&#x3E;&#x3C;showjumpbar&#x3E;0&#x3C;/showjumpbar&#x3E;&#x3C;enablepaging&#x3E;1&#x3C;/enablepaging&#x3E;&#x3C;oId&#x3E;&#x26;&#x23;x7B&#x3B;9ff144dc-c3f2-db11-8d7d-001143dcb18d&#x26;&#x23;x7D&#x3B;&#x3C;/oId&#x3E;&#x3C;oType&#x3E;4400&#x3C;/oType&#x3E;&#x3C;/parameters&#x3E;&#x3C;columns&#x3E;&#x3C;column width&#x3D;&#x22;250&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Subject&#x22; fieldname&#x3D;&#x22;subject&#x22; entityname&#x3D;&#x22;task&#x22;&#x3E;subject&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Regarding&#x22; fieldname&#x3D;&#x22;regardingobjectid&#x22; entityname&#x3D;&#x22;task&#x22;&#x3E;regardingobjectid&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;150&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Owner&#x22; fieldname&#x3D;&#x22;ownerid&#x22; entityname&#x3D;&#x22;task&#x22;&#x3E;ownerid&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Priority&#x22; fieldname&#x3D;&#x22;prioritycode&#x22; entityname&#x3D;&#x22;task&#x22;&#x3E;prioritycode&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Due Date&#x22; fieldname&#x3D;&#x22;scheduledend&#x22; entityname&#x3D;&#x22;task&#x22;&#x3E;scheduledend&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Status Reason&#x22; fieldname&#x3D;&#x22;statuscode&#x22; entityname&#x3D;&#x22;task&#x22;&#x3E;statuscode&#x3C;/column&#x3E;&#x3C;/columns&#x3E;&#x3C;/grid&#x3E;";

                grid = HttpUtility.HtmlDecode(grid);
                grid = grid.Replace("<pagingCookie/>", "<pagingCookie></pagingCookie>");
                grid = grid.Remove(grid.IndexOf("<viewid>") + 14, 36);
                grid = grid.Insert(grid.IndexOf("<viewid>") + 14, guid);

                if (objectGuid != null)
                {
                    grid = grid.Remove(grid.IndexOf("<oId>") + 11, 36);
                    grid = grid.Insert(grid.IndexOf("<oId>") + 11, objectGuid);
                }

                if (objectType != null)
                {
                    grid = grid.Remove(grid.IndexOf("<oType>") + 7, grid.IndexOf("</oType>") - (grid.IndexOf("<oType>") + 7));
                    grid = grid.Insert(grid.IndexOf("<oType>") + 7, objectType);
                }
                
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }
                return grid;                
            }

            public static string TaskPageView(int pageNum, string guid, string pagingCookie, string otc, string preview)
            {
                string grid = @"<grid><sortColumns>scheduledend&#x3A;1</sortColumns><pageNum>1</pageNum><recsPerPage>50</recsPerPage><dataProvider>Microsoft.Crm.Application.Controls.ActivitiesGridDataProvider</dataProvider><uiProvider>Microsoft.Crm.Application.Controls.AppGridUIProvider</uiProvider><cols/><max>-1</max><refreshAsync>False</refreshAsync><pagingCookie></pagingCookie><enableMultiSort>true</enableMultiSort><enablePagingWhenOnePage>true</enablePagingWhenOnePage><parameters><autorefresh>1</autorefresh><viewid>&#x7B;VIEW_GUID&#x7D;</viewid><viewtype>1039</viewtype><otc>4200</otc><preview>1</preview><quickfind/><filter/><filterDisplay/></parameters></grid>";
                grid = grid.Replace("VIEW_GUID", guid);
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }
                if (otc != null)
                {
                    grid = grid.Replace("<otc>4200", "<otc>" + otc);
                }
                if (preview != null)
                {
                    grid = grid.Replace("<preview>1", "<preview>" + preview);
                }
                return grid;
            }

            public static string TaskPageViewRefresh(int pageNum, string guid, string pagingCookie)
            {
                string grid = @"&#x3C;grid&#x3E;&#x3C;sortColumns&#x3E;scheduledend&#x26;&#x23;x3A&#x3B;1&#x3C;/sortColumns&#x3E;&#x3C;pageNum&#x3E;1&#x3C;/pageNum&#x3E;&#x3C;recsPerPage&#x3E;50&#x3C;/recsPerPage&#x3E;&#x3C;dataProvider&#x3E;Microsoft.Crm.Application.Controls.ActivitiesGridDataProvider&#x3C;/dataProvider&#x3E;&#x3C;uiProvider&#x3E;Microsoft.Crm.Application.Controls.AppGridUIProvider&#x3C;/uiProvider&#x3E;&#x3C;cols/&#x3E;&#x3C;max&#x3E;-1&#x3C;/max&#x3E;&#x3C;refreshAsync&#x3E;False&#x3C;/refreshAsync&#x3E;&#x3C;pagingCookie/&#x3E;&#x3C;enableMultiSort&#x3E;true&#x3C;/enableMultiSort&#x3E;&#x3C;enablePagingWhenOnePage&#x3E;true&#x3C;/enablePagingWhenOnePage&#x3E;&#x3C;parameters&#x3E;&#x3C;autorefresh&#x3E;1&#x3C;/autorefresh&#x3E;&#x3C;viewid&#x3E;&#x26;&#x23;x7B&#x3B;00000000-0000-0000-00AA-000010001899&#x26;&#x23;x7D&#x3B;&#x3C;/viewid&#x3E;&#x3C;viewtype&#x3E;1039&#x3C;/viewtype&#x3E;&#x3C;otc&#x3E;4200&#x3C;/otc&#x3E;&#x3C;preview&#x3E;1&#x3C;/preview&#x3E;&#x3C;/parameters&#x3E;&#x3C;columns&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Activity Type&#x22; fieldname&#x3D;&#x22;activitytypecode&#x22; entityname&#x3D;&#x22;activitypointer&#x22;&#x3E;activitytypecode&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;300&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Subject&#x22; fieldname&#x3D;&#x22;subject&#x22; entityname&#x3D;&#x22;activitypointer&#x22;&#x3E;subject&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Regarding&#x22; fieldname&#x3D;&#x22;regardingobjectid&#x22; entityname&#x3D;&#x22;activitypointer&#x22;&#x3E;regardingobjectid&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Priority&#x22; fieldname&#x3D;&#x22;prioritycode&#x22; entityname&#x3D;&#x22;activitypointer&#x22;&#x3E;prioritycode&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;140&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Start Date&#x22; fieldname&#x3D;&#x22;scheduledstart&#x22; entityname&#x3D;&#x22;activitypointer&#x22;&#x3E;scheduledstart&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;140&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Due Date&#x22; fieldname&#x3D;&#x22;scheduledend&#x22; entityname&#x3D;&#x22;activitypointer&#x22;&#x3E;scheduledend&#x3C;/column&#x3E;&#x3C;/columns&#x3E;&#x3C;/grid&#x3E;";

                grid = HttpUtility.HtmlDecode(grid);
                grid = grid.Replace("<pagingCookie/>", "<pagingCookie></pagingCookie>");
                grid = grid.Remove(grid.IndexOf("<viewid>") + 14, 36);
                grid = grid.Insert(grid.IndexOf("<viewid>") + 14, guid);
                
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }
                return grid;
            }

            public static string TaskPageSearchViewRefresh(int pageNum, string guid, string random, string pagingCookie)
            {
                string grid = @"&#x3C;grid&#x3E;&#x3C;sortColumns&#x3E;subject&#x26;&#x23;x3A&#x3B;1&#x3C;/sortColumns&#x3E;&#x3C;pageNum&#x3E;1&#x3C;/pageNum&#x3E;&#x3C;recsPerPage&#x3E;50&#x3C;/recsPerPage&#x3E;&#x3C;dataProvider&#x3E;Microsoft.Crm.Application.Controls.ActivitiesGridDataProvider&#x3C;/dataProvider&#x3E;&#x3C;uiProvider&#x3E;Microsoft.Crm.Application.Controls.AppGridUIProvider&#x3C;/uiProvider&#x3E;&#x3C;cols/&#x3E;&#x3C;max&#x3E;-1&#x3C;/max&#x3E;&#x3C;refreshAsync&#x3E;False&#x3C;/refreshAsync&#x3E;&#x3C;pagingCookie/&#x3E;&#x3C;enableMultiSort&#x3E;true&#x3C;/enableMultiSort&#x3E;&#x3C;enablePagingWhenOnePage&#x3E;true&#x3C;/enablePagingWhenOnePage&#x3E;&#x3C;parameters&#x3E;&#x3C;autorefresh&#x3E;1&#x3C;/autorefresh&#x3E;&#x3C;viewid&#x3E;&#x26;&#x23;x7B&#x3B;B2397CA2-8672-4863-BC69-B65802061665&#x26;&#x23;x7D&#x3B;&#x3C;/viewid&#x3E;&#x3C;viewtype&#x3E;1039&#x3C;/viewtype&#x3E;&#x3C;otc&#x3E;4212&#x3C;/otc&#x3E;&#x3C;preview&#x3E;1&#x26;&#x23;x2C&#x3B;1&#x26;&#x23;x2C&#x3B;1&#x26;&#x23;x2C&#x3B;1&#x3C;/preview&#x3E;&#x3C;quickfind&#x3E;a&#x3C;/quickfind&#x3E;&#x3C;filter/&#x3E;&#x3C;filterDisplay/&#x3E;&#x3C;scheduledend&#x3E;All&#x3C;/scheduledend&#x3E;&#x3C;/parameters&#x3E;&#x3C;columns&#x3E;&#x3C;column width&#x3D;&#x22;250&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Subject&#x22; fieldname&#x3D;&#x22;subject&#x22; entityname&#x3D;&#x22;task&#x22;&#x3E;subject&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Activity Status&#x22; fieldname&#x3D;&#x22;statecode&#x22; entityname&#x3D;&#x22;task&#x22;&#x3E;statecode&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Priority&#x22; fieldname&#x3D;&#x22;prioritycode&#x22; entityname&#x3D;&#x22;task&#x22;&#x3E;prioritycode&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Due Date&#x22; fieldname&#x3D;&#x22;scheduledend&#x22; entityname&#x3D;&#x22;task&#x22;&#x3E;scheduledend&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Created By&#x22; fieldname&#x3D;&#x22;createdby&#x22; entityname&#x3D;&#x22;task&#x22;&#x3E;createdby&#x3C;/column&#x3E;&#x3C;column width&#x3D;&#x22;100&#x22; isHidden&#x3D;&#x22;false&#x22; isMetadataBound&#x3D;&#x22;true&#x22; isSortable&#x3D;&#x22;true&#x22; label&#x3D;&#x22;Regarding&#x22; fieldname&#x3D;&#x22;regardingobjectid&#x22; entityname&#x3D;&#x22;task&#x22;&#x3E;regardingobjectid&#x3C;/column&#x3E;&#x3C;/columns&#x3E;&#x3C;/grid&#x3E;";

                grid = HttpUtility.HtmlDecode(grid);
                grid = grid.Replace("<pagingCookie/>", "<pagingCookie></pagingCookie>");
                grid = grid.Remove(grid.IndexOf("<viewid>") + 14, 36);
                grid = grid.Insert(grid.IndexOf("<viewid>") + 14, guid);

                grid = grid.Remove(grid.IndexOf("<quickfind>") + 11, grid.IndexOf("</quickfind>") - (grid.IndexOf("<quickfind>") + 11));
                grid = grid.Insert(grid.IndexOf("<quickfind>") + 11, random);
                
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }
                return grid;
            }

            public static string TaskSearchView(int pageNum, string guid, string random, string pagingCookie, string otc, string preview)
            {
                string grid = @"<grid><sortColumns>subject&#x3A;1</sortColumns><pageNum>1</pageNum><recsPerPage>50</recsPerPage><dataProvider>Microsoft.Crm.Application.Controls.ActivitiesGridDataProvider</dataProvider><uiProvider>Microsoft.Crm.Application.Controls.AppGridUIProvider</uiProvider><cols/><max>-1</max><refreshAsync>False</refreshAsync><pagingCookie></pagingCookie><enableMultiSort>true</enableMultiSort><enablePagingWhenOnePage>true</enablePagingWhenOnePage><parameters><autorefresh>1</autorefresh><viewid>&#x7B;VIEW_GUID&#x7D;</viewid><viewtype>1039</viewtype><otc>4212</otc><preview>1</preview><quickfind>RANDOM_STRING</quickfind><filter/><filterDisplay/><scheduledend>All</scheduledend></parameters></grid>";
                //string grid = @"<grid><sortColumns>subject&#x3A;1</sortColumns><pageNum>1</pageNum><recsPerPage>50</recsPerPage><dataProvider>Microsoft.Crm.Application.Controls.ActivitiesGridDataProvider</dataProvider><uiProvider>Microsoft.Crm.Application.Controls.AppGridUIProvider</uiProvider><cols/><max>-1</max><refreshAsync>False</refreshAsync><pagingCookie></pagingCookie><enableMultiSort>true</enableMultiSort><enablePagingWhenOnePage>true</enablePagingWhenOnePage><parameters><autorefresh>1</autorefresh><viewid>&#x7B;VIEW_GUID&#x7D;</viewid><viewtype>1039</viewtype><otc>4200</otc><preview>1</preview><quickfind>" + Utils.GetRandomString(2, 2) + "</quickfind><filter/><filterDisplay/><scheduledend>All</scheduledend></parameters></grid>";
                grid = grid.Replace("VIEW_GUID", guid);
                grid = grid.Replace("RANDOM_STRING", random);
                if (pagingCookie != null)
                {
                    grid = grid.Replace("<pagingCookie>", "<pagingCookie>" + pagingCookie);
                }
                if (pageNum != -1)
                {
                    grid = grid.Replace("<pageNum>1", "<pageNum>" + pageNum.ToString());
                }
                if (otc != null)
                {
                    grid = grid.Replace("<otc>4200", "<otc>" + otc);
                }
                if (preview != null)
                {
                    grid = grid.Replace("<preview>1", "<preview>" + preview);
                }

                return grid;
            }

            public static string UserRolePageRefresh(string userGuid)
            {
                string xmlString = @"<gridXml><grid><sortColumns>name&#58;1</sortColumns><pageNum>1</pageNum><recsPerPage>50</recsPerPage><dataProvider>Microsoft.Crm.Application.Controls.AppGridDataProvider</dataProvider><uiProvider>Microsoft.Crm.Application.Controls.AppGridUIProvider</uiProvider><cols/><max>-1</max><refreshAsync>False</refreshAsync><pagingCookie></pagingCookie><enableMultiSort>true</enableMultiSort><enablePagingWhenOnePage>true</enablePagingWhenOnePage><parameters><autorefresh>1</autorefresh><viewid>&#123;00000000-0000-0000-00AA-000010001208&#125;</viewid><viewtype>1039</viewtype><otc>1036</otc><queryapi/><showjumpbar>0</showjumpbar><enablepaging>1</enablepaging><oId>_USER_GUID_</oId><oType>8</oType><relName/><roleOrd>-1</roleOrd><isCustomRelationship>false</isCustomRelationship></parameters><columns><column width=""300"" isHidden=""false"" isMetadataBound=""true"" isSortable=""true"" label=""Name"" fieldname=""name"" entityname=""role"">name</column><column width=""300"" isHidden=""false"" isMetadataBound=""true"" isSortable=""true"" label=""Business Unit"" fieldname=""businessunitid"" entityname=""role"">businessunitid</column></columns></grid></gridXml>";
                
                xmlString = xmlString.Replace("_USER_GUID_", userGuid);

                return xmlString;
            }
    	}
    
        public class Helper
        {
            public static string privilegeXml()
            {
                string[] identifiers = new string[] { "prvCreateWorkflowProcessInstance", "prvReadWorkflowProcessInstance", "prvAppendToUserSettings", "prvDeleteUserSettings", "prvWriteUserSettings", "prvReadOrganizationUI", "prvCreateUserSettings", "prvReadUserSettings", "prvReadArticleTemplate", "prvReadContractTemplate", "prvCreateAccount", "prvReadAccount", "prvWriteAccount", "prvDeleteAccount", "prvAppendAccount", "prvCreateContact", "prvAppendToAccount", "prvAssignAccount", "prvShareAccount", "prvReadContact", "prvWriteContact", "prvDeleteContact", "prvAppendContact", "prvAppendToContact", "prvAssignContact", "prvShareContact", "prvCreateLead", "prvReadLead", "prvWriteLead", "prvDeleteLead", "prvAppendLead", "prvAppendToLead", "prvAssignLead", "prvShareLead", "prvCreateOpportunity", "prvReadOpportunity", "prvWriteOpportunity", "prvDeleteOpportunity", "prvAppendOpportunity", "prvAppendToOpportunity", "prvAssignOpportunity", "prvShareOpportunity", "prvCreateActivity", "prvReadActivity", "prvWriteActivity", "prvDeleteActivity", "prvAppendActivity", "prvAppendToActivity", "prvAssignActivity", "prvShareActivity", "prvCreateNote", "prvReadNote", "prvWriteNote", "prvDeleteNote", "prvAppendNote", "prvAppendToNote", "prvAssignNote", "prvShareNote", "prvCreateEmailTemplate", "prvReadEmailTemplate", "prvWriteEmailTemplate", "prvDeleteEmailTemplate", "prvAppendEmailTemplate", "prvAssignEmailTemplate", "prvShareEmailTemplate", "prvReadNewsArticle", "prvReadSubject", "prvReadQueue", "prvCreateUserQuery", "prvReadUserQuery", "prvDeleteUserQuery", "prvWriteUserQuery", "prvAssignUserQuery", "prvShareUserQuery", "prvCreateBulkImport", "prvReadBulkImport", "prvWriteBulkImport", "prvDeleteBulkImport", "prvAppendBulkImport", "prvAppendToBulkImport", "prvAssignBulkImport", "prvShareBulkImport", "prvCreateImportConfig", "prvReadImportConfig", "prvWriteImportConfig", "prvWriteImportConfig", "prvAppendImportConfig", "prvAppendToImportConfig", "prvAssignImportConfig", "prvShareImportConfig", "prvCreateCustomerOpportunityRole", "prvReadCustomerOpportunityRole", "prvWriteCustomerOpportunityRole", "prvDeleteCustomerOpportunityRole", "prvAppendCustomerOpportunityRole", "prvReadRelationshipRole", "prvCreateCustomerRelationship", "prvReadCustomerRelationship", "prvWriteCustomerRelationship", "prvDeleteCustomerRelationship", "prvAppendCustomerRelationship", "prvAppendToCustomerRelationship", "prvCreateList", "prvReadList", "prvWriteList", "prvDeleteList", "prvAppendList", "prvAppendToList", "prvAssignList", "prvShareList", "prvCreateCampaign", "prvReadCampaign", "prvWriteCampaign", "prvDeleteCampaign", "prvAppendCampaign", "prvAppendToCampaign", "prvAssignCampaign", "prvShareCampaign", "prvReadProduct", "prvAppendProduct", "prvAppendToProduct", "prvReadCompetitor", "prvReadSalesLiterature", "prvAppendSalesLiterature", "prvAppendToSalesLiterature", "prvCreateQuote", "prvReadQuote", "prvWriteQuote", "prvDeleteQuote", "prvAppendQuote", "prvAppendToQuote", "prvAssignQuote", "prvShareQuote", "prvCreateOrder", "prvReadOrder", "prvWriteOrder", "prvDeleteOrder", "prvAppendOrder", "prvAppendToOrder", "prvAssignOrder", "prvShareOrder", "prvCreateInvoice", "prvReadInvoice", "prvWriteInvoice", "prvDeleteInvoice", "prvAppendInvoice", "prvAppendToInvoice", "prvAssignInvoice", "prvShareInvoice", "prvReadTerritory", "prvReadArticle", "prvWriteArticle", "prvCreateIncident", "prvReadIncident", "prvWriteIncident", "prvDeleteIncident", "prvAppendIncident", "prvAppendToIncident", "prvAssignIncident", "prvShareIncident", "prvCreateContract", "prvReadContract", "prvWriteContract", "prvAppendContract", "prvAppendToContract", "prvAssignContract", "prvShareContract", "prvReadOrganization", "prvReadBusinessUnit", "prvReadUser", "prvAppendToUser", "prvReadTeam", "prvReadRole", "prvReadLicense", "prvBulkEdit", "prvQuickCreate", "prvPrint", "prvExportToExcel", "prvMerge", "prvMailMerge", "prvGoOffline", "prvSyncToOutlook", "prvAddressBook", "prvGoMobile", "prvAppendCalendar", "prvAppendToCalendar", "prvCreateOwnCalendar", "prvReadOwnCalendar", "prvWriteOwnCalendar", "prvDeleteOwnCalendar", "prvReadService", "prvAppendService", "prvAppendToService", "prvReadEquipment", "prvAppendToEquipment", "prvReadSite", "prvAppendToSite", "prvSearchAvailability", "prvReadCustomization", "prvReadQuery", "prvReadWorkflowProcess", "prvAppendToQuery", "prvShareCustomerRelationship", "prvReadAttributeMap", "prvReadEntityMap", "prvShareCustomerOpportunityRole", "prvAssignCustomerRelationship", "prvAppendToCustomerOpportunityRole", "prvAssignCustomerOpportunityRole", "prvDeleteImportConfig" };
                string xml = @"<role><privilegeinfo>&#x7B;prvCreateAccount&#x7D;&#x3B;B&#x3B;&#x7B;prvReadAccount&#x7D;&#x3B;G&#x3B;&#x7B;prvWriteAccount&#x7D;&#x3B;G&#x3B;&#x7B;prvDeleteAccount&#x7D;&#x3B;B&#x3B;&#x7B;prvAppendAccount&#x7D;&#x3B;G&#x3B;&#x7B;prvAppendToAccount&#x7D;&#x3B;G&#x3B;&#x7B;prvAssignAccount&#x7D;&#x3B;B&#x3B;&#x7B;prvShareAccount&#x7D;&#x3B;G&#x3B;&#x7B;prvCreateContact&#x7D;&#x3B;B&#x3B;&#x7B;prvReadContact&#x7D;&#x3B;G&#x3B;&#x7B;prvWriteContact&#x7D;&#x3B;G&#x3B;&#x7B;prvDeleteContact&#x7D;&#x3B;B&#x3B;&#x7B;prvAppendContact&#x7D;&#x3B;G&#x3B;&#x7B;prvAppendToContact&#x7D;&#x3B;G&#x3B;&#x7B;prvAssignContact&#x7D;&#x3B;B&#x3B;&#x7B;prvShareContact&#x7D;&#x3B;G&#x3B;&#x7B;prvCreateLead&#x7D;&#x3B;B&#x3B;&#x7B;prvReadLead&#x7D;&#x3B;G&#x3B;&#x7B;prvWriteLead&#x7D;&#x3B;G&#x3B;&#x7B;prvDeleteLead&#x7D;&#x3B;B&#x3B;&#x7B;prvAppendLead&#x7D;&#x3B;G&#x3B;&#x7B;prvAppendToLead&#x7D;&#x3B;G&#x3B;&#x7B;prvAssignLead&#x7D;&#x3B;B&#x3B;&#x7B;prvShareLead&#x7D;&#x3B;G&#x3B;&#x7B;prvCreateOpportunity&#x7D;&#x3B;B&#x3B;&#x7B;prvReadOpportunity&#x7D;&#x3B;G&#x3B;&#x7B;prvWriteOpportunity&#x7D;&#x3B;G&#x3B;&#x7B;prvDeleteOpportunity&#x7D;&#x3B;B&#x3B;&#x7B;prvAppendOpportunity&#x7D;&#x3B;G&#x3B;&#x7B;prvAppendToOpportunity&#x7D;&#x3B;G&#x3B;&#x7B;prvAssignOpportunity&#x7D;&#x3B;B&#x3B;&#x7B;prvShareOpportunity&#x7D;&#x3B;G&#x3B;&#x7B;prvCreateActivity&#x7D;&#x3B;B&#x3B;&#x7B;prvReadActivity&#x7D;&#x3B;G&#x3B;&#x7B;prvWriteActivity&#x7D;&#x3B;L&#x3B;&#x7B;prvDeleteActivity&#x7D;&#x3B;B&#x3B;&#x7B;prvAppendActivity&#x7D;&#x3B;L&#x3B;&#x7B;prvAppendToActivity&#x7D;&#x3B;L&#x3B;&#x7B;prvAssignActivity&#x7D;&#x3B;B&#x3B;&#x7B;prvShareActivity&#x7D;&#x3B;G&#x3B;&#x7B;prvCreateNote&#x7D;&#x3B;B&#x3B;&#x7B;prvReadNote&#x7D;&#x3B;G&#x3B;&#x7B;prvWriteNote&#x7D;&#x3B;B&#x3B;&#x7B;prvDeleteNote&#x7D;&#x3B;B&#x3B;&#x7B;prvAppendNote&#x7D;&#x3B;L&#x3B;&#x7B;prvAppendToNote&#x7D;&#x3B;L&#x3B;&#x7B;prvAssignNote&#x7D;&#x3B;B&#x3B;&#x7B;prvShareNote&#x7D;&#x3B;G&#x3B;&#x7B;prvCreateEmailTemplate&#x7D;&#x3B;B&#x3B;&#x7B;prvReadEmailTemplate&#x7D;&#x3B;G&#x3B;&#x7B;prvWriteEmailTemplate&#x7D;&#x3B;B&#x3B;&#x7B;prvDeleteEmailTemplate&#x7D;&#x3B;B&#x3B;&#x7B;prvAppendEmailTemplate&#x7D;&#x3B;L&#x3B;&#x7B;prvAssignEmailTemplate&#x7D;&#x3B;L&#x3B;&#x7B;prvShareEmailTemplate&#x7D;&#x3B;L&#x3B;&#x7B;prvReadNewsArticle&#x7D;&#x3B;G&#x3B;&#x7B;prvReadSubject&#x7D;&#x3B;G&#x3B;&#x7B;prvReadQueue&#x7D;&#x3B;G&#x3B;&#x7B;prvCreateUserQuery&#x7D;&#x3B;B&#x3B;&#x7B;prvReadUserQuery&#x7D;&#x3B;B&#x3B;&#x7B;prvWriteUserQuery&#x7D;&#x3B;B&#x3B;&#x7B;prvDeleteUserQuery&#x7D;&#x3B;B&#x3B;&#x7B;prvAssignUserQuery&#x7D;&#x3B;B&#x3B;&#x7B;prvShareUserQuery&#x7D;&#x3B;B&#x3B;&#x7B;prvCreateBulkImport&#x7D;&#x3B;B&#x3B;&#x7B;prvReadBulkImport&#x7D;&#x3B;G&#x3B;&#x7B;prvWriteBulkImport&#x7D;&#x3B;L&#x3B;&#x7B;prvDeleteBulkImport&#x7D;&#x3B;B&#x3B;&#x7B;prvAppendBulkImport&#x7D;&#x3B;L&#x3B;&#x7B;prvAppendToBulkImport&#x7D;&#x3B;L&#x3B;&#x7B;prvAssignBulkImport&#x7D;&#x3B;B&#x3B;&#x7B;prvShareBulkImport&#x7D;&#x3B;D&#x3B;&#x7B;prvCreateImportConfig&#x7D;&#x3B;B&#x3B;&#x7B;prvReadImportConfig&#x7D;&#x3B;G&#x3B;&#x7B;prvWriteImportConfig&#x7D;&#x3B;L&#x3B;&#x7B;prvDeleteImportConfig&#x7D;&#x3B;B&#x3B;&#x7B;prvAppendImportConfig&#x7D;&#x3B;D&#x3B;&#x7B;prvAppendToImportConfig&#x7D;&#x3B;D&#x3B;&#x7B;prvAssignImportConfig&#x7D;&#x3B;B&#x3B;&#x7B;prvShareImportConfig&#x7D;&#x3B;B&#x3B;&#x7B;prvCreateCustomerOpportunityRole&#x7D;&#x3B;B&#x3B;&#x7B;prvReadCustomerOpportunityRole&#x7D;&#x3B;G&#x3B;&#x7B;prvWriteCustomerOpportunityRole&#x7D;&#x3B;B&#x3B;&#x7B;prvDeleteCustomerOpportunityRole&#x7D;&#x3B;B&#x3B;&#x7B;prvAppendCustomerOpportunityRole&#x7D;&#x3B;G&#x3B;&#x7B;prvReadRelationshipRole&#x7D;&#x3B;G&#x3B;&#x7B;prvCreateCustomerRelationship&#x7D;&#x3B;B&#x3B;&#x7B;prvReadCustomerRelationship&#x7D;&#x3B;G&#x3B;&#x7B;prvWriteCustomerRelationship&#x7D;&#x3B;B&#x3B;&#x7B;prvDeleteCustomerRelationship&#x7D;&#x3B;B&#x3B;&#x7B;prvAppendCustomerRelationship&#x7D;&#x3B;G&#x3B;&#x7B;prvAppendToCustomerRelationship&#x7D;&#x3B;G&#x3B;&#x7B;prvCreateList&#x7D;&#x3B;B&#x3B;&#x7B;prvReadList&#x7D;&#x3B;G&#x3B;&#x7B;prvWriteList&#x7D;&#x3B;D&#x3B;&#x7B;prvDeleteList&#x7D;&#x3B;B&#x3B;&#x7B;prvAppendList&#x7D;&#x3B;D&#x3B;&#x7B;prvAppendToList&#x7D;&#x3B;D&#x3B;&#x7B;prvAssignList&#x7D;&#x3B;B&#x3B;&#x7B;prvShareList&#x7D;&#x3B;D&#x3B;&#x7B;prvCreateCampaign&#x7D;&#x3B;B&#x3B;&#x7B;prvReadCampaign&#x7D;&#x3B;G&#x3B;&#x7B;prvWriteCampaign&#x7D;&#x3B;L&#x3B;&#x7B;prvDeleteCampaign&#x7D;&#x3B;B&#x3B;&#x7B;prvAppendCampaign&#x7D;&#x3B;L&#x3B;&#x7B;prvAppendToCampaign&#x7D;&#x3B;L&#x3B;&#x7B;prvAssignCampaign&#x7D;&#x3B;B&#x3B;&#x7B;prvShareCampaign&#x7D;&#x3B;D&#x3B;&#x7B;prvReadProduct&#x7D;&#x3B;G&#x3B;&#x7B;prvAppendProduct&#x7D;&#x3B;G&#x3B;&#x7B;prvAppendToProduct&#x7D;&#x3B;G&#x3B;&#x7B;prvReadCompetitor&#x7D;&#x3B;G&#x3B;&#x7B;prvReadSalesLiterature&#x7D;&#x3B;G&#x3B;&#x7B;prvAppendSalesLiterature&#x7D;&#x3B;G&#x3B;&#x7B;prvAppendToSalesLiterature&#x7D;&#x3B;G&#x3B;&#x7B;prvCreateQuote&#x7D;&#x3B;G&#x3B;&#x7B;prvReadQuote&#x7D;&#x3B;G&#x3B;&#x7B;prvWriteQuote&#x7D;&#x3B;G&#x3B;&#x7B;prvDeleteQuote&#x7D;&#x3B;G&#x3B;&#x7B;prvAppendQuote&#x7D;&#x3B;G&#x3B;&#x7B;prvAppendToQuote&#x7D;&#x3B;G&#x3B;&#x7B;prvAssignQuote&#x7D;&#x3B;G&#x3B;&#x7B;prvShareQuote&#x7D;&#x3B;G&#x3B;&#x7B;prvCreateOrder&#x7D;&#x3B;G&#x3B;&#x7B;prvReadOrder&#x7D;&#x3B;G&#x3B;&#x7B;prvWriteOrder&#x7D;&#x3B;G&#x3B;&#x7B;prvDeleteOrder&#x7D;&#x3B;G&#x3B;&#x7B;prvAppendOrder&#x7D;&#x3B;G&#x3B;&#x7B;prvAppendToOrder&#x7D;&#x3B;G&#x3B;&#x7B;prvAssignOrder&#x7D;&#x3B;G&#x3B;&#x7B;prvShareOrder&#x7D;&#x3B;G&#x3B;&#x7B;prvCreateInvoice&#x7D;&#x3B;G&#x3B;&#x7B;prvReadInvoice&#x7D;&#x3B;G&#x3B;&#x7B;prvWriteInvoice&#x7D;&#x3B;G&#x3B;&#x7B;prvDeleteInvoice&#x7D;&#x3B;G&#x3B;&#x7B;prvAppendInvoice&#x7D;&#x3B;G&#x3B;&#x7B;prvAppendToInvoice&#x7D;&#x3B;G&#x3B;&#x7B;prvAssignInvoice&#x7D;&#x3B;G&#x3B;&#x7B;prvShareInvoice&#x7D;&#x3B;G&#x3B;&#x7B;prvReadTerritory&#x7D;&#x3B;G&#x3B;&#x7B;prvReadArticle&#x7D;&#x3B;G&#x3B;&#x7B;prvWriteArticle&#x7D;&#x3B;G&#x3B;&#x7B;prvReadArticleTemplate&#x7D;&#x3B;G&#x3B;&#x7B;prvCreateIncident&#x7D;&#x3B;G&#x3B;&#x7B;prvReadIncident&#x7D;&#x3B;G&#x3B;&#x7B;prvWriteIncident&#x7D;&#x3B;G&#x3B;&#x7B;prvDeleteIncident&#x7D;&#x3B;G&#x3B;&#x7B;prvAppendIncident&#x7D;&#x3B;G&#x3B;&#x7B;prvAppendToIncident&#x7D;&#x3B;G&#x3B;&#x7B;prvAssignIncident&#x7D;&#x3B;G&#x3B;&#x7B;prvShareIncident&#x7D;&#x3B;G&#x3B;&#x7B;prvCreateContract&#x7D;&#x3B;L&#x3B;&#x7B;prvReadContract&#x7D;&#x3B;G&#x3B;&#x7B;prvWriteContract&#x7D;&#x3B;G&#x3B;&#x7B;prvAppendContract&#x7D;&#x3B;L&#x3B;&#x7B;prvAppendToContract&#x7D;&#x3B;L&#x3B;&#x7B;prvAssignContract&#x7D;&#x3B;B&#x3B;&#x7B;prvShareContract&#x7D;&#x3B;G&#x3B;&#x7B;prvReadContractTemplate&#x7D;&#x3B;G&#x3B;&#x7B;prvReadOrganization&#x7D;&#x3B;G&#x3B;&#x7B;prvReadBusinessUnit&#x7D;&#x3B;G&#x3B;&#x7B;prvReadUser&#x7D;&#x3B;G&#x3B;&#x7B;prvAppendToUser&#x7D;&#x3B;L&#x3B;&#x7B;prvCreateUserSettings&#x7D;&#x3B;L&#x3B;&#x7B;prvReadUserSettings&#x7D;&#x3B;G&#x3B;&#x7B;prvWriteUserSettings&#x7D;&#x3B;B&#x3B;&#x7B;prvReadTeam&#x7D;&#x3B;G&#x3B;&#x7B;prvReadRole&#x7D;&#x3B;L&#x3B;&#x7B;prvReadLicense&#x7D;&#x3B;G&#x3B;&#x7B;prvBulkEdit&#x7D;&#x3B;G&#x3B;&#x7B;prvQuickCreate&#x7D;&#x3B;G&#x3B;&#x7B;prvPrint&#x7D;&#x3B;G&#x3B;&#x7B;prvExportToExcel&#x7D;&#x3B;G&#x3B;&#x7B;prvMerge&#x7D;&#x3B;G&#x3B;&#x7B;prvMailMerge&#x7D;&#x3B;G&#x3B;&#x7B;prvGoOffline&#x7D;&#x3B;G&#x3B;&#x7B;prvSyncToOutlook&#x7D;&#x3B;G&#x3B;&#x7B;prvAddressBook&#x7D;&#x3B;G&#x3B;&#x7B;prvGoMobile&#x7D;&#x3B;G&#x3B;&#x7B;prvAppendCalendar&#x7D;&#x3B;L&#x3B;&#x7B;prvAppendToCalendar&#x7D;&#x3B;G&#x3B;&#x7B;prvCreateOwnCalendar&#x7D;&#x3B;G&#x3B;&#x7B;prvReadOwnCalendar&#x7D;&#x3B;G&#x3B;&#x7B;prvWriteOwnCalendar&#x7D;&#x3B;G&#x3B;&#x7B;prvDeleteOwnCalendar&#x7D;&#x3B;G&#x3B;&#x7B;prvReadService&#x7D;&#x3B;G&#x3B;&#x7B;prvAppendService&#x7D;&#x3B;G&#x3B;&#x7B;prvAppendToService&#x7D;&#x3B;G&#x3B;&#x7B;prvReadEquipment&#x7D;&#x3B;G&#x3B;&#x7B;prvAppendToEquipment&#x7D;&#x3B;G&#x3B;&#x7B;prvReadSite&#x7D;&#x3B;G&#x3B;&#x7B;prvAppendToSite&#x7D;&#x3B;G&#x3B;&#x7B;prvSearchAvailability&#x7D;&#x3B;G&#x3B;&#x7B;prvReadCustomization&#x7D;&#x3B;G&#x3B;&#x7B;prvReadQuery&#x7D;&#x3B;G&#x3B;&#x7B;prvReadWorkflowProcess&#x7D;&#x3B;G&#x3B;&#x7B;prvCreateWorkflowProcessInstance&#x7D;&#x3B;L&#x3B;&#x7B;prvReadWorkflowProcessInstance&#x7D;&#x3B;G&#x3B;&#x7B;prvAppendToQuery&#x7D;&#x3B;G&#x3B;&#x7B;prvShareCustomerRelationship&#x7D;&#x3B;G&#x3B;&#x7B;prvReadAttributeMap&#x7D;&#x3B;G&#x3B;&#x7B;prvReadEntityMap&#x7D;&#x3B;G&#x3B;&#x7B;prvReadOrganizationUI&#x7D;&#x3B;G&#x3B;&#x7B;prvShareCustomerOpportunityRole&#x7D;&#x3B;G&#x3B;&#x7B;prvAssignCustomerRelationship&#x7D;&#x3B;B&#x3B;&#x7B;prvAppendToCustomerOpportunityRole&#x7D;&#x3B;G&#x3B;&#x7B;prvDeleteUserSettings&#x7D;&#x3B;B&#x3B;&#x7B;prvAssignCustomerOpportunityRole&#x7D;&#x3B;B&#x3B;&#x7B;prvAppendToUserSettings&#x7D;&#x3B;B</privilegeinfo></role>";
    
                for (int i = 0; i < identifiers.Length; i++)
                {
    				EntityRequest er = new EntityRequest();
    				er.Type = EntityNames.Privileges;
    				er.ReturnAs = EntityNames.Privileges;
    				er.ParentID = EntityManager.Instance.OrgId;
    
    				er.Props = new System.Collections.Hashtable();
    				er.Props.Add("name", identifiers[i]);
    
    				System.Diagnostics.Trace.WriteLine("Looking for priv: " + identifiers[i]);
    				CRMEntity priv = (CRMEntity)(EntityManager.Instance.GetEntities(er)[EntityNames.Privileges]);
    
    				EntityManager.Instance.FreeEntity(priv);
    
                    if (priv == null)
                    {
                        // Something wrong.
                        throw new Exception("Unable to find entity for the privilege" + identifiers[i]);
                    }
                    else
                    {
                        xml = xml.Replace(identifiers[i], priv["privilegeid"]);
                    }
                }
                return xml;
            }
    
            public static XmlNode GenerateEntityXml()
            {    
                string dataXml = @"<entity><entityid></entityid><singularname></singularname><pluralname></pluralname><schemaname></schemaname><ownershiptype>UserOwned</ownershiptype><description></description><displayareas><displayarea id=""Workplace"" displayed=""false""/><displayarea id=""SFA"" displayed=""true""/><displayarea id=""MA"" displayed=""false""/><displayarea id=""CS"" displayed=""false""/><displayarea id=""Settings"" displayed=""false""/><displayarea id=""ResourceCenter"" displayed=""false""/></displayareas><richclient><offline>false</offline></richclient><duplicatecheck>true</duplicatecheck><associatedentity><notes>true</notes><activities>true</activities></associatedentity><attribute><displayname>Name</displayname><schemaname>name</schemaname><reqlevel>Required</reqlevel><description>The name of the custom entity.</description><type><name>nvarchar</name><format>text</format><length>100</length></type></attribute></entity>";
  
                string name = Utils.GetRandomString(5, 10);
                string description = Utils.GetRandomString(100, 120);
  
                dataXml = dataXml.Replace("<singularname>", "<singularname>" + name);
                dataXml = dataXml.Replace("<pluralname>", "<pluralname>" + name);
                dataXml = dataXml.Replace("<schemaname>", "<schemaname>" + name);
                dataXml = dataXml.Replace("<description>", "<description>" + description);
        
                XmlDocument data = new XmlDocument();
                data.LoadXml(dataXml);
                return (data);
            }

            public static XmlNode GeneratePublishXml(Guid entityId)
            {
                // use a validating reader to read the grid xml fragment and pull in the namespace
                NameTable nt = new NameTable();
                XmlNamespaceManager nsmgr = new XmlNamespaceManager(nt);
                nsmgr.AddNamespace(String.Empty, ConfigSettings.Default.WebServiceNS);

                XmlParserContext context = new XmlParserContext(null, nsmgr, null, XmlSpace.None);

                string dataXml = "<publish><entities><entity>" + entityId.ToString("B") + "</entity></entities></publish>";

                XmlTextReader reader = new XmlTextReader(dataXml,
                    XmlNodeType.Element, context);

                XmlDocument data = new XmlDocument();
                data.Load(reader);
                return (data);
            }
    
            public static XmlNode GenerateAttributeXml(Guid entityId)
            {
                // use a validating reader to read the grid xml fragment and pull in the namespace
                NameTable nt = new NameTable();
                XmlNamespaceManager nsmgr = new XmlNamespaceManager(nt);
                nsmgr.AddNamespace(String.Empty, ConfigSettings.Default.WebServiceNS);
    
                XmlParserContext context = new XmlParserContext(null, nsmgr, null, XmlSpace.None);
                string name = Utils.GetRandomString(5,7);
    
                string dataXml = "<attribute><entityid>" + entityId.ToString("B") + "</entityid><attributeid></attributeid><displayname>" + name + "</displayname><schemaname>" + name + "</schemaname><reqlevel>None</reqlevel><imemode>auto</imemode><description>test cust atttribute</description><type><name>nvarchar</name><format>text</format><length>100</length></type></attribute>";
    
                XmlTextReader reader = new XmlTextReader(dataXml,
                    XmlNodeType.Element, context);
    
                XmlDocument data = new XmlDocument();
                data.Load(reader);
                return (data);
            }
        }

      class BandWidthUtilities
      {
          //all input parameters follow the NetCap command line help  see description by netcap /?
          public static bool StartCapture(
              string path,
              string sMbyte,
              string sCapFile,
              string sFilter,
              string sNIC,
              string sOffset,
              string sTrigStr
              )
          {
              try
              {
                  string strProgName = "NetCap";

                  //clear the existing same process(es) first
                  Process[] iProcesses = Process.GetProcessesByName(strProgName);
                  if (iProcesses.Length > 0)
                  {
                      foreach (Process iProcess in iProcesses)
                      {
                          iProcess.Kill();
                      }
                  }

                  //e.g. sOffset = '1e' or '1a', then sTrigStr = 01010101
                  string strArgs = "/B:" + sMbyte + " /C:\"" + sCapFile + "\" /T P "
                      + sOffset + " " + sTrigStr + " /N:" + sNIC + " /F:" + sFilter;
                  return StartProgram(strProgName, strArgs, path, true);
              }
              catch (Exception e)
              {
                  WriteErrorLog(e.ToString());
                  return false;
              }
          }


          public static bool StopCapture(string sCommandName, string sParams)
          {
              //e.g. sCommandName = "Ping"; sParams = "-n 1  1.1.1.1";
              return StartProgram(sCommandName, sParams, null, true);
          }


          public static bool StartProgram(string exe, string sArgs, string path, bool waitForExit)
          {
              try
              {
                  Process MyProcess = new Process();

                  MyProcess.StartInfo.CreateNoWindow = true;
                  MyProcess.StartInfo.UseShellExecute = false;
                  MyProcess.StartInfo.FileName = exe;
                  if (path != string.Empty)
                      MyProcess.StartInfo.WorkingDirectory = path;

                  if (sArgs.Trim().ToLower() != "null")
                      MyProcess.StartInfo.Arguments = sArgs;

                  if (MyProcess.Start() == true)
                  {
                      if (waitForExit)
                      {
                          MyProcess.WaitForExit();
                          return true;
                      }
                      return true;
                  }
              }
              catch (Exception e)
              {
                  //write to error log
                  WriteErrorLog("failed in StartProgram, exception: " + e.ToString());

              }
              return false;
          }

          //only two cases in code block below return true at this point
          public static bool RunNetMonExpert(string ExpertName, string CapFileFullName)
          {
              string Exe = "\"" + Environment.GetEnvironmentVariable("ProgramFiles") +
                  "\\Microsoft Network Monitor\\RunExpert.exe\"";

              string ProgArgs = null;
              string SrcFolder = Environment.GetEnvironmentVariable("APPDATA");
              string VRTAFolder = Environment.GetEnvironmentVariable("ProgramFiles") + "\\VRTA";
              string destFolder = Path.GetDirectoryName(CapFileFullName);
              if (destFolder == "")
                  destFolder = ".";

              Console.WriteLine("destFolder:" + destFolder);
              string fileName = Path.GetFileNameWithoutExtension(CapFileFullName);

              try
              {
                  if (ExpertName.ToLower() == "rta")
                  {
                      ProgArgs = "-E:RTAnalyze -C:\"" + CapFileFullName + "\"";
                      if (StartProgram(Exe, ProgArgs, string.Empty, true))
                      {
                          File.Copy(SrcFolder + "\\RTA\\RTA.txt", destFolder + "\\" + fileName + "_rta.txt", true);
                          return true;
                      }
                      else
                          WriteErrorLog("Run RTA failed! " + CapFileFullName + ".cap");
                  }
                  else if (ExpertName.ToLower() == "vrta")
                  {
                      ProgArgs = "-E:BWAnalyze -C:\"" + CapFileFullName + "\"";
                      if (StartProgram(Exe, ProgArgs, string.Empty, true))
                      {
                          //copy files to VRTA's working directory
                          File.Copy(SrcFolder + "\\BW\\BW.txt", VRTAFolder + "\\bw.txt", true);
                          File.Copy(SrcFolder + "\\RTA\\RTA.txt", VRTAFolder + "\\rta.txt", true);
                          if (Directory.Exists(VRTAFolder + "\\output"))
                              Directory.Delete(VRTAFolder + "\\output", true);
                          if (StartProgram(VRTAFolder + "\\vrta.exe", "/q", string.Empty, true))
                          {
                              //this part of code is not strict enough...
                              string[] FileNames = Directory.GetFiles(VRTAFolder + "\\output");
                              foreach (string filename in FileNames)
                              {
                                  File.Copy(filename, destFolder + "\\" + fileName + ".gif", true);
                                  File.Delete(filename);
                              }
                              //recopy the sorted RTA log after VRTA process
                              File.Copy(SrcFolder + "\\RTA\\RTA.txt", destFolder + "\\" + fileName + "_rta.txt", true);
                              return true;
                          }
                          else
                              WriteErrorLog("Run VRTA failed! " + CapFileFullName);
                      }
                      else
                          WriteErrorLog("Run BWA failed! " + CapFileFullName);
                  }
                  else if (ExpertName.ToLower() == "genius")
                  {
                      ProgArgs = "-E:Genius -C:\"" + CapFileFullName + "\"";
                      string gSrcFolder = "c:\\temp";
                      if (StartProgram(Exe, ProgArgs, string.Empty, true))
                      {
                          File.Copy(gSrcFolder + "\\ProtocolBody.bin", destFolder + "\\" + fileName + "_genius.txt", true);
                          return true;
                      }
                      else
                          WriteErrorLog("Run RTA failed! " + CapFileFullName + ".cap");

                  }
                  else
                  {
                      WriteErrorLog("Expert name doesn't match!");
                  }
              }
              catch (Exception e)
              {
                  WriteErrorLog(e.ToString());
              }
              return false;
          }

          public static void ParseRTAFile(string fileName, string title)
          {
              string dir = Path.GetDirectoryName(fileName);


              if (dir == "")
                  dir = ".";

              Console.WriteLine("Dir:" + dir);


              string fName = dir + "\\" + Path.GetFileNameWithoutExtension(fileName) + ".html";
              StreamWriter htmlwriter = new StreamWriter(fName, false);


              htmlwriter.WriteLine("<html><head><title>" + title + "</title></head><body>");
              htmlwriter.WriteLine("<p>&nbsp;</p><p align=\"center\"><b><font size=\"6\">" + title + "</font></b></p>");
              htmlwriter.WriteLine("<p>&nbsp;<b><font size=\"3\"><a href=" + fileName + ">Open Netcom capture File</a></font></b></p>");
              htmlwriter.WriteLine("<p>&nbsp;<b><font size=\"3\"><a href=" + Path.GetFileNameWithoutExtension(fileName) + "_genius.txt>Open Capture file as text</a></font></b></p>");
              htmlwriter.WriteLine("<table border=\"1\" cellpadding=\"0\" cellspacing=\"0\" style=\"border-collapse: collapse\" bordercolor=\"#111111\" width=\"100%\">");
              htmlwriter.WriteLine("<tr>");
              htmlwriter.WriteLine("<td width=\"25%\" align=\"center\"><b>Request URI</b></td>");
              htmlwriter.WriteLine("<td width=\"25%\" align=\"center\"><b>Status Code</b></td>");
              htmlwriter.WriteLine("<td width=\"25%\" align=\"center\"><b>Total Bytes</b></td>");
              htmlwriter.WriteLine("<td width=\"25%\" align=\"center\"><b>Number of Roundtrips </b></td>");
              htmlwriter.WriteLine("<td width=\"25%\" align=\"center\"><b>Request</b></td>");
              htmlwriter.WriteLine("<td width=\"25%\" align=\"center\"><b>Relative Time</b></td>");
              htmlwriter.WriteLine("<td width=\"25%\" align=\"center\"><b>Time to 1st Byte</b></td>");
              htmlwriter.WriteLine("<td width=\"25%\" align=\"center\"><b>Duration</b></td>");
              htmlwriter.WriteLine("<td width=\"25%\" align=\"center\"><b>Referrer Request</b></td>");
              htmlwriter.WriteLine("<td width=\"25%\" align=\"center\"><b>URI Args</b></td>");
              htmlwriter.WriteLine("</tr>");


              StreamReader fileReader = new StreamReader(dir + "\\" + Path.GetFileNameWithoutExtension(fileName) + "_rta.txt");


              try
              {


                  string line1 = fileReader.ReadLine();

                  int Rts = 0;
                  int numRequests = 0;


                  string[] strs1 = null;
                  string[] strs2 = null;
                  string[] nodes = line1.Split(new char[] { '\t' });




                  line1 = fileReader.ReadLine();
                  string line2 = fileReader.ReadLine();


                  strs1 = line1.Split(new char[] { '\t' });

                  //lets make sure we are starting from teh right frame

                  while (line1 != null)
                  {
                      strs1 = line1.Split(new char[] { '\t' });
                      if (line2 != null)
                          strs2 = line2.Split(new char[] { '\t' });
                      if (strs1[3].IndexOf("Stop Capture") >= 0)
                      {
                          line1 = line2;
                          line2 = fileReader.ReadLine();
                          continue;

                      }

                      else
                      {
                          if (strs1[0].IndexOf("TOTALS") >= 0)
                          {
                              htmlwriter.WriteLine("<tr>");
                              htmlwriter.WriteLine("<td width=\"25%\"><b> TOTALS: Number of Requests:" + numRequests + "</b></td>");
                              htmlwriter.WriteLine("<td width=\"25%\" align=\"center\"><b>" + strs1[4] + "</b></td>");
                              htmlwriter.WriteLine("<td width=\"25%\" align=\"center\"><b>" + strs1[6] + "</b></td>");
                              htmlwriter.WriteLine("<td width=\"25%\" align=\"center\"><b>" + Rts + "</b></td>");
                              htmlwriter.WriteLine("<td width=\"25%\" align=\"center\"><b>" + strs1[18] + "</b></td>");
                              htmlwriter.WriteLine("<td width=\"25%\" align=\"center\"><b>" + strs1[1] + "</b></td>");
                              htmlwriter.WriteLine("<td width=\"25%\" align=\"center\"><b>" + strs1[12] + "</b></td>");
                              htmlwriter.WriteLine("<td width=\"25%\" align=\"center\"><b>" + strs1[10] + "</b></td>");
                              htmlwriter.WriteLine("<td width=\"25%\"><b>" + strs1[19] + "</b></td>");
                              htmlwriter.WriteLine("<td width=\"25%\"><b>" + strs1[25] + "</b></td>");
                              htmlwriter.WriteLine("</tr>");
                              line1 = line2;
                              line2 = fileReader.ReadLine();

                          }
                          else
                          {

                              htmlwriter.WriteLine("<tr>");
                              htmlwriter.WriteLine("<td width=\"25%\"> " + strs1[3] + "</td>");
                              htmlwriter.WriteLine("<td width=\"25%\" align=\"center\">" + strs1[4] + "</td>");
                              htmlwriter.WriteLine("<td width=\"25%\" align=\"center\">" + strs1[6] + "</td>");
                              htmlwriter.WriteLine("<td width=\"25%\" align=\"center\">" + strs1[15] + "</td>");
                              htmlwriter.WriteLine("<td width=\"25%\" align=\"center\">" + strs1[18] + "</td>");
                              htmlwriter.WriteLine("<td width=\"25%\" align=\"center\">" + strs1[1] + "</td>");
                              htmlwriter.WriteLine("<td width=\"25%\" align=\"center\">" + strs1[12] + "</td>");
                              htmlwriter.WriteLine("<td width=\"25%\" align=\"center\">" + strs1[10] + "</td>");
                              htmlwriter.WriteLine("<td width=\"25%\">" + strs1[19] + "</td>");
                              htmlwriter.WriteLine("<td width=\"25%\">" + strs1[25] + "</td>");
                              htmlwriter.WriteLine("</tr>");


                              numRequests++;
                              Rts = Rts + Convert.ToInt32(strs1[15]);
                              line1 = line2;
                              line2 = fileReader.ReadLine();
                          }
                      }
                  }
              }

              catch (Exception e)
              {
                  Console.WriteLine("Exception:" + e.ToString());

              }

              finally
              {

                  htmlwriter.WriteLine("</table</body></html>");
                  htmlwriter.Close();

                  fileReader.Close();
              }


          }

          public static void WriteErrorLog(string LogString)
          {
              try
              {
                  string ErrLogName = "error.log";
                  StreamWriter sw = new StreamWriter(ErrLogName, true);
                  sw.WriteLine(DateTime.Now.ToString() + " - " + LogString);
                  sw.Close();
              }
              catch (Exception e)
              {
                  Console.WriteLine(e.ToString());
              }
          }
      }    
    }